@@ -2,6 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2016-2016 Planets Communications B.V.
+   Copyright (C) 2016-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -41,18 +43,17 @@ typedef enum {
 } bvfs_handler_type;
 
 typedef enum {
-   BVFS_Type    = 0,            /* Could be D, F, V */
+   BVFS_Type    = 0,            /**< Could be D, F, V */
    BVFS_PathId  = 1,
-   BVFS_FilenameId = 2,
 
-   BVFS_Name    = 3,
-   BVFS_JobId   = 4,
+   BVFS_Name    = 2,
+   BVFS_JobId   = 3,
 
-   BVFS_LStat   = 5,            /* Can be empty for missing directories */
-   BVFS_FileId  = 6,            /* Can be empty for missing directories */
+   BVFS_LStat   = 4,            /**< Can be empty for missing directories */
+   BVFS_FileId  = 5,            /**< Can be empty for missing directories */
 
    /* Only if File Version record */
-   BVFS_Md5     = 3,
+   BVFS_Md5     = 6,
    BVFS_VolName = 7,
    BVFS_VolInchanger = 8
 } bvfs_row_index;
@@ -77,13 +78,14 @@ public:
    void set_pattern(char *p) {
       uint32_t len = strlen(p);
       pattern = check_pool_memory_size(pattern, len * 2 + 1);
-      db_escape_string(jcr, db, pattern, p, len);
+      db->escape_string(jcr, pattern, p, len);
    }
 
    /* Get the root point */
    DBId_t get_root();
 
-   /* It's much better to access Path though their PathId, it
+   /*
+    * It's much better to access Path though their PathId, it
     * avoids mistakes with string encoding
     */
    void ch_dir(DBId_t pathid) {
@@ -98,8 +100,8 @@ public:
 
    bool ls_files();             /* Returns true if we have more files to read */
    bool ls_dirs();              /* Returns true if we have more dir to read */
-   void ls_special_dirs();      /* get . and .. */
-   void get_all_file_versions(DBId_t pathid, DBId_t fnid, const char *client);
+   void get_all_file_versions(const char *path, const char *fname, const char *client);
+   void get_all_file_versions(DBId_t pathid, const char *fname, const char *client);
 
    void update_cache();
 
@@ -161,15 +163,12 @@ private:
    uint32_t nb_record;          /* number of records of the last query */
    POOLMEM *pattern;
    DBId_t pwd_id;               /* Current pathid */
-   DBId_t dir_filenameid;       /* special FilenameId where Name='' */
    POOLMEM *prev_dir; /* ls_dirs query returns all versions, take the 1st one */
    ATTR *attr;        /* Can be use by handler to call decode_stat() */
 
    bool see_all_versions;
    bool see_copies;
 
-   DBId_t get_dir_filenameid();
-
    DB_RESULT_HANDLER *list_entries;
    void *user_data;
 };
@@ -178,16 +177,12 @@ private:
 #define bvfs_is_file(row) ((row)[BVFS_Type][0] == BVFS_FILE_RECORD)
 #define bvfs_is_version(row) ((row)[BVFS_Type][0] == BVFS_FILE_VERSION)
 
-
-bool bvfs_update_path_hierarchy_cache(JCR *jcr, B_DB *mdb, char *jobids);
-void bvfs_update_cache(JCR *jcr, B_DB *mdb);
 char *bvfs_parent_dir(char *path);
 
-/* Return the basename of the with the trailing /  (update the given string)
- * TODO: see in the rest of bareos if we don't have
- * this function already
+/*
+ * Return the basename of the with the trailing / (update the given string)
+ * TODO: see in the rest of bareos if we don't have this function already
  */
 char *bvfs_basename_dir(char *path);
 
-
 #endif /* __BVFS_H_ */
