@@ -33,16 +33,6 @@
 
 SET AUTOCOMMIT ON\g
 
-CREATE SEQUENCE Filename_Seq;
-CREATE TABLE Filename
-(
-   FilenameId		INTEGER 	NOT NULL DEFAULT Filename_Seq.nextval,
-   Name 		VARBYTE(32000)	NOT NULL,
-   PRIMARY KEY (FilenameId)
-);
-
-CREATE UNIQUE INDEX (filename_name_idx ON filename (Name) WITH STRUCTURE=HASH,PAGE_SIZE=32768);
-
 CREATE SEQUENCE Path_Seq;
 CREATE TABLE path
 (
@@ -60,7 +50,7 @@ CREATE TABLE file
    FileIndex		INTEGER 	NOT NULL DEFAULT 0,
    JobId		INTEGER 	NOT NULL,
    PathId		INTEGER 	NOT NULL,
-   FilenameId		INTEGER 	NOT NULL,
+   Name 		VARBYTE(32000)	NOT NULL,
    DeltaSeq		INTEGER 	NOT NULL DEFAULT 0,
    MarkId		INTEGER 	NOT NULL DEFAULT 0,
    LStat		VARBYTE(255)	NOT NULL,
@@ -68,7 +58,7 @@ CREATE TABLE file
    PRIMARY KEY (fileid)
 );
 
-CREATE INDEX file_jpfid_idx ON File (jobid, pathid, filenameid);
+CREATE INDEX file_jpfid_idx ON File (jobid, pathid, name);
 
 -- If you need performances, you can remove this index
 -- the database engine is able to use the composite index
@@ -395,14 +385,15 @@ CREATE TABLE BaseFiles
 
 CREATE INDEX basefiles_jobid_idx ON BaseFiles (JobId);
 
-CREATE TABLE UnsavedFiles
-(
-   UnsavedId		INTEGER 	NOT NULL,
-   JobId		INTEGER 	NOT NULL,
-   PathId		INTEGER 	NOT NULL,
-   FilenameId		INTEGER 	NOT NULL,
-   PRIMARY KEY (UnsavedId)
-);
+-- This table seems to be obsolete
+-- CREATE TABLE UnsavedFiles
+-- (
+--    UnsavedId		INTEGER 	NOT NULL,
+--    JobId		INTEGER 	NOT NULL,
+--    PathId		INTEGER 	NOT NULL,
+--    FilenameId		INTEGER 	NOT NULL,
+--    PRIMARY KEY (UnsavedId)
+-- );
 
 CREATE TABLE PathHierarchy
 (
@@ -545,7 +536,7 @@ INSERT INTO Status (JobStatus,JobStatusLong,Severity) VALUES
 INSERT INTO Status (JobStatus,JobStatusLong,Severity) VALUES
    ('i', 'Doing batch insert file records',15);
 
-INSERT INTO Version (VersionId) VALUES (2004);
+INSERT INTO Version (VersionId) VALUES (2170);
 
 -- Make sure we have appropriate permissions
 \g
