@@ -1,13 +1,3 @@
-CREATE TABLE Filename
-(
-    FilenameId        SERIAL      NOT NULL,
-    Name              TEXT        NOT NULL,
-    PRIMARY KEY (FilenameId)
-);
-
-ALTER TABLE Filename ALTER COLUMN Name SET STATISTICS 1000;
-CREATE UNIQUE INDEX filename_name_idx ON Filename (Name);
-
 CREATE TABLE Path
 (
     PathId            SERIAL      NOT NULL,
@@ -20,34 +10,40 @@ CREATE UNIQUE INDEX path_name_idx ON Path (Path);
 
 -- We strongly recommend to avoid the temptation to add new indexes.
 -- In general, these will cause very significant performance
--- problems in other areas.  A better approch is to carefully check
--- that all your memory configuation parameters are
--- suitable for the size of your installation.  If you backup
+-- problems in other areas.  A better approach is to carefully check
+-- that all your memory configuration parameters are
+-- suitable for the size of your installation. If you backup
 -- millions of files, you need to adapt the database memory
 -- configuration parameters concerning sorting, joining and global
--- memory.  By DEFAULT, sort and join parameters are very small
+-- memory.  By default, sort and join parameters are very small
 -- (sometimes 8Kb), and having sufficient memory specified by those
 -- parameters is extremely important to run fast.
 
 -- In File table
--- FileIndex can be 0 for FT_DELETED files
--- FileNameId can link to Filename.Name='' for directories
-CREATE TABLE File
-(
-    FileId            BIGSERIAL   NOT NULL,
-    FileIndex         INTEGER     NOT NULL  DEFAULT 0,
-    JobId             INTEGER     NOT NULL,
-    PathId            INTEGER     NOT NULL,
-    FilenameId        INTEGER     NOT NULL,
-    DeltaSeq          SMALLINT    NOT NULL  DEFAULT 0,
-    MarkId            INTEGER     NOT NULL  DEFAULT 0,
-    LStat             TEXT        NOT NULL,
-    Md5               TEXT        NOT NULL,
-    PRIMARY KEY (FileId)
+-- FileIndex is 0 for FT_DELETED files
+-- Name is '' for directories
+CREATE TABLE File (
+   FileId           BIGSERIAL   NOT NULL,
+   FileIndex        INTEGER     NOT NULL  DEFAULT 0,
+   JobId            INTEGER     NOT NULL,
+   PathId           INTEGER     NOT NULL,
+   DeltaSeq         SMALLINT    NOT NULL  DEFAULT 0,
+   MarkId           INTEGER     NOT NULL  DEFAULT 0,
+   Fhinfo           NUMERIC(20) NOT NULL  DEFAULT 0,
+   Fhnode           NUMERIC(20) NOT NULL  DEFAULT 0,
+   LStat            TEXT        NOT NULL,
+   Md5              TEXT        NOT NULL,
+   Name             TEXT        NOT NULL,
+   PRIMARY KEY (FileId)
 );
-
-CREATE INDEX file_jpfid_idx ON File (JobId, PathId, FilenameId);
-CREATE INDEX file_jobid_idx ON File (JobId);
+CREATE INDEX file_jpfid_idx ON File (JobId, PathId, Name);
+-- This index is important for bvfs performance, especially
+-- for .bvfs_lsdirs which is used by bareos-webui.
+-- As it's a partial index, it will only contain data from
+-- from accurate jobs with delete directories, so that the
+-- impact on backups will be low. Nevertheless, it will
+-- improve the performance, even when not using accurate.
+CREATE INDEX file_pjidpart_idx ON File(PathId,JobId) WHERE FileIndex = 0 AND Name = '';
 
 --
 -- Add this if you have a good number of job
@@ -56,11 +52,10 @@ CREATE INDEX file_jobid_idx ON File (JobId);
 
 --
 -- Possibly add one or more of the following indexes
---  if your Verifies are too slow, but they can slow down
---  backups.
+--  to the above File table if your Verifies are
+--  too slow, but they can slow down backups.
 --
 -- CREATE INDEX file_pathid_idx ON file(pathid);
--- CREATE INDEX file_filenameid_idx ON file(filenameid);
 
 CREATE TABLE RestoreObject (
     RestoreObjectId   SERIAL      NOT NULL,
@@ -128,6 +123,7 @@ CREATE TABLE Fileset
 (
     FileSetId         SERIAL      NOT NULL,
     Fileset           TEXT        NOT NULL,
+    FileSetText       TEXT        DEFAULT '',
     Md5               TEXT        NOT NULL,
     CreateTime        TIMESTAMP WITHOUT TIME ZONE NOT NULL,
     PRIMARY KEY (filesetid)
@@ -146,6 +142,7 @@ CREATE TABLE JobMedia
     EndFile           INTEGER     DEFAULT 0,
     StartBlock        BIGINT      DEFAULT 0,
     EndBlock          BIGINT      DEFAULT 0,
+    JobBytes          NUMERIC(20) DEFAULT 0,
     VolIndex          INTEGER     DEFAULT 0,
     PRIMARY KEY (jobmediaid)
 );
@@ -335,14 +332,15 @@ CREATE TABLE basefiles
 
 CREATE INDEX basefiles_jobid_idx ON BaseFiles (JobId);
 
-CREATE TABLE UnsavedFiles
-(
-    UnsavedId         INTEGER     NOT NULL,
-    JobId             INTEGER     NOT NULL,
-    PathId            INTEGER     NOT NULL,
-    FilenameId        INTEGER     NOT NULL,
-    PRIMARY KEY (UnsavedId)
-);
+-- This table seems to be obsolete
+-- CREATE TABLE UnsavedFiles
+-- (
+--     UnsavedId         INTEGER     NOT NULL,
+--     JobId             INTEGER     NOT NULL,
+--     PathId            INTEGER     NOT NULL,
+--     FilenameId        INTEGER     NOT NULL,
+--     PRIMARY KEY (UnsavedId)
+-- );
 
 CREATE TABLE PathHierarchy
 (
@@ -486,7 +484,7 @@ INSERT INTO Status (JobStatus,JobStatusLong,Severity) VALUES
 -- Initialize Version
 --   DELETE should not be required,
 --   but prevents errors if create script is called multiple times
-DELETE FROM Version WHERE VersionId<=2003;
-INSERT INTO Version (VersionId) VALUES (2003);
+DELETE FROM Version WHERE VersionId<=2171;
+INSERT INTO Version (VersionId) VALUES (2171);
 
 -- Make sure we have appropriate permissions
