@@ -1,6 +1,5 @@
 -- read-only access for third-party applications
 -- for tables
-GRANT SELECT ON TABLE Filename TO @DB_USER@;
 GRANT SELECT ON TABLE Path TO @DB_USER@;
 GRANT SELECT ON TABLE File TO @DB_USER@;
 GRANT SELECT ON TABLE RestoreObject TO @DB_USER@;
@@ -19,7 +18,6 @@ GRANT SELECT ON TABLE Log TO @DB_USER@;
 GRANT SELECT ON TABLE LocationLog TO @DB_USER@;
 GRANT SELECT ON TABLE Counters TO @DB_USER@;
 GRANT SELECT ON TABLE BaseFiles TO @DB_USER@;
-GRANT SELECT ON TABLE UnsavedFiles TO @DB_USER@;
 GRANT SELECT ON TABLE PathHierarchy TO @DB_USER@;
 GRANT SELECT ON TABLE PathVisibility TO @DB_USER@;
 GRANT SELECT ON TABLE Version TO @DB_USER@;
