@@ -1,7 +1,6 @@
 -- read-only access for third party applications
 CREATE USER @DB_USER@ @DB_PASS@;
 
-GRANT SELECT ON UnSavedFiles TO @DB_USER@;
 GRANT SELECT ON BaseFiles TO @DB_USER@;
 GRANT SELECT ON JobMedia TO @DB_USER@;
 GRANT SELECT ON File TO @DB_USER@;
@@ -11,7 +10,6 @@ GRANT SELECT ON Client TO @DB_USER@;
 GRANT SELECT ON Pool TO @DB_USER@;
 GRANT SELECT ON Fileset TO @DB_USER@;
 GRANT SELECT ON Path TO @DB_USER@;
-GRANT SELECT ON Filename TO @DB_USER@;
 GRANT SELECT ON Counters TO @DB_USER@;
 GRANT SELECT ON Version TO @DB_USER@;
 GRANT SELECT ON MediaType TO @DB_USER@;
