@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,13 +21,14 @@
    02110-1301, USA.
 */
 /*
+ * Kern Sibbald, March 2000
+ * Major rewrite by Marco van Wieringen, January 2010 for catalog refactoring.
+ */
+/**
+ * @file
  * BAREOS Catalog Database routines specific to MySQL
  * These are MySQL specific routines -- hopefully all
  * other files are generic.
- *
- * Kern Sibbald, March 2000
- *
- * Major rewrite by Marco van Wieringen, January 2010 for catalog refactoring.
  */
 
 #include "bareos.h"
@@ -35,10 +36,12 @@
 #ifdef HAVE_MYSQL
 
 #include "cats.h"
-#include "bdb_priv.h"
 #include <mysql.h>
 #include <errmsg.h>
-#include <bdb_mysql.h>
+#include "bdb_mysql.h"
+
+/* pull in the generated queries definitions */
+#include "mysql_queries.inc"
 
 /* -----------------------------------------------------------------------
  *
@@ -47,7 +50,7 @@
  * -----------------------------------------------------------------------
  */
 
-/*
+/**
  * List of open databases
  */
 static dlist *db_list = NULL;
@@ -66,7 +69,8 @@ B_DB_MYSQL::B_DB_MYSQL(JCR *jcr,
                        bool disable_batch_insert,
                        bool try_reconnect,
                        bool exit_on_fatal,
-                       bool need_private)
+                       bool need_private
+                       )
 {
    /*
     * Initialize the parent class members.
@@ -117,6 +121,8 @@ B_DB_MYSQL::B_DB_MYSQL(JCR *jcr,
    m_is_private = need_private;
    m_try_reconnect = try_reconnect;
    m_exit_on_fatal = exit_on_fatal;
+   m_last_hash_key = 0;
+   m_last_query_text = NULL;
 
    /*
     * Initialize the private members.
@@ -131,22 +137,26 @@ B_DB_MYSQL::B_DB_MYSQL(JCR *jcr,
       db_list = New(dlist(this, &this->m_link));
    }
    db_list->append(this);
+
+   /* make the queries available using the queries variable from the parent class */
+   queries = query_definitions;
 }
 
 B_DB_MYSQL::~B_DB_MYSQL()
 {
 }
 
-/*
+/**
  * Now actually open the database.  This can generate errors,
  *  which are returned in the errmsg
  *
  * DO NOT close the database or delete mdb here !!!!
  */
-bool B_DB_MYSQL::db_open_database(JCR *jcr)
+bool B_DB_MYSQL::open_database(JCR *jcr)
 {
    bool retval = false;
    int errstat;
+   my_bool reconnect = 1;
 
    P(mutex);
    if (m_connected) {
@@ -191,7 +201,7 @@ bool B_DB_MYSQL::db_open_database(JCR *jcr)
       bmicrosleep(5,0);
    }
 
-   m_instance.reconnect = 1;             /* so connection does not timeout */
+   mysql_options(&m_instance, MYSQL_OPT_RECONNECT, &reconnect);    /* so connection does not timeout */
    Dmsg0(50, "mysql_real_connect done\n");
    Dmsg3(50, "db_user=%s db_name=%s db_password=%s\n", m_db_user, m_db_name,
         (m_db_password == NULL) ? "(NULL)" : m_db_password);
@@ -213,7 +223,7 @@ bool B_DB_MYSQL::db_open_database(JCR *jcr)
    }
 
    m_connected = true;
-   if (!check_tables_version(jcr, this)) {
+   if (!check_tables_version(jcr)) {
       goto bail_out;
    }
 
@@ -222,8 +232,8 @@ bool B_DB_MYSQL::db_open_database(JCR *jcr)
    /*
     * Set connection timeout to 8 days specialy for batch mode
     */
-   sql_query("SET wait_timeout=691200");
-   sql_query("SET interactive_timeout=691200");
+   sql_query_without_handler("SET wait_timeout=691200");
+   sql_query_without_handler("SET interactive_timeout=691200");
 
    retval = true;
 
@@ -232,10 +242,10 @@ bail_out:
    return retval;
 }
 
-void B_DB_MYSQL::db_close_database(JCR *jcr)
+void B_DB_MYSQL::close_database(JCR *jcr)
 {
    if (m_connected) {
-      db_end_transaction(jcr);
+      end_transaction(jcr);
    }
    P(mutex);
    m_ref_count--;
@@ -291,7 +301,7 @@ void B_DB_MYSQL::db_close_database(JCR *jcr)
    V(mutex);
 }
 
-bool B_DB_MYSQL::db_validate_connection(void)
+bool B_DB_MYSQL::validate_connection(void)
 {
    bool retval;
    unsigned long mysql_threadid;
@@ -325,7 +335,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * This call is needed because the message channel thread
  *  opens a database on behalf of a jcr that was created in
  *  a different thread. MySQL then allocates thread specific
@@ -333,41 +343,41 @@ bail_out:
  *  closes the database.  Thus the msgchan must call here
  *  to cleanup any thread specific data that it created.
  */
-void B_DB_MYSQL::db_thread_cleanup(void)
+void B_DB_MYSQL::thread_cleanup(void)
 {
 #ifndef HAVE_WIN32
    mysql_thread_end();
 #endif
 }
 
-/*
+/**
  * Escape strings so that MySQL is happy
  *
  *   NOTE! len is the length of the old string. Your new
  *         string must be long enough (max 2*old+1) to hold
  *         the escaped output.
  */
-void B_DB_MYSQL::db_escape_string(JCR *jcr, char *snew, char *old, int len)
+void B_DB_MYSQL::escape_string(JCR *jcr, char *snew, char *old, int len)
 {
    mysql_real_escape_string(m_db_handle, snew, old, len);
 }
 
-/*
+/**
  * Escape binary object so that MySQL is happy
  * Memory is stored in B_DB struct, no need to free it
  */
-char *B_DB_MYSQL::db_escape_object(JCR *jcr, char *old, int len)
+char *B_DB_MYSQL::escape_object(JCR *jcr, char *old, int len)
 {
    esc_obj = check_pool_memory_size(esc_obj, len*2+1);
    mysql_real_escape_string(m_db_handle, esc_obj, old, len);
    return esc_obj;
 }
 
-/*
+/**
  * Unescape binary object so that MySQL is happy
  */
-void B_DB_MYSQL::db_unescape_object(JCR *jcr, char *from, int32_t expected_len,
-                                    POOLMEM *&dest, int32_t *dest_len)
+void B_DB_MYSQL::unescape_object(JCR *jcr, char *from, int32_t expected_len,
+                                 POOLMEM *&dest, int32_t *dest_len)
 {
    if (!from) {
       dest[0] = '\0';
@@ -380,7 +390,7 @@ void B_DB_MYSQL::db_unescape_object(JCR *jcr, char *from, int32_t expected_len,
    dest[expected_len] = '\0';
 }
 
-void B_DB_MYSQL::db_start_transaction(JCR *jcr)
+void B_DB_MYSQL::start_transaction(JCR *jcr)
 {
    if (!jcr->attr) {
       jcr->attr = get_pool_memory(PM_FNAME);
@@ -390,22 +400,22 @@ void B_DB_MYSQL::db_start_transaction(JCR *jcr)
    }
 }
 
-void B_DB_MYSQL::db_end_transaction(JCR *jcr)
+void B_DB_MYSQL::end_transaction(JCR *jcr)
 {
    if (jcr && jcr->cached_attribute) {
       Dmsg0(400, "Flush last cached attribute.\n");
-      if (!db_create_attributes_record(jcr, this, jcr->ar)) {
-         Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), db_strerror(jcr->db));
+      if (!create_attributes_record(jcr, jcr->ar)) {
+         Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), strerror());
       }
       jcr->cached_attribute = false;
    }
 }
 
-/*
+/**
  * Submit a general SQL command (cmd), and for each row returned,
  * the result_handler is called with the ctx.
  */
-bool B_DB_MYSQL::db_sql_query(const char *query, DB_RESULT_HANDLER *result_handler, void *ctx)
+bool B_DB_MYSQL::sql_query_with_handler(const char *query, DB_RESULT_HANDLER *result_handler, void *ctx)
 {
    int status;
    SQL_ROW row;
@@ -413,7 +423,7 @@ bool B_DB_MYSQL::db_sql_query(const char *query, DB_RESULT_HANDLER *result_handl
    bool retry = true;
    bool retval = false;
 
-   Dmsg1(500, "db_sql_query starts with %s\n", query);
+   Dmsg1(500, "sql_query_with_handler starts with %s\n", query);
 
    db_lock(this);
 
@@ -460,11 +470,11 @@ retry_query:
       /* FALL THROUGH */
    default:
       Mmsg(errmsg, _("Query failed: %s: ERR=%s\n"), query, sql_strerror());
-      Dmsg0(500, "db_sql_query failed\n");
+      Dmsg0(500, "sql_query_with_handler failed\n");
       goto bail_out;
    }
 
-   Dmsg0(500, "db_sql_query succeeded. checking handler\n");
+   Dmsg0(500, "sql_query_with_handler succeeded. checking handler\n");
 
    if (result_handler != NULL) {
       if ((m_result = mysql_use_result(m_db_handle)) != NULL) {
@@ -488,7 +498,7 @@ retry_query:
       }
    }
 
-   Dmsg0(500, "db_sql_query finished\n");
+   Dmsg0(500, "sql_query_with_handler finished\n");
    retval = true;
 
 bail_out:
@@ -496,13 +506,13 @@ bail_out:
    return retval;
 }
 
-bool B_DB_MYSQL::sql_query(const char *query, int flags)
+bool B_DB_MYSQL::sql_query_without_handler(const char *query, int flags)
 {
    int status;
    bool retry = true;
    bool retval = true;
 
-   Dmsg1(500, "sql_query starts with '%s'\n", query);
+   Dmsg1(500, "sql_query_without_handler starts with '%s'\n", query);
 
    /*
     * We are starting a new query. reset everything.
@@ -517,6 +527,11 @@ retry_query:
       m_result = NULL;
    }
 
+   /*
+    * query has to be a single SQL statement (not multiple joined by columns (";")).
+    * If multiple SQL statements have to be used, they can produce multiple results,
+    * each of them needs handling.
+    */
    status = mysql_query(m_db_handle, query);
    switch (status) {
    case 0:
@@ -686,7 +701,7 @@ bool B_DB_MYSQL::sql_field_is_numeric(int field_type)
    return IS_NUM(field_type);
 }
 
-/*
+/**
  * Returns true if OK
  *         false if failed
  */
@@ -702,7 +717,9 @@ bool B_DB_MYSQL::sql_batch_start(JCR *jcr)
                               "Name blob,"
                               "LStat tinyblob,"
                               "MD5 tinyblob,"
-                              "DeltaSeq integer)");
+                              "DeltaSeq integer,"
+                              "Fhinfo NUMERIC(20),"
+                              "Fhnode NUMERIC(20) )");
    db_unlock(this);
 
    /*
@@ -714,7 +731,7 @@ bool B_DB_MYSQL::sql_batch_start(JCR *jcr)
 }
 
 /* set error to something to abort operation */
-/*
+/**
  * Returns true if OK
  *         false if failed
  */
@@ -732,20 +749,20 @@ bool B_DB_MYSQL::sql_batch_end(JCR *jcr, const char *error)
    return true;
 }
 
-/*
+/**
  * Returns true if OK
  *         false if failed
  */
 bool B_DB_MYSQL::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
 {
    const char *digest;
-   char ed1[50];
+   char ed1[50], ed2[50], ed3[50];
 
    esc_name = check_pool_memory_size(esc_name, fnl*2+1);
-   db_escape_string(jcr, esc_name, fname, fnl);
+   escape_string(jcr, esc_name, fname, fnl);
 
    esc_path = check_pool_memory_size(esc_path, pnl*2+1);
-   db_escape_string(jcr, esc_path, path, pnl);
+   escape_string(jcr, esc_path, path, pnl);
 
    if (ar->Digest == NULL || ar->Digest[0] == 0) {
       digest = "0";
@@ -758,18 +775,20 @@ bool B_DB_MYSQL::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
     */
    if (changes == 0) {
       Mmsg(cmd, "INSERT INTO batch VALUES "
-           "(%u,%s,'%s','%s','%s','%s',%u)",
+           "(%u,%s,'%s','%s','%s','%s',%u,'%s','%s')",
            ar->FileIndex, edit_int64(ar->JobId,ed1), esc_path,
-           esc_name, ar->attr, digest, ar->DeltaSeq);
+           esc_name, ar->attr, digest, ar->DeltaSeq,
+           edit_uint64(ar->Fhinfo,ed2),
+           edit_uint64(ar->Fhnode,ed3));
       changes++;
    } else {
       /*
        * We use the esc_obj for temporary storage otherwise
        * we keep on copying data.
        */
-      Mmsg(esc_obj, ",(%u,%s,'%s','%s','%s','%s',%u)",
+      Mmsg(esc_obj, ",(%u,%s,'%s','%s','%s','%s',%u,%u,%u)",
            ar->FileIndex, edit_int64(ar->JobId,ed1), esc_path,
-           esc_name, ar->attr, digest, ar->DeltaSeq);
+           esc_name, ar->attr, digest, ar->DeltaSeq, ar->Fhinfo, ar->Fhnode);
       pm_strcat(cmd, esc_obj);
       changes++;
    }
@@ -789,7 +808,7 @@ bool B_DB_MYSQL::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
    return true;
 }
 
-/*
+/**
  * Initialize database data structure. In principal this should
  * never have errors, or it is really fatal.
  */
@@ -840,7 +859,7 @@ B_DB *db_init_database(JCR *jcr,
             continue;
          }
 
-         if (mdb->db_match_database(db_driver, db_name, db_address, db_port)) {
+         if (mdb->match_database(db_driver, db_name, db_address, db_port)) {
             Dmsg1(100, "DB REopen %s\n", db_name);
             mdb->increment_refcount();
             goto bail_out;
@@ -860,7 +879,8 @@ B_DB *db_init_database(JCR *jcr,
                         disable_batch_insert,
                         try_reconnect,
                         exit_on_fatal,
-                        need_private));
+                        need_private
+                        ));
 
 bail_out:
    V(mutex);
