@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,13 +21,15 @@
    02110-1301, USA.
 */
 /*
+ * Kern Sibbald, December 2000
+ */
+/**
+ * @file
  * BAREOS Catalog Database Find record interface routines
  *
  * Note, generally, these routines are more complicated
  * that a simple search by name or id. Such simple
  * request are in get.c
- *
- * Kern Sibbald, December 2000
  */
 
 #include "bareos.h"
@@ -35,8 +37,6 @@
 #if HAVE_SQLITE3 || HAVE_MYSQL || HAVE_POSTGRESQL || HAVE_INGRES || HAVE_DBI
 
 #include "cats.h"
-#include "bdb_priv.h"
-#include "sql_glue.h"
 
 /* -----------------------------------------------------------------------
  *
@@ -45,7 +45,7 @@
  * -----------------------------------------------------------------------
  */
 
-/*
+/**
  * Find job start time if JobId specified, otherwise
  * find last Job start time Incremental and Differential saves.
  *
@@ -55,26 +55,26 @@
  * Returns: 0 on failure
  *          1 on success, jr is unchanged, but stime and job are set
  */
-bool db_find_job_start_time(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM *&stime, char *job)
+bool B_DB::find_job_start_time(JCR *jcr, JOB_DBR *jr, POOLMEM *&stime, char *job)
 {
    bool retval = false;
    SQL_ROW row;
    char ed1[50], ed2[50];
-   char esc_name[MAX_ESCAPE_NAME_LENGTH];
+   char esc_jobname[MAX_ESCAPE_NAME_LENGTH];
 
-   db_lock(mdb);
-   mdb->db_escape_string(jcr, esc_name, jr->Name, strlen(jr->Name));
+   db_lock(this);
+   escape_string(jcr, esc_jobname, jr->Name, strlen(jr->Name));
    pm_strcpy(stime, "0000-00-00 00:00:00");   /* default */
    job[0] = 0;
 
    /* If no Id given, we must find corresponding job */
    if (jr->JobId == 0) {
       /* Differential is since last Full backup */
-      Mmsg(mdb->cmd,
+      Mmsg(cmd,
 "SELECT StartTime, Job FROM Job WHERE JobStatus IN ('T','W') AND Type='%c' AND "
 "Level='%c' AND Name='%s' AND ClientId=%s AND FileSetId=%s "
 "ORDER BY StartTime DESC LIMIT 1",
-           jr->JobType, L_FULL, esc_name,
+           jr->JobType, L_FULL, esc_jobname,
            edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2));
 
       if (jr->JobLevel == L_DIFFERENTIAL) {
@@ -88,58 +88,58 @@ bool db_find_job_start_time(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM *&stime, c
           *  then we do a second look to find the most recent
           *  backup
           */
-         if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
-            Mmsg2(mdb->errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), sql_strerror(mdb), mdb->cmd);
+         if (!QUERY_DB(jcr, cmd)) {
+            Mmsg2(errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), sql_strerror(), cmd);
             goto bail_out;
          }
-         if ((row = sql_fetch_row(mdb)) == NULL) {
-            sql_free_result(mdb);
-            Mmsg(mdb->errmsg, _("No prior Full backup Job record found.\n"));
+         if ((row = sql_fetch_row()) == NULL) {
+            sql_free_result();
+            Mmsg(errmsg, _("No prior Full backup Job record found.\n"));
             goto bail_out;
          }
-         sql_free_result(mdb);
+         sql_free_result();
          /* Now edit SQL command for Incremental Job */
-         Mmsg(mdb->cmd,
+         Mmsg(cmd,
 "SELECT StartTime, Job FROM Job WHERE JobStatus IN ('T','W') AND Type='%c' AND "
 "Level IN ('%c','%c','%c') AND Name='%s' AND ClientId=%s "
 "AND FileSetId=%s ORDER BY StartTime DESC LIMIT 1",
-            jr->JobType, L_INCREMENTAL, L_DIFFERENTIAL, L_FULL, esc_name,
+            jr->JobType, L_INCREMENTAL, L_DIFFERENTIAL, L_FULL, esc_jobname,
             edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2));
       } else {
-         Mmsg1(mdb->errmsg, _("Unknown level=%d\n"), jr->JobLevel);
+         Mmsg1(errmsg, _("Unknown level=%d\n"), jr->JobLevel);
          goto bail_out;
       }
    } else {
-      Dmsg1(100, "Submitting: %s\n", mdb->cmd);
-      Mmsg(mdb->cmd, "SELECT StartTime, Job FROM Job WHERE Job.JobId=%s",
+      Dmsg1(100, "Submitting: %s\n", cmd);
+      Mmsg(cmd, "SELECT StartTime, Job FROM Job WHERE Job.JobId=%s",
            edit_int64(jr->JobId, ed1));
    }
 
-   if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
+   if (!QUERY_DB(jcr, cmd)) {
       pm_strcpy(stime, "");                   /* set EOS */
-      Mmsg2(mdb->errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), sql_strerror(mdb),  mdb->cmd);
+      Mmsg2(errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), sql_strerror(), cmd);
       goto bail_out;
    }
 
-   if ((row = sql_fetch_row(mdb)) == NULL) {
-      Mmsg2(mdb->errmsg, _("No Job record found: ERR=%s\nCMD=%s\n"), sql_strerror(mdb),  mdb->cmd);
-      sql_free_result(mdb);
+   if ((row = sql_fetch_row()) == NULL) {
+      Mmsg2(errmsg, _("No Job record found: ERR=%s\nCMD=%s\n"), sql_strerror(), cmd);
+      sql_free_result();
       goto bail_out;
    }
    Dmsg2(100, "Got start time: %s, job: %s\n", row[0], row[1]);
    pm_strcpy(stime, row[0]);
    bstrncpy(job, row[1], MAX_NAME_LENGTH);
 
-   sql_free_result(mdb);
+   sql_free_result();
    retval = true;
 
 bail_out:
-   db_unlock(mdb);
+   db_unlock(this);
    return retval;
 }
 
 
-/*
+/**
  * Find the last job start time for the specified JobLevel
  *
  *  StartTime is returned in stime
@@ -148,47 +148,47 @@ bail_out:
  * Returns: false on failure
  *          true  on success, jr is unchanged, but stime and job are set
  */
-bool db_find_last_job_start_time(JCR *jcr, B_DB *mdb, JOB_DBR *jr,
-                                 POOLMEM *&stime, char *job, int JobLevel)
+bool B_DB::find_last_job_start_time(JCR *jcr, JOB_DBR *jr, POOLMEM *&stime,
+                                    char *job, int JobLevel)
 {
    bool retval = false;
    SQL_ROW row;
    char ed1[50], ed2[50];
-   char esc_name[MAX_ESCAPE_NAME_LENGTH];
+   char esc_jobname[MAX_ESCAPE_NAME_LENGTH];
 
-   db_lock(mdb);
-   mdb->db_escape_string(jcr, esc_name, jr->Name, strlen(jr->Name));
+   db_lock(this);
+   escape_string(jcr, esc_jobname, jr->Name, strlen(jr->Name));
    pm_strcpy(stime, "0000-00-00 00:00:00");   /* default */
    job[0] = 0;
 
-   Mmsg(mdb->cmd,
+   Mmsg(cmd,
 "SELECT StartTime, Job FROM Job WHERE JobStatus IN ('T','W') AND Type='%c' AND "
 "Level='%c' AND Name='%s' AND ClientId=%s AND FileSetId=%s "
 "ORDER BY StartTime DESC LIMIT 1",
-      jr->JobType, JobLevel, esc_name,
+      jr->JobType, JobLevel, esc_jobname,
       edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2));
-   if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
-      Mmsg2(mdb->errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), sql_strerror(mdb), mdb->cmd);
+   if (!QUERY_DB(jcr, cmd)) {
+      Mmsg2(errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), sql_strerror(), cmd);
       goto bail_out;
    }
-   if ((row = sql_fetch_row(mdb)) == NULL) {
-      sql_free_result(mdb);
-      Mmsg(mdb->errmsg, _("No prior Full backup Job record found.\n"));
+   if ((row = sql_fetch_row()) == NULL) {
+      sql_free_result();
+      Mmsg(errmsg, _("No prior Full backup Job record found.\n"));
       goto bail_out;
    }
    Dmsg1(100, "Got start time: %s\n", row[0]);
    pm_strcpy(stime, row[0]);
    bstrncpy(job, row[1], MAX_NAME_LENGTH);
 
-   sql_free_result(mdb);
+   sql_free_result();
    retval = true;
 
 bail_out:
-   db_unlock(mdb);
+   db_unlock(this);
    return retval;
 }
 
-/*
+/**
  * Find last failed job since given start-time
  *   it must be either Full or Diff.
  *
@@ -196,44 +196,43 @@ bail_out:
  *          true  on success, jr is unchanged and stime unchanged
  *                level returned in JobLevel
  */
-bool db_find_failed_job_since(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM *stime, int &JobLevel)
+bool B_DB::find_failed_job_since(JCR *jcr, JOB_DBR *jr, POOLMEM *stime, int &JobLevel)
 {
    bool retval = false;
    SQL_ROW row;
    char ed1[50], ed2[50];
-   char esc_name[MAX_ESCAPE_NAME_LENGTH];
+   char esc_jobname[MAX_ESCAPE_NAME_LENGTH];
 
-   db_lock(mdb);
-   mdb->db_escape_string(jcr, esc_name, jr->Name, strlen(jr->Name));
+   db_lock(this);
+   escape_string(jcr, esc_jobname, jr->Name, strlen(jr->Name));
 
    /* Differential is since last Full backup */
-   Mmsg(mdb->cmd,
+   Mmsg(cmd,
 "SELECT Level FROM Job WHERE JobStatus NOT IN ('T','W') AND "
 "Type='%c' AND Level IN ('%c','%c') AND Name='%s' AND ClientId=%s "
 "AND FileSetId=%s AND StartTime>'%s' "
 "ORDER BY StartTime DESC LIMIT 1",
-         jr->JobType, L_FULL, L_DIFFERENTIAL, esc_name,
+         jr->JobType, L_FULL, L_DIFFERENTIAL, esc_jobname,
          edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2),
          stime);
-   if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
+   if (!QUERY_DB(jcr, cmd)) {
       goto bail_out;
    }
 
-   if ((row = sql_fetch_row(mdb)) == NULL) {
-      sql_free_result(mdb);
+   if ((row = sql_fetch_row()) == NULL) {
+      sql_free_result();
       goto bail_out;
    }
    JobLevel = (int)*row[0];
-   sql_free_result(mdb);
+   sql_free_result();
    retval = true;
 
 bail_out:
-   db_unlock(mdb);
+   db_unlock(this);
    return retval;
 }
 
-
-/*
+/**
  * Find JobId of last job that ran.  E.g. for
  *   VERIFY_CATALOG we want the JobId of the last INIT.
  *   For VERIFY_VOLUME_TO_CATALOG, we want the JobId of the last Job.
@@ -241,69 +240,69 @@ bail_out:
  * Returns: true  on success
  *          false on failure
  */
-bool db_find_last_jobid(JCR *jcr, B_DB *mdb, const char *Name, JOB_DBR *jr)
+bool B_DB::find_last_jobid(JCR *jcr, const char *Name, JOB_DBR *jr)
 {
    bool retval = false;
    SQL_ROW row;
    char ed1[50];
-   char esc_name[MAX_ESCAPE_NAME_LENGTH];
+   char esc_jobname[MAX_ESCAPE_NAME_LENGTH];
 
-   db_lock(mdb);
+   db_lock(this);
    /* Find last full */
    Dmsg2(100, "JobLevel=%d JobType=%d\n", jr->JobLevel, jr->JobType);
    if (jr->JobLevel == L_VERIFY_CATALOG) {
-      mdb->db_escape_string(jcr, esc_name, jr->Name, strlen(jr->Name));
-      Mmsg(mdb->cmd,
+      escape_string(jcr, esc_jobname, jr->Name, strlen(jr->Name));
+      Mmsg(cmd,
 "SELECT JobId FROM Job WHERE Type='V' AND Level='%c' AND "
 " JobStatus IN ('T','W') AND Name='%s' AND "
 "ClientId=%s ORDER BY StartTime DESC LIMIT 1",
-           L_VERIFY_INIT, esc_name,
+           L_VERIFY_INIT, esc_jobname,
            edit_int64(jr->ClientId, ed1));
    } else if (jr->JobLevel == L_VERIFY_VOLUME_TO_CATALOG ||
               jr->JobLevel == L_VERIFY_DISK_TO_CATALOG ||
               jr->JobType == JT_BACKUP) {
       if (Name) {
-         mdb->db_escape_string(jcr, esc_name, (char*)Name,
-                               MIN(strlen(Name), sizeof(esc_name)));
-         Mmsg(mdb->cmd,
+         escape_string(jcr, esc_jobname, (char*)Name,
+                               MIN(strlen(Name), sizeof(esc_jobname)));
+         Mmsg(cmd,
 "SELECT JobId FROM Job WHERE Type='B' AND JobStatus IN ('T','W') AND "
-"Name='%s' ORDER BY StartTime DESC LIMIT 1", esc_name);
+"Name='%s' ORDER BY StartTime DESC LIMIT 1", esc_jobname);
       } else {
-         Mmsg(mdb->cmd,
+         Mmsg(cmd,
 "SELECT JobId FROM Job WHERE Type='B' AND JobStatus IN ('T','W') AND "
 "ClientId=%s ORDER BY StartTime DESC LIMIT 1",
            edit_int64(jr->ClientId, ed1));
       }
    } else {
-      Mmsg1(mdb->errmsg, _("Unknown Job level=%d\n"), jr->JobLevel);
+      Mmsg1(errmsg, _("Unknown Job level=%d\n"), jr->JobLevel);
       goto bail_out;
    }
-   Dmsg1(100, "Query: %s\n", mdb->cmd);
-   if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
+   Dmsg1(100, "Query: %s\n", cmd);
+   if (!QUERY_DB(jcr, cmd)) {
       goto bail_out;
    }
-   if ((row = sql_fetch_row(mdb)) == NULL) {
-      Mmsg1(mdb->errmsg, _("No Job found for: %s.\n"), mdb->cmd);
-      sql_free_result(mdb);
+   if ((row = sql_fetch_row()) == NULL) {
+      Mmsg1(errmsg, _("No Job found for: %s.\n"), cmd);
+      sql_free_result();
       goto bail_out;
    }
 
    jr->JobId = str_to_int64(row[0]);
-   sql_free_result(mdb);
+   sql_free_result();
 
    Dmsg1(100, "db_get_last_jobid: got JobId=%d\n", jr->JobId);
    if (jr->JobId <= 0) {
-      Mmsg1(mdb->errmsg, _("No Job found for: %s\n"), mdb->cmd);
+      Mmsg1(errmsg, _("No Job found for: %s\n"), cmd);
       goto bail_out;
    }
    retval = true;
 
 bail_out:
-   db_unlock(mdb);
+   db_unlock(this);
    return retval;
 }
 
-/*
+/**
  * Search a comma separated list of unwanted volume names and see if given VolumeName is on it.
  */
 static inline bool is_on_unwanted_volumes_list(const char *VolumeName,
@@ -332,7 +331,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Find Available Media (Volume) for Pool
  *
  * Find a Volume for a given PoolId, MediaType, and Status.
@@ -341,7 +340,7 @@ bail_out:
  * Returns: 0 on failure
  *          numrows on success
  */
-int db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR *mr, const char *unwanted_volumes)
+int B_DB::find_next_volume(JCR *jcr, int item, bool InChanger, MEDIA_DBR *mr, const char *unwanted_volumes)
 {
    char ed1[50];
    int num_rows = 0;
@@ -351,10 +350,10 @@ int db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR
    char esc_type[MAX_ESCAPE_NAME_LENGTH];
    char esc_status[MAX_ESCAPE_NAME_LENGTH];
 
-   db_lock(mdb);
+   db_lock(this);
 
-   mdb->db_escape_string(jcr, esc_type, mr->MediaType, strlen(mr->MediaType));
-   mdb->db_escape_string(jcr, esc_status, mr->VolStatus, strlen(mr->VolStatus));
+   escape_string(jcr, esc_type, mr->MediaType, strlen(mr->MediaType));
+   escape_string(jcr, esc_status, mr->VolStatus, strlen(mr->VolStatus));
 
    if (item == -1) {
       find_oldest = true;
@@ -366,21 +365,21 @@ retry_fetch:
       /*
        * Find oldest volume(s)
        */
-      Mmsg(mdb->cmd, "SELECT MediaId,VolumeName,VolJobs,VolFiles,VolBlocks,"
-                     "VolBytes,VolMounts,VolErrors,VolWrites,MaxVolBytes,VolCapacityBytes,"
-                     "MediaType,VolStatus,PoolId,VolRetention,VolUseDuration,MaxVolJobs,"
-                     "MaxVolFiles,Recycle,Slot,FirstWritten,LastWritten,InChanger,"
-                     "EndFile,EndBlock,LabelType,LabelDate,StorageId,"
-                     "Enabled,LocationId,RecycleCount,InitialWrite,"
-                     "ScratchPoolId,RecyclePoolId,VolReadTime,VolWriteTime,"
-                     "ActionOnPurge,EncryptionKey,MinBlocksize,MaxBlocksize "
-                     "FROM Media WHERE PoolId=%s AND MediaType='%s' AND VolStatus IN ('Full',"
-                     "'Recycle','Purged','Used','Append') AND Enabled=1 "
-                     "ORDER BY LastWritten LIMIT %d",
+      Mmsg(cmd, "SELECT MediaId,VolumeName,VolJobs,VolFiles,VolBlocks,"
+                "VolBytes,VolMounts,VolErrors,VolWrites,MaxVolBytes,VolCapacityBytes,"
+                "MediaType,VolStatus,PoolId,VolRetention,VolUseDuration,MaxVolJobs,"
+                "MaxVolFiles,Recycle,Slot,FirstWritten,LastWritten,InChanger,"
+                "EndFile,EndBlock,LabelType,LabelDate,StorageId,"
+                "Enabled,LocationId,RecycleCount,InitialWrite,"
+                "ScratchPoolId,RecyclePoolId,VolReadTime,VolWriteTime,"
+                "ActionOnPurge,EncryptionKey,MinBlocksize,MaxBlocksize "
+                "FROM Media WHERE PoolId=%s AND MediaType='%s' AND VolStatus IN ('Full',"
+                "'Recycle','Purged','Used','Append') AND Enabled=1 "
+                "ORDER BY LastWritten LIMIT %d",
            edit_int64(mr->PoolId, ed1), esc_type, item);
    } else {
-      POOL_MEM changer(PM_FNAME);
-      const char *order;
+      POOL_MEM changer(PM_MESSAGE);
+      POOL_MEM order(PM_MESSAGE);
 
       /*
        * Find next available volume
@@ -391,12 +390,12 @@ retry_fetch:
 
       if (bstrcmp(mr->VolStatus, "Recycle") ||
           bstrcmp(mr->VolStatus, "Purged")) {
-         order = "AND Recycle=1 ORDER BY LastWritten ASC,MediaId";  /* take oldest that can be recycled */
+         pm_strcpy(order, "AND Recycle=1 ORDER BY LastWritten ASC,MediaId"); /* Take oldest that can be recycled */
       } else {
-         order = sql_media_order_most_recently_written[db_get_type_index(mdb)];    /* take most recently written */
+         fill_query(order, SQL_QUERY_sql_media_order_most_recently_written);    /* Take most recently written */
       }
 
-      Mmsg(mdb->cmd, "SELECT MediaId,VolumeName,VolJobs,VolFiles,VolBlocks,"
+      Mmsg(cmd, "SELECT MediaId,VolumeName,VolJobs,VolFiles,VolBlocks,"
                      "VolBytes,VolMounts,VolErrors,VolWrites,MaxVolBytes,VolCapacityBytes,"
                      "MediaType,VolStatus,PoolId,VolRetention,VolUseDuration,MaxVolJobs,"
                      "MaxVolFiles,Recycle,Slot,FirstWritten,LastWritten,InChanger,"
@@ -409,27 +408,27 @@ retry_fetch:
                      "%s "
                      "%s LIMIT %d",
            edit_int64(mr->PoolId, ed1), esc_type,
-           esc_status, changer.c_str(), order, item);
+           esc_status, changer.c_str(), order.c_str(), item);
    }
 
-   Dmsg1(100, "fnextvol=%s\n", mdb->cmd);
-   if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
+   Dmsg1(100, "fnextvol=%s\n", cmd);
+   if (!QUERY_DB(jcr, cmd)) {
       goto bail_out;
    }
 
-   num_rows = sql_num_rows(mdb);
+   num_rows = sql_num_rows();
    if (item > num_rows || item < 1) {
       Dmsg2(050, "item=%d got=%d\n", item, num_rows);
-      Mmsg2(mdb->errmsg, _("Request for Volume item %d greater than max %d or less than 1\n"), item, num_rows);
+      Mmsg2(errmsg, _("Request for Volume item %d greater than max %d or less than 1\n"), item, num_rows);
       num_rows = 0;
       goto bail_out;
    }
 
    for (int i = 0 ; i < item; i++) {
-      if ((row = sql_fetch_row(mdb)) == NULL) {
+      if ((row = sql_fetch_row()) == NULL) {
          Dmsg1(050, "Fail fetch item=%d\n", i);
-         Mmsg1(mdb->errmsg, _("No Volume record found for item %d.\n"), i);
-         sql_free_result(mdb);
+         Mmsg1(errmsg, _("No Volume record found for item %d.\n"), i);
+         sql_free_result();
          num_rows = 0;
          goto bail_out;
       }
@@ -441,10 +440,10 @@ retry_fetch:
        */
       if (unwanted_volumes && is_on_unwanted_volumes_list(row[1], unwanted_volumes)) {
          Dmsg1(50, "Volume %s is on unwanted_volume_list, skipping\n", row[1]);
-         num_rows-- ;
-         if ( num_rows <= 0) {
+         num_rows--;
+         if (num_rows <= 0) {
             Dmsg1(50, "No more volumes in result, bailing out\n", row[1]);
-            sql_free_result(mdb);
+            sql_free_result();
             goto bail_out;
          }
          continue;
@@ -498,7 +497,7 @@ retry_fetch:
       mr->MinBlocksize = str_to_int32(row[38]);
       mr->MaxBlocksize = str_to_int32(row[39]);
 
-      sql_free_result(mdb);
+      sql_free_result();
       found_candidate = true;
       break;
    }
@@ -509,10 +508,9 @@ retry_fetch:
    }
 
 bail_out:
-   db_unlock(mdb);
+   db_unlock(this);
    Dmsg1(050, "Rtn numrows=%d\n", num_rows);
 
    return num_rows;
 }
-
 #endif /* HAVE_SQLITE3 || HAVE_MYSQL || HAVE_POSTGRESQL || HAVE_INGRES || HAVE_DBI */
