@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,22 +21,25 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Catalog Database routines specific to SQLite
- *
  * Kern Sibbald, January 2002
- *
  * Major rewrite by Marco van Wieringen, January 2010 for catalog refactoring.
  */
+/**
+ * @file
+ * BAREOS Catalog Database routines specific to SQLite
+ */
 
 #include "bareos.h"
 
 #if HAVE_SQLITE3
 
 #include "cats.h"
-#include "bdb_priv.h"
 #include <sqlite3.h>
 #include "bdb_sqlite.h"
 
+/* pull in the generated queries definitions */
+#include "sqlite_queries.inc"
+
 /* -----------------------------------------------------------------------
  *
  *    SQLite dependent defines and subroutines
@@ -73,7 +76,8 @@ B_DB_SQLITE::B_DB_SQLITE(JCR *jcr,
                          bool disable_batch_insert,
                          bool try_reconnect,
                          bool exit_on_fatal,
-                         bool need_private)
+                         bool need_private
+                         )
 {
    /*
     * Initialize the parent class members.
@@ -127,19 +131,22 @@ B_DB_SQLITE::B_DB_SQLITE(JCR *jcr,
       db_list = New(dlist(this, &this->m_link));
    }
    db_list->append(this);
+
+   /* make the queries available using the queries variable from the parent class */
+   queries = query_definitions;
 }
 
 B_DB_SQLITE::~B_DB_SQLITE()
 {
 }
 
-/*
+/**
  * Now actually open the database.  This can generate errors,
  * which are returned in the errmsg
  *
  * DO NOT close the database or delete mdb here !!!!
  */
-bool B_DB_SQLITE::db_open_database(JCR *jcr)
+bool B_DB_SQLITE::open_database(JCR *jcr)
 {
    bool retval = false;
    char *db_path;
@@ -206,10 +213,10 @@ bool B_DB_SQLITE::db_open_database(JCR *jcr)
    sqlite3_busy_handler(m_db_handle, sqlite_busy_handler, NULL);
 
 #if defined(SQLITE3_INIT_QUERY)
-   sql_query(SQLITE3_INIT_QUERY);
+   sql_query_without_handler(SQLITE3_INIT_QUERY);
 #endif
 
-   if (!check_tables_version(jcr, this)) {
+   if (!check_tables_version(jcr)) {
       goto bail_out;
    }
 
@@ -220,10 +227,10 @@ bail_out:
    return retval;
 }
 
-void B_DB_SQLITE::db_close_database(JCR *jcr)
+void B_DB_SQLITE::close_database(JCR *jcr)
 {
    if (m_connected) {
-      db_end_transaction(jcr);
+      end_transaction(jcr);
    }
    P(mutex);
    m_ref_count--;
@@ -261,12 +268,12 @@ void B_DB_SQLITE::db_close_database(JCR *jcr)
    V(mutex);
 }
 
-bool B_DB_SQLITE::db_validate_connection(void)
+bool B_DB_SQLITE::validate_connection(void)
 {
    bool retval;
 
    db_lock(this);
-   if (sql_query("SELECT 1", true)) {
+   if (sql_query_without_handler("SELECT 1", true)) {
       sql_free_result();
       retval = true;
       goto bail_out;
@@ -280,23 +287,25 @@ bail_out:
    return retval;
 }
 
-void B_DB_SQLITE::db_thread_cleanup(void)
+void B_DB_SQLITE::thread_cleanup(void)
 {
    sqlite3_thread_cleanup();
 }
 
-/*
+/**
  * Start a transaction. This groups inserts and makes things
  * much more efficient. Usually started when inserting
  * file attributes.
  */
-void B_DB_SQLITE::db_start_transaction(JCR *jcr)
+void B_DB_SQLITE::start_transaction(JCR *jcr)
 {
    if (!jcr->attr) {
       jcr->attr = get_pool_memory(PM_FNAME);
    }
+
    if (!jcr->ar) {
       jcr->ar = (ATTR_DBR *)malloc(sizeof(ATTR_DBR));
+      jcr->ar->Digest = NULL;
    }
 
    if (!m_allow_transactions) {
@@ -308,22 +317,22 @@ void B_DB_SQLITE::db_start_transaction(JCR *jcr)
     * Allow only 10,000 changes per transaction
     */
    if (m_transaction && changes > 10000) {
-      db_end_transaction(jcr);
+      end_transaction(jcr);
    }
    if (!m_transaction) {
-      sql_query("BEGIN");  /* begin transaction */
+      sql_query_without_handler("BEGIN");  /* begin transaction */
       Dmsg0(400, "Start SQLite transaction\n");
       m_transaction = true;
    }
    db_unlock(this);
 }
 
-void B_DB_SQLITE::db_end_transaction(JCR *jcr)
+void B_DB_SQLITE::end_transaction(JCR *jcr)
 {
    if (jcr && jcr->cached_attribute) {
       Dmsg0(400, "Flush last cached attribute.\n");
-      if (!db_create_attributes_record(jcr, this, jcr->ar)) {
-         Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), db_strerror(jcr->db));
+      if (!create_attributes_record(jcr, jcr->ar)) {
+         Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), strerror());
       }
       jcr->cached_attribute = false;
    }
@@ -334,7 +343,7 @@ void B_DB_SQLITE::db_end_transaction(JCR *jcr)
 
    db_lock(this);
    if (m_transaction) {
-      sql_query("COMMIT"); /* end transaction */
+      sql_query_without_handler("COMMIT"); /* end transaction */
       m_transaction = false;
       Dmsg1(400, "End SQLite transaction changes=%d\n", changes);
    }
@@ -349,15 +358,16 @@ struct rh_data {
    bool initialized;
 };
 
-/*
+/**
  * Convert SQLite's callback into BAREOS DB callback
  */
 static int sqlite_result_handler(void *arh_data, int num_fields, char **rows, char **col_names)
 {
    struct rh_data *rh_data = (struct rh_data *)arh_data;
 
-   /* The db_sql_query doesn't have access to m_results, so if we wan't to get
-    * fields information, we need to use col_names
+   /*
+    * The sql_query_with_handler doesn't have access to m_results,
+    * so if we wan't to get fields information, we need to use col_names
     */
    if (!rh_data->initialized) {
       rh_data->mdb->set_column_names(col_names, num_fields);
@@ -370,17 +380,17 @@ static int sqlite_result_handler(void *arh_data, int num_fields, char **rows, ch
    return 0;
 }
 
-/*
+/**
  * Submit a general SQL command (cmd), and for each row returned,
- *  the result_handler is called with the ctx.
+ * the result_handler is called with the ctx.
  */
-bool B_DB_SQLITE::db_sql_query(const char *query, DB_RESULT_HANDLER *result_handler, void *ctx)
+bool B_DB_SQLITE::sql_query_with_handler(const char *query, DB_RESULT_HANDLER *result_handler, void *ctx)
 {
    bool retval = false;
    int status;
    struct rh_data rh_data;
 
-   Dmsg1(500, "db_sql_query starts with '%s'\n", query);
+   Dmsg1(500, "sql_query_with_handler starts with '%s'\n", query);
 
    db_lock(this);
    if (m_lowlevel_errmsg) {
@@ -399,7 +409,7 @@ bool B_DB_SQLITE::db_sql_query(const char *query, DB_RESULT_HANDLER *result_hand
 
    if (status != SQLITE_OK) {
       Mmsg(errmsg, _("Query failed: %s: ERR=%s\n"), query, sql_strerror());
-      Dmsg0(500, "db_sql_query finished\n");
+      Dmsg0(500, "sql_query_with_handler finished\n");
       goto bail_out;
    }
    Dmsg0(500, "db_sql_query finished\n");
@@ -411,15 +421,15 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Submit a sqlite query and retrieve all the data
  */
-bool B_DB_SQLITE::sql_query(const char *query, int flags)
+bool B_DB_SQLITE::sql_query_without_handler(const char *query, int flags)
 {
    int status;
    bool retval = false;
 
-   Dmsg1(500, "sql_query starts with '%s'\n", query);
+   Dmsg1(500, "sql_query_without_handler starts with '%s'\n", query);
 
    sql_free_result();
    if (m_lowlevel_errmsg) {
@@ -433,9 +443,9 @@ bool B_DB_SQLITE::sql_query(const char *query, int flags)
    m_row_number = 0;               /* no row fetched */
    if (status != 0) {                   /* something went wrong */
       m_num_rows = m_num_fields = 0;
-      Dmsg0(500, "sql_query finished\n");
+      Dmsg0(500, "sql_query_without_handler finished\n");
    } else {
-      Dmsg0(500, "sql_query finished\n");
+      Dmsg0(500, "sql_query_without_handler finished\n");
       retval = true;
    }
    return retval;
@@ -457,7 +467,7 @@ void B_DB_SQLITE::sql_free_result(void)
    db_unlock(this);
 }
 
-/*
+/**
  * Fetch one row at a time
  */
 SQL_ROW B_DB_SQLITE::sql_fetch_row(void)
@@ -492,7 +502,7 @@ uint64_t B_DB_SQLITE::sql_insert_autokey_record(const char *query, const char *t
    /*
     * First execute the insert query and then retrieve the currval.
     */
-   if (!sql_query(query)) {
+   if (!sql_query_without_handler(query)) {
       return 0;
    }
 
@@ -587,7 +597,7 @@ bool B_DB_SQLITE::sql_field_is_numeric(int field_type)
    }
 }
 
-/*
+/**
  * Returns true if OK
  *         false if failed
  */
@@ -596,21 +606,25 @@ bool B_DB_SQLITE::sql_batch_start(JCR *jcr)
    bool retval;
 
    db_lock(this);
-   retval = sql_query("CREATE TEMPORARY TABLE batch ("
-                              "FileIndex integer,"
-                              "JobId integer,"
-                              "Path blob,"
-                              "Name blob,"
-                              "LStat tinyblob,"
-                              "MD5 tinyblob,"
-                              "DeltaSeq integer)");
+   retval = sql_query_without_handler("CREATE TEMPORARY TABLE batch ("
+                                      "FileIndex integer,"
+                                      "JobId integer,"
+                                      "Path blob,"
+                                      "Name blob,"
+                                      "LStat tinyblob,"
+                                      "MD5 tinyblob,"
+                                      "DeltaSeq integer,"
+                                      "Fhinfo TEXT,"
+                                      "Fhnode TEXT "
+                                      ")"
+         );
    db_unlock(this);
 
    return retval;
 }
 
 /* set error to something to abort operation */
-/*
+/**
  * Returns true if OK
  *         false if failed
  */
@@ -621,20 +635,20 @@ bool B_DB_SQLITE::sql_batch_end(JCR *jcr, const char *error)
    return true;
 }
 
-/*
+/**
  * Returns true if OK
  *         false if failed
  */
 bool B_DB_SQLITE::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
 {
    const char *digest;
-   char ed1[50];
+   char ed1[50], ed2[50], ed3[50];
 
    esc_name = check_pool_memory_size(esc_name, fnl*2+1);
-   db_escape_string(jcr, esc_name, fname, fnl);
+   escape_string(jcr, esc_name, fname, fnl);
 
    esc_path = check_pool_memory_size(esc_path, pnl*2+1);
-   db_escape_string(jcr, esc_path, path, pnl);
+   escape_string(jcr, esc_path, path, pnl);
 
    if (ar->Digest == NULL || ar->Digest[0] == 0) {
       digest = "0";
@@ -643,14 +657,16 @@ bool B_DB_SQLITE::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
    }
 
    Mmsg(cmd, "INSERT INTO batch VALUES "
-        "(%u,%s,'%s','%s','%s','%s',%u)",
+        "(%u,%s,'%s','%s','%s','%s',%u,'%s','%s')",
         ar->FileIndex, edit_int64(ar->JobId,ed1), esc_path,
-        esc_name, ar->attr, digest, ar->DeltaSeq);
+        esc_name, ar->attr, digest, ar->DeltaSeq,
+        edit_uint64(ar->Fhinfo,ed2),
+        edit_uint64(ar->Fhnode,ed3));
 
-   return sql_query(cmd);
+   return sql_query_without_handler(cmd);
 }
 
-/*
+/**
  * Initialize database data structure. In principal this should
  * never have errors, or it is really fatal.
  */
@@ -697,7 +713,7 @@ B_DB *db_init_database(JCR *jcr,
             continue;
          }
 
-         if (mdb->db_match_database(db_driver, db_name, db_address, db_port)) {
+         if (mdb->match_database(db_driver, db_name, db_address, db_port)) {
             Dmsg1(300, "DB REopen %s\n", db_name);
             mdb->increment_refcount();
             goto bail_out;
@@ -717,7 +733,8 @@ B_DB *db_init_database(JCR *jcr,
                          disable_batch_insert,
                          try_reconnect,
                          exit_on_fatal,
-                         need_private));
+                         need_private
+                         ));
 
 bail_out:
    V(mutex);
