@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
+   Copyright (C) 2011-2016 Planets Communications B.V.
    Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -21,10 +21,11 @@
    02110-1301, USA.
 */
 /*
- * BAREOS pluginloader
- *
  * Kern Sibbald, October 2007
  */
+/** @file
+ * BAREOS pluginloader
+ */
 #include "bareos.h"
 #include "dird.h"
 #include "dir_plugins.h"
@@ -259,7 +260,7 @@ bail_out:
    return rc;
 }
 
-/*
+/**
  * Print to file the plugin info.
  */
 void dump_dir_plugin(Plugin *plugin, FILE *fp)
@@ -381,7 +382,7 @@ static bool is_plugin_compatible(Plugin *plugin)
    return true;
 }
 
-/*
+/**
  * Instantiate a new plugin instance.
  */
 static inline bpContext *instantiate_plugin(JCR *jcr, Plugin *plugin, uint32_t instance)
@@ -411,7 +412,7 @@ static inline bpContext *instantiate_plugin(JCR *jcr, Plugin *plugin, uint32_t i
    return ctx;
 }
 
-/*
+/**
  * Send a bDirEventNewPluginOptions event to all plugins configured in
  * jcr->res.Job.DirPluginOptions
  */
@@ -509,7 +510,7 @@ void dispatch_new_plugin_options(JCR *jcr)
    }
 }
 
-/*
+/**
  * Create a new instance of each plugin for this Job
  */
 void new_plugins(JCR *jcr)
@@ -541,7 +542,7 @@ void new_plugins(JCR *jcr)
    }
 }
 
-/*
+/**
  * Free the plugin instances for this Job
  */
 void free_plugins(JCR *jcr)
@@ -565,11 +566,10 @@ void free_plugins(JCR *jcr)
    jcr->plugin_ctx_list = NULL;
 }
 
-/* ==============================================================
+/**
  *
  * Callbacks from the plugin
  *
- * ==============================================================
  */
 static bRC bareosGetValue(bpContext *ctx, brDirVariable var, void *value)
 {
@@ -627,7 +627,7 @@ static bRC bareosGetValue(bpContext *ctx, brDirVariable var, void *value)
 
          memset(&pr, 0, sizeof(pr));
          bstrncpy(pr.Name, jcr->res.pool->hdr.name, sizeof(pr.Name));
-         if (!db_get_pool_record(jcr, jcr->db, &pr)) {
+         if (!jcr->db->get_pool_record(jcr, &pr)) {
             retval = bRC_Error;
          }
          *((int *)value) = pr.NumVols;
