@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director daemon -- this is the main program
- *
  * Kern Sibbald, March MM
  */
+/**
+ * @file
+ * the main program
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -108,7 +110,7 @@ static void free_saved_resources(resource_table_reference *table)
    free(table->res_table);
 }
 
-/*
+/**
  * Called here at the end of every job that was hooked decrementing the active job_count.
  * When it goes to zero, no one is using the associated resource table, so free it.
  */
@@ -122,12 +124,15 @@ static void reload_job_end_cb(JCR *jcr, void *ctx)
 
    foreach_alist_index(i, table, reload_table) {
       if (table == (resource_table_reference *)ctx) {
-         if (--table->job_count <= 0) {
-            Dmsg0(100, "Last reference to old configuration, removing saved configuration\n");
-            free_saved_resources(table);
-            reload_table->remove(i);
-            free(table);
-            break;
+         if (table->job_count) {
+            table->job_count--;
+            if (table->job_count == 0) {
+               Dmsg0(100, "Last reference to old configuration, removing saved configuration\n");
+               free_saved_resources(table);
+               reload_table->remove(i);
+               free(table);
+               break;
+            }
          }
       }
    }
@@ -136,7 +141,7 @@ static void reload_job_end_cb(JCR *jcr, void *ctx)
    unlock_jobs();
 }
 
-/*
+/**
  * This allows the message handler to operate on the database by using a pointer
  * to this function. The pointer is needed because the other daemons do not have
  * access to the database. If the pointer is not defined (other daemons), then
@@ -155,13 +160,13 @@ static bool dir_db_log_insert(JCR *jcr, utime_t mtime, char *msg)
    }
    length = strlen(msg);
    esc_msg.check_size(length * 2 + 1);
-   db_escape_string(jcr, jcr->db, esc_msg.c_str(), msg, length);
+   jcr->db->escape_string(jcr, esc_msg.c_str(), msg, length);
 
    bstrutime(dt, sizeof(dt), mtime);
    Mmsg(query, "INSERT INTO Log (JobId, Time, LogText) VALUES (%s,'%s','%s')",
         edit_int64(jcr->JobId, ed1), dt, esc_msg.c_str());
 
-   return db_sql_query(jcr->db, query.c_str());
+   return jcr->db->sql_query(query.c_str());
 }
 
 static void usage()
@@ -374,12 +379,11 @@ int main (int argc, char *argv[])
    char *backend_dir;
 
    foreach_alist(backend_dir, me->backend_directories) {
-        Dmsg1(100, "backend path: %s\n", backend_dir);
+      Dmsg1(100, "backend path: %s\n", backend_dir);
    }
 
    db_set_backend_dirs(me->backend_directories);
 #endif
-
    load_dir_plugins(me->plugin_directory, me->plugin_names);
 
    /*
@@ -445,20 +449,23 @@ bail_out:
    return 0;
 }
 
-/* Cleanup and then exit */
+/**
+ * Cleanup and then exit
+ *
+ */
 #if !defined(HAVE_WIN32)
 static
 #endif
 void terminate_dird(int sig)
 {
-   static bool already_here = false;
+   static bool is_reloading = false;
 
-   if (already_here) {                /* avoid recursive temination problems */
+   if (is_reloading) {                /* avoid recursive temination problems */
       bmicrosleep(2, 0);              /* yield */
       exit(1);
    }
 
-   already_here = true;
+   is_reloading = true;
    debug_level = 0;                   /* turn off debug */
 
    destroy_configure_usage_string();
@@ -499,16 +506,16 @@ void terminate_dird(int sig)
    exit(sig);
 }
 
-/*
+/**
  * If we get here, we have received a SIGHUP, which means to reread our configuration file.
  */
 #if !defined(HAVE_WIN32)
 extern "C"
 void sighandler_reload_config(int sig, siginfo_t *siginfo, void *ptr)
 {
-   static bool already_here = false;
+   static bool is_reloading = false;
 
-   if (already_here) {
+   if (is_reloading) {
       /*
        * Note: don't use Jmsg here, as it could produce a race condition
        * on multiple parallel reloads
@@ -516,9 +523,9 @@ void sighandler_reload_config(int sig, siginfo_t *siginfo, void *ptr)
       Qmsg(NULL, M_ERROR, 0, _("Already reloading. Request ignored.\n"));
       return;
    }
-   already_here = true;
+   is_reloading = true;
    do_reload_config();
-   already_here = false;
+   is_reloading = false;
 }
 #endif
 
@@ -547,7 +554,7 @@ static bool init_sighandler_sighup()
    return retval;
 }
 
-/*
+/**
  * The algorithm used is as follows: we count how many jobs are
  * running and mark the running jobs to make a callback on
  * exiting. The old config is saved with the reload table
@@ -561,18 +568,18 @@ static bool init_sighandler_sighup()
  * last reload, then the old resources will be immediately release.
  * A console is considered a job because it may have pointers to
  * resources, but a SYSTEM job is not since it *should* not have any
- * permanent pointers to jobs.
+ * permanent pointers to resources.
  */
 bool do_reload_config()
 {
-   static bool already_here = false;
+   static bool is_reloading = false;
    bool ok = false;
    bool reloaded = false;
    JCR *jcr;
-   int njobs = 0;                     /* Number of running jobs */
+   int num_running_jobs = 0;                     /* Number of running jobs */
    resource_table_reference prev_config;
 
-   if (already_here) {
+   if (is_reloading) {
       /*
        * Note: don't use Jmsg here, as it could produce a race condition
        * on multiple parallel reloads
@@ -580,19 +587,15 @@ bool do_reload_config()
       Qmsg(NULL, M_ERROR, 0, _("Already reloading. Request ignored.\n"));
       return false;
    }
-   already_here = true;
+   is_reloading = true;
+
+   stop_statistics_thread();
 
    lock_jobs();
    LockRes();
 
-   /**
-    * Flush the sql connection pools.
-    */
    db_sql_pool_flush();
 
-   /*
-    * Save the previous config so we can restore it.
-    */
    prev_config.res_table = my_config->save_resources();
    prev_config.job_count = 0;
 
@@ -613,17 +616,13 @@ bool do_reload_config()
        */
       failed_config.res_table = my_config->save_resources();
 
-      /*
-       * Now restore old resource values,
-       */
       num = my_config->m_r_last - my_config->m_r_first + 1;
       for (int i = 0; i < num; i++) {
+         // restore original config
          my_config->m_res_head[i] = prev_config.res_table[i];
       }
 
-      /*
-       * Reset director resource to old config as check_resources() changed it
-       */
+      // me changed above in check_resources()
       me = (DIRRES *)GetNextRes(R_DIRECTOR, NULL);
 
       /*
@@ -642,8 +641,8 @@ bool do_reload_config()
                memcpy(new_table, &prev_config, sizeof(resource_table_reference));
             }
             new_table->job_count++;
-            job_end_push(jcr, reload_job_end_cb, (void *)new_table);
-            njobs++;
+            register_job_end_callback(jcr, reload_job_end_cb, (void *)new_table);
+            num_running_jobs++;
          }
       }
       endeach_jcr(jcr);
@@ -655,7 +654,7 @@ bool do_reload_config()
       set_working_directory(me->working_directory);
       Dmsg0(10, "Director's configuration file reread.\n");
 
-      if (njobs > 0) {
+      if (num_running_jobs > 0) {
          /*
           * See if we already initialized the alist.
           */
@@ -673,16 +672,17 @@ bool do_reload_config()
           */
          free_saved_resources(&prev_config);
       }
+      start_statistics_thread();
    }
 
 bail_out:
    UnlockRes();
    unlock_jobs();
-   already_here = false;
+   is_reloading = false;
    return reloaded;
 }
 
-/*
+/**
  * See if two storage definitions point to the same Storage Daemon.
  *
  * We compare:
@@ -697,7 +697,7 @@ static inline bool is_same_storage_daemon(STORERES *store1, STORERES *store2)
           bstrcasecmp(store1->password.value, store2->password.value);
 }
 
-/*
+/**
  * Make a quick check to see that we have all the
  * resources needed.
  *
@@ -1085,7 +1085,7 @@ bail_out:
    return OK;
 }
 
-/*
+/**
  * Initialize the sql pooling.
  */
 static bool initialize_sql_pooling(void)
@@ -1120,7 +1120,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * In this routine,
  *  - we can check the connection (mode=CHECK_CONNECTION)
  *  - we can synchronize the catalog with the configuration (mode=UPDATE_CATALOG)
@@ -1152,29 +1152,29 @@ static bool check_catalog(cat_op mode)
                             catalog->try_reconnect,
                             catalog->exit_on_fatal);
 
-      if (!db || !db_open_database(NULL, db)) {
+      if (!db || !db->open_database(NULL)) {
          Pmsg2(000, _("Could not open Catalog \"%s\", database \"%s\".\n"),
               catalog->name(), catalog->db_name);
          Jmsg(NULL, M_FATAL, 0, _("Could not open Catalog \"%s\", database \"%s\".\n"),
               catalog->name(), catalog->db_name);
          if (db) {
-            Jmsg(NULL, M_FATAL, 0, _("%s"), db_strerror(db));
-            Pmsg1(000, "%s", db_strerror(db));
-            db_close_database(NULL, db);
+            Jmsg(NULL, M_FATAL, 0, _("%s"), db->strerror());
+            Pmsg1(000, "%s", db->strerror());
+            db->close_database(NULL);
          }
          OK = false;
          goto bail_out;
       }
 
       /* Display a message if the db max_connections is too low */
-      if (!db_check_max_connections(NULL, db, me->MaxConcurrentJobs)) {
+      if (!db->check_max_connections(NULL, me->MaxConcurrentJobs)) {
          Pmsg1(000, "Warning, settings problem for Catalog=%s\n", catalog->name());
-         Pmsg1(000, "%s", db_strerror(db));
+         Pmsg1(000, "%s", db->strerror());
       }
 
       /* we are in testing mode, so don't touch anything in the catalog */
       if (mode == CHECK_CONNECTION) {
-         db_close_database(NULL, db);
+         db->close_database(NULL);
          continue;
       }
 
@@ -1217,7 +1217,7 @@ static bool check_catalog(cat_op mode)
                client->catalog->name(), client->name());
          memset(&cr, 0, sizeof(cr));
          bstrncpy(cr.Name, client->name(), sizeof(cr.Name));
-         db_create_client_record(NULL, db, &cr);
+         db->create_client_record(NULL, &cr);
       }
 
       /* Ensure basic storage record is in DB */
@@ -1230,13 +1230,13 @@ static bool check_catalog(cat_op mode)
          if (store->media_type) {
             bstrncpy(mtr.MediaType, store->media_type, sizeof(mtr.MediaType));
             mtr.ReadOnly = 0;
-            db_create_mediatype_record(NULL, db, &mtr);
+            db->create_mediatype_record(NULL, &mtr);
          } else {
             mtr.MediaTypeId = 0;
          }
          bstrncpy(sr.Name, store->name(), sizeof(sr.Name));
          sr.AutoChanger = store->autochanger;
-         if (!db_create_storage_record(NULL, db, &sr)) {
+         if (!db->create_storage_record(NULL, &sr)) {
             Jmsg(NULL, M_FATAL, 0, _("Could not create storage record for %s\n"),
                  store->name());
             OK = false;
@@ -1245,7 +1245,7 @@ static bool check_catalog(cat_op mode)
          store->StorageId = sr.StorageId;   /* set storage Id */
          if (!sr.created) {                 /* if not created, update it */
             sr.AutoChanger = store->autochanger;
-            if (!db_update_storage_record(NULL, db, &sr)) {
+            if (!db->update_storage_record(NULL, &sr)) {
                Jmsg(NULL, M_FATAL, 0, _("Could not update storage record for %s\n"),
                     store->name());
                OK = false;
@@ -1270,7 +1270,7 @@ static bool check_catalog(cat_op mode)
             } else {
                cr.WrapCounter[0] = 0;  /* empty string */
             }
-            if (db_create_counter_record(NULL, db, &cr)) {
+            if (db->create_counter_record(NULL, &cr)) {
                counter->CurrentValue = cr.CurrentValue;
                counter->created = true;
                Dmsg2(100, "Create counter %s val=%d\n", counter->name(), counter->CurrentValue);
@@ -1282,14 +1282,14 @@ static bool check_catalog(cat_op mode)
       }
       /* cleanup old job records */
       if (mode == UPDATE_AND_FIX) {
-         db_sql_query(db, cleanup_created_job);
-         db_sql_query(db, cleanup_running_job);
+         db->sql_query(B_DB::SQL_QUERY_cleanup_created_job);
+         db->sql_query(B_DB::SQL_QUERY_cleanup_running_job);
       }
 
       /* Set type in global for debugging */
-      set_db_type(db_get_type(db));
+      set_db_type(db->get_type());
 
-      db_close_database(NULL, db);
+      db->close_database(NULL);
    }
 
 bail_out:
@@ -1299,7 +1299,10 @@ bail_out:
 static void cleanup_old_files()
 {
    DIR* dp;
-   struct dirent *entry, *result;
+   struct dirent *result;
+#ifdef USE_READDIR_R
+   struct dirent *entry;
+#endif
    int rc, name_max;
    int my_name_len = strlen(my_name);
    int len = strlen(me->working_directory);
@@ -1340,9 +1343,16 @@ static void cleanup_old_files()
       return;
    }
 
+#ifdef USE_READDIR_R
    entry = (struct dirent *)malloc(sizeof(struct dirent) + name_max + 1000);
    while (1) {
       if ((readdir_r(dp, entry, &result) != 0) || (result == NULL)) {
+#else
+   while (1) {
+      result = readdir(dp);
+      if (result == NULL) {
+#endif
+
          break;
       }
       /* Exclude any name with ., .., not my_name or containing a space */
@@ -1361,7 +1371,9 @@ static void cleanup_old_files()
       }
    }
 
+#ifdef USE_READDIR_R
    free(entry);
+#endif
    closedir(dp);
 /* Be careful to free up the correct resources */
 get_out1:
