@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2003-2006 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,11 +21,13 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- expand.c -- does variable expansion
- * in particular for the LabelFormat specification.
- *
  * Kern Sibbald, June MMIII
  */
+/**
+ * @file
+ * does variable expansion
+ * in particular for the LabelFormat specification.
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -174,7 +176,7 @@ static struct s_built_in_vars built_in_vars[] = {
    { NULL, 0, NULL }
 };
 
-/*
+/**
  * Search the table of built-in variables, and if found,
  * call the appropriate subroutine to do the work.
  */
@@ -204,7 +206,7 @@ static var_rc_t lookup_built_in_var(var_t *ctx,
    return VAR_ERR_UNDEFINED_VARIABLE;
 }
 
-/*
+/**
  * Search counter variables
  */
 static var_rc_t lookup_counter_var(var_t *ctx,
@@ -264,9 +266,9 @@ static var_rc_t lookup_counter_var(var_t *ctx,
                } else {
                   cr.WrapCounter[0] = 0;
                }
-               if (!db_update_counter_record(jcr, jcr->db, &cr)) {
+               if (!jcr->db->update_counter_record(jcr, &cr)) {
                   Jmsg(jcr, M_ERROR, 0, _("Count not update counter %s: ERR=%s\n"),
-                     counter->name(), db_strerror(jcr->db));
+                       counter->name(), jcr->db->strerror());
                }
             }
          }
@@ -279,7 +281,7 @@ static var_rc_t lookup_counter_var(var_t *ctx,
    return status;
 }
 
-/*
+/**
  * Called here from "core" expand code to look up a variable
  */
 static var_rc_t lookup_var(var_t *ctx,
@@ -393,7 +395,7 @@ static var_rc_t lookup_var(var_t *ctx,
    return VAR_OK;
 }
 
-/*
+/**
  * Called here to do a special operation on a variable
  *   op_ptr  points to the special operation code (not EOS terminated)
  *   arg_ptr points to argument to special op code
@@ -447,7 +449,7 @@ static var_rc_t operate_var(var_t *var,
    return status;
 }
 
-/*
+/**
  * Expand an input line and return it.
  *
  * Returns: 0 on failure
