@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
+   Copyright (C) 2011-2016 Planets Communications B.V.
    Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -21,9 +21,10 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- fd_cmds.c -- send commands to File daemon
- *
  * Kern Sibbald, October MM
+ */
+/**
+ * fd_cmds.c -- send commands to File daemon
  *
  * This routine is run as a separate thread.  There may be more
  * work to be done to make it totally reentrant!!!!
@@ -32,6 +33,7 @@
  * These functions are used by both backup and verify.
  */
 
+
 #include "bareos.h"
 #include "dird.h"
 #include "findlib/find.h"
@@ -101,7 +103,7 @@ static inline utime_t get_heartbeat_interval(CLIENTRES *res)
    return heartbeat;
 }
 
-/*
+/**
  * Open connection to File daemon.
  *
  * Try connecting every retry_interval (default 10 sec), and
@@ -228,8 +230,8 @@ int send_job_info(JCR *jcr)
          cr.FileRetention = jcr->res.client->FileRetention;
          cr.JobRetention = jcr->res.client->JobRetention;
          bstrncpy(cr.Uname, fd->msg+strlen(OKjob)+1, sizeof(cr.Uname));
-         if (!db_update_client_record(jcr, jcr->db, &cr)) {
-            Jmsg(jcr, M_WARNING, 0, _("Error updating Client record. ERR=%s\n"), db_strerror(jcr->db));
+         if (!jcr->db->update_client_record(jcr, &cr)) {
+            Jmsg(jcr, M_WARNING, 0, _("Error updating Client record. ERR=%s\n"), jcr->db->strerror());
          }
       }
    } else {
@@ -321,7 +323,7 @@ static inline void send_since_time(JCR *jcr)
    }
 }
 
-/*
+/**
  * Send level command to FD.
  * Used for backup jobs and estimate command.
  */
@@ -370,7 +372,7 @@ bool send_level_command(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * Send either an Included or an Excluded list to FD
  */
 static bool send_fileset(JCR *jcr)
@@ -621,7 +623,7 @@ static bool send_list_item(JCR *jcr, const char *code, char *item, BSOCK *fd)
    return true;
 }
 
-/*
+/**
  * Send include list to File daemon
  */
 bool send_include_list(JCR *jcr)
@@ -634,7 +636,7 @@ bool send_include_list(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * Send exclude list to File daemon
  *   Under the new scheme, the Exclude list
  *   is part of the FileSet sent with the
@@ -666,7 +668,7 @@ static inline bool have_client_runscripts(alist *run_scripts)
    return retval;
 }
 
-/*
+/**
  * Send RunScripts to File daemon
  * 1) We send all runscript to FD, they can be executed Before, After, or twice
  * 2) Then, we send a "RunBeforeNow" command to the FD to tell him to do the
@@ -759,7 +761,7 @@ struct OBJ_CTX {
    int count;
 };
 
-/*
+/**
  * restore_object_handler is called for each file found
  */
 static int restore_object_handler(void *ctx, int num_fields, char **row)
@@ -802,12 +804,11 @@ static int restore_object_handler(void *ctx, int num_fields, char **row)
 
    Dmsg1(010, "Send obj: %s\n", fd->msg);
 
-   db_unescape_object(jcr,
-                      jcr->db,
-                      row[8],                /* Object  */
-                      str_to_uint64(row[1]), /* Object length */
-                      fd->msg,
-                      &fd->msglen);
+   jcr->db->unescape_object(jcr,
+                            row[8],                /* Object  */
+                            str_to_uint64(row[1]), /* Object length */
+                            fd->msg,
+                            &fd->msglen);
    fd->send();                           /* send object */
    octx->count++;
 
@@ -852,8 +853,8 @@ bool send_plugin_options(JCR *jcr)
 
 static inline void send_global_restore_objects(JCR *jcr, OBJ_CTX *octx)
 {
-   POOL_MEM query(PM_MESSAGE);
    char ed1[50];
+   POOL_MEM query(PM_MESSAGE);
 
    if (!jcr->JobIds || !jcr->JobIds[0]) {
       return;
@@ -862,35 +863,32 @@ static inline void send_global_restore_objects(JCR *jcr, OBJ_CTX *octx)
    /*
     * Send restore objects for all jobs involved
     */
-   Mmsg(query, get_restore_objects, jcr->JobIds, FT_RESTORE_FIRST);
-   db_sql_query(jcr->db, query.c_str(), restore_object_handler, (void *)octx);
+   jcr->db->fill_query(query, B_DB::SQL_QUERY_get_restore_objects, jcr->JobIds, FT_RESTORE_FIRST);
+   jcr->db->sql_query(query.c_str(), restore_object_handler, (void *)octx);
 
-   Mmsg(query, get_restore_objects, jcr->JobIds, FT_PLUGIN_CONFIG);
-   db_sql_query(jcr->db, query.c_str(), restore_object_handler, (void *)octx);
+   jcr->db->fill_query(query, B_DB::SQL_QUERY_get_restore_objects, jcr->JobIds, FT_PLUGIN_CONFIG);
+   jcr->db->sql_query(query.c_str(), restore_object_handler, (void *)octx);
 
    /*
     * Send config objects for the current restore job
     */
-   Mmsg(query, get_restore_objects,
-        edit_uint64(jcr->JobId, ed1), FT_PLUGIN_CONFIG_FILLED);
-   db_sql_query(jcr->db, query.c_str(), restore_object_handler, (void *)octx);
+   jcr->db->fill_query(query, B_DB::SQL_QUERY_get_restore_objects, edit_uint64(jcr->JobId, ed1), FT_PLUGIN_CONFIG_FILLED);
+   jcr->db->sql_query(query.c_str(), restore_object_handler, (void *)octx);
 }
 
 static inline void send_job_specific_restore_objects(JCR *jcr, JobId_t JobId, OBJ_CTX *octx)
 {
-   POOL_MEM query(PM_MESSAGE);
    char ed1[50];
+   POOL_MEM query(PM_MESSAGE);
 
    /*
     * Send restore objects for specific JobId.
     */
-   Mmsg(query, get_restore_objects,
-        edit_uint64(JobId, ed1), FT_RESTORE_FIRST);
-   db_sql_query(jcr->db, query.c_str(), restore_object_handler, (void *)octx);
+   jcr->db->fill_query(query, B_DB::SQL_QUERY_get_restore_objects, edit_uint64(JobId, ed1), FT_RESTORE_FIRST);
+   jcr->db->sql_query(query.c_str(), restore_object_handler, (void *)octx);
 
-   Mmsg(query, get_restore_objects,
-        edit_uint64(JobId, ed1), FT_PLUGIN_CONFIG);
-   db_sql_query(jcr->db, query.c_str(), restore_object_handler, (void *)octx);
+   jcr->db->fill_query(query, B_DB::SQL_QUERY_get_restore_objects, edit_uint64(JobId, ed1), FT_PLUGIN_CONFIG);
+   jcr->db->sql_query(query.c_str(), restore_object_handler, (void *)octx);
 }
 
 bool send_restore_objects(JCR *jcr, JobId_t JobId, bool send_global)
@@ -923,7 +921,7 @@ bool send_restore_objects(JCR *jcr, JobId_t JobId, bool send_global)
    return true;
 }
 
-/*
+/**
  * Read the attributes from the File daemon for
  * a Verify job and store them in the catalog.
  */
@@ -941,7 +939,7 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
    /*
     * Start transaction allocates jcr->attr and jcr->ar if needed
     */
-   db_start_transaction(jcr, jcr->db);     /* start transaction if not already open */
+   jcr->db->start_transaction(jcr); /* start transaction if not already open */
    ar = jcr->ar;
 
    Dmsg0(120, "dird: waiting to receive file attributes\n");
@@ -977,8 +975,8 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
          if (jcr->cached_attribute) {
             Dmsg3(dbglvl, "Cached attr. Stream=%d fname=%s\n", ar->Stream, ar->fname,
                ar->attr);
-            if (!db_create_file_attributes_record(jcr, jcr->db, ar)) {
-               Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), db_strerror(jcr->db));
+            if (!jcr->db->create_file_attributes_record(jcr, ar)) {
+               Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), jcr->db->strerror());
             }
          }
 
@@ -1001,7 +999,6 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
          ar->JobId = jcr->JobId;
          ar->ClientId = jcr->ClientId;
          ar->PathId = 0;
-         ar->FilenameId = 0;
          ar->Digest = NULL;
          ar->DigestType = CRYPTO_DIGEST_NONE;
          ar->DeltaSeq = 0;
@@ -1029,7 +1026,7 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
          ar->DigestType = crypto_digest_stream_type(stream);
          length = strlen(Digest.c_str());
          digest.check_size(length * 2 + 1);
-         db_escape_string(jcr, jcr->db, digest.c_str(), Digest.c_str(), length);
+         jcr->db->escape_string(jcr, digest.c_str(), Digest.c_str(), length);
          Dmsg4(dbglvl, "stream=%d DigestLen=%d Digest=%s type=%d\n", stream,
                strlen(digest.c_str()), digest.c_str(), ar->DigestType);
       }
@@ -1046,8 +1043,8 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
    if (jcr->cached_attribute) {
       Dmsg3(dbglvl, "Cached attr with digest. Stream=%d fname=%s attr=%s\n", ar->Stream,
          ar->fname, ar->attr);
-      if (!db_create_file_attributes_record(jcr, jcr->db, ar)) {
-         Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), db_strerror(jcr->db));
+      if (!jcr->db->create_file_attributes_record(jcr, ar)) {
+         Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), jcr->db->strerror());
       }
       jcr->cached_attribute = false;
    }
@@ -1057,7 +1054,7 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
    return 1;
 }
 
-/*
+/**
  * Cancel a job running in the File daemon
  */
 bool cancel_file_daemon_job(UAContext *ua, JCR *jcr)
@@ -1084,7 +1081,7 @@ bool cancel_file_daemon_job(UAContext *ua, JCR *jcr)
    return true;
 }
 
-/*
+/**
  * Get the status of a remote File Daemon.
  */
 void do_native_client_status(UAContext *ua, CLIENTRES *client, char *cmd)
@@ -1135,7 +1132,7 @@ void do_native_client_status(UAContext *ua, CLIENTRES *client, char *cmd)
    return;
 }
 
-/*
+/**
  * resolve a host on a filedaemon
  */
 void do_client_resolve(UAContext *ua, CLIENTRES *client)
@@ -1188,7 +1185,7 @@ void do_client_resolve(UAContext *ua, CLIENTRES *client)
    return;
 }
 
-/*
+/**
  * After receiving a connection (in socket_server.c) if it is
  * from the File daemon, this routine is called.
  */
