@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
+   Copyright (C) 2011-2016 Planets Communications B.V.
    Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director Job processing routines
- *
  * Kern Sibbald, October MM
  */
+/**
+ * @file
+ * BAREOS Director Job processing routines
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -66,7 +68,7 @@ void term_job_server()
    jobq_destroy(&job_queue);          /* ignore any errors */
 }
 
-/*
+/**
  * Run a job -- typically called by the scheduler, but may also
  *              be called by the UA (Console program).
  *
@@ -188,8 +190,8 @@ bool setup_job(JCR *jcr, bool suppress_output)
       }
    }
 
-   if (!db_create_job_record(jcr, jcr->db, &jcr->jr)) {
-      Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
+   if (!jcr->db->create_job_record(jcr, &jcr->jr)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", jcr->db->strerror());
       goto bail_out;
    }
 
@@ -243,12 +245,18 @@ bool setup_job(JCR *jcr, bool suppress_output)
       }
 
       switch (jcr->getJobProtocol()) {
-      case PT_NDMP:
+      case PT_NDMP_BAREOS:
          if (!do_ndmp_backup_init(jcr)) {
             ndmp_backup_cleanup(jcr, JS_ErrorTerminated);
             goto bail_out;
          }
          break;
+      case PT_NDMP_NATIVE:
+         if (!do_ndmp_backup_init_ndmp_native(jcr)) {
+            ndmp_backup_cleanup(jcr, JS_ErrorTerminated);
+            goto bail_out;
+         }
+         break;
       default:
          if (jcr->is_JobLevel(L_VIRTUAL_FULL)) {
             if (!do_native_vbackup_init(jcr)) {
@@ -272,7 +280,8 @@ bool setup_job(JCR *jcr, bool suppress_output)
       break;
    case JT_RESTORE:
       switch (jcr->getJobProtocol()) {
-      case PT_NDMP:
+      case PT_NDMP_BAREOS:
+      case PT_NDMP_NATIVE:
          if (!do_ndmp_restore_init(jcr)) {
             ndmp_restore_cleanup(jcr, JS_ErrorTerminated);
             goto bail_out;
@@ -401,7 +410,7 @@ void update_job_end(JCR *jcr, int TermCode)
    update_job_end_record(jcr);
 }
 
-/*
+/**
  * This is the engine called by jobq.c:jobq_add() when we were pulled from the work queue.
  *
  * At this point, we are running in our own thread and all necessary resources are
@@ -443,8 +452,8 @@ static void *job_thread(void *arg)
       jcr->res.job->RunScripts = New(alist(10, not_owned_by_alist));
    }
 
-   if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
-      Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
+   if (!jcr->db->update_job_start_record(jcr, &jcr->jr)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", jcr->db->strerror());
    }
 
    /*
@@ -462,8 +471,8 @@ static void *job_thread(void *arg)
     */
    jcr->start_time = time(NULL);
    jcr->jr.StartTime = jcr->start_time;
-   if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
-      Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
+   if (!jcr->db->update_job_start_record(jcr, &jcr->jr)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", jcr->db->strerror());
    }
 
    generate_plugin_event(jcr, bDirEventJobRun);
@@ -471,7 +480,7 @@ static void *job_thread(void *arg)
    switch (jcr->getJobType()) {
    case JT_BACKUP:
       switch (jcr->getJobProtocol()) {
-      case PT_NDMP:
+      case PT_NDMP_BAREOS:
          if (!job_canceled(jcr)) {
             if (do_ndmp_backup(jcr)) {
                do_autoprune(jcr);
@@ -482,6 +491,17 @@ static void *job_thread(void *arg)
             ndmp_backup_cleanup(jcr, JS_Canceled);
          }
          break;
+      case PT_NDMP_NATIVE:
+         if (!job_canceled(jcr)) {
+            if (do_ndmp_backup_ndmp_native(jcr)) {
+               do_autoprune(jcr);
+            } else {
+               ndmp_backup_cleanup(jcr, JS_ErrorTerminated);
+            }
+         } else {
+            ndmp_backup_cleanup(jcr, JS_Canceled);
+         }
+         break;
       default:
          if (!job_canceled(jcr)) {
             if (jcr->is_JobLevel(L_VIRTUAL_FULL)) {
@@ -520,7 +540,7 @@ static void *job_thread(void *arg)
       break;
    case JT_RESTORE:
       switch (jcr->getJobProtocol()) {
-      case PT_NDMP:
+      case PT_NDMP_BAREOS:
          if (!job_canceled(jcr)) {
             if (do_ndmp_restore(jcr)) {
                do_autoprune(jcr);
@@ -531,6 +551,17 @@ static void *job_thread(void *arg)
             ndmp_restore_cleanup(jcr, JS_Canceled);
          }
          break;
+      case PT_NDMP_NATIVE:
+         if (!job_canceled(jcr)) {
+            if (do_ndmp_restore_ndmp_native(jcr)) {
+               do_autoprune(jcr);
+            } else {
+               ndmp_restore_cleanup(jcr, JS_ErrorTerminated);
+            }
+         } else {
+            ndmp_restore_cleanup(jcr, JS_Canceled);
+         }
+         break;
       default:
          if (!job_canceled(jcr)) {
             if (do_native_restore(jcr)) {
@@ -631,7 +662,7 @@ void sd_msg_thread_send_signal(JCR *jcr, int sig)
    jcr->unlock();
 }
 
-/*
+/**
  * Cancel a job -- typically called by the UA (Console program), but may also
  *              be called by the job watchdog.
  *
@@ -749,7 +780,7 @@ static void job_monitor_watchdog(watchdog_t *self)
    endeach_jcr(jcr);
 }
 
-/*
+/**
  * Check if the maxwaittime has expired and it is possible
  *  to cancel the job.
  */
@@ -777,7 +808,7 @@ static bool job_check_maxwaittime(JCR *jcr)
    return cancel;
 }
 
-/*
+/**
  * Check if maxruntime has expired and if the job can be
  *   canceled.
  */
@@ -819,7 +850,7 @@ static bool job_check_maxruntime(JCR *jcr)
    return cancel;
 }
 
-/*
+/**
  * Check if MaxRunSchedTime has expired and if the job can be
  *   canceled.
  */
@@ -837,7 +868,7 @@ static bool job_check_maxrunschedtime(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * Get or create a Pool record with the given name.
  * Returns: 0 on error
  *          poolid if OK
@@ -850,11 +881,10 @@ DBId_t get_or_create_pool_record(JCR *jcr, char *pool_name)
    bstrncpy(pr.Name, pool_name, sizeof(pr.Name));
    Dmsg1(110, "get_or_create_pool=%s\n", pool_name);
 
-   while (!db_get_pool_record(jcr, jcr->db, &pr)) { /* get by Name */
+   while (!jcr->db->get_pool_record(jcr, &pr)) { /* get by Name */
       /* Try to create the pool */
       if (create_pool(jcr, jcr->db, jcr->res.pool, POOL_OP_CREATE) < 0) {
-         Jmsg(jcr, M_FATAL, 0, _("Pool \"%s\" not in database. ERR=%s"), pr.Name,
-            db_strerror(jcr->db));
+         Jmsg(jcr, M_FATAL, 0, _("Pool \"%s\" not in database. ERR=%s"), pr.Name, jcr->db->strerror());
          return 0;
       } else {
          Jmsg(jcr, M_INFO, 0, _("Created database record for Pool \"%s\".\n"), pr.Name);
@@ -863,7 +893,7 @@ DBId_t get_or_create_pool_record(JCR *jcr, char *pool_name)
    return pr.PoolId;
 }
 
-/*
+/**
  * Check for duplicate jobs.
  *  Returns: true  if current job should continue
  *           false if current job should terminate
@@ -996,7 +1026,7 @@ bool allow_duplicate_job(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * This subroutine edits the last job start time into a
  * "since=date/time" buffer that is returned in the
  * variable since.  This is used for display purposes in
@@ -1056,12 +1086,11 @@ bool get_level_since_time(JCR *jcr)
        * This is probably redundant, but some of the code below
        * uses jcr->stime, so don't remove unless you are sure.
        */
-      if (!db_find_job_start_time(jcr,jcr->db, &jcr->jr, jcr->stime, jcr->PrevJob)) {
+      if (!jcr->db->find_job_start_time(jcr, &jcr->jr, jcr->stime, jcr->PrevJob)) {
          do_full = true;
       }
 
-      have_full = db_find_last_job_start_time(jcr, jcr->db, &jcr->jr,
-                                              stime, prev_job, L_FULL);
+      have_full = jcr->db->find_last_job_start_time(jcr, &jcr->jr, stime, prev_job, L_FULL);
       if (have_full) {
          last_full_time = str_to_utime(stime);
       } else {
@@ -1078,8 +1107,7 @@ bool get_level_since_time(JCR *jcr)
          /*
           * Lookup last diff job
           */
-         if (db_find_last_job_start_time(jcr, jcr->db, &jcr->jr,
-                                         stime, prev_job, L_DIFFERENTIAL)) {
+         if (jcr->db->find_last_job_start_time(jcr, &jcr->jr, stime, prev_job, L_DIFFERENTIAL)) {
             last_diff_time = str_to_utime(stime);
             /*
              * If no Diff since Full, use Full time
@@ -1114,7 +1142,7 @@ bool get_level_since_time(JCR *jcr)
          /*
           * No recent Full job found, so upgrade this one to Full
           */
-         Jmsg(jcr, M_INFO, 0, "%s", db_strerror(jcr->db));
+         Jmsg(jcr, M_INFO, 0, "%s", jcr->db->strerror());
          Jmsg(jcr, M_INFO, 0, _("No prior or suitable Full backup found in catalog. Doing FULL backup.\n"));
          bsnprintf(jcr->since, sizeof(jcr->since), _(" (upgraded from %s)"), level_to_str(JobLevel));
          jcr->setJobLevel(jcr->jr.JobLevel = L_FULL);
@@ -1123,7 +1151,7 @@ bool get_level_since_time(JCR *jcr)
          /*
           * No recent Full job found, and MaxVirtualFull is set so upgrade this one to Virtual Full
           */
-         Jmsg(jcr, M_INFO, 0, "%s", db_strerror(jcr->db));
+         Jmsg(jcr, M_INFO, 0, "%s", jcr->db->strerror());
          Jmsg(jcr, M_INFO, 0, _("No prior or suitable Full backup found in catalog. Doing Virtual FULL backup.\n"));
          bsnprintf(jcr->since, sizeof(jcr->since), _(" (upgraded from %s)"), level_to_str(jcr->getJobLevel()));
          jcr->setJobLevel(jcr->jr.JobLevel = L_VIRTUAL_FULL);
@@ -1146,7 +1174,7 @@ bool get_level_since_time(JCR *jcr)
          pool_updated = true;
       } else {
          if (jcr->res.job->rerun_failed_levels) {
-            if (db_find_failed_job_since(jcr, jcr->db, &jcr->jr, jcr->stime, JobLevel)) {
+            if (jcr->db->find_failed_job_since(jcr, &jcr->jr, jcr->stime, JobLevel)) {
                Jmsg(jcr, M_INFO, 0, _("Prior failed job found in catalog. Upgrading to %s.\n"), level_to_str(JobLevel));
                bsnprintf(jcr->since, sizeof(jcr->since), _(" (upgraded from %s)"), level_to_str(JobLevel));
                jcr->setJobLevel(jcr->jr.JobLevel = JobLevel);
@@ -1166,8 +1194,8 @@ bool get_level_since_time(JCR *jcr)
        */
       if (jcr->PrevJob[0]) {
          bstrncpy(jcr->previous_jr.Job, jcr->PrevJob, sizeof(jcr->previous_jr.Job));
-         if (!db_get_job_record(jcr, jcr->db, &jcr->previous_jr)) {
-            Jmsg(jcr, M_FATAL, 0, _("Could not get job record for previous Job. ERR=%s"), db_strerror(jcr->db));
+         if (!jcr->db->get_job_record(jcr, &jcr->previous_jr)) {
+            Jmsg(jcr, M_FATAL, 0, _("Could not get job record for previous Job. ERR=%s"), jcr->db->strerror());
          }
       }
 
@@ -1239,10 +1267,10 @@ void apply_pool_overrides(JCR *jcr, bool force)
             pool_override = true;
             if (jcr->res.run_inc_pool_override) {
                pm_strcpy(jcr->res.pool_source, _("Run IncPool override"));
-               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.full_pool->name(), "Run IncPool override\n");
+               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.inc_pool->name(), "Run IncPool override\n");
             } else {
                pm_strcpy(jcr->res.pool_source, _("Job IncPool override"));
-               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.full_pool->name(), "Job IncPool override\n");
+               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.inc_pool->name(), "Job IncPool override\n");
             }
          }
          break;
@@ -1252,10 +1280,10 @@ void apply_pool_overrides(JCR *jcr, bool force)
             pool_override = true;
             if (jcr->res.run_diff_pool_override) {
                pm_strcpy(jcr->res.pool_source, _("Run DiffPool override"));
-               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.full_pool->name(), "Run DiffPool override\n");
+               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.diff_pool->name(), "Run DiffPool override\n");
             } else {
                pm_strcpy(jcr->res.pool_source, _("Job DiffPool override"));
-               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.full_pool->name(), "Job DiffPool override\n");
+               Dmsg2(100, "Pool set to '%s' because of %s", jcr->res.diff_pool->name(), "Job DiffPool override\n");
             }
          }
          break;
@@ -1271,7 +1299,7 @@ void apply_pool_overrides(JCR *jcr, bool force)
    }
 }
 
-/*
+/**
  * Get or create a Client record for this Job
  */
 bool get_or_create_client_record(JCR *jcr)
@@ -1287,9 +1315,8 @@ bool get_or_create_client_record(JCR *jcr)
       jcr->client_name = get_pool_memory(PM_NAME);
    }
    pm_strcpy(jcr->client_name, jcr->res.client->hdr.name);
-   if (!db_create_client_record(jcr, jcr->db, &cr)) {
-      Jmsg(jcr, M_FATAL, 0, _("Could not create Client record. ERR=%s\n"),
-         db_strerror(jcr->db));
+   if (!jcr->db->create_client_record(jcr, &cr)) {
+      Jmsg(jcr, M_FATAL, 0, _("Could not create Client record. ERR=%s\n"), jcr->db->strerror());
       return false;
    }
    /*
@@ -1297,10 +1324,9 @@ bool get_or_create_client_record(JCR *jcr)
     */
    if (jcr->res.client->HardQuota != 0 ||
        jcr->res.client->SoftQuota != 0) {
-      if (!db_get_quota_record(jcr, jcr->db, &cr)) {
-         if (!db_create_quota_record(jcr, jcr->db, &cr)) {
-            Jmsg(jcr, M_FATAL, 0, _("Could not create Quota record. ERR=%s\n"),
-               db_strerror(jcr->db));
+      if (!jcr->db->get_quota_record(jcr, &cr)) {
+         if (!jcr->db->create_quota_record(jcr, &cr)) {
+            Jmsg(jcr, M_FATAL, 0, _("Could not create Quota record. ERR=%s\n"), jcr->db->strerror());
          }
          jcr->res.client->QuotaLimit = 0;
          jcr->res.client->GraceTime = 0;
@@ -1344,15 +1370,15 @@ bool get_or_create_fileset_record(JCR *jcr)
       Jmsg(jcr, M_WARNING, 0, _("FileSet MD5 digest not found.\n"));
    }
    if (!jcr->res.fileset->ignore_fs_changes ||
-       !db_get_fileset_record(jcr, jcr->db, &fsr)) {
+       !jcr->db->get_fileset_record(jcr, &fsr)) {
       POOL_MEM FileSetText(PM_MESSAGE);
 
       jcr->res.fileset->print_config(FileSetText, false, false);
       fsr.FileSetText = FileSetText.c_str();
 
-      if (!db_create_fileset_record(jcr, jcr->db, &fsr)) {
+      if (!jcr->db->create_fileset_record(jcr, &fsr)) {
          Jmsg(jcr, M_ERROR, 0, _("Could not create FileSet \"%s\" record. ERR=%s\n"),
-              fsr.FileSet, db_strerror(jcr->db));
+              fsr.FileSet, jcr->db->strerror());
          return false;
       }
    }
@@ -1379,7 +1405,7 @@ void init_jcr_job_record(JCR *jcr)
    bstrncpy(jcr->jr.Job, jcr->Job, sizeof(jcr->jr.Job));
 }
 
-/*
+/**
  * Write status and such in DB
  */
 void update_job_end_record(JCR *jcr)
@@ -1395,13 +1421,12 @@ void update_job_end_record(JCR *jcr)
    jcr->jr.VolSessionTime = jcr->VolSessionTime;
    jcr->jr.JobErrors = jcr->JobErrors;
    jcr->jr.HasBase = jcr->HasBase;
-   if (!db_update_job_end_record(jcr, jcr->db, &jcr->jr)) {
-      Jmsg(jcr, M_WARNING, 0, _("Error updating job record. %s"),
-         db_strerror(jcr->db));
+   if (!jcr->db->update_job_end_record(jcr, &jcr->jr)) {
+      Jmsg(jcr, M_WARNING, 0, _("Error updating job record. %s"), jcr->db->strerror());
    }
 }
 
-/*
+/**
  * Takes base_name and appends (unique) current
  *   date and time to form unique job name.
  *
@@ -1459,7 +1484,9 @@ void create_unique_job_name(JCR *jcr, const char *base_name)
    Dmsg2(100, "JobId=%u created Job=%s\n", jcr->JobId, jcr->Job);
 }
 
-/* Called directly from job rescheduling */
+/**
+ * Called directly from job rescheduling
+ */
 void dird_free_jcr_pointers(JCR *jcr)
 {
    if (jcr->file_bsock) {
@@ -1488,7 +1515,7 @@ void dird_free_jcr_pointers(JCR *jcr)
    free_and_null_pool_memory(jcr->fname);
 }
 
-/*
+/**
  * Free the Job Control Record if no one is still using it.
  *  Called from main free_jcr() routine in src/lib/jcr.c so
  *  that we can do our Director specific cleanup of the jcr.
@@ -1551,7 +1578,7 @@ void dird_free_jcr(JCR *jcr)
     */
    free_rwstorage(jcr);
 
-   jcr->job_end_push.destroy();
+   jcr->job_end_callbacks.destroy();
 
    if (jcr->JobId != 0) {
       write_state_file(me->working_directory, "bareos-dir", get_first_port_host_order(me->DIRaddrs));
@@ -1562,7 +1589,7 @@ void dird_free_jcr(JCR *jcr)
    Dmsg0(200, "End dird free_jcr\n");
 }
 
-/*
+/**
  * The Job storage definition must be either in the Job record
  * or in the Pool record.  The Pool record overrides the Job record.
  */
@@ -1589,7 +1616,7 @@ void get_job_storage(USTORERES *store, JOBRES *job, RUNRES *run)
    }
 }
 
-/*
+/**
  * Set some defaults in the JCR necessary to
  * run. These items are pulled from the job
  * definition as defaults, but can be overridden
@@ -1754,7 +1781,7 @@ void create_clones(JCR *jcr)
    }
 }
 
-/*
+/**
  * Given: a JobId in jcr->previous_jr.JobId,
  *  this subroutine writes a bsr file to restore that job.
  * Returns: -1 on error
