@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2004-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -22,11 +22,15 @@
 */
 /*
  * BAREOS Director -- migrate.c -- responsible for doing migration and copy jobs.
- *
  * Also handles Copy jobs (March 2008)
- *
  * Kern Sibbald, September 2004
  * SD-SD Migration by Marco van Wieringen, November 2012
+ */
+/**
+ * @file
+ * responsible for doing migration and copy jobs.
+ *
+ * Also handles Copy jobs
  *
  * Basic tasks done here:
  *    Open DB and create records for this job.
@@ -47,14 +51,14 @@
 static char replicatecmd[]  =
    "replicate Job=%s address=%s port=%d ssl=%d Authorization=%s\n";
 
-/*
+/**
  * Get Job names in Pool
  */
 static const char *sql_job =
    "SELECT DISTINCT Job.Name from Job,Pool"
    " WHERE Pool.Name='%s' AND Job.PoolId=Pool.PoolId";
 
-/*
+/**
  * Get JobIds from regex'ed Job names
  */
 static const char *sql_jobids_from_job =
@@ -62,7 +66,7 @@ static const char *sql_jobids_from_job =
    " WHERE Job.Name='%s' AND Pool.Name='%s' AND Job.PoolId=Pool.PoolId"
    " ORDER by Job.StartTime";
 
-/*
+/**
  * Get Client names in Pool
  */
 static const char *sql_client =
@@ -70,7 +74,7 @@ static const char *sql_client =
    " WHERE Pool.Name='%s' AND Job.ClientId=Client.ClientId AND"
    " Job.PoolId=Pool.PoolId";
 
-/*
+/**
  * Get JobIds from regex'ed Client names
  */
 static const char *sql_jobids_from_client =
@@ -80,7 +84,7 @@ static const char *sql_jobids_from_client =
    " AND Job.JobStatus IN ('T','W')"
    " ORDER by Job.StartTime";
 
-/*
+/**
  * Get Volume names in Pool
  */
 static const char *sql_vol =
@@ -88,7 +92,7 @@ static const char *sql_vol =
    " VolStatus in ('Full','Used','Error') AND Media.Enabled=1 AND"
    " Media.PoolId=Pool.PoolId AND Pool.Name='%s'";
 
-/*
+/**
  * Get JobIds from regex'ed Volume names
  */
 static const char *sql_jobids_from_vol =
@@ -98,7 +102,7 @@ static const char *sql_jobids_from_vol =
    " AND Job.JobStatus IN ('T','W') AND Media.Enabled=1"
    " ORDER by Job.StartTime";
 
-/*
+/**
  * Get JobIds from the smallest volume
  */
 static const char *sql_smallest_vol =
@@ -108,7 +112,7 @@ static const char *sql_smallest_vol =
    " Media.PoolId=Pool.PoolId AND Pool.Name='%s'"
    " ORDER BY VolBytes ASC LIMIT 1";
 
-/*
+/**
  * Get JobIds from the oldest volume
  */
 static const char *sql_oldest_vol =
@@ -118,7 +122,7 @@ static const char *sql_oldest_vol =
    " Media.PoolId=Pool.PoolId AND Pool.Name='%s'"
    " ORDER BY LastWritten ASC LIMIT 1";
 
-/*
+/**
  * Get JobIds when we have selected MediaId
  */
 static const char *sql_jobids_from_mediaid =
@@ -127,7 +131,7 @@ static const char *sql_jobids_from_mediaid =
    " AND Job.Type IN ('B','C') AND Job.JobStatus IN ('T','W')"
    " ORDER by Job.StartTime";
 
-/*
+/**
  * Get the number of bytes in the pool
  */
 static const char *sql_pool_bytes =
@@ -138,13 +142,13 @@ static const char *sql_pool_bytes =
    " Job.Type IN ('B','C') AND Job.JobStatus IN ('T','W') AND"
    " JobMedia.JobId=Job.JobId AND Job.PoolId=Media.PoolId)";
 
-/*
+/**
  * Get the number of bytes in the Jobs
  */
 static const char *sql_job_bytes =
    "SELECT SUM(JobBytes) FROM Job WHERE JobId IN (%s)";
 
-/*
+/**
  * Get Media Ids in Pool
  */
 static const char *sql_mediaids =
@@ -152,7 +156,7 @@ static const char *sql_mediaids =
    " VolStatus in ('Full','Used','Error') AND Media.Enabled=1 AND"
    " Media.PoolId=Pool.PoolId AND Pool.Name='%s' ORDER BY LastWritten ASC";
 
-/*
+/**
  * Get JobIds in Pool longer than specified time
  */
 static const char *sql_pool_time =
@@ -163,7 +167,7 @@ static const char *sql_pool_time =
    " JobMedia.JobId=Job.JobId AND Job.PoolId=Media.PoolId"
    " AND Job.RealEndTime<='%s'";
 
-/*
+/**
  * Get JobIds from successfully completed backup jobs which have not been copied before
  */
 static const char *sql_jobids_of_pool_uncopied_jobs =
@@ -177,27 +181,27 @@ static const char *sql_jobids_of_pool_uncopied_jobs =
    " AND PriorJobId != 0)"
    " ORDER by Job.StartTime";
 
-/*
+/**
  * Migrate NDMP Job MetaData.
  */
 static const char *sql_migrate_ndmp_metadata =
    "UPDATE File SET JobId=%s "
    "WHERE JobId=%s "
-   "AND FilenameId NOT IN ("
-   "SELECT FilenameId "
+   "AND Name NOT IN ("
+   "SELECT Name "
    "FROM File "
    "WHERE JobId=%s)";
 
-/*
+/**
  * Copy NDMP Job MetaData.
  */
 static const char *sql_copy_ndmp_metadata =
-   "INSERT INTO File (FileIndex, JobId, PathId, FilenameId, DeltaSeq, MarkId, LStat, MD5) "
-   "SELECT FileIndex, %s, PathId, FilenameId, DeltaSeq, MarkId, LStat, MD5 "
+   "INSERT INTO File (FileIndex, JobId, PathId, Name, DeltaSeq, MarkId, LStat, MD5) "
+   "SELECT FileIndex, %s, PathId, Name, DeltaSeq, MarkId, LStat, MD5 "
    "FROM File "
    "WHERE JobId=%s "
-   "AND FilenameId NOT IN ("
-   "SELECT FilenameId "
+   "AND Name NOT IN ("
+   "SELECT Name "
    "FROM File "
    "WHERE JobId=%s)";
 
@@ -208,7 +212,7 @@ struct idpkt {
    uint32_t count;
 };
 
-/*
+/**
  * See if two storage definitions point to the same Storage Daemon.
  *
  * We compare:
@@ -242,7 +246,7 @@ bool set_migration_wstorage(JCR *jcr, POOLRES *pool, POOLRES *next_pool, const c
    return true;
 }
 
-/*
+/**
  * set_migration_next_pool() called by do_migration_init()
  * at differents stages.
  *
@@ -263,9 +267,9 @@ static inline bool set_migration_next_pool(JCR *jcr, POOLRES **retpool)
     */
    memset(&pr, 0, sizeof(pr));
    pr.PoolId = jcr->jr.PoolId;
-   if (!db_get_pool_record(jcr, jcr->db, &pr)) {
+   if (!jcr->db->get_pool_record(jcr, &pr)) {
       Jmsg(jcr, M_FATAL, 0, _("Pool for JobId %s not in database. ERR=%s\n"),
-           edit_int64(pr.PoolId, ed1), db_strerror(jcr->db));
+           edit_int64(pr.PoolId, ed1), jcr->db->strerror());
          return false;
    }
 
@@ -328,7 +332,7 @@ static inline bool set_migration_next_pool(JCR *jcr, POOLRES **retpool)
    return true;
 }
 
-/*
+/**
  * Sanity check that we are not using the same storage for reading and writing.
  */
 static inline bool same_storage(JCR *jcr)
@@ -392,7 +396,7 @@ static inline void start_new_migration_job(JCR *jcr)
    free_ua_context(ua);
 }
 
-/*
+/**
  * Return next DBId from comma separated list
  *
  * Returns:
@@ -431,7 +435,7 @@ static inline int get_next_dbid_from_list(char **p, DBId_t *DBId)
    return 1;
 }
 
-/*
+/**
  * Add an item to the list if it is unique
  */
 static void add_unique_id(idpkt *ids, char *item)
@@ -475,7 +479,7 @@ static void add_unique_id(idpkt *ids, char *item)
    return;
 }
 
-/*
+/**
  * Callback handler make list of DB Ids
  */
 static int unique_dbid_handler(void *ctx, int num_fields, char **row)
@@ -529,7 +533,7 @@ static int unique_name_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-/*
+/**
  * This routine returns:
  *    false       if an error occurred
  *    true        otherwise
@@ -542,8 +546,8 @@ static bool find_jobids_from_mediaid_list(JCR *jcr, idpkt *ids, const char *type
 
    Mmsg(query, sql_jobids_from_mediaid, ids->list);
    ids->count = 0;
-   if (!db_sql_query(jcr->db, query.c_str(), unique_dbid_handler, (void *)ids)) {
-      Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+   if (!jcr->db->sql_query(query.c_str(), unique_dbid_handler, (void *)ids)) {
+      Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), jcr->db->strerror());
       goto bail_out;
    }
    if (ids->count == 0) {
@@ -568,8 +572,8 @@ static bool find_mediaid_then_jobids(JCR *jcr, idpkt *ids,
     * Basic query for MediaId
     */
    Mmsg(query, query1, jcr->res.rpool->name());
-   if (!db_sql_query(jcr->db, query.c_str(), unique_dbid_handler, (void *)ids)) {
-      Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+   if (!jcr->db->sql_query(query.c_str(), unique_dbid_handler, (void *)ids)) {
+      Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), jcr->db->strerror());
       goto bail_out;
    }
 
@@ -590,7 +594,7 @@ bail_out:
    return ok;
 }
 
-/*
+/**
  * This routine returns:
  *    false       if an error occurred
  *    true        otherwise
@@ -613,9 +617,9 @@ static inline bool find_jobids_of_pool_uncopied_jobs(JCR *jcr, idpkt *ids)
    Dmsg1(dbglevel, "copy selection pattern=%s\n", jcr->res.rpool->name());
    Mmsg(query, sql_jobids_of_pool_uncopied_jobs, jcr->res.rpool->name());
    Dmsg1(dbglevel, "get uncopied jobs query=%s\n", query.c_str());
-   if (!db_sql_query(jcr->db, query.c_str(), unique_dbid_handler, (void *)ids)) {
+   if (!jcr->db->sql_query(query.c_str(), unique_dbid_handler, (void *)ids)) {
       Jmsg(jcr, M_FATAL, 0,
-           _("SQL to get uncopied jobs failed. ERR=%s\n"), db_strerror(jcr->db));
+           _("SQL to get uncopied jobs failed. ERR=%s\n"), jcr->db->strerror());
       goto bail_out;
    }
    ok = true;
@@ -651,10 +655,10 @@ static bool regex_find_jobids(JCR *jcr, idpkt *ids,
     */
    Mmsg(query, query1, jcr->res.rpool->name());
    Dmsg1(dbglevel, "get name query1=%s\n", query.c_str());
-   if (!db_sql_query(jcr->db, query.c_str(), unique_name_handler,
+   if (!jcr->db->sql_query(query.c_str(), unique_name_handler,
         (void *)item_chain)) {
       Jmsg(jcr, M_FATAL, 0,
-           _("SQL to get %s failed. ERR=%s\n"), type, db_strerror(jcr->db));
+           _("SQL to get %s failed. ERR=%s\n"), type, jcr->db->strerror());
       goto bail_out;
    }
    Dmsg1(dbglevel, "query1 returned %d names\n", item_chain->size());
@@ -718,9 +722,9 @@ static bool regex_find_jobids(JCR *jcr, idpkt *ids,
       Dmsg2(dbglevel, "Got %s: %s\n", type, item->item);
       Mmsg(query, query2, item->item, jcr->res.rpool->name());
       Dmsg1(dbglevel, "get id from name query2=%s\n", query.c_str());
-      if (!db_sql_query(jcr->db, query.c_str(), unique_dbid_handler, (void *)ids)) {
+      if (!jcr->db->sql_query(query.c_str(), unique_dbid_handler, (void *)ids)) {
          Jmsg(jcr, M_FATAL, 0,
-              _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+              _("SQL failed. ERR=%s\n"), jcr->db->strerror());
          goto bail_out;
       }
    }
@@ -740,7 +744,7 @@ bail_out:
    return ok;
 }
 
-/*
+/**
  * This is the central piece of code that finds jobs actually JobIds to migrate.
  * It examines the Selection Type to see what kind of migration we are doing
  * (Volume, Job, Client, ...) and applies any Selection Pattern if appropriate
@@ -793,10 +797,10 @@ static inline bool getJobs_to_migrate(JCR *jcr)
          goto bail_out;
       }
       Dmsg1(dbglevel, "SQL=%s\n", jcr->res.job->selection_pattern);
-      if (!db_sql_query(jcr->db, jcr->res.job->selection_pattern,
+      if (!jcr->db->sql_query(jcr->res.job->selection_pattern,
            unique_dbid_handler, (void *)&ids)) {
          Jmsg(jcr, M_FATAL, 0,
-              _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+              _("SQL failed. ERR=%s\n"), jcr->db->strerror());
          goto bail_out;
       }
       break;
@@ -827,8 +831,8 @@ static inline bool getJobs_to_migrate(JCR *jcr)
        */
       Mmsg(query, sql_pool_bytes, jcr->res.rpool->name());
 
-      if (!db_sql_query(jcr->db, query.c_str(), db_int64_handler, (void *)&ctx)) {
-         Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+      if (!jcr->db->sql_query(query.c_str(), db_int64_handler, (void *)&ctx)) {
+         Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), jcr->db->strerror());
          goto bail_out;
       }
 
@@ -856,8 +860,8 @@ static inline bool getJobs_to_migrate(JCR *jcr)
       Mmsg(query, sql_mediaids, jcr->res.rpool->name());
       Dmsg1(dbglevel, "query=%s\n", query.c_str());
 
-      if (!db_sql_query(jcr->db, query.c_str(), unique_dbid_handler, (void *)&ids)) {
-         Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+      if (!jcr->db->sql_query(query.c_str(), unique_dbid_handler, (void *)&ids)) {
+         Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), jcr->db->strerror());
          goto bail_out;
       }
 
@@ -900,8 +904,8 @@ static inline bool getJobs_to_migrate(JCR *jcr)
           */
          Mmsg(query, sql_job_bytes, mid.list);
          Dmsg1(dbglevel, "Jobbytes query: %s\n", query.c_str());
-         if (!db_sql_query(jcr->db, query.c_str(), db_int64_handler, (void *)&ctx)) {
-            Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+         if (!jcr->db->sql_query(query.c_str(), db_int64_handler, (void *)&ctx)) {
+            Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), jcr->db->strerror());
             goto bail_out;
          }
          pool_bytes -= ctx.value;
@@ -935,8 +939,8 @@ static inline bool getJobs_to_migrate(JCR *jcr)
       Mmsg(query, sql_pool_time, jcr->res.rpool->name(), dt);
       Dmsg1(dbglevel, "query=%s\n", query.c_str());
 
-      if (!db_sql_query(jcr->db, query.c_str(), unique_dbid_handler, (void *)&ids)) {
-         Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), db_strerror(jcr->db));
+      if (!jcr->db->sql_query(query.c_str(), unique_dbid_handler, (void *)&ids)) {
+         Jmsg(jcr, M_FATAL, 0, _("SQL failed. ERR=%s\n"), jcr->db->strerror());
          goto bail_out;
       }
 
@@ -1025,7 +1029,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Called here before the job is run to do the job
  * specific setup. Note, one of the important things to
  * complete in this init code is to make the definitive
@@ -1076,10 +1080,10 @@ bool do_migration_init(JCR *jcr)
       jcr->previous_jr.JobId = jcr->MigrateJobId;
       Dmsg1(dbglevel, "Previous jobid=%d\n", (int)jcr->previous_jr.JobId);
 
-      if (!db_get_job_record(jcr, jcr->db, &jcr->previous_jr)) {
+      if (!jcr->db->get_job_record(jcr, &jcr->previous_jr)) {
          Jmsg(jcr, M_FATAL, 0, _("Could not get job record for JobId %s to %s. ERR=%s"),
               edit_int64(jcr->previous_jr.JobId, ed1),
-              jcr->get_ActionName(), db_strerror(jcr->db));
+              jcr->get_ActionName(), jcr->db->strerror());
          return false;
       }
 
@@ -1177,7 +1181,7 @@ bool do_migration_init(JCR *jcr)
       set_jcr_defaults(mig_jcr, prev_job);
 
       /*
-       * Don't let WatchDog checks Max*Time value on this Job
+       * Don't let Watchdog checks Max*Time value on this Job
        */
       mig_jcr->no_maxtime = true;
 
@@ -1192,13 +1196,14 @@ bool do_migration_init(JCR *jcr)
       mig_jcr->spool_data = jcr->spool_data;
       mig_jcr->spool_size = jcr->spool_size;
 
+
       if (!setup_job(mig_jcr, true)) {
          Jmsg(jcr, M_FATAL, 0, _("setup job failed.\n"));
          return false;
       }
 
       /*
-       * Keep track that the mig_jcr has a controling JCR.
+       * Keep track that the mig_jcr has a controlling JCR.
        */
       mig_jcr->cjcr = jcr;
 
@@ -1213,10 +1218,6 @@ bool do_migration_init(JCR *jcr)
       mig_jcr->jr.PoolId = jcr->jr.PoolId;
       mig_jcr->jr.JobId = mig_jcr->JobId;
 
-      Dmsg4(dbglevel, "mig_jcr: Name=%s JobId=%d Type=%c Level=%c\n",
-            mig_jcr->jr.Name, (int)mig_jcr->jr.JobId, mig_jcr->jr.JobType,
-            mig_jcr->jr.JobLevel);
-
       if (set_migration_next_pool(jcr, &pool)) {
          /*
           * If pool storage specified, use it as source
@@ -1244,12 +1245,23 @@ bool do_migration_init(JCR *jcr)
        * one to the writing SD.
        */
       jcr->remote_replicate = !is_same_storage_daemon(jcr->res.rstore, jcr->res.wstore);
+
+      /*
+       * set the JobLevel to what the original job was
+       */
+      mig_jcr->setJobLevel(mig_jcr->previous_jr.JobLevel);
+
+
+      Dmsg4(dbglevel, "mig_jcr: Name=%s JobId=%d Type=%c Level=%c\n",
+            mig_jcr->jr.Name, (int)mig_jcr->jr.JobId, mig_jcr->jr.JobType,
+            mig_jcr->jr.JobLevel);
+
    }
 
    return true;
 }
 
-/*
+/**
  * Do a Migration of a previous job
  *
  * - previous_jr refers to the job DB record of the Job that is
@@ -1326,38 +1338,14 @@ static inline bool do_actual_migration(JCR *jcr)
          ((STORERES *)jcr->res.rstorage->first())->name(),
          ((STORERES *)jcr->res.wstorage->first())->name());
 
-   if (!jcr->remote_replicate) {
-      /*
-       * Open a message channel connection with the Storage daemon.
-       */
-      Dmsg0(110, "Open connection with storage daemon\n");
-      jcr->setJobStatus(JS_WaitSD);
-      mig_jcr->setJobStatus(JS_WaitSD);
+   if (jcr->remote_replicate) {
 
-      /*
-       * Start conversation with Storage daemon
-       */
-      if (!connect_to_storage_daemon(jcr, 10, me->SDConnectTimeout, true)) {
-         free_paired_storage(jcr);
-         return false;
-      }
-
-      /*
-       * Now start a job with the Storage daemon
-       */
-      if (!start_storage_daemon_job(jcr, jcr->res.rstorage, jcr->res.wstorage, /* send_bsr */ true)) {
-         free_paired_storage(jcr);
-         return false;
-      }
-
-      Dmsg0(150, "Storage daemon connection OK\n");
-   } else {
       alist *wstorage;
 
       /*
        * See if we need to apply any bandwidth limiting.
        * We search the bandwidth limiting in the following way:
-       * - Job bandwith limiting
+       * - Job bandwidth limiting
        * - Writing Storage Daemon bandwidth limiting
        * - Reading Storage Daemon bandwidth limiting
        */
@@ -1422,11 +1410,39 @@ static inline bool do_actual_migration(JCR *jcr)
       /*
        * Now start a job with the Writing Storage daemon
        */
+
       if (!start_storage_daemon_job(mig_jcr, NULL, mig_jcr->res.wstorage, /* send_bsr */ false)) {
          goto bail_out;
       }
 
       Dmsg0(150, "Writing Storage daemon connection OK\n");
+
+   } else { /* local replicate */
+
+      /*
+       * Open a message channel connection with the Storage daemon.
+       */
+      Dmsg0(110, "Open connection with storage daemon\n");
+      jcr->setJobStatus(JS_WaitSD);
+      mig_jcr->setJobStatus(JS_WaitSD);
+
+      /*
+       * Start conversation with Storage daemon
+       */
+      if (!connect_to_storage_daemon(jcr, 10, me->SDConnectTimeout, true)) {
+         free_paired_storage(jcr);
+         return false;
+      }
+
+      /*
+       * Now start a job with the Storage daemon
+       */
+      if (!start_storage_daemon_job(jcr, jcr->res.rstorage, jcr->res.wstorage, /* send_bsr */ true)) {
+         free_paired_storage(jcr);
+         return false;
+      }
+
+      Dmsg0(150, "Storage daemon connection OK\n");
    }
 
    /*
@@ -1447,8 +1463,8 @@ static inline bool do_actual_migration(JCR *jcr)
    /*
     * Update job start record for this migration control job
     */
-   if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
-      Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
+   if (!jcr->db->update_job_start_record(jcr, &jcr->jr)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", jcr->db->strerror());
       goto bail_out;
    }
 
@@ -1465,8 +1481,8 @@ static inline bool do_actual_migration(JCR *jcr)
    /*
     * Update job start record for the real migration backup job
     */
-   if (!db_update_job_start_record(mig_jcr, mig_jcr->db, &mig_jcr->jr)) {
-      Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(mig_jcr->db));
+   if (!mig_jcr->db->update_job_start_record(mig_jcr, &mig_jcr->jr)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", mig_jcr->db->strerror());
       goto bail_out;
    }
 
@@ -1560,11 +1576,11 @@ static inline bool do_actual_migration(JCR *jcr)
       wait_for_storage_daemon_termination(jcr);
       wait_for_storage_daemon_termination(mig_jcr);
       jcr->setJobStatus(jcr->SDJobStatus);
-      db_write_batch_file_records(mig_jcr);
+      mig_jcr->db_batch->write_batch_file_records(mig_jcr);
    } else {
       wait_for_storage_daemon_termination(jcr);
       jcr->setJobStatus(jcr->SDJobStatus);
-      db_write_batch_file_records(jcr);
+      jcr->db_batch->write_batch_file_records(jcr);
    }
 
 bail_out:
@@ -1600,7 +1616,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Select the Jobs to Migrate/Copy using the getJobs_to_migrate function and then exit.
  */
 static inline bool do_migration_selection(JCR *jcr)
@@ -1750,7 +1766,7 @@ static inline void generate_migrate_summary(JCR *jcr, MEDIA_DBR *mr, int msg_typ
    }
 }
 
-/*
+/**
  * Release resources allocated during backup.
  */
 void migration_cleanup(JCR *jcr, int TermCode)
@@ -1808,7 +1824,7 @@ void migration_cleanup(JCR *jcr, int TermCode)
            jcr->previous_jr.cStartTime, jcr->previous_jr.cEndTime,
            edit_uint64(jcr->previous_jr.JobTDate, ec1),
            new_jobid);
-      db_sql_query(mig_jcr->db, query.c_str());
+      jcr->db->sql_query(query.c_str());
 
       if (jcr->is_terminated_ok()) {
          UAContext *ua;
@@ -1824,14 +1840,14 @@ void migration_cleanup(JCR *jcr, int TermCode)
              */
             Mmsg(query, "UPDATE Job SET Type='%c' WHERE JobId=%s",
                  (char)JT_MIGRATED_JOB, old_jobid);
-            db_sql_query(mig_jcr->db, query.c_str());
+            mig_jcr->db->sql_query(query.c_str());
 
             /*
              * Move JobLog to new JobId
              */
             Mmsg(query, "UPDATE Log SET JobId=%s WHERE JobId=%s",
                  new_jobid, old_jobid);
-            db_sql_query(mig_jcr->db, query.c_str());
+            mig_jcr->db->sql_query(query.c_str());
 
             /*
              * If we just migrated a NDMP job, we need to move the file MetaData
@@ -1845,7 +1861,7 @@ void migration_cleanup(JCR *jcr, int TermCode)
             case APT_NDMPV3:
             case APT_NDMPV4:
                Mmsg(query, sql_migrate_ndmp_metadata, new_jobid, old_jobid, new_jobid);
-               db_sql_query(mig_jcr->db, query.c_str());
+               mig_jcr->db->sql_query(query.c_str());
                break;
             default:
                break;
@@ -1876,7 +1892,7 @@ void migration_cleanup(JCR *jcr, int TermCode)
             Mmsg(query, "INSERT INTO Log (JobId, Time, LogText ) "
                         "SELECT %s, Time, LogText FROM Log WHERE JobId=%s",
                  new_jobid, old_jobid);
-            db_sql_query(mig_jcr->db, query.c_str());
+            mig_jcr->db->sql_query(query.c_str());
 
             /*
              * If we just copied a NDMP job, we need to copy the file MetaData
@@ -1890,7 +1906,7 @@ void migration_cleanup(JCR *jcr, int TermCode)
             case APT_NDMPV3:
             case APT_NDMPV4:
                Mmsg(query, sql_copy_ndmp_metadata, new_jobid, old_jobid, new_jobid);
-               db_sql_query(mig_jcr->db, query.c_str());
+               mig_jcr->db->sql_query(query.c_str());
                break;
             default:
                break;
@@ -1898,22 +1914,21 @@ void migration_cleanup(JCR *jcr, int TermCode)
 
             Mmsg(query, "UPDATE Job SET Type='%c' WHERE JobId=%s",
                  (char)JT_JOB_COPY, new_jobid);
-            db_sql_query(mig_jcr->db, query.c_str());
+            mig_jcr->db->sql_query(query.c_str());
             break;
          default:
             break;
          }
       }
 
-      if (!db_get_job_record(jcr, jcr->db, &jcr->jr)) {
-         Jmsg(jcr, M_WARNING, 0, _("Error getting Job record for Job report: ERR=%s"),
-              db_strerror(jcr->db));
+      if (!jcr->db->get_job_record(jcr, &jcr->jr)) {
+         Jmsg(jcr, M_WARNING, 0, _("Error getting Job record for Job report: ERR=%s"), jcr->db->strerror());
          jcr->setJobStatus(JS_ErrorTerminated);
       }
 
       update_bootstrap_file(mig_jcr);
 
-      if (!db_get_job_volume_names(mig_jcr, mig_jcr->db, mig_jcr->jr.JobId, mig_jcr->VolumeName)) {
+      if (!mig_jcr->db->get_job_volume_names(mig_jcr, mig_jcr->jr.JobId, mig_jcr->VolumeName)) {
          /*
           * Note, if the job has failed, most likely it did not write any
           * tape, so suppress this "error" message since in that case
@@ -1921,7 +1936,7 @@ void migration_cleanup(JCR *jcr, int TermCode)
           * normal exit should we complain about this error.
           */
          if (jcr->is_terminated_ok() && jcr->jr.JobBytes) {
-            Jmsg(jcr, M_ERROR, 0, "%s", db_strerror(mig_jcr->db));
+            Jmsg(jcr, M_ERROR, 0, "%s", mig_jcr->db->strerror());
          }
          mig_jcr->VolumeName[0] = 0;         /* none */
       }
@@ -1937,9 +1952,9 @@ void migration_cleanup(JCR *jcr, int TermCode)
             p = mig_jcr->VolumeName;     /* no |, take full name */
          }
          bstrncpy(mr.VolumeName, p, sizeof(mr.VolumeName));
-         if (!db_get_media_record(jcr, jcr->db, &mr)) {
+         if (!jcr->db->get_media_record(jcr, &mr)) {
             Jmsg(jcr, M_WARNING, 0, _("Error getting Media record for Volume \"%s\": ERR=%s"),
-                 mr.VolumeName, db_strerror(jcr->db));
+                 mr.VolumeName, jcr->db->strerror());
          }
       }
 
