@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,20 +21,20 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- msgchan.c -- handles the message channel
- *                                 to the Storage daemon and the File daemon.
- *
  * Kern Sibbald, August MM
- *
- * This routine runs as a thread and must be thread reentrant.
- *
- * Basic tasks done here:
- *    Open a message channel with the Storage daemon
- *      to authenticate ourself and to pass the JobId.
- *    Create a thread to interact with the Storage daemon
- *      who returns a job status and requests Catalog services, etc.
  */
 
+/**
+ * @file
+ *
+ * handles the message channel to the Storage daemon and the File daemon.
+ *
+ * * Basic tasks done here:
+ *    * Open a message channel with the Storage daemon
+ *      to authenticate ourself and to pass the JobId.
+ *    * Create a thread to interact with the Storage daemon
+ *      who returns a job status and requests Catalog services, etc.
+ */
 #include "bareos.h"
 #include "dird.h"
 
@@ -99,10 +99,13 @@ bool update_device_res(JCR *jcr, DEVICERES *dev)
 }
 #endif
 
-/*
- * Send bootstrap file to Storage daemon.
- *  This is used for restore, verify VolumeToCatalog, migration,
- *    and copy Jobs.
+/** Send bootstrap file to Storage daemon.
+ *  This is used for
+ *    * restore
+ *    * verify
+ *    * VolumeToCatalog
+ *    * migration and
+ *    * copy Jobs
  */
 static inline bool send_bootstrap_file_to_sd(JCR *jcr, BSOCK *sd)
 {
@@ -135,8 +138,7 @@ static inline bool send_bootstrap_file_to_sd(JCR *jcr, BSOCK *sd)
    return true;
 }
 
-/*
- * Start a job with the Storage daemon
+/** Start a job with the Storage daemon
  */
 bool start_storage_daemon_job(JCR *jcr, alist *rstore, alist *wstore, bool send_bsr)
 {
@@ -359,8 +361,7 @@ bool start_storage_daemon_job(JCR *jcr, alist *rstore, alist *wstore, bool send_
    return ok;
 }
 
-/*
- * Start a thread to handle Storage daemon messages and
+/** Start a thread to handle Storage daemon messages and
  *  Catalog requests.
  */
 bool start_storage_daemon_message_thread(JCR *jcr)
@@ -391,7 +392,7 @@ extern "C" void msg_thread_cleanup(void *arg)
 {
    JCR *jcr = (JCR *)arg;
 
-   db_end_transaction(jcr, jcr->db);        /* terminate any open transaction */
+   jcr->db->end_transaction(jcr);           /* terminate any open transaction */
    jcr->lock();
    jcr->sd_msg_thread_done = true;
    jcr->SD_msg_chan_started = false;
@@ -399,12 +400,11 @@ extern "C" void msg_thread_cleanup(void *arg)
    pthread_cond_broadcast(&jcr->nextrun_ready); /* wakeup any waiting threads */
    pthread_cond_broadcast(&jcr->term_wait); /* wakeup any waiting threads */
    Dmsg2(100, "=== End msg_thread. JobId=%d usecnt=%d\n", jcr->JobId, jcr->use_count());
-   db_thread_cleanup(jcr->db);              /* remove thread specific data */
+   jcr->db->thread_cleanup();               /* remove thread specific data */
    free_jcr(jcr);                           /* release jcr */
 }
 
-/*
- * Handle the message channel (i.e. requests from the
+/** Handle the message channel (i.e. requests from the
  *  Storage daemon).
  * Note, we are running in a separate thread.
  */
@@ -468,10 +468,12 @@ extern "C" void *msg_thread(void *arg)
    }
    if (n == BNET_HARDEOF) {
       /*
-       * This probably should be M_FATAL, but I am not 100% sure
-       *  that this return *always* corresponds to a dropped line.
+       * A lost connection to the storage daemon is FATAL.
+       * This is required, as otherwise
+       * the job could failed to write data
+       * but still end as JS_Warnings (OK -- with warnings).
        */
-      Qmsg(jcr, M_ERROR, 0, _("Director's comm line to SD dropped.\n"));
+      Qmsg(jcr, M_FATAL, 0, _("Director's comm line to SD dropped.\n"));
    }
    if (is_bnet_error(sd)) {
       jcr->SDJobStatus = JS_ErrorTerminated;
@@ -547,8 +549,7 @@ extern "C" void *device_thread(void *arg)
    return NULL;
 }
 
-/*
- * Start a thread to handle getting Device resource information
+/** Start a thread to handle getting Device resource information
  *  from SD. This is called once at startup of the Director.
  */
 void init_device_resources()
