@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2011-2015 Planets Communications B.V.
-   Copyright (C) 2013-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,19 +20,21 @@
    02110-1301, USA.
 */
 /*
- * Generic NDMP Data Management Application (DMA) routines
- *
  * Marco van Wieringen, May 2015
  */
+/**
+ * @file
+ * Generic NDMP Data Management Application (DMA) routines
+ */
 
 #include "bareos.h"
 #include "dird.h"
 
 #if HAVE_NDMP
 
-#define SMTAPE_MIN_BLOCKSIZE 4096 /* 4 Kb */
-#define SMTAPE_MAX_BLOCKSIZE 262144 /* 256 Kb */
-#define SMTAPE_BLOCKSIZE_INCREMENTS 4096 /* 4 Kb */
+#define SMTAPE_MIN_BLOCKSIZE 4096 /**< 4 Kb */
+#define SMTAPE_MAX_BLOCKSIZE 262144 /**< 256 Kb */
+#define SMTAPE_BLOCKSIZE_INCREMENTS 4096 /**< 4 Kb */
 
 #include "ndmp/ndmagents.h"
 #include "ndmp_dma_priv.h"
@@ -41,10 +43,11 @@
 
 /* Forward referenced functions */
 
-/*
+/**
  * Per NDMP format specific options.
  */
 static ndmp_backup_format_option ndmp_backup_format_options[] = {
+   /* format uses_file_history  uses_level restore_prefix_relative needs_namelist */
    { (char *)"dump", true, true, true, true },
    { (char *)"tar", true, false, true, true },
    { (char *)"smtape", false, true, false, true },
@@ -54,6 +57,11 @@ static ndmp_backup_format_option ndmp_backup_format_options[] = {
    { NULL, false, false, false }
 };
 
+/**
+ * find ndmp_backup_format_option for given format
+ * @param [in] backup_format string specifying the backup format
+ * @return returns a pointer to the ndmp_backup_format_option
+ */
 ndmp_backup_format_option *ndmp_lookup_backup_format_options(const char *backup_format)
 {
    int i = 0;
@@ -72,7 +80,7 @@ ndmp_backup_format_option *ndmp_lookup_backup_format_options(const char *backup_
    return (ndmp_backup_format_option *)NULL;
 }
 
-/*
+/**
  * Validation functions.
  */
 bool ndmp_validate_client(JCR *jcr)
@@ -166,7 +174,7 @@ bool ndmp_validate_job(JCR *jcr, struct ndm_job_param *job)
    return true;
 }
 
-/*
+/**
  * Fill a ndmagent structure with the correct info. Instead of calling ndmagent_from_str
  * we fill the structure ourself from info provides in a resource.
  */
@@ -212,6 +220,28 @@ static inline bool fill_ndmp_agent_config(JCR *jcr,
       Jmsg(jcr, M_FATAL, 0, _("Illegal authtype %d for NDMP Job\n"), authtype);
       return false;
    }
+   /*
+    *  sanity checks
+    */
+   if (!address)  {
+      Jmsg(jcr, M_FATAL, 0, _("fill_ndmp_agent_config: address is missing\n"));
+      return false;
+   }
+
+   if (!username) {
+      Jmsg(jcr, M_FATAL, 0, _("fill_ndmp_agent_config: username is missing\n"));
+      return false;
+   }
+
+   if (!password) {
+      Jmsg(jcr, M_FATAL, 0, _("fill_ndmp_agent_config: password is missing\n"));
+      return false;
+   }
+
+   if (!port)     {
+      Jmsg(jcr, M_FATAL, 0, _("fill_ndmp_agent_config: port is missing\n"));
+      return false;
+   }
 
    agent->port = port;
    bstrncpy(agent->host, address, sizeof(agent->host));
@@ -221,7 +251,7 @@ static inline bool fill_ndmp_agent_config(JCR *jcr,
    return true;
 }
 
-/*
+/**
  * Parse a meta-tag and convert it into a ndmp_pval
  */
 void ndmp_parse_meta_tag(struct ndm_env_table *env_tab, char *meta_tag)
@@ -250,7 +280,7 @@ void ndmp_parse_meta_tag(struct ndm_env_table *env_tab, char *meta_tag)
    *p = '=';
 }
 
-/*
+/**
  * Calculate the wanted NDMP loglevel from the current debug level and
  * any configure minimum level.
  */
@@ -356,6 +386,9 @@ bail_out:
    return false;
 }
 
+
+
+
 bool ndmp_build_storage_job(JCR *jcr,
                             STORERES *store,
                             bool init_tape,
@@ -368,29 +401,73 @@ bool ndmp_build_storage_job(JCR *jcr,
    job->operation = operation;
    job->bu_type = jcr->backup_format;
 
-   if (init_tape) {
-      /*
-       * Setup the TAPE agent of the NDMP job.
-       */
-      ASSERT(store->password.encoding == p_encoding_clear);
-      if (!fill_ndmp_agent_config(jcr, &job->tape_agent,
-                                  store->Protocol, store->AuthType,
-                                  store->address, store->SDport,
-                                  store->username, store->password.value)) {
-         goto bail_out;
-      }
+   if (!fill_ndmp_agent_config(jcr, &job->data_agent, store->Protocol,
+                               store->AuthType, store->address,
+                               store->SDport, store->username,
+                               store->password.value)) {
+      goto bail_out;
    }
 
-   if (init_robot) {
-      /*
-       * Setup the ROBOT agent of the NDMP job.
-       */
-      if (!fill_ndmp_agent_config(jcr, &job->robot_agent,
-                                  store->Protocol, store->AuthType,
-                                  store->address, store->SDport,
-                                  store->username, store->password.value)) {
-         goto bail_out;
-      }
+
+  if (init_tape) {
+     /*
+      * Setup the TAPE agent of the NDMP job.
+      */
+     ASSERT(store->password.encoding == p_encoding_clear);
+     if (!fill_ndmp_agent_config(jcr, &job->tape_agent,
+                                 store->Protocol, store->AuthType,
+                                 store->address, store->SDport,
+                                 store->username, store->password.value)) {
+        goto bail_out;
+     }
+  }
+
+  if (init_robot) {
+     /*
+      * Setup the ROBOT agent of the NDMP job.
+      */
+     if (!fill_ndmp_agent_config(jcr, &job->robot_agent,
+                                 store->Protocol, store->AuthType,
+                                 store->address, store->SDport,
+                                 store->username, store->password.value)) {
+        goto bail_out;
+     }
+  }
+
+
+
+   return true;
+
+bail_out:
+   return false;
+}
+
+bool ndmp_build_client_and_storage_job(JCR *jcr,
+                            STORERES *store,
+                            CLIENTRES *client,
+                            bool init_tape,
+                            bool init_robot,
+                            int operation,
+                            struct ndm_job_param *job)
+{
+   /*
+    * setup storage job
+    * i.e. setup tape_agent and robot_agent
+    */
+   if ( !ndmp_build_storage_job(jcr, store, init_tape, init_robot,
+            operation, job) ) {
+      goto bail_out;
+   }
+
+   /*
+    * now configure client job
+    * i.e. setup data_agent
+    */
+   if (!fill_ndmp_agent_config(jcr, &job->data_agent, client->Protocol,
+                               client->AuthType, client->address,
+                               client->FDport, client->username,
+                               client->password.value)) {
+      goto bail_out;
    }
 
    return true;
@@ -399,7 +476,7 @@ bail_out:
    return false;
 }
 
-/*
+/**
  * Interface function which glues the logging infra of the NDMP lib with the daemon.
  */
 extern "C" void ndmp_loghandler(struct ndmlog *log, char *tag, int level, char *msg)
@@ -476,7 +553,7 @@ extern "C" void ndmp_loghandler(struct ndmlog *log, char *tag, int level, char *
    Dmsg3(internal_level, "NDMP: [%s] [%d] %s\n", tag, level, msg);
 }
 
-/*
+/**
  * Interface function which glues the logging infra of the NDMP lib with the user context.
  */
 extern "C" void ndmp_client_status_handler(struct ndmlog *log, char *tag, int lev, char *msg)
@@ -494,7 +571,7 @@ extern "C" void ndmp_client_status_handler(struct ndmlog *log, char *tag, int le
    nis->ua->send_msg("%s\n", msg);
 }
 
-/*
+/**
  * Generic function to query the NDMP server using the NDM_JOB_OP_QUERY_AGENTS
  * operation. Callback is the above ndmp_client_status_handler which prints
  * the data to the user context.
@@ -577,7 +654,7 @@ bail_out:
    ndmp_sess.param = NULL;
 }
 
-/*
+/**
  * Output the status of a NDMP client. Query the DATA agent of a
  * native NDMP server to give some info.
  */
