@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2011-2015 Planets Communications B.V.
-   Copyright (C) 2013-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,10 +20,12 @@
    02110-1301, USA.
 */
 /*
- * Storage specific NDMP Data Management Application (DMA) routines
- *
  * Marco van Wieringen, May 2015
  */
+/**
+ * @file
+ * Storage specific NDMP Data Management Application (DMA) routines
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -37,7 +39,7 @@
 
 /* Forward referenced functions */
 
-/*
+/**
  * Output the status of a storage daemon when its a normal storage
  * daemon accessed via the NDMP protocol or query the TAPE and ROBOT
  * agent of a native NDMP server.
@@ -65,7 +67,7 @@ void do_ndmp_storage_status(UAContext *ua, STORERES *store, char *cmd)
    }
 }
 
-/*
+/**
  * Interface function which glues the logging infra of the NDMP lib for debugging.
  */
 extern "C" void ndmp_robot_status_handler(struct ndmlog *log, char *tag, int lev, char *msg)
@@ -83,7 +85,7 @@ extern "C" void ndmp_robot_status_handler(struct ndmlog *log, char *tag, int lev
    Dmsg1(100, "%s\n", msg);
 }
 
-/*
+/**
  * Generic cleanup function that can be used after a successfull or failed NDMP Job ran.
  */
 static void cleanup_ndmp_session(struct ndm_session *ndmp_sess)
@@ -102,7 +104,7 @@ static void cleanup_ndmp_session(struct ndm_session *ndmp_sess)
    free(ndmp_sess);
 }
 
-/*
+/**
  * Generic function to run a storage Job on a remote NDMP server.
  */
 static bool ndmp_run_storage_job(JCR *jcr, STORERES *store, struct ndm_session *ndmp_sess, struct ndm_job_param *ndmp_job)
@@ -166,7 +168,7 @@ bail_out:
    return false;
 }
 
-/*
+/**
  * Generic function to get the current element status of a NDMP robot.
  */
 static bool get_robot_element_status(JCR *jcr, STORERES *store, struct ndm_session **ndmp_sess)
@@ -176,7 +178,7 @@ static bool get_robot_element_status(JCR *jcr, STORERES *store, struct ndm_sessi
    /*
     * See if this is an autochanger.
     */
-   if (!store->autochanger || !store->changer_device) {
+   if (!store->autochanger || !store->ndmp_changer_device) {
       return false;
    }
 
@@ -195,11 +197,12 @@ static bool get_robot_element_status(JCR *jcr, STORERES *store, struct ndm_sessi
     * device in the form NAME[,[CNUM,]SID[,LUN]
     */
    ndmp_job.robot_target = (struct ndmscsi_target *)actuallymalloc(sizeof(struct ndmscsi_target));
-   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->changer_device) != 0) {
+   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->ndmp_changer_device) != 0) {
       actuallyfree(ndmp_job.robot_target);
       return false;
    }
    ndmp_job.have_robot = 1;
+   ndmp_job.auto_remedy = 1;
 
    /*
     * Initialize a new NDMP session
@@ -215,7 +218,7 @@ static bool get_robot_element_status(JCR *jcr, STORERES *store, struct ndm_sessi
    return true;
 }
 
-/*
+/**
  * Get the volume names from a smc_element_descriptor.
  */
 static void fill_volume_name(vol_list_t *vl, struct smc_element_descriptor *edp)
@@ -223,15 +226,13 @@ static void fill_volume_name(vol_list_t *vl, struct smc_element_descriptor *edp)
    if (edp->PVolTag) {
       vl->VolName = bstrdup((char *)edp->primary_vol_tag->volume_id);
       strip_trailing_junk(vl->VolName);
-   }
-
-   if (edp->AVolTag) {
+   } else if (edp->AVolTag) {
       vl->VolName = bstrdup((char *)edp->alternate_vol_tag->volume_id);
       strip_trailing_junk(vl->VolName);
    }
 }
 
-/*
+/**
  * Fill the mapping table from logical to physical storage addresses.
  *
  * The robot mapping table is used when we need to map from a logical
@@ -242,7 +243,7 @@ static void fill_volume_name(vol_list_t *vl, struct smc_element_descriptor *edp)
  * the fact if things are full or empty as that data is kind of volatile
  * and you should use a vol_list for that.
  */
-static void ndmp_fill_storage_mappings(UAContext *ua, STORERES *store, struct ndm_session *ndmp_sess)
+static void ndmp_fill_storage_mappings(STORERES *store, struct ndm_session *ndmp_sess)
 {
    drive_number_t drive;
    slot_number_t slot,
@@ -296,6 +297,9 @@ static void ndmp_fill_storage_mappings(UAContext *ua, STORERES *store, struct nd
    /*
     * Loop over each mapping entry ordered by the Index field and assign a
     * logical number to it.
+    * For the slots,
+    * - first do the normal slots
+    * - second the I/E slots so that they are always at the end
     */
    foreach_dlist(mapping, store->rss->storage_mappings) {
       switch (mapping->Type) {
@@ -313,9 +317,10 @@ static void ndmp_fill_storage_mappings(UAContext *ua, STORERES *store, struct nd
          break;
       }
    }
+
 }
 
-/*
+/**
  * Get the current content of the autochanger as a generic vol_list dlist.
  */
 dlist *ndmp_get_vol_list(UAContext *ua, STORERES *store, bool listall, bool scan)
@@ -326,6 +331,7 @@ dlist *ndmp_get_vol_list(UAContext *ua, STORERES *store, bool listall, bool scan
    vol_list_t *vl = NULL;
    dlist *vol_list = NULL;
 
+   ua->warning_msg(_("get ndmp_vol_list...\n"));
    if (!get_robot_element_status(ua->jcr, store, &ndmp_sess)) {
       return (dlist *)NULL;
    }
@@ -334,7 +340,7 @@ dlist *ndmp_get_vol_list(UAContext *ua, STORERES *store, bool listall, bool scan
     * If we have no storage mappings create them now from the data we just retrieved.
     */
    if (!store->rss->storage_mappings) {
-      ndmp_fill_storage_mappings(ua, store, ndmp_sess);
+      ndmp_fill_storage_mappings(store, ndmp_sess);
    }
 
    /*
@@ -493,10 +499,29 @@ dlist *ndmp_get_vol_list(UAContext *ua, STORERES *store, bool listall, bool scan
    return vol_list;
 }
 
-/*
+/**
  * Update the mapping table from logical to physical storage addresses.
  */
-static bool ndmp_update_storage_mappings(UAContext *ua, STORERES *store)
+bool ndmp_update_storage_mappings(JCR* jcr, STORERES *store) 
+{
+   struct ndm_session *ndmp_sess;
+
+   if (!get_robot_element_status(jcr, store, &ndmp_sess)) {
+      return false;
+   }
+
+   ndmp_fill_storage_mappings(store, ndmp_sess);
+
+   cleanup_ndmp_session(ndmp_sess);
+
+   return true;
+   
+} 
+
+/**
+ * Update the mapping table from logical to physical storage addresses.
+ */
+bool ndmp_update_storage_mappings(UAContext *ua, STORERES *store)
 {
    struct ndm_session *ndmp_sess;
 
@@ -504,14 +529,14 @@ static bool ndmp_update_storage_mappings(UAContext *ua, STORERES *store)
       return false;
    }
 
-   ndmp_fill_storage_mappings(ua, store, ndmp_sess);
+   ndmp_fill_storage_mappings(store, ndmp_sess);
 
    cleanup_ndmp_session(ndmp_sess);
 
    return true;
 }
 
-/*
+/**
  * Number of slots in a NDMP autochanger.
  */
 slot_number_t ndmp_get_num_slots(UAContext *ua, STORERES *store)
@@ -545,7 +570,7 @@ slot_number_t ndmp_get_num_slots(UAContext *ua, STORERES *store)
    return slots;
 }
 
-/*
+/**
  * Number of drives in a NDMP autochanger.
  */
 drive_number_t ndmp_get_num_drives(UAContext *ua, STORERES *store)
@@ -578,7 +603,7 @@ drive_number_t ndmp_get_num_drives(UAContext *ua, STORERES *store)
    return drives;
 }
 
-/*
+/**
  * Move a volume from one slot to an other in a NDMP autochanger.
  */
 bool ndmp_transfer_volume(UAContext *ua, STORERES *store,
@@ -592,7 +617,7 @@ bool ndmp_transfer_volume(UAContext *ua, STORERES *store,
    /*
     * See if this is an autochanger.
     */
-   if (!store->autochanger || !store->changer_device) {
+   if (!store->autochanger || !store->ndmp_changer_device) {
       return retval;
    }
 
@@ -627,17 +652,20 @@ bool ndmp_transfer_volume(UAContext *ua, STORERES *store,
    ndmp_job.to_addr = slot_mapping;
    ndmp_job.to_addr_given = 1;
 
+   ua->warning_msg(_ ("transferring form slot %hd to slot %hd...\n"), src_slot, dst_slot );
+
    /*
     * Set the remote robotics name to use.
     * We use the ndmscsi_target_from_str() function which parses the NDMJOB format of a
     * device in the form NAME[,[CNUM,]SID[,LUN]
     */
    ndmp_job.robot_target = (struct ndmscsi_target *)actuallymalloc(sizeof(struct ndmscsi_target));
-   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->changer_device) != 0) {
+   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->ndmp_changer_device) != 0) {
       actuallyfree(ndmp_job.robot_target);
       return retval;
    }
    ndmp_job.have_robot = 1;
+   ndmp_job.auto_remedy = 1;
 
    /*
     * Initialize a new NDMP session
@@ -657,17 +685,19 @@ bool ndmp_transfer_volume(UAContext *ua, STORERES *store,
    return retval;
 }
 
-/*
+/**
  * Lookup the name of a drive in a NDMP autochanger.
  */
-static char *lookup_ndmp_drive(STORERES *store, drive_number_t drive)
+char *lookup_ndmp_drive(STORERES *store, drive_number_t drivenumber)
 {
    int cnt = 0;
    char *tapedevice;
+   RES *tapedeviceres;
 
-   if (store->tape_devices) {
-      foreach_alist(tapedevice, store->tape_devices) {
-         if (cnt == drive) {
+   if (store->device) {
+      foreach_alist(tapedeviceres, store->device) {
+         if (cnt == drivenumber) {
+            tapedevice = tapedeviceres->name;
             return tapedevice;
          }
          cnt++;
@@ -677,7 +707,7 @@ static char *lookup_ndmp_drive(STORERES *store, drive_number_t drive)
    return NULL;
 }
 
-/*
+/**
  * Perform an autochanger operation in a NDMP autochanger.
  */
 bool ndmp_autochanger_volume_operation(UAContext *ua, STORERES *store, const char *operation,
@@ -689,13 +719,12 @@ bool ndmp_autochanger_volume_operation(UAContext *ua, STORERES *store, const cha
    struct ndm_job_param ndmp_job;
    struct ndm_session *ndmp_sess;
 
-   Dmsg3(100, "ndmp_autochanger_volume_operation: operation %s, drive %hd, slot %hd\n",
-         operation, drive, slot);
-
+   Dmsg3(100, "ndmp_autochanger_volume_operation: operation %s, drive %hd, slot %hd\n", operation, drive, slot);
+   ua->warning_msg(_("ndmp_autochanger_volume_operation: operation %s, drive %hd, slot %hd\n"), operation, drive, slot);
    /*
     * See if this is an autochanger.
     */
-   if (!store->autochanger || !store->changer_device) {
+   if (!store->autochanger || !store->ndmp_changer_device) {
       return retval;
    }
 
@@ -744,7 +773,7 @@ bool ndmp_autochanger_volume_operation(UAContext *ua, STORERES *store, const cha
    /*
     * Map the logical address to a physical one.
     */
-   drive_mapping = lookup_storage_mapping(store, slot_type_drive, LOGICAL_TO_PHYSICAL, slot);
+   drive_mapping = lookup_storage_mapping(store, slot_type_drive, PHYSICAL_TO_LOGICAL, slot);
    if (drive_mapping == -1) {
       ua->error_msg("No slot mapping for drive %hd\n", drive);
       return retval;
@@ -758,11 +787,12 @@ bool ndmp_autochanger_volume_operation(UAContext *ua, STORERES *store, const cha
     * device in the form NAME[,[CNUM,]SID[,LUN]
     */
    ndmp_job.robot_target = (struct ndmscsi_target *)actuallymalloc(sizeof(struct ndmscsi_target));
-   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->changer_device) != 0) {
+   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->ndmp_changer_device) != 0) {
       actuallyfree(ndmp_job.robot_target);
       return retval;
    }
    ndmp_job.have_robot = 1;
+   ndmp_job.auto_remedy = 1;
 
    /*
     * Initialize a new NDMP session
@@ -782,7 +812,7 @@ bool ndmp_autochanger_volume_operation(UAContext *ua, STORERES *store, const cha
    return retval;
 }
 
-/*
+/**
  * Label a volume in a NDMP autochanger.
  */
 bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
@@ -794,13 +824,13 @@ bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
    struct ndm_session *ndmp_sess;
    struct ndmmedia *media;
 
-   Dmsg4(100,"ndmp_send_label_request: VolumeName = %s MediaType=%s PoolName=%s drive = %hd\n",
-         mr->VolumeName, mr->MediaType, pr->Name, drive);
+   Dmsg4(100,"ndmp_send_label_request: VolumeName=%s MediaType=%s PoolName=%s drive=%hd\n", mr->VolumeName, mr->MediaType, pr->Name, drive);
+   ua->warning_msg(_("ndmp_send_label_request: VolumeName=%s MediaType=%s PoolName=%s drive=%hd\n"), mr->VolumeName, mr->MediaType, pr->Name, drive);
 
    /*
     * See if this is an autochanger.
     */
-   if (!store->autochanger || !store->changer_device) {
+   if (!store->autochanger || !store->ndmp_changer_device) {
       return retval;
    }
 
@@ -819,11 +849,13 @@ bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
     * device in the form NAME[,[CNUM,]SID[,LUN]
     */
    ndmp_job.robot_target = (struct ndmscsi_target *)actuallymalloc(sizeof(struct ndmscsi_target));
-   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->changer_device) != 0) {
+   if (ndmscsi_target_from_str(ndmp_job.robot_target, store->ndmp_changer_device) != 0) {
       actuallyfree(ndmp_job.robot_target);
+      Dmsg0(100,"ndmp_send_label_request: no robot to use\n");
       return retval;
    }
    ndmp_job.have_robot = 1;
+   ndmp_job.auto_remedy = 1;
 
    /*
     * Set the remote tape drive to use.
@@ -831,7 +863,6 @@ bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
    ndmp_job.tape_device = lookup_ndmp_drive(store, drive);
    if (!ndmp_job.tape_device) {
       actuallyfree(ndmp_job.robot_target);
-      return retval;
    }
 
    /*
@@ -843,7 +874,6 @@ bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
       slot_mapping = lookup_storage_mapping(store, slot_type_normal, LOGICAL_TO_PHYSICAL, slot);
       if (slot_mapping == -1) {
          ua->error_msg("No slot mapping for slot %hd\n", slot);
-         actuallyfree(ndmp_job.robot_target);
          return retval;
       }
       media = ndma_store_media(&ndmp_job.media_tab, slot_mapping);
@@ -851,6 +881,8 @@ bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
       media = ndma_store_media(&ndmp_job.media_tab, 0);
    }
    bstrncpy(media->label, mr->VolumeName, NDMMEDIA_LABEL_MAX - 1);
+   media->valid_label =  NDMP9_VALIDITY_VALID;
+
 
    /*
     * Initialize a new NDMP session
@@ -870,7 +902,7 @@ bool ndmp_send_label_request(UAContext *ua, STORERES *store, MEDIA_DBR *mr,
    return retval;
 }
 #else
-/*
+/**
  * Dummy entry points when NDMP not enabled.
  */
 void do_ndmp_storage_status(UAContext *ua, STORERES *store, char *cmd)
