@@ -20,10 +20,12 @@
    02110-1301, USA.
 */
 /*
- * FHDB using LMDB for NDMP Data Management Application (DMA)
- *
  * Marco van Wieringen & Philipp Storz, May 2015
  */
+/*
+ * @file
+ * FHDB using LMDB for NDMP Data Management Application (DMA)
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -131,7 +133,7 @@ retry:
          }
          break;
       default:
-         Dmsg2(dbglvl, _("Unable insert new data at %llu: %s\n"), payload->node,  mdb_strerror(result));
+         Dmsg2(dbglvl, _("Unable to insert new data at %llu: %s\n"), payload->node,  mdb_strerror(result));
          Jmsg2(nis->jcr, M_FATAL, 0, _("Unable insert new data at %llu: %s\n"), payload->node,  mdb_strerror(result));
          goto bail_out;
       }
@@ -365,7 +367,7 @@ bail_out:
    return 1;
 }
 
-/*
+/**
  * This glues the NDMP File Handle DB with internal code.
  */
 void ndmp_fhdb_lmdb_register(struct ndmlog *ixlog)
