@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,11 +21,13 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- next_vol -- handles finding the next volume for append.
+ * Kern Sibbald, March MMI
+ */
+/**
+ * @file
+ * handles finding the next volume for append.
  *
  * Split out of catreq.c August MMIII catalog request from the Storage daemon.
- *
- * Kern Sibbald, March MMI
  */
 
 #include "bareos.h"
@@ -33,7 +35,7 @@
 
 static int const dbglvl = 50;   /* debug level */
 
-/*
+/**
  * Set storage id if possible
  */
 void set_storageid_in_mr(STORERES *store, MEDIA_DBR *mr)
@@ -43,7 +45,7 @@ void set_storageid_in_mr(STORERES *store, MEDIA_DBR *mr)
    }
 }
 
-/*
+/**
  *  Items needed:
  *
  *  mr.PoolId must be set
@@ -84,7 +86,7 @@ int find_next_volume_for_append(JCR *jcr, MEDIA_DBR *mr, int index,
       set_storageid_in_mr(store, mr);
 
       bstrncpy(mr->VolStatus, "Append", sizeof(mr->VolStatus));
-      ok = db_find_next_volume(jcr, jcr->db, index, InChanger, mr, unwanted_volumes);
+      ok = jcr->db->find_next_volume(jcr, index, InChanger, mr, unwanted_volumes);
       if (!ok) {
          /*
           * No volume found, apply algorithm
@@ -155,7 +157,7 @@ int find_next_volume_for_append(JCR *jcr, MEDIA_DBR *mr, int index,
              * Find oldest volume to recycle
              */
             set_storageid_in_mr(store, mr);
-            ok = db_find_next_volume(jcr, jcr->db, -1, InChanger, mr, unwanted_volumes);
+            ok = jcr->db->find_next_volume(jcr, -1, InChanger, mr, unwanted_volumes);
             set_storageid_in_mr(store, mr);
             Dmsg1(dbglvl, "Find oldest=%d Volume\n", ok);
             if (ok && prune) {
@@ -208,7 +210,7 @@ int find_next_volume_for_append(JCR *jcr, MEDIA_DBR *mr, int index,
    return ok;
 }
 
-/*
+/**
  * Check if any time limits or use limits have expired if so,
  * set the VolStatus appropriately.
  */
@@ -275,9 +277,9 @@ bool has_volume_expired(JCR *jcr, MEDIA_DBR *mr)
        */
       Dmsg1(dbglvl, "Vol=%s has expired update media record\n", mr->VolumeName);
       set_storageid_in_mr(NULL, mr);
-      if (!db_update_media_record(jcr, jcr->db, mr)) {
+      if (!jcr->db->update_media_record(jcr, mr)) {
          Jmsg(jcr, M_ERROR, 0, _("Catalog error updating volume \"%s\". ERR=%s"),
-              mr->VolumeName, db_strerror(jcr->db));
+              mr->VolumeName, jcr->db->strerror());
       }
    }
    Dmsg2(dbglvl, "Vol=%s expired=%d\n", mr->VolumeName, expired);
@@ -285,7 +287,7 @@ bool has_volume_expired(JCR *jcr, MEDIA_DBR *mr)
    return expired;
 }
 
-/*
+/**
  * Try hard to recycle the current volume
  *
  * Returns: on failure - reason = NULL
@@ -400,7 +402,7 @@ bool get_scratch_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr, STORERES *store
    /*
     * Get Pool record for Scratch Pool
     * choose between ScratchPoolId and Scratch
-    * db_get_pool_record will first try ScratchPoolId,
+    * get_pool_record will first try ScratchPoolId,
     * and then try the pool named Scratch
     */
    memset(&smr, 0, sizeof(smr));
@@ -408,7 +410,7 @@ bool get_scratch_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr, STORERES *store
 
    bstrncpy(spr.Name, "Scratch", sizeof(spr.Name));
    spr.PoolId = mr->ScratchPoolId;
-   if (db_get_pool_record(jcr, jcr->db, &spr)) {
+   if (jcr->db->get_pool_record(jcr, &spr)) {
       smr.PoolId = spr.PoolId;
       if (InChanger) {
          smr.StorageId = mr->StorageId;  /* want only Scratch Volumes in changer */
@@ -422,7 +424,7 @@ bool get_scratch_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr, STORERES *store
        * then try to take the oldest volume.
        */
       set_storageid_in_mr(store, &smr);  /* put StorageId in new record */
-      if (db_find_next_volume(jcr, jcr->db, 1, InChanger, &smr, NULL)) {
+      if (jcr->db->find_next_volume(jcr, 1, InChanger, &smr, NULL)) {
          found = true;
       } else if (find_recycled_volume(jcr, InChanger, &smr, store, NULL)) {
          found = true;
@@ -439,9 +441,8 @@ bool get_scratch_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr, STORERES *store
          memset(&pr, 0, sizeof(pr));
          bstrncpy(pr.Name, jcr->res.pool->name(), sizeof(pr.Name));
 
-         if (!db_get_pool_record(jcr, jcr->db, &pr)) {
-            Jmsg(jcr, M_WARNING, 0, _("Unable to get Pool record: ERR=%s"),
-                 db_strerror(jcr->db));
+         if (!jcr->db->get_pool_record(jcr, &pr)) {
+            Jmsg(jcr, M_WARNING, 0, _("Unable to get Pool record: ERR=%s"), jcr->db->strerror());
             goto bail_out;
          }
 
@@ -469,8 +470,8 @@ bool get_scratch_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr, STORERES *store
          bstrncpy(mr->VolStatus, smr.VolStatus, sizeof(smr.VolStatus));
          mr->RecyclePoolId = smr.RecyclePoolId;
 
-         if (!db_update_media_record(jcr, jcr->db, mr)) {
-            Jmsg(jcr, M_WARNING, 0, _("Failed to move Scratch Volume. ERR=%s\n"), db_strerror(jcr->db));
+         if (!jcr->db->update_media_record(jcr, mr)) {
+            Jmsg(jcr, M_WARNING, 0, _("Failed to move Scratch Volume. ERR=%s\n"), jcr->db->strerror());
             goto bail_out;
          }
 
