@@ -2,6 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2002-2012 Free Software Foundation Europe e.V.
+   Copyright (C) 2016-2016 Planets Communications B.V.
+   Copyright (C) 2016-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -19,12 +21,13 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- Automatic Recycling of Volumes
- *                    Recycles Volumes that have been purged
- *
  * Kern Sibbald, May MMII
  */
-
+/**
+ * @file
+ * BAREOS Director -- Automatic Recycling of Volumes
+ *                    Recycles Volumes that have been purged
+ */
 #include "bareos.h"
 #include "dird.h"
 
@@ -35,7 +38,7 @@ bool find_recycled_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr,
 {
    bstrncpy(mr->VolStatus, "Recycle", sizeof(mr->VolStatus));
    set_storageid_in_mr(store, mr);
-   if (db_find_next_volume(jcr, jcr->db, 1, InChanger, mr, unwanted_volumes)) {
+   if (jcr->db->find_next_volume(jcr, 1, InChanger, mr, unwanted_volumes)) {
       jcr->MediaId = mr->MediaId;
       Dmsg1(20, "Find_next_vol MediaId=%u\n", jcr->MediaId);
       pm_strcpy(jcr->VolumeName, mr->VolumeName);
@@ -47,7 +50,7 @@ bool find_recycled_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr,
    return false;
 }
 
-/*
+/**
  * Look for oldest Purged volume
  */
 bool recycle_oldest_purged_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr,
@@ -56,7 +59,7 @@ bool recycle_oldest_purged_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr,
    bstrncpy(mr->VolStatus, "Purged", sizeof(mr->VolStatus));
    set_storageid_in_mr(store, mr);
 
-   if (db_find_next_volume(jcr, jcr->db, 1, InChanger, mr, unwanted_volumes)) {
+   if (jcr->db->find_next_volume(jcr, 1, InChanger, mr, unwanted_volumes)) {
       Dmsg1(20, "Find_next_vol MediaId=%u\n", mr->MediaId);
       set_storageid_in_mr(store, mr);
       if (recycle_volume(jcr, mr)) {
@@ -71,7 +74,7 @@ bool recycle_oldest_purged_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr,
    return false;
 }
 
-/*
+/**
  * Recycle the specified volume
  */
 bool recycle_volume(JCR *jcr, MEDIA_DBR *mr)
@@ -84,5 +87,5 @@ bool recycle_volume(JCR *jcr, MEDIA_DBR *mr)
    mr->set_first_written = true;
    set_storageid_in_mr(NULL, mr);
 
-   return db_update_media_record(jcr, jcr->db, mr);
+   return jcr->db->update_media_record(jcr, mr);
 }
