@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,10 +20,12 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 */
-/**
- * BAREOS Director -- restore.c -- responsible for restoring files
- *
+/*
  * Kern Sibbald, November MM
+ */
+/**
+ * @file
+ * responsible for restoring files
  *
  * This routine is run as a separate thread.
  *
@@ -35,6 +37,7 @@
  *    Open connection with File daemon and pass him commands to do the restore.
  */
 
+
 #include "bareos.h"
 #include "dird.h"
 
@@ -397,6 +400,36 @@ bail_out:
    return false;
 }
 
+static inline bool validate_client(JCR *jcr)
+{
+   switch (jcr->res.client->Protocol) {
+   case APT_NATIVE:
+      return true;
+   default:
+      Jmsg(jcr, M_FATAL, 0, _("Client %s has illegal backup protocol %s for Native restore\n"),
+           jcr->res.client->name(), auth_protocol_to_str(jcr->res.client->Protocol));
+      return false;
+   }
+}
+
+static inline bool validate_storage(JCR *jcr)
+{
+   STORERES *store;
+
+   foreach_alist(store, jcr->res.rstorage) {
+      switch (store->Protocol) {
+      case APT_NATIVE:
+         continue;
+      default:
+         Jmsg(jcr, M_FATAL, 0, _("Storage %s has illegal backup protocol %s for Native restore\n"),
+              store->name(), auth_protocol_to_str(store->Protocol));
+         return false;
+      }
+   }
+
+   return true;
+}
+
 /**
  * Do a restore initialization.
  *
@@ -407,6 +440,13 @@ bool do_native_restore_init(JCR *jcr)
 {
    free_wstorage(jcr);                /* we don't write */
 
+   /*
+    * Validate that we have a native client and storage(s).
+    */
+   if (!validate_client(jcr) || !validate_storage(jcr)) {
+      return false;
+   }
+
    return true;
 }
 
@@ -423,8 +463,8 @@ bool do_native_restore(JCR *jcr)
 
    memset(&rjr, 0, sizeof(rjr));
    jcr->jr.JobLevel = L_FULL;         /* Full restore */
-   if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
-      Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
+   if (!jcr->db->update_job_start_record(jcr, &jcr->jr)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", jcr->db->strerror());
       goto bail_out;
    }
    Dmsg0(20, "Updated job start record\n");
@@ -556,7 +596,8 @@ void generate_restore_summary(JCR *jcr, int msg_type, const char *term_msg)
    jobstatus_to_ascii(jcr->SDJobStatus, sd_term_msg, sizeof(sd_term_msg));
 
    switch (jcr->getJobProtocol()) {
-   case PT_NDMP:
+   case PT_NDMP_BAREOS:
+   case PT_NDMP_NATIVE:
       Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
            "  Build OS:               %s %s %s\n"
            "  JobId:                  %d\n"
