@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2016 Planets Communications B.V.
-   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Commands
- *
  * Kern Sibbald, September MM
  */
+/**
+ * @file
+ * User Agent Commands
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -133,7 +135,7 @@ static bool do_truncate(UAContext *ua, MEDIA_DBR &mr);
 
 bool quit_cmd(UAContext *ua, const char *cmd);
 
-/*
+/**
  * Not all in alphabetical order.
  * New commands are added after existing commands with similar letters
  * to prevent breakage of existing user scripts.
@@ -194,7 +196,7 @@ static struct ua_cmdstruct commands[] = {
      NULL, true, false },
    { NT_(".quit"), quit_cmd, _("Close connection"),
      NULL, false, false },
-   { NT_(".sql"), dot_sql_cmd, _("Send an arbitary SQL command"),
+   { NT_(".sql"), dot_sql_cmd, _("Send an arbitrary SQL command"),
      NT_("query=<sqlquery>"), false, true },
    { NT_(".schedule"), dot_schedule_cmd, _("List all schedule resources"),
      NT_("[enabled | disabled]"), false, false },
@@ -218,7 +220,7 @@ static struct ua_cmdstruct commands[] = {
    { NT_(".bvfs_get_jobids"), dot_bvfs_get_jobids_cmd, _("Get jobids required for a restore"),
      NT_("jobid=<jobid> | ujobid=<unique-jobid> [all]"), true, true },
    { NT_(".bvfs_versions"), dot_bvfs_versions_cmd, _("List versions of a file"),
-     NT_("jobid=0 client=<client-name> pathid=<path-id> fnid=<filename-id> [copies] [versions]"), true, true },
+     NT_("jobid=0 client=<client-name> pathid=<path-id> filename=<file-name> [copies] [versions]"), true, true },
    { NT_(".bvfs_restore"), dot_bvfs_restore_cmd, _("Mark BVFS files/directories for restore. Stored in handle."),
      NT_("path=<handle> jobid=<jobid> [fileid=<file-id>] [dirid=<dirid>] [hardlink=<hardlink>]"), true, true },
    { NT_(".bvfs_cleanup"), dot_bvfs_cleanup_cmd, _("Cleanup BVFS cache for a certain handle"),
@@ -270,12 +272,13 @@ static struct ua_cmdstruct commands[] = {
          "filesets |\n"
          "fileset [ jobid=<jobid> ] | fileset [ ujobid=<complete_name> ] |\n"
          "fileset [ filesetid=<filesetid> ] | fileset [ jobid=<jobid> ] |\n"
-         "jobs [job=<job-name>] [client=<client-name>] [jobstatus=<status>] [volume=<volumename>] [days=<number>] [hours=<number>] [last] [count] |\n"
+         "jobs [job=<job-name>] [client=<client-name>] [jobstatus=<status>] [joblevel=<joblevel>] [volume=<volumename>] [days=<number>] [hours=<number>] [last] [count] |\n"
          "job=<job-name> [client=<client-name>] [jobstatus=<status>] [volume=<volumename>] [days=<number>] [hours=<number>] |\n"
          "jobid=<jobid> | ujobid=<complete_name> |\n"
          "joblog jobid=<jobid> | joblog ujobid=<complete_name> |\n"
          "jobmedia jobid=<jobid> | jobmedia ujobid=<complete_name> |\n"
          "jobtotals |\n"
+         "jobstatistics jobid=<jobid> |\n"
          "log [ limit=<number> [ offset=<number> ] ] [reverse]|\n"
          "media [ jobid=<jobid> | ujobid=<complete_name> | pool=<pool-name> | all ] |\n"
          "media=<media-name> |\n"
@@ -283,7 +286,7 @@ static struct ua_cmdstruct commands[] = {
          "pools |\n"
          "pool=<pool-name> |\n"
          "storages |\n"
-         "volumes [ jobid=<jobid> | ujobid=<complete_name> | pool=<pool-name> | all ] |\n"
+         "volumes [ jobid=<jobid> | ujobid=<complete_name> | pool=<pool-name> | all ] [count] |\n"
          "volume=<volume-name> |\n"
          "[current] | [enabled | disabled] |\n"
          "[limit=<number> [offset=<number>]]"), true, true },
@@ -296,9 +299,9 @@ static struct ua_cmdstruct commands[] = {
          "fileset jobid=<jobid> | fileset ujobid=<complete_name> |\n"
          "fileset [ filesetid=<filesetid> ] | fileset [ jobid=<jobid> ] |\n"
          "jobs [job=<job-name>] [client=<client-name>] [jobstatus=<status>] [volume=<volumename>] [days=<number>] [hours=<number>] [last] [count] |\n"
-         "job=<job-name> [client=<client-name>] [jobstatus=<status>] [volume=<volumename>] [days=<number>] [hours=<number>] |\n"
+         "job=<job-name> [client=<client-name>] [jobstatus=<status>] [joblevel=<joblevel>] [volume=<volumename>] [days=<number>] [hours=<number>] |\n"
          "jobid=<jobid> | ujobid=<complete_name> |\n"
-         "joblog jobid=<jobid> | joblog ujobid=<complete_name> |\n"
+         "joblog jobid=<jobid> [count] | joblog ujobid=<complete_name> [count] |\n"
          "jobmedia jobid=<jobid> | jobmedia ujobid=<complete_name> |\n"
          "jobtotals |\n"
          "media [ jobid=<jobid> | ujobid=<complete_name> | pool=<pool-name> | all ] |\n"
@@ -374,10 +377,18 @@ static struct ua_cmdstruct commands[] = {
    { NT_("setip"), setip_cmd, _("Sets new client address -- if authorized"),
      NT_(""), false, true },
    { NT_("show"), show_cmd, _("Show resource records"),
-     NT_("jobdefs=<job-defaults> | job=<job-name> | pool=<pool-name> | fileset=<fileset-name> |\n"
-         "schedule=<schedule-name> | client=<client-name> | message=<message-resource-name> |\n"
-         "profile=<profile-name> | jobdefs | jobs | pools | filesets | schedules | clients |\n"
-         "messages | profiles | consoles | disabled [ clients | jobs | schedules ] | all [verbose]"), true, true },
+     NT_("catalog=<catalog-name> | client=<client-name> | console=<console-name> | "
+         "director=<director-name> | fileset=<fileset-name> | "
+         "jobdefs=<job-defaults> | job=<job-name> | message=<message-resource-name> | "
+         "pool=<pool-name> | profile=<profile-name> | "
+         "schedule=<schedule-name> | storage=<storage-name> "
+         "|\n"
+         "catalog | clients | consoles | directors | filesets | jobdefs | jobs | "
+         "messages | pools | profiles | schedules | storages "
+         "|\n"
+         "disabled [ clients | jobs | schedules ] "
+         "|\n"
+         "all [verbose]"), true, true },
    { NT_("sqlquery"), sqlquery_cmd, _("Use SQL to query catalog"),
      NT_(""), false, true },
    { NT_("time"), time_cmd, _("Print current time"),
@@ -392,17 +403,21 @@ static struct ua_cmdstruct commands[] = {
    { NT_("umount"), unmount_cmd, _("Umount - for old-time Unix guys, see unmount"),
      NT_("storage=<storage-name> [ drive=<drivenum> ]\n"
          "\tjobid=<jobid> | job=<job-name> | ujobid=<complete_name>"), false, true },
-   { NT_("update"), update_cmd, _("Update volume, pool, job or stats"),
-     NT_("stats\n"
-         "\tpool=<pool-name>\n"
-         "\tslots storage=<storage-name> scan\n"
-         "\tvolume=<volume-name> volstatus=<status> volretention=<time-def>\n"
-         "\tpool=<pool-name> recycle=<yes/no> slot=<number>\n"
-         "\tinchanger=<yes/no>\n"
-         "\tmaxvolbytes=<size> maxvolfiles=<nb> maxvoljobs=<nb>\n"
-         "\tenabled=<yes/no> recyclepool=<pool-name> actiononpurge=<action>\n"
-         "\tjobid=<jobid> jobname=<name> starttime=<time-def> client=<client-name>\n"
-         "\tfilesetid=<fileset-id> jobtype=<job-type>"), true, true },
+   { NT_("update"), update_cmd, _("Update volume, pool, slots, job or statistics"),
+     NT_("[volume=<volume-name> "
+         "[volstatus=<status>] [volretention=<time-def>] actiononpurge=<action>] "
+         "[pool=<pool-name>] [recycle=<yes/no>] [slot=<number>] [inchanger=<yes/no>]] "
+         "|\n"
+         "[pool=<pool-name> "
+         "[maxvolbytes=<size>] [maxvolfiles=<nb>] [maxvoljobs=<nb>]"
+         "[enabled=<yes/no>] [recyclepool=<pool-name>] [actiononpurge=<action>] |\n"
+         "slots [storage=<storage-name>] [scan]] "
+         "|\n"
+         "[jobid=<jobid> [jobname=<name>] [starttime=<time-def>] [client=<client-name>]\n"
+         "[filesetid=<fileset-id>] [jobtype=<job-type>]] "
+         "|\n"
+         "[stats "
+         "[days=<number>]]"), true, true },
    { NT_("use"), use_cmd, _("Use specific catalog"),
      NT_("catalog=<catalog>"), false, true },
    { NT_("var"), var_cmd, _("Does variable expansion"),
@@ -421,7 +436,7 @@ bool UAContext::execute(ua_cmdstruct *cmd)
    return (cmd->func)(this, this->cmd);
 }
 
-/*
+/**
  * Execute a command from the UA
  */
 bool do_a_command(UAContext *ua)
@@ -507,7 +522,7 @@ static bool is_dot_command(const char *cmd)
    return false;
 }
 
-/*
+/**
  * Add Volumes to an existing Pool
  */
 static bool add_cmd(UAContext *ua, const char *cmd)
@@ -642,15 +657,15 @@ static bool add_cmd(UAContext *ua, const char *cmd)
       mr.Enabled = VOL_ENABLED;
       set_storageid_in_mr(store, &mr);
       Dmsg1(200, "Create Volume %s\n", mr.VolumeName);
-      if (!db_create_media_record(ua->jcr, ua->db, &mr)) {
-         ua->error_msg("%s", db_strerror(ua->db));
+      if (!ua->db->create_media_record(ua->jcr, &mr)) {
+         ua->error_msg("%s", ua->db->strerror());
          return true;
       }
    }
    pr.NumVols += num;
    Dmsg0(200, "Update pool record.\n");
-   if (db_update_pool_record(ua->jcr, ua->db, &pr) != 1) {
-      ua->warning_msg("%s", db_strerror(ua->db));
+   if (ua->db->update_pool_record(ua->jcr, &pr) != 1) {
+      ua->warning_msg("%s", ua->db->strerror());
       return true;
    }
    ua->send_msg(_("%d Volumes created in pool %s\n"), num, pr.Name);
@@ -658,7 +673,7 @@ static bool add_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Turn auto mount on/off
  *
  * automount on
@@ -734,7 +749,7 @@ static inline bool cancel_jobs(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Cancel a job
  */
 static bool cancel_cmd(UAContext *ua, const char *cmd)
@@ -752,7 +767,7 @@ static bool cancel_cmd(UAContext *ua, const char *cmd)
    }
 }
 
-/*
+/**
  * Create a Pool Record in the database.
  * It is always created from the Resource record.
  */
@@ -776,7 +791,7 @@ static bool create_cmd(UAContext *ua, const char *cmd)
       break;
 
    case -1:
-      ua->error_msg("%s", db_strerror(ua->db));
+      ua->error_msg("%s", ua->db->strerror());
       break;
 
    default:
@@ -951,7 +966,7 @@ static bool setbwlimit_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Set a new address in a Client resource. We do this only
  * if the Console name is the same as the Client name
  * and the Console can access the client.
@@ -1108,7 +1123,7 @@ static void do_storage_setdebug(UAContext *ua, STORERES *store, int level,
    return;
 }
 
-/*
+/**
  * For the client, we have the following values that can be set :
  *
  * level = debug level
@@ -1274,7 +1289,7 @@ static void do_all_setdebug(UAContext *ua, int level, int trace_flag,
    free(unique_client);
 }
 
-/*
+/**
  * setdebug level=nn all trace=1/0 timestamp=1/0
  */
 static bool setdebug_cmd(UAContext *ua, const char *cmd)
@@ -1443,7 +1458,7 @@ static bool setdebug_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Resolve a hostname.
  */
 static bool resolve_cmd(UAContext *ua, const char *cmd)
@@ -1515,7 +1530,7 @@ static bool resolve_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Turn debug tracing to file on/off
  */
 static bool trace_cmd(UAContext *ua, const char *cmd)
@@ -1781,7 +1796,7 @@ bail_out:
    return true;
 }
 
-/*
+/**
  * Print time
  */
 static bool time_cmd(UAContext *ua, const char *cmd)
@@ -1813,8 +1828,7 @@ static bool time_cmd(UAContext *ua, const char *cmd)
 }
 
 
-
-/*
+/**
  * truncate command. Truncates volumes (volume files) on the storage daemon.
  *
  * usage:
@@ -1914,7 +1928,7 @@ static bool truncate_cmd(UAContext *ua, const char *cmd)
    i = find_arg_with_value(ua, "volume");
    if (i >= 0) {
       if (is_name_valid(ua->argv[i])) {
-         db_escape_string(ua->jcr, ua->db, esc, ua->argv[i], strlen(ua->argv[i]));
+         ua->db->escape_string(ua->jcr, esc, ua->argv[i], strlen(ua->argv[i]));
          if (!*volumes.c_str()) {
             Mmsg(tmp, "'%s'", esc);
          } else {
@@ -1936,13 +1950,13 @@ static bool truncate_cmd(UAContext *ua, const char *cmd)
    }
 
    /* create sql query string (in ua->db->cmd) */
-   if (!prepare_media_sql_query(ua->jcr, ua->db, &mr, volumes)) {
+   if (!ua->db->prepare_media_sql_query(ua->jcr, &mr, &tmp , volumes)) {
       ua->error_msg(_("Invalid parameter (failed to create sql query).\n"));
       goto bail_out;
    }
 
    /* execute query and display result */
-   db_list_sql_query(ua->jcr, ua->db, ua->db->cmd,
+   ua->db->list_sql_query(ua->jcr, tmp.c_str(),
                      ua->send, HORZ_LIST, "volumes", true);
 
    /*
@@ -1950,8 +1964,8 @@ static bool truncate_cmd(UAContext *ua, const char *cmd)
     * Second execution is only required,
     * because function is also used in other contextes.
     */
-   tmp.strcpy(ua->db->cmd);
-   if (!db_get_query_dbids(ua->jcr, ua->db, tmp, mediaIds)) {
+   //tmp.strcpy(ua->db->cmd);
+   if (!ua->db->get_query_dbids(ua->jcr, tmp, mediaIds)) {
       Dmsg0(100, "No results from db_get_query_dbids\n");
       goto bail_out;
    };
@@ -1961,14 +1975,14 @@ static bool truncate_cmd(UAContext *ua, const char *cmd)
       goto bail_out;
    }
 
-   if (!verify_media_ids_from_single_storage(ua->jcr, ua->db, mediaIds)) {
+   if (!ua->db->verify_media_ids_from_single_storage(ua->jcr, mediaIds)) {
       ua->error_msg("Selected volumes are from different storages. "
                     "This is not supported. Please choose only volumes from a single storage.\n");
       goto bail_out;
    }
 
    mr.MediaId = mediaIds.get(0);
-   if (!db_get_media_record(ua->jcr, ua->db, &mr)) {
+   if (!ua->db->get_media_record(ua->jcr, &mr)) {
       goto bail_out;
    }
 
@@ -1987,7 +2001,7 @@ static bool truncate_cmd(UAContext *ua, const char *cmd)
    for (int i = 0; i < mediaIds.size(); i++) {
       memset(&mr, 0, sizeof(mr));
       mr.MediaId = mediaIds.get(i);
-      if (!db_get_media_record(ua->jcr, ua->db, &mr)) {
+      if (!ua->db->get_media_record(ua->jcr, &mr)) {
          Dmsg1(0, "Can't find MediaId=%lld\n", (uint64_t) mr.MediaId);
       } else {
          do_truncate(ua, mr);
@@ -2002,7 +2016,6 @@ bail_out:
       delete ua->jcr->store_bsock;
       ua->jcr->store_bsock = NULL;
    }
-
    return result;
 }
 
@@ -2016,13 +2029,13 @@ static bool do_truncate(UAContext *ua, MEDIA_DBR &mr)
    memset(&pool_dbr, 0, sizeof(pool_dbr));
 
    storage_dbr.StorageId = mr.StorageId;
-   if (!db_get_storage_record(ua->jcr, ua->db, &storage_dbr)) {
+   if (!ua->db->get_storage_record(ua->jcr, &storage_dbr)) {
       ua->error_msg("failed to determine storage for id %lld\n", mr.StorageId);
       goto bail_out;
    }
 
    pool_dbr.PoolId = mr.PoolId;
-   if (!db_get_pool_record(ua->jcr, ua->db, &pool_dbr)) {
+   if (!ua->db->get_pool_record(ua->jcr, &pool_dbr)) {
       ua->error_msg("failed to determine pool for id %lld\n", mr.PoolId);
       goto bail_out;
    }
@@ -2056,7 +2069,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Reload the conf file
  */
 static bool reload_cmd(UAContext *ua, const char *cmd)
@@ -2076,7 +2089,7 @@ static bool reload_cmd(UAContext *ua, const char *cmd)
    return result;
 }
 
-/*
+/**
  * Delete Pool records (should purge Media with it).
  *
  * delete pool=<pool-name>
@@ -2134,7 +2147,7 @@ static bool delete_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * delete_job has been modified to parse JobID lists like the following:
  * delete JobID=3,4,6,7-11,14
  *
@@ -2203,7 +2216,7 @@ static void delete_job(UAContext *ua)
    }
 }
 
-/*
+/**
  * We call delete_job_id_range to parse range tokens and iterate over ranges
  */
 static bool delete_job_id_range(UAContext *ua, char *tok)
@@ -2249,7 +2262,7 @@ static bool delete_job_id_range(UAContext *ua, char *tok)
    return true;
 }
 
-/*
+/**
  * do_job_delete now performs the actual delete operation atomically
  */
 static void do_job_delete(UAContext *ua, JobId_t JobId)
@@ -2261,7 +2274,7 @@ static void do_job_delete(UAContext *ua, JobId_t JobId)
    ua->send_msg(_("Jobid %s and associated records deleted from the catalog.\n"), ed1);
 }
 
-/*
+/**
  * Delete media records from database -- dangerous
  */
 static bool delete_volume(UAContext *ua)
@@ -2294,7 +2307,7 @@ static bool delete_volume(UAContext *ua)
     * If not purged, do it
     */
    if (!bstrcmp(mr.VolStatus, "Purged")) {
-      if (!db_get_volume_jobids(ua->jcr, ua->db, &mr, &lst)) {
+      if (!ua->db->get_volume_jobids(ua->jcr, &mr, &lst)) {
          ua->error_msg(_("Can't list jobs on this volume\n"));
          return true;
       }
@@ -2303,11 +2316,11 @@ static bool delete_volume(UAContext *ua)
       }
    }
 
-   db_delete_media_record(ua->jcr, ua->db, &mr);
+   ua->db->delete_media_record(ua->jcr, &mr);
    return true;
 }
 
-/*
+/**
  * Delete a pool record from the database -- dangerous
  */
 static bool delete_pool(UAContext *ua)
@@ -2326,7 +2339,7 @@ static bool delete_pool(UAContext *ua)
       return true;
    }
    if (ua->pint32_val) {
-      db_delete_pool_record(ua->jcr, ua->db, &pr);
+      ua->db->delete_pool_record(ua->jcr, &pr);
    }
    return true;
 }
@@ -2389,7 +2402,7 @@ static void do_mount_cmd(UAContext *ua, const char *cmd)
    invalidate_vol_list(store.store);
 }
 
-/*
+/**
  * mount [storage=<name>] [drive=nn] [slot=mm]
  */
 static bool mount_cmd(UAContext *ua, const char *cmd)
@@ -2398,7 +2411,7 @@ static bool mount_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * unmount [storage=<name>] [drive=nn]
  */
 static bool unmount_cmd(UAContext *ua, const char *cmd)
@@ -2407,7 +2420,7 @@ static bool unmount_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Perform a NO-OP.
  */
 static bool noop_cmd(UAContext *ua, const char *cmd)
@@ -2423,7 +2436,7 @@ static bool noop_cmd(UAContext *ua, const char *cmd)
    return true;                    /* no op */
 }
 
-/*
+/**
  * release [storage=<name>] [drive=nn]
  */
 static bool release_cmd(UAContext *ua, const char *cmd)
@@ -2432,7 +2445,7 @@ static bool release_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Switch databases
  * use catalog=<name>
  */
@@ -2462,7 +2475,7 @@ bool quit_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Handler to get job status
  */
 static int status_handler(void *ctx, int num_fields, char **row)
@@ -2478,7 +2491,7 @@ static int status_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-/*
+/**
  * Wait until no job is running
  */
 static bool wait_cmd(UAContext *ua, const char *cmd)
@@ -2609,7 +2622,7 @@ static bool wait_cmd(UAContext *ua, const char *cmd)
     * We have to get JobStatus
     */
    Mmsg(temp, "SELECT JobStatus FROM Job WHERE JobId='%s'", edit_int64(JobId, ed1));
-   db_sql_query(ua->db, temp.c_str(), status_handler, (void *)&jobstatus);
+   ua->db->sql_query(temp.c_str(), status_handler, (void *)&jobstatus);
 
    switch (jobstatus) {
    case JS_Error:
@@ -2758,7 +2771,7 @@ static bool version_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 #else
-/*
+/**
  *  Test code -- turned on only for debug testing
  */
 static bool version_cmd(UAContext *ua, const char *cmd)
@@ -2767,7 +2780,7 @@ static bool version_cmd(UAContext *ua, const char *cmd)
    POOL_MEM query(PM_MESSAGE);
    open_db(ua);
    Mmsg(query, "select MediaId from Media,Pool where Pool.PoolId=Media.PoolId and Pool.Name='Full'");
-   db_get_query_dbids(ua->jcr, ua->db, query, ids);
+   get_query_dbids(ua->jcr, ua->db, query, ids);
    ua->send_msg("num_ids=%d max_ids=%d tot_ids=%d\n", ids.num_ids, ids.max_ids, ids.tot_ids);
    for (int i=0; i < ids.num_ids; i++) {
       ua->send_msg("id=%d\n", ids.DBId[i]);
