@@ -3,7 +3,7 @@
 
    Copyright (C) 2002-2011 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2016 Planets Communications B.V.
-   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,14 +21,16 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Commands
+ * Kern Sibbald, April MMII
+ */
+/**
+ * @file
+ * User Agent Commands
  *
  * These are "dot" commands, i.e. commands preceded
  * by a period. These commands are meant to be used
  * by a program, so there is no prompting, and the
  * returned results are (supposed to be) predictable.
- *
- * Kern Sibbald, April MMII
  */
 
 #include "bareos.h"
@@ -105,12 +107,12 @@ bool dot_bvfs_update_cmd(UAContext *ua, const char *cmd)
    }
    pos = find_arg_with_value(ua, "jobid");
    if (pos != -1 && is_a_number_list(ua->argv[pos])) {
-      if (!bvfs_update_path_hierarchy_cache(ua->jcr, ua->db, ua->argv[pos])) {
+      if (!ua->db->bvfs_update_path_hierarchy_cache(ua->jcr, ua->argv[pos])) {
          ua->error_msg("ERROR: BVFS reported a problem for %s\n", ua->argv[pos]);
       }
    } else {
       /* update cache for all jobids */
-      bvfs_update_cache(ua->jcr, ua->db);
+      ua->db->bvfs_update_cache(ua->jcr);
    }
 
    return true;
@@ -186,7 +188,6 @@ static int bvfs_result_handler(void *ctx, int fields, char **row)
       ua->send->object_start();
       ua->send->object_key_value("Type", row[BVFS_Type]);
       ua->send->object_key_value("PathId", str_to_uint64(row[BVFS_PathId]), "%lld\t");
-      ua->send->object_key_value("FilenameId", (uint64_t)0, "%lld\t");
       ua->send->object_key_value("FileId", str_to_uint64(fileid), "%lld\t");
       ua->send->object_key_value("JobId", str_to_uint64(jobid), "%lld\t");
       ua->send->object_key_value("lstat", lstat, "%s\t");
@@ -199,19 +200,18 @@ static int bvfs_result_handler(void *ctx, int fields, char **row)
       ua->send->object_start();
       ua->send->object_key_value("Type", row[BVFS_Type]);
       ua->send->object_key_value("PathId", str_to_uint64(row[BVFS_PathId]), "%lld\t");
-      ua->send->object_key_value("FilenameId", str_to_uint64(row[BVFS_FilenameId]), "%lld\t");
       ua->send->object_key_value("FileId", str_to_uint64(fileid), "%lld\t");
       ua->send->object_key_value("JobId", str_to_uint64(jobid), "%lld\t");
       ua->send->object_key_value("lstat", lstat, "%s\t");
       ua->send->object_key_value("MD5", row[BVFS_Md5], "%s\t");
       ua->send->object_key_value("VolumeName", row[BVFS_VolName], "%s\t");
       ua->send->object_key_value("VolumeInChanger", str_to_uint64(row[BVFS_VolInchanger]), "%lld\n");
+      bvfs_stat(ua, lstat, &LinkFI);
       ua->send->object_end();
    } else if (bvfs_is_file(row)) {
       ua->send->object_start();
       ua->send->object_key_value("Type", row[BVFS_Type]);
       ua->send->object_key_value("PathId", str_to_uint64(row[BVFS_PathId]), "%lld\t");
-      ua->send->object_key_value("FilenameId", str_to_uint64(row[BVFS_FilenameId]), "%lld\t");
       ua->send->object_key_value("FileId", str_to_uint64(fileid), "%lld\t");
       ua->send->object_key_value("JobId", str_to_uint64(jobid), "%lld\t");
       ua->send->object_key_value("lstat", lstat, "%s\t");
@@ -224,19 +224,18 @@ static int bvfs_result_handler(void *ctx, int fields, char **row)
    return 0;
 }
 
-static inline bool bvfs_parse_arg_version(UAContext *ua, char **client, DBId_t *fnid, bool *versions, bool *copies)
+static inline bool bvfs_parse_arg_version(UAContext *ua, char **client, char **fname, bool *versions, bool *copies)
 {
-   *fnid = 0;
+   *fname = NULL;
    *client = NULL;
    *versions = false;
    *copies = false;
 
    for (int i = 1; i < ua->argc; i++) {
-      if (bstrcasecmp(ua->argk[i], NT_("fnid")) ||
-          bstrcasecmp(ua->argk[i], NT_("filenameid"))) {
-         if (ua->argv[i] && is_a_number(ua->argv[i])) {
-            *fnid = str_to_int64(ua->argv[i]);
-         }
+      if (bstrcasecmp(ua->argk[i], NT_("name")) ||
+          bstrcasecmp(ua->argk[i], NT_("fname")) ||
+          bstrcasecmp(ua->argk[i], NT_("filename"))) {
+         *fname = ua->argv[i];
       }
 
       if (bstrcasecmp(ua->argk[i], NT_("client"))) {
@@ -252,7 +251,7 @@ static inline bool bvfs_parse_arg_version(UAContext *ua, char **client, DBId_t *
       }
    }
 
-   return (*client && *fnid > 0);
+   return (*client && *fname);
 }
 
 static bool bvfs_parse_arg(UAContext *ua, DBId_t *pathid, char **path, char **jobid, int *limit, int *offset)
@@ -304,7 +303,7 @@ static bool bvfs_parse_arg(UAContext *ua, DBId_t *pathid, char **path, char **jo
    return true;
 }
 
-/*
+/**
  * This checks to see if the JobId given is allowed under the current
  * ACLs e.g. comparing the JobName against the Job_ACL and the client
  * against the Client_ACL.
@@ -318,14 +317,14 @@ static inline bool bvfs_validate_jobid(UAContext *ua, const char *jobid, bool au
    memset(&jr, 0, sizeof(jr));
    jr.JobId = str_to_int64(jobid);
 
-   if (db_get_job_record(ua->jcr, ua->db, &jr)) {
+   if (ua->db->get_job_record(ua->jcr, &jr)) {
       if (!ua->acl_access_ok(Job_ACL, jr.Name, audit_event)) {
          goto bail_out;
       }
 
       if (jr.ClientId) {
          cr.ClientId = jr.ClientId;
-         if (db_get_client_record(ua->jcr, ua->db, &cr)) {
+         if (ua->db->get_client_record(ua->jcr, &cr)) {
             if (!ua->acl_access_ok(Client_ACL, cr.Name, audit_event)) {
                goto bail_out;
             }
@@ -339,7 +338,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * This returns in filtered_jobids the list of allowed jobids in the
  * jobids variable under the current ACLs e.g. using bvfs_validate_jobid().
  */
@@ -380,32 +379,37 @@ static bool bvfs_validate_jobids(UAContext *ua, const char *jobids, POOL_MEM &fi
    return (cnt > 0) ? true : false;
 }
 
-/*
+/**
  * .bvfs_cleanup path=b2XXXXX
  */
 bool dot_bvfs_cleanup_cmd(UAContext *ua, const char *cmd)
 {
    int i;
 
-   if ((i = find_arg_with_value(ua, "path")) >= 0) {
-      if (!open_client_db(ua, true)) {
-         return false;
-      }
-      Bvfs fs(ua->jcr, ua->db);
-      fs.drop_restore_list(ua->argv[i]);
+   if ((i = find_arg_with_value(ua, "path")) < 0) {
+      ua->error_msg("Can't find path argument\n");
+      return false;             /* not enough param */
    }
 
+   if (!open_client_db(ua, true)) {
+      return false;
+   }
+
+   Bvfs fs(ua->jcr, ua->db);
+   fs.drop_restore_list(ua->argv[i]);
+
    return true;
 }
 
-/*
+/**
  * .bvfs_restore path=b2XXXXX jobid=1,2 fileid=1,2 dirid=1,2 hardlink=1,2,3,4
  */
 bool dot_bvfs_restore_cmd(UAContext *ua, const char *cmd)
 {
    DBId_t pathid = 0;
    char *empty = (char *)"";
-   int limit = 2000, offset = 0, i;
+   int limit = 2000, offset = 0;
+   int i = 0;
    char *path = NULL, *jobid = NULL;
    char *fileid, *dirid, *hardlink;
    POOL_MEM filtered_jobids(PM_FNAME);
@@ -443,9 +447,9 @@ bool dot_bvfs_restore_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
- * .bvfs_lsfiles jobid=1,2,3,4 pathid=10
+/**
  * .bvfs_lsfiles jobid=1,2,3,4 path=/
+ * .bvfs_lsfiles jobid=1,2,3,4 pathid=10
  */
 bool dot_bvfs_lsfiles_cmd(UAContext *ua, const char *cmd)
 {
@@ -496,10 +500,10 @@ bool dot_bvfs_lsfiles_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
- * .bvfs_lsdirs jobid=1,2,3,4 pathid=10
- * .bvfs_lsdirs jobid=1,2,3,4 path=/
+/**
  * .bvfs_lsdirs jobid=1,2,3,4 path=
+ * .bvfs_lsdirs jobid=1,2,3,4 path=/
+ * .bvfs_lsdirs jobid=1,2,3,4 pathid=10
  */
 bool dot_bvfs_lsdirs_cmd(UAContext *ua, const char *cmd)
 {
@@ -524,33 +528,38 @@ bool dot_bvfs_lsdirs_cmd(UAContext *ua, const char *cmd)
 
    Bvfs fs(ua->jcr, ua->db);
    fs.set_jobids(filtered_jobids.c_str());
-   fs.set_limit(limit);
-   fs.set_handler(bvfs_result_handler, ua);
 
    if (pathid) {
       fs.ch_dir(pathid);
    } else {
-      fs.ch_dir(path);
+      if (!fs.ch_dir(path)) {
+         /* path could not be found. Giving up. */
+         return false;
+      }
    }
 
+   fs.set_handler(bvfs_result_handler, ua);
    fs.set_offset(offset);
+   fs.set_limit(limit);
 
    ua->send->array_start("directories");
-   fs.ls_special_dirs();
    fs.ls_dirs();
    ua->send->array_end("directories");
 
    return true;
 }
 
-/*
- * .bvfs_versions jobid=0 client=<client-name> fnid=10 pathid=10 copies versions (jobid isn't used)
+/**
+ * .bvfs_versions jobid=0 client=<client-name> filename=<file-name> pathid=<number> [copies] [versions]
+ *
+ * jobid isn't used.
+ * versions is set, but not used.
  */
 bool dot_bvfs_versions_cmd(UAContext *ua, const char *cmd)
 {
-   DBId_t pathid = 0, fnid = 0;
+   DBId_t pathid = 0;
    int limit = 2000, offset = 0;
-   char *path = NULL, *jobid = NULL, *client = NULL;
+   char *path = NULL, *jobid = NULL, *client = NULL, *fname = NULL;
    bool copies = false, versions = false;
 
    if (!bvfs_parse_arg(ua, &pathid, &path, &jobid, &limit, &offset)) {
@@ -558,8 +567,8 @@ bool dot_bvfs_versions_cmd(UAContext *ua, const char *cmd)
       return false;             /* not enough param */
    }
 
-   if (!bvfs_parse_arg_version(ua, &client, &fnid, &versions, &copies)) {
-      ua->error_msg("Can't find client or fnid argument\n");
+   if (!bvfs_parse_arg_version(ua, &client, &fname, &versions, &copies)) {
+      ua->error_msg("Can't find client or fname argument\n");
       return false;              /* not enough param */
    }
 
@@ -573,19 +582,23 @@ bool dot_bvfs_versions_cmd(UAContext *ua, const char *cmd)
    }
 
    Bvfs fs(ua->jcr, ua->db);
-   fs.set_limit(limit);
    fs.set_see_all_versions(versions);
    fs.set_see_copies(copies);
    fs.set_handler(bvfs_result_handler, ua);
+   fs.set_limit(limit);
    fs.set_offset(offset);
    ua->send->array_start("versions");
-   fs.get_all_file_versions(pathid, fnid, client);
+   if (pathid) {
+      fs.get_all_file_versions(pathid, fname, client);
+   } else {
+      fs.get_all_file_versions(path, fname, client);
+   }
    ua->send->array_end("versions");
 
    return true;
 }
 
-/*
+/**
  * .bvfs_get_jobids jobid=1
  *  -> returns needed jobids to restore
  * .bvfs_get_jobids jobid=1 all
@@ -618,9 +631,9 @@ bool dot_bvfs_get_jobids_cmd(UAContext *ua, const char *cmd)
       return false;
    }
 
-   if (!db_get_job_record(ua->jcr, ua->db, &jr)) {
+   if (!ua->db->get_job_record(ua->jcr, &jr)) {
       ua->error_msg(_("Unable to get Job record for JobId=%s: ERR=%s\n"),
-                    ua->argv[pos], db_strerror(ua->db));
+                    ua->argv[pos], ua->db->strerror());
       return false;
    }
 
@@ -636,9 +649,8 @@ bool dot_bvfs_get_jobids_cmd(UAContext *ua, const char *cmd)
        * If we have the "all" option, we do a search on all defined fileset for this client
        */
       if (find_arg(ua, "all") > 0) {
-         edit_int64(jr.ClientId, ed1);
-         Mmsg(query, uar_sel_filesetid, ed1);
-         db_get_query_dbids(ua->jcr, ua->db, query, ids);
+         ua->db->fill_query(query, B_DB::SQL_QUERY_uar_sel_filesetid, edit_int64(jr.ClientId, ed1));
+         ua->db->get_query_dbids(ua->jcr, query, ids);
       } else {
          ids.num_ids = 1;
          ids.DBId[0] = jr.FileSetId;
@@ -656,7 +668,7 @@ bool dot_bvfs_get_jobids_cmd(UAContext *ua, const char *cmd)
           * Lookup the FileSet.
           */
          fs.FileSetId = ids.DBId[i];
-         if (!db_get_fileset_record(ua->jcr, ua->db, &fs)) {
+         if (!ua->db->get_fileset_record(ua->jcr, &fs)) {
             continue;
          }
 
@@ -671,7 +683,7 @@ bool dot_bvfs_get_jobids_cmd(UAContext *ua, const char *cmd)
           * Lookup the actual JobIds for the given fileset.
           */
          jr.FileSetId = fs.FileSetId;
-         if (!db_accurate_get_jobids(ua->jcr, ua->db, &jr, &tempids)) {
+         if (!ua->db->accurate_get_jobids(ua->jcr, &jr, &tempids)) {
             return true;
          }
          jobids.add(tempids);
@@ -769,7 +781,7 @@ static void do_client_cmd(UAContext *ua, CLIENTRES *client, const char *cmd)
    return;
 }
 
-/*
+/**
  * .die (seg fault)
  * .dump (sm_dump)
  * .exit (no arg => .quit)
@@ -907,7 +919,7 @@ bool dot_admin_cmds(UAContext *ua, const char *cmd)
    return result;
 }
 #else
-/*
+/**
  * Dummy routine for non-development version
  */
 bool dot_admin_cmds(UAContext *ua, const char *cmd)
@@ -936,7 +948,7 @@ bool dot_jobdefs_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Can use an argument to filter on JobType
  * .jobs [type=B]
  */
@@ -994,14 +1006,15 @@ bool dot_jobstatus_cmd(UAContext *ua, const char *cmd)
          return false;
       }
    }
-   Mmsg(select, get_jobstatus_details, where.c_str());
+
+   ua->db->fill_query(select, B_DB::SQL_QUERY_get_jobstatus_details, where.c_str());
 
    if (!open_client_db(ua)) {
       return false;
    }
 
    ua->send->array_start("jobstatus");
-   retval = db_list_sql_query(ua->jcr, ua->db, select.c_str(), ua->send, HORZ_LIST, false);
+   retval = ua->db->list_sql_query(ua->jcr, select.c_str(), ua->send, HORZ_LIST, false);
    ua->send->array_end("jobstatus");
 
    return retval;
@@ -1215,7 +1228,7 @@ bool dot_types_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * If this command is called, it tells the director that we
  * are a program that wants a sort of API, and hence,
  * we will probably suppress certain output, include more
@@ -1302,19 +1315,19 @@ bool dot_sql_cmd(UAContext *ua, const char *cmd)
       /*
        * BAT uses the ".sql" command and expects this format
        */
-      retval = db_sql_query(ua->db, ua->argv[pos], sql_handler, (void *)ua);
+      retval = ua->db->sql_query(ua->argv[pos], sql_handler, (void *)ua);
       break;
    default:
       /*
        * General format
        */
-      retval = db_list_sql_query(ua->jcr, ua->db, ua->argv[pos], ua->send, HORZ_LIST, false);
+      retval = ua->db->list_sql_query(ua->jcr, ua->argv[pos], ua->send, HORZ_LIST, false);
       break;
    }
 
    if (!retval) {
-      Dmsg1(100, "Query failed: ERR=%s", db_strerror(ua->db));
-      ua->error_msg(_("Query failed: %s. ERR=%s"), ua->cmd, db_strerror(ua->db));
+      Dmsg1(100, "Query failed: ERR=%s", ua->db->strerror());
+      ua->error_msg(_("Query failed: %s. ERR=%s"), ua->cmd, ua->db->strerror());
    }
 
    return retval;
@@ -1338,10 +1351,8 @@ bool dot_mediatypes_cmd(UAContext *ua, const char *cmd)
    }
 
    ua->send->array_start("mediatypes");
-   if (!db_sql_query(ua->db,
-                     "SELECT DISTINCT MediaType FROM MediaType ORDER BY MediaType",
-                     one_handler, (void *)ua)) {
-      ua->error_msg(_("List MediaType failed: ERR=%s\n"), db_strerror(ua->db));
+   if (!ua->db->sql_query("SELECT DISTINCT MediaType FROM MediaType ORDER BY MediaType", one_handler, (void *)ua)) {
+      ua->error_msg(_("List MediaType failed: ERR=%s\n"), ua->db->strerror());
    }
    ua->send->array_end("mediatypes");
 
@@ -1355,10 +1366,8 @@ bool dot_media_cmd(UAContext *ua, const char *cmd)
    }
 
    ua->send->array_start("media");
-   if (!db_sql_query(ua->db,
-                     "SELECT DISTINCT Media.VolumeName FROM Media ORDER BY VolumeName",
-                     one_handler, (void *)ua)) {
-      ua->error_msg(_("List Media failed: ERR=%s\n"), db_strerror(ua->db));
+   if (!ua->db->sql_query("SELECT DISTINCT Media.VolumeName FROM Media ORDER BY VolumeName", one_handler, (void *)ua)) {
+      ua->error_msg(_("List Media failed: ERR=%s\n"), ua->db->strerror());
    }
    ua->send->array_end("media");
 
@@ -1404,10 +1413,8 @@ bool dot_locations_cmd(UAContext *ua, const char *cmd)
    }
 
    ua->send->array_start("locations");
-   if (!db_sql_query(ua->db,
-                     "SELECT DISTINCT Location FROM Location ORDER BY Location",
-                     one_handler, (void *)ua)) {
-      ua->error_msg(_("List Location failed: ERR=%s\n"), db_strerror(ua->db));
+   if (!ua->db->sql_query( "SELECT DISTINCT Location FROM Location ORDER BY Location", one_handler, (void *)ua)) {
+      ua->error_msg(_("List Location failed: ERR=%s\n"), ua->db->strerror());
    }
    ua->send->array_end("locations");
 
@@ -1471,7 +1478,7 @@ bool dot_volstatus_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Return default values for a job
  */
 bool dot_defaults_cmd(UAContext *ua, const char *cmd)
@@ -1504,7 +1511,7 @@ bool dot_defaults_cmd(UAContext *ua, const char *cmd)
          ua->send->object_key_value("client", "%s=", ((job->client) ? job->client->name() : _("*None*")), "%s\n");
          ua->send->send_buffer();
          get_job_storage(&store, job, NULL);
-         ua->send->object_key_value("storage", "%s=", store.store->name(), "%s\n");
+         ua->send->object_key_value("storage", "%s=", ((store.store) ? store.store->name() : _("*None*")), "%s\n");
          ua->send->send_buffer();
          ua->send->object_key_value("where", "%s=", (job->RestoreWhere ? job->RestoreWhere : ""), "%s\n");
          ua->send->send_buffer();
