@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2016 Planets Communications B.V.
-   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,11 +21,13 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Output Commands
+ * Kern Sibbald, September MM
+ */
+/**
+ * @file
+ * User Agent Output Commands
  *
  * I.e. messages, listing database, showing resources, ...
- *
- * Kern Sibbald, September MM
  */
 
 #include "bareos.h"
@@ -43,13 +45,13 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist);
 static bool list_nextvol(UAContext *ua, int ndays);
 static bool parse_list_backups_cmd(UAContext *ua, const char *range, e_list_type llist);
 
-/*
+/**
  * Some defaults.
  */
 #define DEFAULT_LOG_LINES 5
 #define DEFAULT_NR_DAYS 50
 
-/*
+/**
  * Turn auto display of console messages on/off
  */
 bool autodisplay_cmd(UAContext *ua, const char *cmd)
@@ -74,7 +76,7 @@ bool autodisplay_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Turn GUI mode on/off
  */
 bool gui_cmd(UAContext *ua, const char *cmd)
@@ -99,7 +101,7 @@ bool gui_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Enter with Resources locked
  */
 static void show_disabled_jobs(UAContext *ua)
@@ -126,7 +128,7 @@ static void show_disabled_jobs(UAContext *ua)
    }
 }
 
-/*
+/**
  * Enter with Resources locked
  */
 static void show_disabled_clients(UAContext *ua)
@@ -153,7 +155,7 @@ static void show_disabled_clients(UAContext *ua)
    }
 }
 
-/*
+/**
  * Enter with Resources locked
  */
 static void show_disabled_schedules(UAContext *ua)
@@ -205,7 +207,7 @@ static struct showstruct avail_resources[] = {
    { NULL, 0 }
 };
 
-/*
+/**
  *  Displays Resources
  *
  *  show all
@@ -343,7 +345,7 @@ bail_out:
    return true;
 }
 
-/*
+/**
  *  List contents of database
  *
  *  list jobs                   - lists all jobs run
@@ -393,6 +395,12 @@ static int get_jobid_from_cmdline(UAContext *ua)
    i = find_arg_with_value(ua, NT_("ujobid"));
    if (i >= 0) {
       bstrncpy(jr.Job, ua->argv[i], MAX_NAME_LENGTH);
+      jr.JobId = 0;
+      if (ua->db->get_job_record(ua->jcr, &jr)) {
+         jobid = jr.JobId;
+      } else {
+         return -1;
+      }
    } else {
       i = find_arg_with_value(ua, NT_("jobid"));
       if (i >= 0) {
@@ -403,7 +411,7 @@ static int get_jobid_from_cmdline(UAContext *ua)
       }
    }
 
-   if (db_get_job_record(ua->jcr, ua->db, &jr)) {
+   if (ua->db->get_job_record(ua->jcr, &jr)) {
       jobid = jr.JobId;
    } else {
       Dmsg1(200, "get_jobid_from_cmdline: Failed to get job record for JobId %d\n", jr.JobId);
@@ -419,7 +427,7 @@ static int get_jobid_from_cmdline(UAContext *ua)
 
    if (jr.ClientId) {
       cr.ClientId = jr.ClientId;
-      if (db_get_client_record(ua->jcr, ua->db, &cr)) {
+      if (ua->db->get_client_record(ua->jcr, &cr)) {
          if (!ua->acl_access_ok(Client_ACL, cr.Name, true)) {
             Dmsg1(200, "get_jobid_from_cmdline: No access to Client %s\n", cr.Name);
             jobid = -1;
@@ -436,7 +444,7 @@ bail_out:
    return jobid;
 }
 
-/*
+/**
  * Filter convience functions that abstract the actions needed to
  * perform a certain type of acl or resource filtering.
  */
@@ -479,6 +487,14 @@ static void set_query_range(POOL_MEM &query_range, UAContext *ua, JOB_DBR *jr)
 {
    int i;
 
+   /*
+    * Ensure query range is an empty string instead of NULL
+    * to avoid any issues.
+    */
+   if (query_range.c_str() == NULL) {
+      pm_strcpy(query_range, "");
+   }
+
    /*
     * See if this is a second call to set_query_range() if so and any acl
     * filters have been set we setup a new query_range filter including a
@@ -499,16 +515,8 @@ static void set_query_range(POOL_MEM &query_range, UAContext *ua, JOB_DBR *jr)
       POOL_MEM temp(PM_MESSAGE);
 
       jr->limit = atoi(ua->argv[i]);
+      ua->send->add_limit_filter_tuple(jr->limit);
 
-      /*
-       * When any acl filters are set create a limit filter using the filter framework
-       * and increase the database LIMIT clause to 10 * the limit. The filter framework
-       * will make sure the end-user will only see the wanted limit.
-       */
-      if (ua->send->has_acl_filters()) {
-         ua->send->add_limit_filter_tuple(jr->limit);
-         jr->limit = jr->limit * 10;
-      }
       temp.bsprintf(" LIMIT %d", jr->limit);
       pm_strcat(query_range, temp.c_str());
 
@@ -517,6 +525,8 @@ static void set_query_range(POOL_MEM &query_range, UAContext *ua, JOB_DBR *jr)
        */
       i = find_arg_with_value(ua, NT_("offset"));
       if (i >= 0) {
+         jr->offset = atoi(ua->argv[i]);
+         ua->send->add_offset_filter_tuple(jr->offset);
          temp.bsprintf(" OFFSET %d", atoi(ua->argv[i]));
          pm_strcat(query_range, temp.c_str());
       }
@@ -531,7 +541,8 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
    MEDIA_DBR mr;
    int days = 0,
        hours = 0,
-       jobstatus = 0;
+       jobstatus = 0,
+       joblevel = 0;
    bool count,
         last,
         current,
@@ -599,6 +610,14 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
       return false;
    }
 
+   /*
+    * joblevel=X
+    */
+   if (!get_user_job_level_selection(ua, &joblevel)) {
+      ua->error_msg(_("invalid joblevel parameter\n"));
+      return false;
+   }
+
    /*
     * Select what to do based on the first argument.
     */
@@ -662,14 +681,13 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_job_records(ua->jcr, ua->db, &jr, query_range.c_str(), clientname,
-                          jobstatus, volumename, schedtime, last, count,
-                          ua->send, llist);
+      ua->db->list_job_records(ua->jcr, &jr, query_range.c_str(), clientname,
+                               jobstatus, joblevel, volumename, schedtime, last, count, ua->send, llist);
    } else if (bstrcasecmp(ua->argk[1], NT_("jobtotals"))) {
       /*
        * List JOBTOTALS
        */
-      db_list_job_totals(ua->jcr, ua->db, &jr, ua->send);
+      ua->db->list_job_totals(ua->jcr, &jr, ua->send);
    } else if ((bstrcasecmp(ua->argk[1], NT_("jobid")) ||
                bstrcasecmp(ua->argk[1], NT_("ujobid"))) && ua->argv[1]) {
       /*
@@ -696,8 +714,8 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
             set_query_range(query_range, ua, &jr);
 
-            db_list_job_records(ua->jcr, ua->db, &jr, query_range.c_str(), clientname,
-                                jobstatus, volumename, schedtime, last, count, ua->send, llist);
+            ua->db->list_job_records(ua->jcr, &jr, query_range.c_str(), clientname,
+                                     jobstatus, joblevel, volumename, schedtime, last, count, ua->send, llist);
          }
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("basefiles"))) {
@@ -706,9 +724,9 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
        */
       jobid = get_jobid_from_cmdline(ua);
       if (jobid > 0) {
-         db_list_base_files_for_job(ua->jcr, ua->db, jobid, ua->send);
+         ua->db->list_base_files_for_job(ua->jcr, jobid, ua->send);
       } else {
-         ua->error_msg(_("missing parameter: jobid\n"));
+         ua->error_msg(_("jobid not found in db, access to job or client denied by ACL, or client not found in db\n"));
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("files"))) {
       /*
@@ -716,9 +734,9 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
        */
       jobid = get_jobid_from_cmdline(ua);
       if (jobid > 0) {
-         db_list_files_for_job(ua->jcr, ua->db, jobid, ua->send);
+         ua->db->list_files_for_job(ua->jcr, jobid, ua->send);
       } else {
-         ua->error_msg(_("missing parameter: jobid\n"));
+         ua->error_msg(_("jobid not found in db, access to job or client denied by ACL, or client not found in db\n"));
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("fileset"))) {
       int filesetid = 0;
@@ -743,9 +761,9 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
          set_query_range(query_range, ua, &jr);
 
-         db_list_filesets(ua->jcr, ua->db, &jr, query_range.c_str(), ua->send, llist);
+         ua->db->list_filesets(ua->jcr, &jr, query_range.c_str(), ua->send, llist);
       } else {
-         ua->error_msg(_("missing parameter: jobid or filesetid\n"));
+         ua->error_msg(_("jobid not found in db, access to job or client denied by ACL, or client not found in db or missing filesetid\n"));
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("filesets"))) {
       /*
@@ -758,16 +776,16 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_filesets(ua->jcr, ua->db, &jr, query_range.c_str(), ua->send, llist);
+      ua->db->list_filesets(ua->jcr, &jr, query_range.c_str(), ua->send, llist);
    } else if (bstrcasecmp(ua->argk[1], NT_("jobmedia"))) {
       /*
        * List JOBMEDIA
        */
       jobid = get_jobid_from_cmdline(ua);
       if (jobid >= 0) {
-         db_list_jobmedia_records(ua->jcr, ua->db, jobid, ua->send, llist);
+         ua->db->list_jobmedia_records(ua->jcr, jobid, ua->send, llist);
       } else {
-         ua->error_msg(_("missing parameter: jobid\n"));
+         ua->error_msg(_("jobid not found in db, access to job or client denied by ACL, or client not found in db\n"));
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("joblog"))) {
       /*
@@ -775,9 +793,9 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
        */
       jobid = get_jobid_from_cmdline(ua);
       if (jobid >= 0) {
-         db_list_joblog_records(ua->jcr, ua->db, jobid, ua->send, llist);
+         ua->db->list_joblog_records(ua->jcr, jobid, query_range.c_str(), count, ua->send, llist);
       } else {
-         ua->error_msg(_("missing parameter: jobid\n"));
+         ua->error_msg(_("jobid not found in db, access to job or client denied by ACL, or client not found in db\n"));
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("log"))) {
       bool reverse;
@@ -804,7 +822,7 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_log_records(ua->jcr, ua->db, clientname, query_range.c_str(), reverse, ua->send, llist);
+      ua->db->list_log_records(ua->jcr, clientname, query_range.c_str(), reverse,  ua->send, llist);
    } else if (bstrcasecmp(ua->argk[1], NT_("pool")) ||
               bstrcasecmp(ua->argk[1], NT_("pools"))) {
       POOL_DBR pr;
@@ -824,7 +842,7 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_pool_records(ua->jcr, ua->db, &pr, ua->send, llist);
+      ua->db->list_pool_records(ua->jcr, &pr, ua->send, llist);
    } else if (bstrcasecmp(ua->argk[1], NT_("clients"))) {
       /*
        * List CLIENTS
@@ -842,7 +860,7 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_client_records(ua->jcr, ua->db, NULL, ua->send, llist);
+      ua->db->list_client_records(ua->jcr, NULL, ua->send, llist);
    } else if (bstrcasecmp(ua->argk[1], NT_("client")) && ua->argv[1]) {
       /*
        * List CLIENT=xxx
@@ -860,7 +878,7 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_client_records(ua->jcr, ua->db, ua->argv[1], ua->send, llist);
+      ua->db->list_client_records(ua->jcr, ua->argv[1], ua->send, llist);
    } else if (bstrcasecmp(ua->argk[1], NT_("storages"))) {
       /*
        * List STORAGES
@@ -878,7 +896,7 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
 
       set_query_range(query_range, ua, &jr);
 
-      db_list_storage_records(ua->jcr, ua->db, ua->send, llist);
+      ua->db->list_sql_query(ua->jcr, "SELECT * FROM Storage", ua->send, llist, "storages");
    } else if (bstrcasecmp(ua->argk[1], NT_("media")) ||
               bstrcasecmp(ua->argk[1], NT_("volume")) ||
               bstrcasecmp(ua->argk[1], NT_("volumes"))) {
@@ -887,12 +905,12 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
        */
       jobid = get_jobid_from_cmdline(ua);
       if (jobid > 0) {
-         int count;
+         int num_vols = 0;
          POOLMEM *VolumeName;
 
          VolumeName = get_pool_memory(PM_FNAME);
-         count = db_get_job_volume_names(ua->jcr, ua->db, jobid, VolumeName);
-         ua->send_msg(_("Jobid %d used %d Volume(s): %s\n"), jobid, count, VolumeName);
+         num_vols = ua->db->get_job_volume_names(ua->jcr, jobid, VolumeName);
+         ua->send_msg(_("Jobid %d used %d Volume(s): %s\n"), jobid, num_vols, VolumeName);
          free_pool_memory(VolumeName);
       } else if (jobid == 0) {
          /*
@@ -901,7 +919,7 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
          if (ua->argv[1]) {
             bstrncpy(mr.VolumeName, ua->argv[1], sizeof(mr.VolumeName));
             ua->send->object_start("volume");
-            db_list_media_records(ua->jcr, ua->db, &mr, ua->send, llist);
+            ua->db->list_media_records(ua->jcr, &mr, query_range.c_str(), count, ua->send, llist);
             ua->send->object_end("volume");
          } else {
             /*
@@ -917,9 +935,11 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
                   return true;
                }
 
+               set_query_range(query_range, ua, &jr);
+
                mr.PoolId = pr.PoolId;
                ua->send->array_start("volumes");
-               db_list_media_records(ua->jcr, ua->db, &mr, ua->send, llist);
+               ua->db->list_media_records(ua->jcr, &mr, query_range.c_str(), count, ua->send, llist);
                ua->send->array_end("volumes");
                return true;
             } else {
@@ -935,14 +955,14 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
                   if (current) {
                      set_res_filter(ua, 1, R_POOL); /* PoolName */
                   }
-                  db_list_media_records(ua->jcr, ua->db, &mr, ua->send, llist);
+                  ua->db->list_media_records(ua->jcr, &mr, query_range.c_str(), count, ua->send, llist);
                   ua->send->array_end("volumes");
                } else {
                   /*
                    * List Volumes in all pools
                    */
-                  if (!db_get_pool_ids(ua->jcr, ua->db, &num_pools, &ids)) {
-                     ua->error_msg(_("Error obtaining pool ids. ERR=%s\n"), db_strerror(ua->db));
+                  if (!ua->db->get_pool_ids(ua->jcr, &num_pools, &ids)) {
+                     ua->error_msg(_("Error obtaining pool ids. ERR=%s\n"), ua->db->strerror());
                      return true;
                   }
 
@@ -953,12 +973,12 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
                   ua->send->object_start("volumes");
                   for (i = 0; i < num_pools; i++) {
                      pr.PoolId = ids[i];
-                     if (db_get_pool_record(ua->jcr, ua->db, &pr)) {
+                     if (ua->db->get_pool_record(ua->jcr, &pr)) {
                         if (ua->acl_access_ok(Pool_ACL, pr.Name, false)) {
                            ua->send->decoration( "Pool: %s\n", pr.Name );
                            ua->send->array_start(pr.Name);
                            mr.PoolId = ids[i];
-                           db_list_media_records(ua->jcr, ua->db, &mr, ua->send, llist);
+                           ua->db->list_media_records(ua->jcr, &mr, query_range.c_str(), count, ua->send, llist);
                            ua->send->array_end(pr.Name);
                         }
                      }
@@ -996,10 +1016,10 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
       i = find_arg_with_value(ua, NT_("jobid"));
       if (i >= 0) {
          if (is_a_number_list(ua->argv[i])) {
-            db_list_copies_records(ua->jcr, ua->db, query_range.c_str(), ua->argv[i], ua->send, llist);
+            ua->db->list_copies_records(ua->jcr, query_range.c_str(), ua->argv[i], ua->send, llist);
          }
       } else {
-         db_list_copies_records(ua->jcr, ua->db, query_range.c_str(), NULL, ua->send, llist);
+         ua->db->list_copies_records(ua->jcr, query_range.c_str(), NULL, ua->send, llist);
       }
    } else if (bstrcasecmp(ua->argk[1], NT_("backups"))) {
       if (parse_list_backups_cmd(ua, query_range.c_str(), llist)) {
@@ -1032,7 +1052,15 @@ static bool do_list_cmd(UAContext *ua, const char *cmd, e_list_type llist)
             break;
          }
 
-         db_list_sql_query(ua->jcr, ua->db, ua->cmd, ua->send, llist, "backups");
+         ua->db->list_sql_query(ua->jcr, ua->cmd, ua->send, llist, "backups");
+      }
+   } else if ( bstrcasecmp(ua->argk[1], NT_("jobstatistics") )
+            || bstrcasecmp(ua->argk[1], NT_("jobstats") ) ) {
+      jobid = get_jobid_from_cmdline(ua);
+      if (jobid > 0) {
+         ua->db->list_jobstatistics_records(ua->jcr, jobid, ua->send, llist);
+      } else {
+         ua->error_msg(_("no jobid given\n"));
       }
    } else {
       ua->error_msg(_("Unknown list keyword: %s\n"), NPRT(ua->argk[1]));
@@ -1258,9 +1286,9 @@ static bool parse_list_backups_cmd(UAContext *ua, const char *range, e_list_type
    pm_strcat(criteria, range);
 
    if (llist == VERT_LIST) {
-      Mmsg(ua->cmd, list_jobs_long, selection.c_str(), criteria.c_str());
+      ua->db->fill_query(ua->cmd, B_DB::SQL_QUERY_list_jobs_long, selection.c_str(), criteria.c_str());
    } else {
-      Mmsg(ua->cmd, list_jobs, selection.c_str(), criteria.c_str());
+      ua->db->fill_query(ua->cmd, B_DB::SQL_QUERY_list_jobs, selection.c_str(), criteria.c_str());
    }
 
    return true;
@@ -1307,7 +1335,7 @@ static bool list_nextvol(UAContext *ua, int ndays)
       }
       memset(&pr, 0, sizeof(pr));
       pr.PoolId = jcr->jr.PoolId;
-      if (!db_get_pool_record(jcr, jcr->db, &pr)) {
+      if (!ua->db->get_pool_record(jcr, &pr)) {
          bstrncpy(pr.Name, "*UnknownPool*", sizeof(pr.Name));
       }
       mr.PoolId = jcr->jr.PoolId;
@@ -1339,7 +1367,7 @@ get_out:
    return true;
 }
 
-/*
+/**
  * For a given job, we examine all his run records
  *  to see if it is scheduled today or tomorrow.
  */
@@ -1431,7 +1459,7 @@ RUNRES *find_next_run(RUNRES *run, JOBRES *job, utime_t &runtime, int ndays)
    return NULL;
 }
 
-/*
+/**
  * Fill in the remaining fields of the jcr as if it is going to run the job.
  */
 bool complete_jcr_for_job(JCR *jcr, JOBRES *job, POOLRES *pool)
@@ -1468,11 +1496,10 @@ bool complete_jcr_for_job(JCR *jcr, JOBRES *job, POOLRES *pool)
       return false;
    }
    bstrncpy(pr.Name, jcr->res.pool->name(), sizeof(pr.Name));
-   while (!db_get_pool_record(jcr, jcr->db, &pr)) { /* get by Name */
+   while (!jcr->db->get_pool_record(jcr, &pr)) { /* get by Name */
       /* Try to create the pool */
       if (create_pool(jcr, jcr->db, jcr->res.pool, POOL_OP_CREATE) < 0) {
-         Jmsg(jcr, M_FATAL, 0, _("Pool %s not in database. %s"), pr.Name,
-            db_strerror(jcr->db));
+         Jmsg(jcr, M_FATAL, 0, _("Pool %s not in database. %s"), pr.Name, jcr->db->strerror());
          if (jcr->db) {
             db_sql_close_pooled_connection(jcr, jcr->db);
             jcr->db = NULL;
@@ -1545,7 +1572,7 @@ bool messages_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Callback routine for "filtering" database listing.
  */
 of_filter_state filterit(void *ctx, void *data, of_filter_tuple *tuple)
@@ -1556,15 +1583,9 @@ of_filter_state filterit(void *ctx, void *data, of_filter_tuple *tuple)
 
    switch (tuple->type) {
    case OF_FILTER_LIMIT:
-      if (tuple->u.limit_filter.limit > 0) {
-         Dmsg1(200, "filterit: limit filter still %d entries to display\n",
-               tuple->u.limit_filter.limit);
-         tuple->u.limit_filter.limit--;
-      } else {
-         Dmsg0(200, "filterit: limit filter reached don't display entry\n");
-         retval = OF_FILTER_STATE_SUPPRESS;
-      }
-      goto bail_out;
+      break;
+   case OF_FILTER_OFFSET:
+      break;
    case OF_FILTER_ACL:
       if (!row[tuple->u.acl_filter.column] ||
           strlen(row[tuple->u.acl_filter.column]) == 0) {
@@ -1663,7 +1684,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Callback routine for "printing" database listing
  */
 bool printit(void *ctx, const char *msg)
@@ -1681,7 +1702,7 @@ bool printit(void *ctx, const char *msg)
    return retval;
 }
 
-/*
+/**
  * Format message and send to other end.
 
  * If the UA_sock is NULL, it means that there is no user
@@ -1774,7 +1795,7 @@ void bsendmsg(void *ctx, const char *fmt, ...)
  * The following UA methods are mainly intended for GUI
  * programs
  */
-/*
+/**
  * This is a message that should be displayed on the user's
  *  console.
  */
@@ -1792,7 +1813,7 @@ void UAContext::send_msg(const char *fmt, ...)
    send->message(NULL, message);
 }
 
-/*
+/**
  * This is an error condition with a command. The gui should put
  *  up an error or critical dialog box.  The command is aborted.
  */
@@ -1812,7 +1833,7 @@ void UAContext::error_msg(const char *fmt, ...)
    send->message(MSG_TYPE_ERROR, message);
 }
 
-/*
+/**
  * This is a warning message, that should bring up a warning
  *  dialog box on the GUI. The command is not aborted, but something
  *  went wrong.
@@ -1833,7 +1854,7 @@ void UAContext::warning_msg(const char *fmt, ...)
    send->message(MSG_TYPE_WARNING, message);
 }
 
-/*
+/**
  * This is an information message that should probably be put
  *  into the status line of a GUI program.
  */
