@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2002-2009 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
+   Copyright (C) 2011-2016 Planets Communications B.V.
    Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -21,11 +21,13 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Database prune Command
+ * Kern Sibbald, February MMII
+ */
+/**
+ * @file
+ * User Agent Database prune Command
  *
  * Applies retention periods
- *
- * Kern Sibbald, February MMII
  */
 
 #include "bareos.h"
@@ -40,7 +42,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client);
 static bool prune_stats(UAContext *ua, utime_t retention);
 static bool grow_del_list(struct del_ctx *del);
 
-/*
+/**
  * Called here to count entries to be deleted
  */
 int del_count_handler(void *ctx, int num_fields, char **row)
@@ -55,7 +57,7 @@ int del_count_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-/*
+/**
  * Called here to make in memory list of JobIds to be
  *  deleted and the associated PurgedFiles flag.
  *  The in memory list will then be transversed
@@ -88,7 +90,7 @@ int file_delete_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-/*
+/**
  * Prune records from database
  */
 bool prune_cmd(UAContext *ua, const char *cmd)
@@ -274,7 +276,7 @@ bool prune_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * Prune Directory meta data records from the database.
  */
 static bool prune_directory(UAContext *ua, CLIENTRES *client)
@@ -332,7 +334,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client)
       len++;
    }
    prune_topdir = (char *)malloc(len * 2 + 1);
-   db_escape_string(ua->jcr, ua->db, prune_topdir, temp.c_str(), len);
+   ua->db->escape_string(ua->jcr, prune_topdir, temp.c_str(), len);
 
    /*
     * Remove all files in particular directory.
@@ -354,7 +356,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client)
 
       memset(&cr, 0, sizeof(cr));
       bstrncpy(cr.Name, client->name(), sizeof(cr.Name));
-      if (!db_create_client_record(ua->jcr, ua->db, &cr)) {
+      if (!ua->db->create_client_record(ua->jcr, &cr)) {
          goto bail_out;
       }
 
@@ -367,7 +369,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client)
    }
 
    db_lock(ua->db);
-   db_sql_query(ua->db, query.c_str());
+   ua->db->sql_query(query.c_str());
    db_unlock(ua->db);
 
    /*
@@ -390,7 +392,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client)
       }
 
       db_lock(ua->db);
-      db_sql_query(ua->db, query.c_str());
+      ua->db->sql_query(query.c_str());
       db_unlock(ua->db);
    }
 
@@ -404,7 +406,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Prune Job stat records from the database.
  */
 static bool prune_stats(UAContext *ua, utime_t retention)
@@ -415,9 +417,8 @@ static bool prune_stats(UAContext *ua, utime_t retention)
    utime_t now = (utime_t)time(NULL);
 
    db_lock(ua->db);
-   Mmsg(query, "DELETE FROM JobHisto WHERE JobTDate < %s",
-        edit_int64(now - retention, ed1));
-   db_sql_query(ua->db, query.c_str());
+   Mmsg(query, "DELETE FROM JobHisto WHERE JobTDate < %s", edit_int64(now - retention, ed1));
+   ua->db->sql_query(query.c_str());
    db_unlock(ua->db);
 
    ua->info_msg(_("Pruned Jobs from JobHisto in catalog.\n"));
@@ -426,14 +427,14 @@ static bool prune_stats(UAContext *ua, utime_t retention)
 
    db_lock(ua->db);
    Mmsg(query, "DELETE FROM DeviceStats WHERE SampleTime < '%s'", dt);
-   db_sql_query(ua->db, query.c_str());
+   ua->db->sql_query(query.c_str());
    db_unlock(ua->db);
 
    ua->info_msg(_("Pruned Statistics from DeviceStats in catalog.\n"));
 
    db_lock(ua->db);
    Mmsg(query, "DELETE FROM JobStats WHERE SampleTime < '%s'", dt);
-   db_sql_query(ua->db, query.c_str());
+   ua->db->sql_query(query.c_str());
    db_unlock(ua->db);
 
    ua->info_msg(_("Pruned Statistics from JobStats in catalog.\n"));
@@ -441,7 +442,7 @@ static bool prune_stats(UAContext *ua, utime_t retention)
    return true;
 }
 
-/*
+/**
  * Use pool and client specified by user to select jobs to prune
  * returns add_from string to add in FROM clause
  *         add_where string to add in WHERE clause
@@ -462,27 +463,25 @@ static bool prune_set_filter(UAContext *ua, CLIENTRES *client,
 
    db_lock(ua->db);
    if (client) {
-      db_escape_string(ua->jcr, ua->db, ed2,
-                       client->name(), strlen(client->name()));
+      ua->db->escape_string(ua->jcr, ed2, client->name(), strlen(client->name()));
       Mmsg(tmp, " AND Client.Name = '%s' ", ed2);
       pm_strcat(*add_where, tmp.c_str());
       pm_strcat(*add_from, " JOIN Client USING (ClientId) ");
    }
 
    if (pool) {
-      db_escape_string(ua->jcr, ua->db, ed2,
-                       pool->name(), strlen(pool->name()));
+      ua->db->escape_string(ua->jcr, ed2, pool->name(), strlen(pool->name()));
       Mmsg(tmp, " AND Pool.Name = '%s' ", ed2);
       pm_strcat(*add_where, tmp.c_str());
       /* Use ON() instead of USING for some old SQLite */
-      pm_strcat(*add_from, " JOIN Pool ON (Job.PoolId = Pool.PoolId) ");
+      pm_strcat(*add_from, " JOIN Pool USING(PoolId) ");
    }
    Dmsg2(150, "f=%s w=%s\n", add_from->c_str(), add_where->c_str());
    db_unlock(ua->db);
    return true;
 }
 
-/*
+/**
  * Prune File records from the database. For any Job which
  * is older than the retention period, we unconditionally delete
  * all File records for that Job.  This is simple enough that no
@@ -532,8 +531,8 @@ bool prune_files(UAContext *ua, CLIENTRES *client, POOLRES *pool)
         sql_from.c_str(), sql_where.c_str());
    Dmsg1(050, "select sql=%s\n", query.c_str());
    cnt.count = 0;
-   if (!db_sql_query(ua->db, query.c_str(), del_count_handler, (void *)&cnt)) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->sql_query(query.c_str(), del_count_handler, (void *)&cnt)) {
+      ua->error_msg("%s", ua->db->strerror());
       Dmsg0(050, "Count failed\n");
       goto bail_out;
    }
@@ -558,7 +557,7 @@ bool prune_files(UAContext *ua, CLIENTRES *client, POOLRES *pool)
    Mmsg(query, "SELECT JobId FROM Job %s WHERE PurgedFiles=0 %s",
         sql_from.c_str(), sql_where.c_str());
    Dmsg1(050, "select sql=%s\n", query.c_str());
-   db_sql_query(ua->db, query.c_str(), file_delete_handler, (void *)&del);
+   ua->db->sql_query(query.c_str(), file_delete_handler, (void *)&del);
 
    purge_files_from_job_list(ua, del);
 
@@ -576,22 +575,24 @@ bail_out:
 
 static void drop_temp_tables(UAContext *ua)
 {
-   db_sql_query(ua->db, drop_deltabs[db_get_type_index(ua->db)]);
+   ua->db->sql_query(B_DB::SQL_QUERY_drop_deltabs);
 }
 
 static bool create_temp_tables(UAContext *ua)
 {
    /* Create temp tables and indicies */
-   if (!db_sql_query(ua->db, create_deltabs[db_get_type_index(ua->db)])) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->sql_query(B_DB::SQL_QUERY_create_deltabs)) {
+      ua->error_msg("%s", ua->db->strerror());
       Dmsg0(050, "create DelTables table failed\n");
       return false;
    }
-   if (!db_sql_query(ua->db, create_delindex)) {
-       ua->error_msg("%s", db_strerror(ua->db));
+
+   if (!ua->db->sql_query(B_DB::SQL_QUERY_create_delindex)) {
+       ua->error_msg("%s", ua->db->strerror());
        Dmsg0(050, "create DelInx1 index failed\n");
        return false;
    }
+
    return true;
 }
 
@@ -603,10 +604,10 @@ static bool grow_del_list(struct del_ctx *del)
 
    if (del->num_ids == del->max_ids) {
       del->max_ids = (del->max_ids * 3) / 2;
-      del->JobId = (JobId_t *)brealloc(del->JobId, sizeof(JobId_t) *
-         del->max_ids);
+      del->JobId = (JobId_t *)brealloc(del->JobId, sizeof(JobId_t) * del->max_ids);
       del->PurgedFiles = (char *)brealloc(del->PurgedFiles, del->max_ids);
    }
+
    return true;
 }
 
@@ -615,7 +616,7 @@ struct accurate_check_ctx {
    DBId_t FileSetId;                  /* Id of FileSet */
 };
 
-/*
+/**
  * row: Job.Name, FileSet, Client.Name, FileSetId, ClientId, Type
  */
 static int job_select_handler(void *ctx, int num_fields, char **row)
@@ -661,7 +662,7 @@ static int job_select_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-/*
+/**
  * Pruning Jobs is a bit more complicated than purging Files
  * because we delete Job records only if there is a more current
  * backup of the FileSet. Otherwise, we keep the Job record.
@@ -728,9 +729,9 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
         sql_from.c_str(), sql_where.c_str());
 
    Dmsg1(050, "select sql=%s\n", query.c_str());
-   if (!db_sql_query(ua->db, query.c_str())) {
+   if (!ua->db->sql_query(query.c_str())) {
       if (ua->verbose) {
-         ua->error_msg("%s", db_strerror(ua->db));
+         ua->error_msg("%s", ua->db->strerror());
       }
       goto bail_out;
    }
@@ -757,8 +758,8 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
     * in the configuration file. Interesting ClientId/FileSetId will be
     * added to jobids_check.
     */
-   if (!db_sql_query(ua->db, query.c_str(), job_select_handler, jobids_check)) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->sql_query(query.c_str(), job_select_handler, jobids_check)) {
+      ua->error_msg("%s", ua->db->strerror());
    }
 
    /*
@@ -775,7 +776,7 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
    foreach_alist(elt, jobids_check) {
       jr.ClientId = elt->ClientId;   /* should be always the same */
       jr.FileSetId = elt->FileSetId;
-      db_accurate_get_jobids(ua->jcr, ua->db, &jr, &tempids);
+      ua->db->accurate_get_jobids(ua->jcr, &jr, &tempids);
       jobids.add(tempids);
    }
 
@@ -791,8 +792,8 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
          "ORDER BY JobTDate DESC LIMIT 1",
         sql_from.c_str(), sql_where.c_str());
 
-   if (!db_sql_query(ua->db, query.c_str(), db_list_handler, &jobids)) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->sql_query(query.c_str(), db_list_handler, &jobids)) {
+      ua->error_msg("%s", ua->db->strerror());
    }
 
    /* If we found jobs to exclude from the DelCandidates list, we should
@@ -801,7 +802,7 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
    if (jobids.count > 0) {
       Dmsg1(60, "jobids to exclude before basejobs = %s\n", jobids.list);
       /* We also need to exclude all basejobs used */
-      db_get_used_base_jobids(ua->jcr, ua->db, jobids.list, &jobids);
+      ua->db->get_used_base_jobids(ua->jcr, jobids.list, &jobids);
 
       /* Removing useful jobs from the DelCandidates list */
       Mmsg(query, "DELETE FROM DelCandidates "
@@ -809,8 +810,8 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
                      "AND JobFiles!=0",          /* Discard when JobFiles=0 */
            jobids.list);
 
-      if (!db_sql_query(ua->db, query.c_str())) {
-         ua->error_msg("%s", db_strerror(ua->db));
+      if (!ua->db->sql_query(query.c_str())) {
+         ua->error_msg("%s", ua->db->strerror());
          goto bail_out;         /* Don't continue if the list isn't clean */
       }
       Dmsg1(60, "jobids to exclude = %s\n", jobids.list);
@@ -820,8 +821,8 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
    Mmsg(query,
         "SELECT DISTINCT DelCandidates.JobId,DelCandidates.PurgedFiles "
           "FROM DelCandidates");
-   if (!db_sql_query(ua->db, query.c_str(), job_delete_handler, (void *)&del)) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->sql_query(query.c_str(), job_delete_handler, (void *)&del)) {
+      ua->error_msg("%s", ua->db->strerror());
    }
 
    purge_job_list_from_catalog(ua, del);
@@ -848,7 +849,7 @@ bail_out:
    return 1;
 }
 
-/*
+/**
  * Dispatch to the right prune jobs function.
  */
 bool prune_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool, int JobType)
@@ -861,7 +862,7 @@ bool prune_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool, int JobType)
    }
 }
 
-/*
+/**
  * Prune a given Volume
  */
 bool prune_volume(UAContext *ua, MEDIA_DBR *mr)
@@ -900,7 +901,7 @@ bool prune_volume(UAContext *ua, MEDIA_DBR *mr)
    return ok;
 }
 
-/*
+/**
  * Get prune list for a volume
  */
 int get_prune_list_for_volume(UAContext *ua, MEDIA_DBR *mr, del_ctx *del)
@@ -917,18 +918,16 @@ int get_prune_list_for_volume(UAContext *ua, MEDIA_DBR *mr, del_ctx *del)
    /*
     * Now add to the  list of JobIds for Jobs written to this Volume
     */
-   edit_int64(mr->MediaId, ed1);
    period = mr->VolRetention;
    now = (utime_t)time(NULL);
-   edit_int64(now-period, ed2);
-   Mmsg(query, sel_JobMedia, ed1, ed2);
-   Dmsg3(250, "Now=%d period=%d now-period=%s\n", (int)now, (int)period,
-      ed2);
+   ua->db->fill_query(query, B_DB::SQL_QUERY_sel_JobMedia, edit_int64(mr->MediaId, ed1), edit_int64(now-period, ed2));
 
+   Dmsg3(250, "Now=%d period=%d now-period=%s\n", (int)now, (int)period, ed2);
    Dmsg1(050, "Query=%s\n", query.c_str());
-   if (!db_sql_query(ua->db, query.c_str(), file_delete_handler, (void *)del)) {
+
+   if (!ua->db->sql_query(query.c_str(), file_delete_handler, (void *)del)) {
       if (ua->verbose) {
-         ua->error_msg("%s", db_strerror(ua->db));
+         ua->error_msg("%s", ua->db->strerror());
       }
       Dmsg0(050, "Count failed\n");
       goto bail_out;
@@ -939,7 +938,7 @@ bail_out:
    return count;
 }
 
-/*
+/**
  * We have a list of jobs to prune or purge. If any of them is
  *   currently running, we set its JobId to zero which effectively
  *   excludes it.
