@@ -1,8 +1,8 @@
 /* BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,11 +20,12 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- Run Command
- *
  * Kern Sibbald, December MMI
  */
-
+/**
+ * @file
+ * BAREOS Director -- Run Command
+ */
 #include "bareos.h"
 #include "dird.h"
 
@@ -39,7 +40,7 @@ static int modify_job_parameters(UAContext *ua, JCR *jcr, RUN_CTX &rc);
 /* Imported variables */
 extern struct s_kw ReplaceOptions[];
 
-/*
+/**
  * Rerun a job by jobid. Lookup the job data and rerun the job with that data.
  *
  * Returns: false on error
@@ -54,9 +55,8 @@ static inline bool rerun_job(UAContext *ua, JobId_t JobId, bool yes, utime_t now
    memset(&jr, 0, sizeof(jr));
    jr.JobId = JobId;
    ua->send_msg("rerunning jobid %d\n", jr.JobId);
-   if (!db_get_job_record(ua->jcr, ua->db, &jr)) {
-      Jmsg(ua->jcr, M_WARNING, 0, _("Error getting Job record for Job rerun: ERR=%s"),
-           db_strerror(ua->jcr->db));
+   if (!ua->db->get_job_record(ua->jcr, &jr)) {
+      Jmsg(ua->jcr, M_WARNING, 0, _("Error getting Job record for Job rerun: ERR=%s"), ua->db->strerror());
       goto bail_out;
    }
 
@@ -84,9 +84,8 @@ static inline bool rerun_job(UAContext *ua, JobId_t JobId, bool yes, utime_t now
 
       memset(&cr, 0, sizeof(cr));
       cr.ClientId = jr.ClientId;
-      if (!db_get_client_record(ua->jcr, ua->db, &cr)) {
-         Jmsg(ua->jcr, M_WARNING, 0, _("Error getting Client record for Job rerun: ERR=%s"),
-              db_strerror(ua->jcr->db));
+      if (!ua->db->get_client_record(ua->jcr, &cr)) {
+         Jmsg(ua->jcr, M_WARNING, 0, _("Error getting Client record for Job rerun: ERR=%s"), ua->db->strerror());
          goto bail_out;
       }
       Mmsg(cmdline, " client=\"%s\"", cr.Name);
@@ -98,9 +97,8 @@ static inline bool rerun_job(UAContext *ua, JobId_t JobId, bool yes, utime_t now
 
       memset(&pr, 0, sizeof(pr));
       pr.PoolId = jr.PoolId;
-      if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
-         Jmsg(ua->jcr, M_WARNING, 0, _("Error getting Pool record for Job rerun: ERR=%s"),
-              db_strerror(ua->jcr->db));
+      if (!ua->db->get_pool_record(ua->jcr, &pr)) {
+         Jmsg(ua->jcr, M_WARNING, 0, _("Error getting Pool record for Job rerun: ERR=%s"), ua->db->strerror());
          goto bail_out;
       }
 
@@ -166,9 +164,8 @@ static inline bool rerun_job(UAContext *ua, JobId_t JobId, bool yes, utime_t now
 
       memset(&fs, 0, sizeof(fs));
       fs.FileSetId = jr.FileSetId;
-      if (!db_get_fileset_record(ua->jcr, ua->db, &fs)) {
-         Jmsg(ua->jcr, M_WARNING, 0, _("Error getting FileSet record for Job rerun: ERR=%s"),
-              db_strerror(ua->jcr->db));
+      if (!ua->db->get_fileset_record(ua->jcr, &fs)) {
+         Jmsg(ua->jcr, M_WARNING, 0, _("Error getting FileSet record for Job rerun: ERR=%s"), ua->db->strerror());
          goto bail_out;
       }
       Mmsg(cmdline, " fileset=\"%s\"", fs.FileSet);
@@ -192,7 +189,7 @@ bail_out:
    return false;
 }
 
-/*
+/**
  * Rerun a job selection.
  *
  * Returns: 0 on error
@@ -254,17 +251,17 @@ bool rerun_cmd(UAContext *ua, const char *cmd)
    }
 
    if (j < 0 && !timeframe && !since_jobid_given) {
-      ua->send_msg("Please specifiy jobid, since_jobid, hours or days\n");
+      ua->send_msg("Please specify jobid, since_jobid, hours or days\n");
       goto bail_out;
    }
 
    if (j >= 0 && since_jobid_given) {
-      ua->send_msg("Please specifiy either jobid or since_jobid (and optionally until_jobid)\n");
+      ua->send_msg("Please specify either jobid or since_jobid (and optionally until_jobid)\n");
       goto bail_out;
    }
 
    if (j >= 0 && timeframe) {
-      ua->send_msg("Please specifiy either jobid or timeframe\n");
+      ua->send_msg("Please specify either jobid or timeframe\n");
       goto bail_out;
    }
 
@@ -297,7 +294,7 @@ bool rerun_cmd(UAContext *ua, const char *cmd)
          Mmsg(query, "SELECT JobId FROM Job WHERE JobStatus = 'f' AND SchedTime > '%s' ORDER BY JobId", dt);
       }
 
-      db_get_query_dbids(ua->jcr, ua->db, query, ids);
+      ua->db->get_query_dbids(ua->jcr, query, ids);
 
       ua->send_msg("The following ids were selected for rerun:\n");
       for (i = 0; i < ids.num_ids; i++) {
@@ -339,7 +336,7 @@ bail_out:
    return false;
 }
 
-/*
+/**
  * For Backup and Verify Jobs
  *     run [job=]<job-name> level=<level-name>
  *
@@ -828,7 +825,7 @@ try_again:
    return 0;
 }
 
-/*
+/**
  * Reset the restore context.
  * This subroutine can be called multiple times, so it must keep any prior settings.
  */
@@ -1412,9 +1409,8 @@ static bool display_job_parameters(UAContext *ua, JCR *jcr, RUN_CTX &rc)
          } else if (jcr->RestoreJobId) { /* Display job name if jobid requested */
             memset(&jr, 0, sizeof(jr));
             jr.JobId = jcr->RestoreJobId;
-            if (!db_get_job_record(jcr, ua->db, &jr)) {
-               ua->error_msg(_("Could not get job record for selected JobId. ERR=%s"),
-                             db_strerror(ua->db));
+            if (!ua->db->get_job_record(jcr, &jr)) {
+               ua->error_msg(_("Could not get job record for selected JobId. ERR=%s"), ua->db->strerror());
                return false;
             }
             Name = jr.Job;
