@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Prompt and Selection code
- *
  * Kern Sibbald, October MMI
  */
+/**
+ * @file
+ * User Agent Prompt and Selection code
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -33,7 +35,7 @@
 extern struct s_jt jobtypes[];
 extern struct s_jl joblevels[];
 
-/*
+/**
  * Confirm a retention period
  */
 bool confirm_retention(UAContext *ua, utime_t *ret, const char *msg)
@@ -73,7 +75,7 @@ bool confirm_retention(UAContext *ua, utime_t *ret, const char *msg)
     return true;
 }
 
-/*
+/**
  * Given a list of keywords, find the first one that is in the argument list.
  *
  * Returns: -1 if not found
@@ -92,7 +94,7 @@ int find_arg_keyword(UAContext *ua, const char **list)
    return -1;
 }
 
-/*
+/**
  * Given one keyword, find the first one that is in the argument list.
  *
  * Returns: argk index (always gt 0)
@@ -109,7 +111,7 @@ int find_arg(UAContext *ua, const char *keyword)
    return -1;
 }
 
-/*
+/**
  * Given a single keyword, find it in the argument list, but it must have a value
  *
  * Returns: -1 if not found or no value
@@ -130,7 +132,7 @@ int find_arg_with_value(UAContext *ua, const char *keyword)
    return -1;
 }
 
-/*
+/**
  * Given a list of keywords, prompt the user to choose one.
  *
  * Returns: -1 on failure
@@ -146,7 +148,7 @@ int do_keyword_prompt(UAContext *ua, const char *msg, const char **list)
    return do_prompt(ua, "", msg, NULL, 0);
 }
 
-/*
+/**
  * Select a Storage resource from prompt list
  */
 STORERES *select_storage_resource(UAContext *ua, bool autochanger_only)
@@ -180,7 +182,7 @@ STORERES *select_storage_resource(UAContext *ua, bool autochanger_only)
    return store;
 }
 
-/*
+/**
  * Select a FileSet resource from prompt list
  */
 FILESETRES *select_fileset_resource(UAContext *ua)
@@ -207,7 +209,7 @@ FILESETRES *select_fileset_resource(UAContext *ua)
    return fs;
 }
 
-/*
+/**
  * Get a catalog resource from prompt list
  */
 CATRES *get_catalog_resource(UAContext *ua)
@@ -261,7 +263,7 @@ CATRES *get_catalog_resource(UAContext *ua)
    return catalog;
 }
 
-/*
+/**
  * Select a job to enable or disable
  */
 JOBRES *select_enable_disable_job_resource(UAContext *ua, bool enable)
@@ -292,7 +294,7 @@ JOBRES *select_enable_disable_job_resource(UAContext *ua, bool enable)
    return job;
 }
 
-/*
+/**
  * Select a Job resource from prompt list
  */
 JOBRES *select_job_resource(UAContext *ua)
@@ -319,7 +321,7 @@ JOBRES *select_job_resource(UAContext *ua)
    return job;
 }
 
-/*
+/**
  * Select a Restore Job resource from argument or prompt
  */
 JOBRES *get_restore_job(UAContext *ua)
@@ -339,7 +341,7 @@ JOBRES *get_restore_job(UAContext *ua)
    return select_restore_job_resource(ua);
 }
 
-/*
+/**
  * Select a Restore Job resource from prompt list
  */
 JOBRES *select_restore_job_resource(UAContext *ua)
@@ -366,7 +368,7 @@ JOBRES *select_restore_job_resource(UAContext *ua)
    return job;
 }
 
-/*
+/**
  * Select a client resource from prompt list
  */
 CLIENTRES *select_client_resource(UAContext *ua)
@@ -393,7 +395,7 @@ CLIENTRES *select_client_resource(UAContext *ua)
    return client;
 }
 
-/*
+/**
  * Select a client to enable or disable
  */
 CLIENTRES *select_enable_disable_client_resource(UAContext *ua, bool enable)
@@ -424,7 +426,7 @@ CLIENTRES *select_enable_disable_client_resource(UAContext *ua, bool enable)
    return client;
 }
 
-/*
+/**
  *  Get client resource, start by looking for
  *   client=<client-name>
  *  if we don't find the keyword, we prompt the user.
@@ -450,7 +452,7 @@ CLIENTRES *get_client_resource(UAContext *ua)
    return select_client_resource(ua);
 }
 
-/*
+/**
  * Select a schedule to enable or disable
  */
 SCHEDRES *select_enable_disable_schedule_resource(UAContext *ua, bool enable)
@@ -481,7 +483,7 @@ SCHEDRES *select_enable_disable_schedule_resource(UAContext *ua, bool enable)
    return sched;
 }
 
-/*
+/**
  * Scan what the user has entered looking for:
  *
  * client=<client-name>
@@ -494,10 +496,10 @@ SCHEDRES *select_enable_disable_schedule_resource(UAContext *ua, bool enable)
 bool get_client_dbr(UAContext *ua, CLIENT_DBR *cr)
 {
    if (cr->Name[0]) {                 /* If name already supplied */
-      if (db_get_client_record(ua->jcr, ua->db, cr)) {
+      if (ua->db->get_client_record(ua->jcr, cr)) {
          return true;
       }
-      ua->error_msg(_("Could not find Client %s: ERR=%s"), cr->Name, db_strerror(ua->db));
+      ua->error_msg(_("Could not find Client %s: ERR=%s"), cr->Name, ua->db->strerror());
    }
 
    for (int i = 1; i < ua->argc; i++) {
@@ -507,9 +509,9 @@ bool get_client_dbr(UAContext *ua, CLIENT_DBR *cr)
             break;
          }
          bstrncpy(cr->Name, ua->argv[i], sizeof(cr->Name));
-         if (!db_get_client_record(ua->jcr, ua->db, cr)) {
+         if (!ua->db->get_client_record(ua->jcr, cr)) {
             ua->error_msg(_("Could not find Client \"%s\": ERR=%s"), ua->argv[i],
-                     db_strerror(ua->db));
+                     ua->db->strerror());
             cr->ClientId = 0;
             break;
          }
@@ -524,7 +526,7 @@ bool get_client_dbr(UAContext *ua, CLIENT_DBR *cr)
    return true;
 }
 
-/*
+/**
  * Select a Client record from the catalog
  *
  * Returns true on success
@@ -538,8 +540,8 @@ bool select_client_dbr(UAContext *ua, CLIENT_DBR *cr)
    char name[MAX_NAME_LENGTH];
 
    cr->ClientId = 0;
-   if (!db_get_client_ids(ua->jcr, ua->db, &num_clients, &ids)) {
-      ua->error_msg(_("Error obtaining client ids. ERR=%s\n"), db_strerror(ua->db));
+   if (!ua->db->get_client_ids(ua->jcr, &num_clients, &ids)) {
+      ua->error_msg(_("Error obtaining client ids. ERR=%s\n"), ua->db->strerror());
       return false;
    }
 
@@ -551,7 +553,7 @@ bool select_client_dbr(UAContext *ua, CLIENT_DBR *cr)
    start_prompt(ua, _("Defined Clients:\n"));
    for (int i = 0; i < num_clients; i++) {
       ocr.ClientId = ids[i];
-      if (!db_get_client_record(ua->jcr, ua->db, &ocr) ||
+      if (!ua->db->get_client_record(ua->jcr, &ocr) ||
           !ua->acl_access_ok(Client_ACL, ocr.Name)) {
          continue;
       }
@@ -566,8 +568,8 @@ bool select_client_dbr(UAContext *ua, CLIENT_DBR *cr)
    memset(&ocr, 0, sizeof(ocr));
    bstrncpy(ocr.Name, name, sizeof(ocr.Name));
 
-   if (!db_get_client_record(ua->jcr, ua->db, &ocr)) {
-      ua->error_msg(_("Could not find Client \"%s\": ERR=%s"), name, db_strerror(ua->db));
+   if (!ua->db->get_client_record(ua->jcr, &ocr)) {
+      ua->error_msg(_("Could not find Client \"%s\": ERR=%s"), name, ua->db->strerror());
       return false;
    }
 
@@ -576,7 +578,7 @@ bool select_client_dbr(UAContext *ua, CLIENT_DBR *cr)
    return true;
 }
 
-/*
+/**
  * Scan what the user has entered looking for:
  *
  * argk=<storage-name>
@@ -591,11 +593,11 @@ bool select_client_dbr(UAContext *ua, CLIENT_DBR *cr)
 bool get_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
 {
    if (sr->Name[0]) {                 /* If name already supplied */
-      if (db_get_storage_record(ua->jcr, ua->db, sr) &&
+      if (ua->db->get_storage_record(ua->jcr, sr) &&
           ua->acl_access_ok(Pool_ACL, sr->Name)) {
          return true;
       }
-      ua->error_msg(_("Could not find Storage \"%s\": ERR=%s"), sr->Name, db_strerror(ua->db));
+      ua->error_msg(_("Could not find Storage \"%s\": ERR=%s"), sr->Name, ua->db->strerror());
    }
 
    if (!select_storage_dbr(ua, sr, argk)) {  /* try once more */
@@ -605,7 +607,7 @@ bool get_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
    return true;
 }
 
-/*
+/**
  * Scan what the user has entered looking for:
  *
  * argk=<pool-name>
@@ -620,11 +622,11 @@ bool get_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
 bool get_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
 {
    if (pr->Name[0]) {                 /* If name already supplied */
-      if (db_get_pool_record(ua->jcr, ua->db, pr) &&
+      if (ua->db->get_pool_record(ua->jcr, pr) &&
           ua->acl_access_ok(Pool_ACL, pr->Name)) {
          return true;
       }
-      ua->error_msg(_("Could not find Pool \"%s\": ERR=%s"), pr->Name, db_strerror(ua->db));
+      ua->error_msg(_("Could not find Pool \"%s\": ERR=%s"), pr->Name, ua->db->strerror());
    }
 
    if (!select_pool_dbr(ua, pr, argk)) {  /* try once more */
@@ -634,7 +636,7 @@ bool get_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
    return true;
 }
 
-/*
+/**
  * Select a Pool record from catalog
  * argk can be pool, recyclepool, scratchpool etc..
  */
@@ -649,8 +651,8 @@ bool select_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
       if (bstrcasecmp(ua->argk[i], argk) && ua->argv[i] &&
           ua->acl_access_ok(Pool_ACL, ua->argv[i])) {
          bstrncpy(pr->Name, ua->argv[i], sizeof(pr->Name));
-         if (!db_get_pool_record(ua->jcr, ua->db, pr)) {
-            ua->error_msg(_("Could not find Pool \"%s\": ERR=%s"), ua->argv[i], db_strerror(ua->db));
+         if (!ua->db->get_pool_record(ua->jcr, pr)) {
+            ua->error_msg(_("Could not find Pool \"%s\": ERR=%s"), ua->argv[i], ua->db->strerror());
             pr->PoolId = 0;
             break;
          }
@@ -659,8 +661,8 @@ bool select_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
    }
 
    pr->PoolId = 0;
-   if (!db_get_pool_ids(ua->jcr, ua->db, &num_pools, &ids)) {
-      ua->error_msg(_("Error obtaining pool ids. ERR=%s\n"), db_strerror(ua->db));
+   if (!ua->db->get_pool_ids(ua->jcr, &num_pools, &ids)) {
+      ua->error_msg(_("Error obtaining pool ids. ERR=%s\n"), ua->db->strerror());
       return 0;
    }
 
@@ -676,7 +678,7 @@ bool select_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
 
    for (int i = 0; i < num_pools; i++) {
       opr.PoolId = ids[i];
-      if (!db_get_pool_record(ua->jcr, ua->db, &opr) ||
+      if (!ua->db->get_pool_record(ua->jcr, &opr) ||
           !ua->acl_access_ok(Pool_ACL, opr.Name)) {
          continue;
       }
@@ -698,8 +700,8 @@ bool select_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
    if (!bstrcmp(name, _("*None*"))) {
      bstrncpy(opr.Name, name, sizeof(opr.Name));
 
-     if (!db_get_pool_record(ua->jcr, ua->db, &opr)) {
-        ua->error_msg(_("Could not find Pool \"%s\": ERR=%s"), name, db_strerror(ua->db));
+     if (!ua->db->get_pool_record(ua->jcr, &opr)) {
+        ua->error_msg(_("Could not find Pool \"%s\": ERR=%s"), name, ua->db->strerror());
         return false;
      }
    }
@@ -709,7 +711,7 @@ bool select_pool_dbr(UAContext *ua, POOL_DBR *pr, const char *argk)
    return true;
 }
 
-/*
+/**
  * Select a Pool and a Media (Volume) record from the database
  */
 bool select_pool_and_media_dbr(UAContext *ua, POOL_DBR *pr, MEDIA_DBR *mr)
@@ -720,8 +722,8 @@ bool select_pool_and_media_dbr(UAContext *ua, POOL_DBR *pr, MEDIA_DBR *mr)
 
    memset(pr, 0, sizeof(POOL_DBR));
    pr->PoolId = mr->PoolId;
-   if (!db_get_pool_record(ua->jcr, ua->db, pr)) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->get_pool_record(ua->jcr, pr)) {
+      ua->error_msg("%s", ua->db->strerror());
       return false;
    }
 
@@ -733,7 +735,7 @@ bool select_pool_and_media_dbr(UAContext *ua, POOL_DBR *pr, MEDIA_DBR *mr)
    return true;
 }
 
-/*
+/**
  * Select a Storage record from catalog
  * argk can be storage
  */
@@ -748,8 +750,8 @@ bool select_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
       if (bstrcasecmp(ua->argk[i], argk) && ua->argv[i] &&
           ua->acl_access_ok(Storage_ACL, ua->argv[i])) {
          bstrncpy(sr->Name, ua->argv[i], sizeof(sr->Name));
-         if (!db_get_storage_record(ua->jcr, ua->db, sr)) {
-            ua->error_msg(_("Could not find Storage \"%s\": ERR=%s"), ua->argv[i], db_strerror(ua->db));
+         if (!ua->db->get_storage_record(ua->jcr, sr)) {
+            ua->error_msg(_("Could not find Storage \"%s\": ERR=%s"), ua->argv[i], ua->db->strerror());
             sr->StorageId = 0;
             break;
          }
@@ -758,9 +760,9 @@ bool select_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
    }
 
    sr->StorageId = 0;
-   if (!db_get_storage_ids(ua->jcr, ua->db, &num_storages, &ids)) {
-      ua->error_msg(_("Error obtaining storage ids. ERR=%s\n"), db_strerror(ua->db));
-      return false;
+   if (!ua->db->get_storage_ids(ua->jcr, &num_storages, &ids)) {
+      ua->error_msg(_("Error obtaining storage ids. ERR=%s\n"), ua->db->strerror());
+      return 0;
    }
 
    if (num_storages <= 0) {
@@ -775,7 +777,7 @@ bool select_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
 
    for (int i = 0; i < num_storages; i++) {
       osr.StorageId = ids[i];
-      if (!db_get_storage_record(ua->jcr, ua->db, &osr) ||
+      if (!ua->db->get_storage_record(ua->jcr, &osr) ||
           !ua->acl_access_ok(Storage_ACL, osr.Name)) {
          continue;
       }
@@ -797,8 +799,8 @@ bool select_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
    if (!bstrcmp(name, _("*None*"))) {
      bstrncpy(osr.Name, name, sizeof(osr.Name));
 
-     if (!db_get_storage_record(ua->jcr, ua->db, &osr)) {
-        ua->error_msg(_("Could not find Storage \"%s\": ERR=%s"), name, db_strerror(ua->db));
+     if (!ua->db->get_storage_record(ua->jcr, &osr)) {
+        ua->error_msg(_("Could not find Storage \"%s\": ERR=%s"), name, ua->db->strerror());
         return false;
      }
    }
@@ -808,7 +810,7 @@ bool select_storage_dbr(UAContext *ua, STORAGE_DBR *sr, const char *argk)
    return true;
 }
 
-/*
+/**
  * Select a Media (Volume) record from the database
  */
 bool select_media_dbr(UAContext *ua, MEDIA_DBR *mr)
@@ -840,7 +842,7 @@ bool select_media_dbr(UAContext *ua, MEDIA_DBR *mr)
       }
 
       mr->PoolId = pr.PoolId;
-      db_list_media_records(ua->jcr, ua->db, mr, ua->send, HORZ_LIST);
+      ua->db->list_media_records(ua->jcr, mr, NULL, false, ua->send, HORZ_LIST);
 
       if (!get_cmd(ua, _("Enter *MediaId or Volume name: "))) {
          goto bail_out;
@@ -855,8 +857,8 @@ bool select_media_dbr(UAContext *ua, MEDIA_DBR *mr)
       }
    }
 
-   if (!db_get_media_record(ua->jcr, ua->db, mr)) {
-      pm_strcpy(err, db_strerror(ua->db));
+   if (!ua->db->get_media_record(ua->jcr, mr)) {
+      pm_strcpy(err, ua->db->strerror());
       goto bail_out;
    }
    retval = true;
@@ -870,7 +872,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Select a pool resource from prompt list
  */
 POOLRES *select_pool_resource(UAContext *ua)
@@ -896,7 +898,7 @@ POOLRES *select_pool_resource(UAContext *ua)
    return pool;
 }
 
-/*
+/**
  *  If you are thinking about using it, you
  *  probably want to use select_pool_dbr()
  *  or get_pool_dbr() above.
@@ -918,26 +920,26 @@ POOLRES *get_pool_resource(UAContext *ua)
    return select_pool_resource(ua);
 }
 
-/*
+/**
  * List all jobs and ask user to select one
  */
 int select_job_dbr(UAContext *ua, JOB_DBR *jr)
 {
-   db_list_job_records(ua->jcr, ua->db, jr, "", NULL, 0, NULL, 0, 0, 0, ua->send, HORZ_LIST);
+   ua->db->list_job_records(ua->jcr, jr, "", NULL, 0, 0, NULL, 0, 0, 0, ua->send, HORZ_LIST);
    if (!get_pint(ua, _("Enter the JobId to select: "))) {
       return 0;
    }
 
    jr->JobId = ua->int64_val;
-   if (!db_get_job_record(ua->jcr, ua->db, jr)) {
-      ua->error_msg("%s", db_strerror(ua->db));
+   if (!ua->db->get_job_record(ua->jcr, jr)) {
+      ua->error_msg("%s", ua->db->strerror());
       return 0;
    }
 
    return jr->JobId;
 }
 
-/*
+/**
  * Scan what the user has entered looking for:
  *
  * jobid=nn
@@ -962,9 +964,9 @@ int get_job_dbr(UAContext *ua, JOB_DBR *jr)
       } else {
          continue;
       }
-      if (!db_get_job_record(ua->jcr, ua->db, jr)) {
+      if (!ua->db->get_job_record(ua->jcr, jr)) {
          ua->error_msg(_("Could not find Job \"%s\": ERR=%s"), ua->argv[i],
-                  db_strerror(ua->db));
+                  ua->db->strerror());
          jr->JobId = 0;
          break;
       }
@@ -990,7 +992,7 @@ int get_job_dbr(UAContext *ua, JOB_DBR *jr)
    return jr->JobId;
 }
 
-/*
+/**
  * Implement unique set of prompts
  */
 void start_prompt(UAContext *ua, const char *msg)
@@ -1003,7 +1005,7 @@ void start_prompt(UAContext *ua, const char *msg)
   ua->prompt[0] = bstrdup(msg);
 }
 
-/*
+/**
  * Add to prompts -- keeping them unique
  */
 void add_prompt(UAContext *ua, const char *prompt)
@@ -1023,7 +1025,7 @@ void add_prompt(UAContext *ua, const char *prompt)
    ua->prompt[ua->num_prompts++] = bstrdup(prompt);
 }
 
-/*
+/**
  * Display prompts and get user's choice
  *
  * Returns: -1 on error
@@ -1044,7 +1046,7 @@ int do_prompt(UAContext *ua, const char *automsg, const char *msg, char *prompt,
       if (prompt) {
          bstrncpy(prompt, ua->prompt[1], max_prompt);
       }
-      if (!ua->api) {
+      if (!ua->api && !ua->runscript) {
          ua->send_msg(_("Automatically selected %s: %s\n"), automsg, ua->prompt[1]);
       }
       goto done;
@@ -1139,7 +1141,7 @@ done:
    return (item > 0) ? (item - 1) : item;
 }
 
-/*
+/**
  * We scan what the user has entered looking for :
  * - storage=<storage-resource>
  * - job=<job_name>
@@ -1266,7 +1268,7 @@ STORERES *get_storage_resource(UAContext *ua, bool use_default, bool autochanger
    return store;
 }
 
-/*
+/**
  * Get drive that we are working with for this storage
  */
 drive_number_t get_storage_drive(UAContext *ua, STORERES *store)
@@ -1316,7 +1318,7 @@ drive_number_t get_storage_drive(UAContext *ua, STORERES *store)
    return drive;
 }
 
-/*
+/**
  * Get slot that we are working with for this storage
  */
 slot_number_t get_storage_slot(UAContext *ua, STORERES *store)
@@ -1345,7 +1347,7 @@ slot_number_t get_storage_slot(UAContext *ua, STORERES *store)
    return slot;
 }
 
-/*
+/**
  * Scan looking for mediatype=
  *
  *  if not found or error, put up selection list
@@ -1395,7 +1397,7 @@ bool get_level_from_name(JCR *jcr, const char *level_name)
    return found;
 }
 
-/*
+/**
  * Insert an JobId into the list of selected JobIds when its a unique new id.
  */
 static inline bool insert_selected_jobid(alist *selected_jobids, JobId_t JobId)
@@ -1421,7 +1423,7 @@ static inline bool insert_selected_jobid(alist *selected_jobids, JobId_t JobId)
    return false;
 }
 
-/*
+/**
  * Get a job selection, "reason" is used in user messages and can be: cancel, limit, ...
  *
  * Returns: NULL on error
@@ -1704,7 +1706,7 @@ bail_out:
    return NULL;
 }
 
-/*
+/**
  * Get a slot selection.
  *
  * Returns: false on error
@@ -1885,3 +1887,18 @@ bool get_user_job_status_selection(UAContext *ua, int *jobstatus)
    }
    return true;
 }
+
+bool get_user_job_level_selection(UAContext *ua, int *joblevel)
+{
+   int i;
+
+   if ((i = find_arg_with_value(ua, NT_("joblevel"))) >= 0) {
+      if (strlen(ua->argv[i]) == 1 && ua->argv[i][0] >= 'A' && ua->argv[i][0] <= 'z') {
+         *joblevel = ua->argv[i][0];
+      } else {
+         /* invalid joblevel */
+         return false;
+      }
+   }
+   return true;
+}
