@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
+   Copyright (C) 2011-2016 Planets Communications B.V.
    Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Status Command
- *
  * Kern Sibbald, August MMI
  */
+/**
+ * @file
+ * User Agent Status Command
+ */
 
 #include "bareos.h"
 #include "dird.h"
@@ -66,7 +68,7 @@ static void client_status(UAContext *ua, CLIENTRES *client, char *cmd)
    }
 }
 
-/*
+/**
  * .status command
  */
 bool dot_status_cmd(UAContext *ua, const char *cmd)
@@ -139,7 +141,7 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/*
+/**
  * status command
  */
 bool status_cmd(UAContext *ua, const char *cmd)
@@ -251,6 +253,7 @@ static void do_all_status(UAContext *ua)
    CLIENTRES *client, **unique_client;
    int i, j;
    bool found;
+   int32_t previous_JobStatus = 0;
 
    do_director_status(ua);
 
@@ -282,9 +285,12 @@ static void do_all_status(UAContext *ua)
    }
    UnlockRes();
 
+   previous_JobStatus = ua->jcr->JobStatus;
+
    /* Call each unique Storage daemon */
    for (j = 0; j < i; j++) {
       storage_status(ua, unique_store[j], NULL);
+      ua->jcr->JobStatus = previous_JobStatus;
    }
    free(unique_store);
 
@@ -316,9 +322,12 @@ static void do_all_status(UAContext *ua)
    }
    UnlockRes();
 
+   previous_JobStatus = ua->jcr->JobStatus;
+
    /* Call each unique File daemon */
    for (j = 0; j < i; j++) {
       client_status(ua, unique_client[j], NULL);
+      ua->jcr->JobStatus = previous_JobStatus;
    }
    free(unique_client);
 
@@ -499,7 +508,7 @@ static bool show_scheduled_preview(UAContext *ua, SCHEDRES *sched,
    return true;
 }
 
-/*
+/**
  * Check the number of clients in the DB against the configured number of subscriptions
  *
  * Return true if (number of clients < number of subscriptions), else
@@ -846,7 +855,7 @@ static void prt_runtime(UAContext *ua, sched_pkt *sp)
    jcr->setJobType(orig_jobtype);
 }
 
-/*
+/**
  * Sort items by runtime, priority
  */
 static int compare_by_runtime_priority(void *item1, void *item2)
@@ -869,7 +878,7 @@ static int compare_by_runtime_priority(void *item1, void *item2)
    return 0;
 }
 
-/*
+/**
  * Find all jobs to be run in roughly the next 24 hours.
  */
 static void list_scheduled_jobs(UAContext *ua)
@@ -1302,9 +1311,9 @@ static void content_send_info_api(UAContext *ua, char type, int Slot, char *vol_
    memset(&mr, 0, sizeof(mr));
 
    bstrncpy(mr.VolumeName, vol_name, sizeof(mr.VolumeName));
-   if (db_get_media_record(ua->jcr, ua->db, &mr)) {
+   if (ua->db->get_media_record(ua->jcr, &mr)) {
       pr.PoolId = mr.PoolId;
-      if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+      if (!ua->db->get_pool_record(ua->jcr, &pr)) {
          strcpy(pr.Name, "?");
       }
       ua->send_msg(slot_api_full_format, type,
@@ -1329,9 +1338,9 @@ static void content_send_info_json(UAContext *ua, const char *type, int Slot, ch
    memset(&mr, 0, sizeof(mr));
 
    bstrncpy(mr.VolumeName, vol_name, sizeof(mr.VolumeName));
-   if (db_get_media_record(ua->jcr, ua->db, &mr)) {
+   if (ua->db->get_media_record(ua->jcr, &mr)) {
       pr.PoolId = mr.PoolId;
-      if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+      if (!ua->db->get_pool_record(ua->jcr, &pr)) {
          strcpy(pr.Name, "?");
       }
 
@@ -1365,7 +1374,7 @@ static void content_send_info_json(UAContext *ua, const char *type, int Slot, ch
    }
 }
 
-/*
+/**
  * Input (output of mxt-changer listall):
  *
  * Drive content:         D:Drive num:F:Slot loaded:Volume Name
@@ -1603,7 +1612,7 @@ bail_out:
    return;
 }
 
-/*
+/**
  * Print slots from AutoChanger
  */
 static void status_slots(UAContext *ua, STORERES *store)
@@ -1730,10 +1739,10 @@ static void status_slots(UAContext *ua, STORERES *store)
                bstrncpy(mr.VolumeName, vl2->VolName, sizeof(mr.VolumeName));
             }
 
-            if (mr.VolumeName[0] && db_get_media_record(ua->jcr, ua->db, &mr)) {
+            if (mr.VolumeName[0] && ua->db->get_media_record(ua->jcr, &mr)) {
                memset(&pr, 0, sizeof(pr));
                pr.PoolId = mr.PoolId;
-               if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+               if (!ua->db->get_pool_record(ua->jcr, &pr)) {
                   strcpy(pr.Name, "?");
                }
 
