@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2002-2011 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,11 +21,14 @@
    02110-1301, USA.
 */
 /*
- * BAREOS Director -- User Agent Database File tree for Restore
+ * Kern Sibbald, July MMII
+ */
+/**
+ * @file
+ * User Agent Database File tree for Restore
  *                    command. This file interacts with the user implementing the
  *                    UA tree commands.
  *
- * Kern Sibbald, July MMII
  */
 
 #include "bareos.h"
@@ -95,7 +98,7 @@ static struct cmdstruct commands[] = {
 };
 #define comsize ((int)(sizeof(commands)/sizeof(struct cmdstruct)))
 
-/*
+/**
  * Enter a prompt mode where the user can select/deselect
  * files to be restored. This is sort of like a mini-shell
  * that allows "cd", "pwd", "add", "rm", ...
@@ -192,7 +195,7 @@ bool user_select_files_from_tree(TREE_CTX *tree)
    return status;
 }
 
-/*
+/**
  * This callback routine is responsible for inserting the
  * items it gets into the directory tree. For each JobId selected
  * this routine is called once for each file. We do not allow
@@ -201,7 +204,7 @@ bool user_select_files_from_tree(TREE_CTX *tree)
  *
  * See uar_sel_files in sql_cmds.c for query that calls us.
  * row[0]=Path, row[1]=Filename, row[2]=FileIndex
- * row[3]=JobId row[4]=LStat row[5]=DeltaSeq
+ * row[3]=JobId row[4]=LStat row[5]=DeltaSeq row[6]=Fhinfo row[7]=Fhnode
  */
 int insert_tree_handler(void *ctx, int num_fields, char **row)
 {
@@ -233,8 +236,10 @@ int insert_tree_handler(void *ctx, int num_fields, char **row)
    JobId = str_to_int64(row[3]);
    FileIndex = str_to_int64(row[2]);
    delta_seq = str_to_int64(row[5]);
-   Dmsg6(150, "node=0x%p JobId=%s FileIndex=%s Delta=%s node.delta=%d LinkFI=%d\n",
-         node, row[3], row[2], row[5], node->delta_seq, LinkFI);
+   node->fhinfo = str_to_int64(row[6]);
+   node->fhnode = str_to_int64(row[7]);
+   Dmsg8(150, "node=0x%p JobId=%s FileIndex=%s Delta=%s node.delta=%d LinkFI=%d, fhinfo=%d, fhnode=%d\n",
+         node, row[3], row[2], row[5], node->delta_seq, LinkFI, node->fhinfo, node->fhnode);
 
    /*
     * TODO: check with hardlinks
@@ -250,7 +255,7 @@ int insert_tree_handler(void *ctx, int num_fields, char **row)
 
          } else {
             tree->ua->warning_msg(_("Something is wrong with the Delta sequence of %s, "
-                                    "skiping new parts. Current sequence is %d\n"),
+                                    "skipping new parts. Current sequence is %d\n"),
                                   row[1], node->delta_seq);
 
             Dmsg3(0, "Something is wrong with Delta, skip it "
@@ -341,7 +346,7 @@ int insert_tree_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-/*
+/**
  * Set extract to value passed. We recursively walk down the tree setting all children
  * if the node is a directory.
  */
@@ -408,7 +413,7 @@ static int set_extract(UAContext *ua, TREE_NODE *node, TREE_CTX *tree, bool extr
                fdbr.FileId = 0;
                fdbr.JobId = node->JobId;
 
-               if (node->hard_link && db_get_file_attributes_record(ua->jcr, ua->db, cwd, NULL, &fdbr)) {
+               if (node->hard_link && ua->db->get_file_attributes_record(ua->jcr, cwd, NULL, &fdbr)) {
                   int32_t LinkFI;
                   struct stat statp;
 
@@ -450,7 +455,7 @@ static void strip_trailing_slash(char *arg)
    }
 }
 
-/*
+/**
  * Recursively mark the current directory to be restored as
  *  well as all directories and files below it.
  */
@@ -707,7 +712,7 @@ static int lscmd(UAContext *ua, TREE_CTX *tree)
    return 1;
 }
 
-/*
+/**
  * Ls command that lists only the marked files
  */
 static int dot_lsmarkcmd(UAContext *ua, TREE_CTX *tree)
@@ -725,7 +730,7 @@ static int dot_lsmarkcmd(UAContext *ua, TREE_CTX *tree)
    return 1;
 }
 
-/*
+/**
  * This recursive ls command that lists only the marked files
  */
 static void rlsmark(UAContext *ua, TREE_NODE *tnode, int level)
@@ -772,7 +777,7 @@ static int lsmarkcmd(UAContext *ua, TREE_CTX *tree)
    return 1;
 }
 
-/*
+/**
  * This is actually the long form used for "dir"
  */
 static inline void ls_output(guid_list *guid, POOLMEM *&buf,
@@ -826,7 +831,7 @@ static inline void ls_output(guid_list *guid, POOLMEM *&buf,
    }
 }
 
-/*
+/**
  * Like ls command, but give more detail on each file
  */
 static int do_dircmd(UAContext *ua, TREE_CTX *tree, bool dot_cmd)
@@ -865,7 +870,7 @@ static int do_dircmd(UAContext *ua, TREE_CTX *tree, bool dot_cmd)
           * Strip / from soft links to directories.
           *
           * This is because soft links to files have a trailing slash
-          * when returned from tree_getpath, but db_get_file_attr...
+          * when returned from tree_getpath, but get_file_attr...
           * treats soft links as files, so they do not have a trailing
           * slash like directory names.
           */
@@ -880,7 +885,7 @@ static int do_dircmd(UAContext *ua, TREE_CTX *tree, bool dot_cmd)
             pcwd = cwd;
          }
 
-         if (db_get_file_attributes_record(ua->jcr, ua->db, pcwd, NULL, &fdbr)) {
+         if (ua->db->get_file_attributes_record(ua->jcr, pcwd, NULL, &fdbr)) {
             int32_t LinkFI;
             decode_stat(fdbr.LStat, &statp, sizeof(statp), &LinkFI); /* decode stat pkt */
          } else {
@@ -936,7 +941,7 @@ static int estimatecmd(UAContext *ua, TREE_CTX *tree)
             fdbr.FileId = 0;
             fdbr.JobId = node->JobId;
 
-            if (db_get_file_attributes_record(ua->jcr, ua->db, cwd, NULL, &fdbr)) {
+            if (ua->db->get_file_attributes_record(ua->jcr, cwd, NULL, &fdbr)) {
                int32_t LinkFI;
                decode_stat(fdbr.LStat, &statp, sizeof(statp), &LinkFI); /* decode stat pkt */
                if (S_ISREG(statp.st_mode) && statp.st_size > 0) {
@@ -975,7 +980,7 @@ static int helpcmd(UAContext *ua, TREE_CTX *tree)
    return 1;
 }
 
-/*
+/**
  * Change directories.  Note, if the user specifies x: and it fails,
  * we assume it is a Win32 absolute cd rather than relative and
  * try a second time with /x: ...  Win32 kludge.
