@@ -40,6 +40,13 @@
  * disadvantage that we need to keep a filenr to index the bitmap which
  * also cost some bytes.
  */
+
+#include "hostconfig.h"
+
+#ifdef HAVE_HPUX_OS
+#pragma pack(push,4)
+#endif
+
 struct accurate_payload {
    int64_t filenr;
    int32_t delta_seq;
@@ -47,6 +54,7 @@ struct accurate_payload {
    char *chksum;
 };
 
+
 /*
  * Accurate payload storage abstraction classes.
  */
@@ -89,6 +97,10 @@ struct CurFile {
    accurate_payload payload;
 };
 
+#ifdef HAVE_HPUX_OS
+#pragma pack(pop)
+#endif
+
 class B_ACCURATE_HTABLE: public B_ACCURATE {
 protected:
    htable *m_file_list;
