@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * Create a file, and reset the modes
- *
  * Kern Sibbald, November MM
  */
+/**
+ * @file
+ * Create a file, and reset the modes
+ */
 
 #include "bareos.h"
 #include "jcr.h"
@@ -42,7 +44,7 @@
 static int separate_path_and_file(JCR *jcr, char *fname, char *ofile);
 static int path_already_seen(JCR *jcr, char *path, int pnl);
 
-/*
+/**
  * Create the file, or the directory
  *
  * fname is the original filename
@@ -81,7 +83,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
    }
 
    new_mode = attr->statp.st_mode;
-   Dmsg3(200, "type=%d newmode=%x file=%s\n", attr->type, new_mode, attr->ofname);
+   Dmsg3(200, "type=%d newmode=%04o file=%s\n", attr->type, (new_mode & ~S_IFMT), attr->ofname);
    parent_mode = S_IWUSR | S_IXUSR | new_mode;
    gid = attr->statp.st_gid;
    uid = attr->statp.st_uid;
@@ -298,7 +300,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
             if (is_bopen(bfd)) {
                Qmsg1(jcr, M_ERROR, 0, _("bpkt already open fid=%d\n"), bfd->fid);
             }
-            Dmsg2(400, "open %s flags=0x%x\n", attr->ofname, flags);
+            Dmsg2(400, "open %s flags=%08o\n", attr->ofname, flags);
             if ((bopen(bfd, attr->ofname, flags, 0, 0)) < 0) {
                berrno be;
                be.set_errno(bfd->berrno);
@@ -414,7 +416,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
        */
    case FT_DIRBEGIN:
    case FT_DIREND:
-      Dmsg2(200, "Make dir mode=%o dir=%s\n", new_mode, attr->ofname);
+      Dmsg2(200, "Make dir mode=%04o dir=%s\n", (new_mode & ~S_IFMT), attr->ofname);
       if (!makepath(attr, attr->ofname, new_mode, parent_mode, uid, gid, 0)) {
          return CF_ERROR;
       }
@@ -472,7 +474,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
    return CF_ERROR;
 }
 
-/*
+/**
  *  Returns: > 0 index into path where last path char is.
  *           0  no path
  *           -1 filename is zero length
@@ -517,7 +519,7 @@ static int separate_path_and_file(JCR *jcr, char *fname, char *ofile)
    return pnl;
 }
 
-/*
+/**
  * Primitive caching of path to prevent recreating a pathname
  *   each time as long as we remain in the same directory.
  */
