@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,7 +20,8 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 */
-/*
+/**
+ * @file
  * Prototypes for lib directory of BAREOS
  */
 
@@ -34,6 +35,7 @@ ATTR *new_attr(JCR *jcr);
 void free_attr(ATTR *attr);
 int unpack_attributes_record(JCR *jcr, int32_t stream, char *rec, int32_t reclen, ATTR *attr);
 void build_attr_output_fnames(JCR *jcr, ATTR *attr);
+const char *attr_to_str(POOL_MEM &resultbuffer, JCR *jcr, ATTR *attr);
 void print_ls_output(JCR *jcr, ATTR *attr);
 
 /* attribs.c */
@@ -138,6 +140,8 @@ bool path_exists(const char *path);
 bool path_exists(POOL_MEM &path);
 bool path_is_directory(const char *path);
 bool path_is_directory(POOL_MEM &path);
+bool path_contains_directory(const char *path);
+bool path_contains_directory(POOL_MEM &path);
 bool path_is_absolute(const char *path);
 bool path_is_absolute(POOL_MEM &path);
 bool path_get_directory(POOL_MEM &directory, POOL_MEM &path);
@@ -233,6 +237,7 @@ bool duration_to_utime(char *str, utime_t *value);
 bool size_to_uint64(char *str, uint64_t *value);
 bool speed_to_uint64(char *str, uint64_t *value);
 char *edit_utime(utime_t val, char *buf, int buf_len);
+char *edit_pthread(pthread_t val, char *buf, int buf_len);
 bool is_a_number(const char *num);
 bool is_a_number_list(const char *n);
 bool is_an_integer(const char *n);
@@ -247,7 +252,7 @@ void unlock_last_jobs_list();
 bool read_last_jobs_list(int fd, uint64_t addr);
 uint64_t write_last_jobs_list(int fd, uint64_t addr);
 void write_state_file(char *dir, const char *progname, int port);
-void job_end_push(JCR *jcr, void job_end_cb(JCR *jcr,void *), void *ctx);
+void register_job_end_callback(JCR *jcr, void job_end_cb(JCR *jcr,void *), void *ctx);
 void lock_jobs();
 void unlock_jobs();
 JCR *jcr_walk_start();
@@ -408,6 +413,7 @@ void bash_spaces(char *str);
 void bash_spaces(POOL_MEM &pm);
 void unbash_spaces(char *str);
 void unbash_spaces(POOL_MEM &pm);
+const char* indent_multiline_string(POOL_MEM &resultbuffer, const char *multilinestring, const char *separator);
 char *encode_time(utime_t time, char *buf);
 bool convert_timeout_to_timespec(timespec &timeout, int timeout_in_seconds);
 char *encode_mode(mode_t mode, char *buf);
