@@ -55,7 +55,7 @@ char *help_text[] = {
 #endif /* !NDMOS_OPTION_NO_CONTROL_AGENT */
 #ifndef NDMOS_EFFECT_NO_SERVER_AGENTS
 	"  -o daemon      -- launch session for incomming connections",
-	"  -o tape-size=SIZE -- specify the length, in bytes of the simulated tape",
+	"  -o tape-limit=SIZE -- specify the length, in bytes of the simulated tape",
 #endif /* !NDMOS_EFFECT_NO_SERVER_AGENTS */
 #ifndef NDMOS_OPTION_NO_CONTROL_AGENT
 	"  -o rewind      -- rewind tape in drive, need -T and -f",
