@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2007-2012 Free Software Foundation Europe e.V.
-   Copyright (C) 2014-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2014-2018 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,10 +20,12 @@
    02110-1301, USA.
 */
 /*
- * A simple pipe plugin for the Bareos File Daemon
- *
  * Kern Sibbald, October 2007
  */
+/**
+ * @file
+ * A simple pipe plugin for the Bareos File Daemon
+ */
 #include "bareos.h"
 #include "fd_plugins.h"
 #include "fd_common.h"
@@ -105,7 +107,7 @@ static pFuncs pluginFuncs = {
    setXattr
 };
 
-/*
+/**
  * Plugin private context
  */
 struct plugin_ctx {
@@ -120,7 +122,7 @@ struct plugin_ctx {
    int replace;
 };
 
-/*
+/**
  * This defines the arguments that the plugin parser understands.
  */
 enum plugin_argument_type {
@@ -147,12 +149,12 @@ static plugin_argument plugin_arguments[] = {
 extern "C" {
 #endif
 
-/*
+/**
  * loadPlugin() and unloadPlugin() are entry points that are
  *  exported, so Bareos can directly call these two entry points
  *  they are common to all Bareos plugins.
  */
-/*
+/**
  * External entry point called by Bareos to "load" the plugin
  */
 bRC DLL_IMP_EXP loadPlugin(bInfo *lbinfo,
@@ -168,7 +170,7 @@ bRC DLL_IMP_EXP loadPlugin(bInfo *lbinfo,
    return bRC_OK;
 }
 
-/*
+/**
  * External entry point to unload the plugin
  */
 bRC DLL_IMP_EXP unloadPlugin()
@@ -180,12 +182,12 @@ bRC DLL_IMP_EXP unloadPlugin()
 }
 #endif
 
-/*
+/**
  * The following entry points are accessed through the function
  *   pointers we supplied to Bareos. Each plugin type (dir, fd, sd)
  *   has its own set of entry points that the plugin must define.
  */
-/*
+/**
  * Create a new instance of the plugin i.e. allocate our private storage
  */
 static bRC newPlugin(bpContext *ctx)
@@ -209,7 +211,7 @@ static bRC newPlugin(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Free a plugin instance, i.e. release our private storage
  */
 static bRC freePlugin(bpContext *ctx)
@@ -241,7 +243,7 @@ static bRC freePlugin(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Return some plugin value (none defined)
  */
 static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
@@ -249,7 +251,7 @@ static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Set a plugin value (none defined)
  */
 static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
@@ -257,7 +259,7 @@ static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Handle an event that was generated in Bareos
  */
 static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
@@ -314,7 +316,7 @@ static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
    return retval;
 }
 
-/*
+/**
  * Start the backup of a specific file
  */
 static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
@@ -345,7 +347,7 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
    return bRC_OK;
 }
 
-/*
+/**
  * Done with backup of this file
  */
 static bRC endBackupFile(bpContext *ctx)
@@ -356,7 +358,7 @@ static bRC endBackupFile(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Bareos is calling us to do the actual I/O
  */
 static bRC pluginIO(bpContext *ctx, struct io_pkt *io)
@@ -448,7 +450,7 @@ static bRC pluginIO(bpContext *ctx, struct io_pkt *io)
    return bRC_OK;
 }
 
-/*
+/**
  * Bareos is notifying us that a plugin name string was found, and
  *   passing us the plugin command, so we can prepare for a restore.
  */
@@ -461,7 +463,7 @@ static bRC startRestoreFile(bpContext *ctx, const char *cmd)
    return bRC_OK;
 }
 
-/*
+/**
  * Bareos is notifying us that the plugin data has terminated, so
  *  the restore for this particular file is done.
  */
@@ -475,7 +477,7 @@ static bRC endRestoreFile(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * This is called during restore to create the file (if necessary)
  * We must return in rp->create_status:
  *
@@ -502,7 +504,7 @@ static bRC setFileAttributes(bpContext *ctx, struct restore_pkt *rp)
    return bRC_OK;
 }
 
-/*
+/**
  * When using Incremental dump, all previous dumps are necessary
  */
 static bRC checkFile(bpContext *ctx, char *fname)
@@ -530,7 +532,7 @@ static bRC setXattr(bpContext *ctx, xattr_pkt *xp)
    return bRC_OK;
 }
 
-/*
+/**
  * Apply codes in writer command:
  * %w -> "where"
  * %r -> "replace"
@@ -619,7 +621,7 @@ static char *apply_rp_codes(bpContext *ctx)
    return omsg;
 }
 
-/*
+/**
  * Strip any backslashes in the string.
  */
 static inline void strip_back_slashes(char *value)
@@ -640,7 +642,7 @@ static inline void strip_back_slashes(char *value)
    }
 }
 
-/*
+/**
  * Only set destination to value when it has no previous setting.
  */
 static inline void set_string_if_null(char **destination, char *value)
@@ -651,7 +653,7 @@ static inline void set_string_if_null(char **destination, char *value)
    }
 }
 
-/*
+/**
  * Always set destination to value and clean any previous one.
  */
 static inline void set_string(char **destination, char *value)
@@ -664,7 +666,7 @@ static inline void set_string(char **destination, char *value)
    strip_back_slashes(*destination);
 }
 
-/*
+/**
  * Parse the plugin definition passed in.
  *
  * The definition is in this form:
@@ -723,8 +725,8 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
          /*
           * Parsing something fishy ? e.g. partly with known keywords.
           */
-         Jmsg(ctx, M_FATAL, "bpipe-fd: Found mixing of old and new syntax, please fix your plugin definition\n", plugin_definition);
-         Dmsg(ctx, dbglvl, "bpipe-fd: Found mixing of old and new syntax, please fix your plugin definition\n", plugin_definition);
+         Jmsg(ctx, M_FATAL, "bpipe-fd: Found mixing of old and new syntax, please fix your plugin definition (%s)\n", plugin_definition);
+         Dmsg(ctx, dbglvl, "bpipe-fd: Found mixing of old and new syntax, please fix your plugin definition (%s)\n", plugin_definition);
          goto bail_out;
       }
 
@@ -826,6 +828,15 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
 
                switch (plugin_arguments[i].type) {
                case argument_file:
+                  if (!path_contains_directory(argument_value)){
+                     Jmsg(ctx, M_FATAL, 
+                           "bpipe-fd: file argument (%s) must contain a directory structure. Please fix your plugin definition\n",
+                           argument_value);
+                     Dmsg(ctx, dbglvl,
+                           "bpipe-fd: file argument (%s) must contain a directory structure. Please fix your plugin definition\n",
+                           argument_value);
+                     goto bail_out;
+                  }
                   str_destination = &p_ctx->fname;
                   break;
                case argument_reader:
