@@ -19,7 +19,8 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 */
-/*
+/**
+ * @file
  * CEPHFS plugin for the Bareos File Daemon
  */
 #include "bareos.h"
@@ -41,7 +42,7 @@ static const int dbglvl = 150;
 
 #define CEPHFS_PATH_MAX 4096
 
-/*
+/**
  * Forward referenced functions
  */
 static bRC newPlugin(bpContext *ctx);
@@ -67,13 +68,13 @@ static bRC setup_backup(bpContext *ctx, void *value);
 static bRC setup_restore(bpContext *ctx, void *value);
 static bRC end_restore_job(bpContext *ctx, void *value);
 
-/*
+/**
  * Pointers to Bareos functions
  */
 static bFuncs *bfuncs = NULL;
 static bInfo  *binfo = NULL;
 
-/*
+/**
  * Plugin Information block
  */
 static genpInfo pluginInfo = {
@@ -88,7 +89,7 @@ static genpInfo pluginInfo = {
    PLUGIN_USAGE
 };
 
-/*
+/**
  * Plugin entry points for Bareos
  */
 static pFuncs pluginFuncs = {
@@ -115,18 +116,23 @@ static pFuncs pluginFuncs = {
    setXattr
 };
 
-/*
+/**
  * Plugin private context
  */
 struct plugin_ctx {
    int32_t backup_level;              /* Backup level e.g. Full/Differential/Incremental */
    utime_t since;                     /* Since time for Differential/Incremental */
    char *plugin_options;              /* Options passed to plugin */
+   char *plugin_definition;           /* Previous plugin definition passed to plugin */
    char *conffile;                    /* Configfile to read to be able to connect to CEPHFS */
    char *basedir;                     /* Basedir to start backup in */
    char flags[FOPTS_BYTES];           /* Bareos internal flags */
    int32_t type;                      /* FT_xx for this file */
+#if HAVE_CEPHFS_CEPH_STATX_H
+   struct ceph_statx statx;           /* Stat struct for next file to save */
+#else
    struct stat statp;                 /* Stat struct for next file to save */
+#endif
    bool processing_xattr;             /* Set when we are processing a xattr list */
    char *next_xattr_name;             /* Next xattr name to process */
    POOLMEM *cwd;                      /* Current Working Directory */
@@ -141,7 +147,7 @@ struct plugin_ctx {
    int cfd;                           /* CEPHFS file handle */
 };
 
-/*
+/**
  * This defines the arguments that the plugin parser understands.
  */
 enum plugin_argument_type {
@@ -163,12 +169,16 @@ static plugin_argument plugin_arguments[] = {
    { NULL, argument_none }
 };
 
-/*
+/**
  * If we recurse into a subdir we push the current directory onto
  * a stack so we can pop it after we have processed the subdir.
  */
 struct dir_stack_entry {
+#if HAVE_CEPHFS_CEPH_STATX_H
+   struct ceph_statx statx;           /* Stat struct of directory */
+#else
    struct stat statp;                 /* Stat struct of directory */
+#endif
    struct ceph_dir_result *cdir;      /* CEPHFS directory handle */
 };
 
@@ -176,7 +186,7 @@ struct dir_stack_entry {
 extern "C" {
 #endif
 
-/*
+/**
  * loadPlugin() and unloadPlugin() are entry points that are exported, so Bareos can
  * directly call these two entry points they are common to all Bareos plugins.
  *
@@ -195,7 +205,7 @@ bRC DLL_IMP_EXP loadPlugin(bInfo *lbinfo,
    return bRC_OK;
 }
 
-/*
+/**
  * External entry point to unload the plugin
  */
 bRC DLL_IMP_EXP unloadPlugin()
@@ -207,7 +217,7 @@ bRC DLL_IMP_EXP unloadPlugin()
 }
 #endif
 
-/*
+/**
  * The following entry points are accessed through the function pointers we supplied to Bareos.
  * Each plugin type (dir, fd, sd) has its own set of entry points that the plugin must define.
  *
@@ -267,7 +277,7 @@ static bRC newPlugin(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Free a plugin instance, i.e. release our private storage
  */
 static bRC freePlugin(bpContext *ctx)
@@ -307,6 +317,10 @@ static bRC freePlugin(bpContext *ctx)
       free(p_ctx->conffile);
    }
 
+   if (p_ctx->plugin_definition) {
+      free(p_ctx->plugin_definition);
+   }
+
    if (p_ctx->plugin_options) {
       free(p_ctx->plugin_options);
    }
@@ -319,7 +333,7 @@ static bRC freePlugin(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Return some plugin value (none defined)
  */
 static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
@@ -327,7 +341,7 @@ static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Set a plugin value (none defined)
  */
 static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
@@ -335,7 +349,7 @@ static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Handle an event that was generated in Bareos
  */
 static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
@@ -400,7 +414,7 @@ static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
    return retval;
 }
 
-/*
+/**
  * Get the next file to backup.
  */
 static bRC get_next_file_to_backup(bpContext *ctx)
@@ -443,7 +457,11 @@ static bRC get_next_file_to_backup(bpContext *ctx)
           * Pop the previous directory handle and continue processing that.
           */
          entry = (struct dir_stack_entry *)p_ctx->dir_stack->pop();
+#if HAVE_CEPHFS_CEPH_STATX_H
+         memcpy(&p_ctx->statx, &entry->statx, sizeof(p_ctx->statx));
+#else
          memcpy(&p_ctx->statp, &entry->statp, sizeof(p_ctx->statp));
+#endif
          p_ctx->cdir = entry->cdir;
          free(entry);
       } else {
@@ -462,17 +480,28 @@ static bRC get_next_file_to_backup(bpContext *ctx)
     * Loop until we know what file is next or when we are done.
     */
    while (1) {
-      int stmask = 0;
 
+#if HAVE_CEPHFS_CEPH_STATX_H
+      unsigned int stmask = 0;
+      memset(&p_ctx->statx, 0, sizeof(p_ctx->statx));
+      status = ceph_readdirplus_r(p_ctx->cmount, p_ctx->cdir, &p_ctx->de, &p_ctx->statx, stmask, CEPH_STATX_ALL_STATS, NULL);
+#else
+      int stmask = 0;
       memset(&p_ctx->statp, 0, sizeof(p_ctx->statp));
-      memset(&p_ctx->de, 0, sizeof(p_ctx->de));
       status = ceph_readdirplus_r(p_ctx->cmount, p_ctx->cdir, &p_ctx->de, &p_ctx->statp, &stmask);
+#endif
+
+      memset(&p_ctx->de, 0, sizeof(p_ctx->de));
 
       /*
        * No more entries in this directory ?
        */
       if (status == 0) {
+#if HAVE_CEPHFS_CEPH_STATX_H
+         status = ceph_statx(p_ctx->cmount, p_ctx->cwd, &p_ctx->statx, CEPH_STATX_MODE, 0);
+#else
          status = ceph_stat(p_ctx->cmount, p_ctx->cwd, &p_ctx->statp);
+#endif
          if (status < 0) {
             berrno be;
 
@@ -514,7 +543,11 @@ static bRC get_next_file_to_backup(bpContext *ctx)
       /*
        * Determine the FileType.
        */
+#if HAVE_CEPHFS_CEPH_STATX_H
+      switch (p_ctx->statx.stx_mode & S_IFMT) {
+#else
       switch (p_ctx->statp.st_mode & S_IFMT) {
+#endif
       case S_IFREG:
          p_ctx->type = FT_REG;
          break;
@@ -542,8 +575,13 @@ static bRC get_next_file_to_backup(bpContext *ctx)
          p_ctx->type = FT_SPEC;
          break;
       default:
+#if HAVE_CEPHFS_CEPH_STATX_H
+         Jmsg(ctx, M_FATAL, "cephfs-fd: Unknown filetype encountered %ld for %s\n",
+              p_ctx->statx.stx_mode & S_IFMT, p_ctx->next_filename);
+#else
          Jmsg(ctx, M_FATAL, "cephfs-fd: Unknown filetype encountered %ld for %s\n",
               p_ctx->statp.st_mode & S_IFMT, p_ctx->next_filename);
+#endif
          return bRC_Error;
       }
 
@@ -555,8 +593,11 @@ static bRC get_next_file_to_backup(bpContext *ctx)
       sp.pkt_end = sizeof(sp);
       sp.fname = p_ctx->next_filename;
       sp.type = p_ctx->type;
+#if HAVE_CEPHFS_CEPH_STATX_H
+      memcpy(&sp.statp, &p_ctx->statx, sizeof(sp.statp));
+#else
       memcpy(&sp.statp, &p_ctx->statp, sizeof(sp.statp));
-
+#endif
       if (bfuncs->AcceptFile(ctx, &sp) == bRC_Skip) {
          Dmsg(ctx, dbglvl, "cephfs-fd: file %s skipped due to current fileset settings\n", p_ctx->next_filename);
          continue;
@@ -573,7 +614,7 @@ static bRC get_next_file_to_backup(bpContext *ctx)
    return bRC_More;
 }
 
-/*
+/**
  * Start the backup of a specific file
  */
 static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
@@ -616,7 +657,11 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
                struct dir_stack_entry *new_entry;
 
                new_entry = (struct dir_stack_entry *)malloc(sizeof(struct dir_stack_entry));
+#if HAVE_CEPHFS_CEPH_STATX_H
+               memcpy(&new_entry->statx, &p_ctx->statx, sizeof(new_entry->statx));
+#else
                memcpy(&new_entry->statp, &p_ctx->statp, sizeof(new_entry->statp));
+#endif
                new_entry->cdir = p_ctx->cdir;
                p_ctx->dir_stack->push(new_entry);
             }
@@ -639,7 +684,11 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
                   struct dir_stack_entry *entry;
 
                   entry = (struct dir_stack_entry *)p_ctx->dir_stack->pop();
+#if HAVE_CEPHFS_CEPH_STATX_H
+                  memcpy(&p_ctx->statx, &entry->statx, sizeof(p_ctx->statx));
+#else
                   memcpy(&p_ctx->statp, &entry->statp, sizeof(p_ctx->statp));
+#endif
                   p_ctx->cdir = entry->cdir;
                   free(entry);
 
@@ -705,7 +754,11 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
 
    sp->fname = p_ctx->next_filename;
    sp->type = p_ctx->type;
+#if HAVE_CEPHFS_CEPH_STATX_H
+   memcpy(&sp->statp, &p_ctx->statx, sizeof(sp->statp));
+#else
    memcpy(&sp->statp, &p_ctx->statp, sizeof(sp->statp));
+#endif
    sp->save_time = p_ctx->since;
 
    /*
@@ -736,7 +789,7 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
    return bRC_OK;
 }
 
-/*
+/**
  * Done with backup of this file
  */
 static bRC endBackupFile(bpContext *ctx)
@@ -753,8 +806,13 @@ static bRC endBackupFile(bpContext *ctx)
    if (bit_is_set(FO_NOATIME, p_ctx->flags)) {
       struct utimbuf times;
 
+#if HAVE_CEPHFS_CEPH_STATX_H
+      times.actime = p_ctx->statx.stx_atime.tv_sec;
+      times.modtime = p_ctx->statx.stx_mtime.tv_sec;
+#else
       times.actime = p_ctx->statp.st_atime;
       times.modtime = p_ctx->statp.st_mtime;
+#endif
 
       ceph_utime(p_ctx->cmount, p_ctx->next_filename, &times);
    }
@@ -762,7 +820,7 @@ static bRC endBackupFile(bpContext *ctx)
    return get_next_file_to_backup(ctx);
 }
 
-/*
+/**
  * Strip any backslashes in the string.
  */
 static inline void strip_back_slashes(char *value)
@@ -783,7 +841,7 @@ static inline void strip_back_slashes(char *value)
    }
 }
 
-/*
+/**
  * Only set destination to value when it has no previous setting.
  */
 static inline void set_string_if_null(char **destination, char *value)
@@ -794,7 +852,7 @@ static inline void set_string_if_null(char **destination, char *value)
    }
 }
 
-/*
+/**
  * Always set destination to value and clean any previous one.
  */
 static inline void set_string(char **destination, char *value)
@@ -807,7 +865,7 @@ static inline void set_string(char **destination, char *value)
    strip_back_slashes(*destination);
 }
 
-/*
+/**
  * Parse the plugin definition passed in.
  *
  * The definition is in this form:
@@ -825,6 +883,22 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
       return bRC_Error;
    }
 
+   /*
+    * See if we already got some plugin definition before and its exactly the same.
+    */
+   if (p_ctx->plugin_definition) {
+      if (bstrcmp(p_ctx->plugin_definition, (char *)value)) {
+         return bRC_OK;
+      }
+
+      free(p_ctx->plugin_definition);
+   }
+
+   /*
+    * Keep track of the last processed plugin definition.
+    */
+   p_ctx->plugin_definition = bstrdup((char *)value);
+
    keep_existing = (p_ctx->plugin_options) ? true : false;
 
    /*
@@ -932,7 +1006,7 @@ bail_out:
    return bRC_Error;
 }
 
-/*
+/**
  * Open a CEPHFS mountpoint.
  */
 static bRC connect_to_cephfs(bpContext *ctx)
@@ -940,6 +1014,14 @@ static bRC connect_to_cephfs(bpContext *ctx)
    int status;
    plugin_ctx *p_ctx = (plugin_ctx *)ctx->pContext;
 
+   /*
+    * If we get called and we already have a handle to cephfs we should tear it down.
+    */
+   if (p_ctx->cmount) {
+      ceph_shutdown(p_ctx->cmount);
+      p_ctx->cmount = NULL;
+   }
+
    status = ceph_create(&p_ctx->cmount, NULL);
    if (status < 0) {
       berrno be;
@@ -971,7 +1053,7 @@ bail_out:
    return bRC_Error;
 }
 
-/*
+/**
  * Generic setup for performing a backup.
  */
 static bRC setup_backup(bpContext *ctx, void *value)
@@ -982,6 +1064,16 @@ static bRC setup_backup(bpContext *ctx, void *value)
       return bRC_Error;
    }
 
+   /*
+    * If we are already having a handle to cepfs and we are getting the
+    * same plugin definition there is no need to tear down the whole stuff and
+    * setup exactly the same.
+    */
+   if (p_ctx->cmount &&
+       bstrcmp((char *)value, p_ctx->plugin_definition)) {
+      return bRC_OK;
+   }
+
    if (connect_to_cephfs(ctx) != bRC_OK) {
       return bRC_Error;
    }
@@ -1001,7 +1093,7 @@ static bRC setup_backup(bpContext *ctx, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Generic setup for performing a restore.
  */
 static bRC setup_restore(bpContext *ctx, void *value)
@@ -1012,14 +1104,20 @@ static bRC setup_restore(bpContext *ctx, void *value)
       return bRC_Error;
    }
 
-   if (connect_to_cephfs(ctx) != bRC_OK) {
-      return bRC_Error;
+   /*
+    * If we are already having a handle to cepfs and we are getting the
+    * same plugin definition there is no need to tear down the whole stuff and
+    * setup exactly the same.
+    */
+   if (p_ctx->cmount &&
+       bstrcmp((char *)value, p_ctx->plugin_definition)) {
+      return bRC_OK;
    }
 
-   return bRC_OK;
+   return connect_to_cephfs(ctx);
 }
 
-/*
+/**
  * Bareos is calling us to do the actual I/O
  */
 static bRC pluginIO(bpContext *ctx, struct io_pkt *io)
@@ -1108,7 +1206,7 @@ bail_out:
    return bRC_Error;
 }
 
-/*
+/**
  * See if we need to do any postprocessing after the restore.
  */
 static bRC end_restore_job(bpContext *ctx, void *value)
@@ -1127,7 +1225,7 @@ static bRC end_restore_job(bpContext *ctx, void *value)
    return retval;
 }
 
-/*
+/**
  * Bareos is notifying us that a plugin name string was found,
  * and passing us the plugin command, so we can prepare for a restore.
  */
@@ -1136,7 +1234,7 @@ static bRC startRestoreFile(bpContext *ctx, const char *cmd)
    return bRC_OK;
 }
 
-/*
+/**
  * Bareos is notifying us that the plugin data has terminated,
  * so the restore for this particular file is done.
  */
@@ -1145,7 +1243,7 @@ static bRC endRestoreFile(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Create a parent directory using the cephfs API.
  *
  * We don't use cephfs_mkdirs as we want to keep track which
@@ -1154,7 +1252,11 @@ static bRC endRestoreFile(bpContext *ctx)
 static inline bool cephfs_makedirs(plugin_ctx *p_ctx, const char *directory)
 {
    char *bp;
+#if HAVE_CEPHFS_CEPH_STATX_H
+   struct ceph_statx stx;
+#else
    struct stat st;
+#endif
    bool retval = false;
    POOL_MEM new_directory(PM_FNAME);
 
@@ -1182,7 +1284,11 @@ static inline bool cephfs_makedirs(plugin_ctx *p_ctx, const char *directory)
       } else {
          *bp = '\0';
 
+#if HAVE_CEPHFS_CEPH_STATX_H
+         if (ceph_statx(p_ctx->cmount, new_directory.c_str(), &stx, CEPH_STATX_SIZE, AT_SYMLINK_NOFOLLOW) != 0) {
+#else
          if (ceph_lstat(p_ctx->cmount, new_directory.c_str(), &st) != 0) {
+#endif
             switch (errno) {
             case ENOENT:
                /*
@@ -1216,7 +1322,7 @@ static inline bool cephfs_makedirs(plugin_ctx *p_ctx, const char *directory)
    return retval;
 }
 
-/*
+/**
  * This is called during restore to create the file (if necessary) We must return in rp->create_status:
  *
  *  CF_ERROR    -- error
@@ -1228,7 +1334,11 @@ static bRC createFile(bpContext *ctx, struct restore_pkt *rp)
 {
    int status;
    bool exists = false;
+#if HAVE_CEPHFS_CEPH_STATX_H
+   struct ceph_statx stx;
+#else
    struct stat st;
+#endif
    plugin_ctx *p_ctx = (plugin_ctx *)ctx->pContext;
 
    if (!p_ctx) {
@@ -1239,20 +1349,32 @@ static bRC createFile(bpContext *ctx, struct restore_pkt *rp)
     * See if the file already exists.
     */
    Dmsg(ctx, 400, "cephfs-fd: Replace=%c %d\n", (char)rp->replace, rp->replace);
+#if HAVE_CEPHFS_CEPH_STATX_H
+   status = ceph_statx(p_ctx->cmount, rp->ofname, &stx, CEPH_STATX_SIZE, AT_SYMLINK_NOFOLLOW);
+#else
    status = ceph_lstat(p_ctx->cmount, rp->ofname, &st);
+#endif
    if (status == 0) {
       exists = true;
 
       switch (rp->replace) {
       case REPLACE_IFNEWER:
+#if HAVE_CEPHFS_CEPH_STATX_H
+         if (rp->statp.st_mtime <= stx.stx_mtime.tv_sec) {
+#else
          if (rp->statp.st_mtime <= st.st_mtime) {
+#endif
             Jmsg(ctx, M_INFO, 0, _("cephfs-fd: File skipped. Not newer: %s\n"), rp->ofname);
             rp->create_status = CF_SKIP;
             goto bail_out;
          }
          break;
       case REPLACE_IFOLDER:
+#if HAVE_CEPHFS_CEPH_STATX_H
+         if (rp->statp.st_mtime >= stx.stx_mtime.tv_sec) {
+#else
          if (rp->statp.st_mtime >= st.st_mtime) {
+#endif
             Jmsg(ctx, M_INFO, 0, _("cephfs-fd: File skipped. Not older: %s\n"), rp->ofname);
             rp->create_status = CF_SKIP;
             goto bail_out;
@@ -1428,7 +1550,7 @@ static bRC setFileAttributes(bpContext *ctx, struct restore_pkt *rp)
    return bRC_OK;
 }
 
-/*
+/**
  * When using Incremental dump, all previous dumps are necessary
  */
 static bRC checkFile(bpContext *ctx, char *fname)
@@ -1442,7 +1564,7 @@ static bRC checkFile(bpContext *ctx, char *fname)
    return bRC_OK;
 }
 
-/*
+/**
  * Acls are saved using extended attributes.
  */
 static const char *xattr_acl_skiplist[3] = {
