@@ -1,7 +1,7 @@
 /*
    BAREOS® - Backup Archiving REcovery Open Sourced
 
-   Copyright (C) 2014-2016 Planets Communications B.V.
+   Copyright (C) 2014-2017 Planets Communications B.V.
    Copyright (C) 2014-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -19,7 +19,8 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 */
-/*
+/**
+ * @file
  * GlusterFS GFAPI plugin for the Bareos File Daemon
  */
 #include "bareos.h"
@@ -28,6 +29,7 @@
 #include "fileopts.h"
 
 #include <api/glfs.h>
+#include <compat-errno.h>
 
 static const int dbglvl = 150;
 
@@ -40,7 +42,7 @@ static const int dbglvl = 150;
 
 #define GLFS_PATH_MAX 4096
 
-/*
+/**
  * Forward referenced functions
  */
 static bRC newPlugin(bpContext *ctx);
@@ -66,13 +68,13 @@ static bRC end_restore_job(bpContext *ctx, void *value);
 static bRC setup_backup(bpContext *ctx, void *value);
 static bRC setup_restore(bpContext *ctx, void *value);
 
-/*
+/**
  * Pointers to Bareos functions
  */
 static bFuncs *bfuncs = NULL;
 static bInfo  *binfo = NULL;
 
-/*
+/**
  * Plugin Information block
  */
 static genpInfo pluginInfo = {
@@ -87,7 +89,7 @@ static genpInfo pluginInfo = {
    PLUGIN_USAGE
 };
 
-/*
+/**
  * Plugin entry points for Bareos
  */
 static pFuncs pluginFuncs = {
@@ -114,13 +116,14 @@ static pFuncs pluginFuncs = {
    setXattr
 };
 
-/*
+/**
  * Plugin private context
  */
 struct plugin_ctx {
    int32_t backup_level;              /* Backup level e.g. Full/Differential/Incremental */
    utime_t since;                     /* Since time for Differential/Incremental */
    char *plugin_options;              /* Options passed to plugin */
+   char *plugin_definition;           /* Previous plugin definition passed to plugin */
    char *gfapi_volume_spec;           /* Unparsed Gluster volume specification */
    char *transport;                   /* Gluster transport protocol to management server */
    char *servername;                  /* Gluster management server */
@@ -135,6 +138,7 @@ struct plugin_ctx {
    char *next_xattr_name;             /* Next xattr name to process */
    bool crawl_fs;                     /* Use local fs crawler to find files to backup */
    char *gf_file_list;                /* File with list of files generated by glusterfind to backup */
+   bool is_accurate;                  /* Backup has accurate option enabled */
    POOLMEM *cwd;                      /* Current Working Directory */
    POOLMEM *next_filename;            /* Next filename to save */
    POOLMEM *link_target;              /* Target symlink points to */
@@ -150,13 +154,14 @@ struct plugin_ctx {
    FILE *file_list_handle;            /* File handle to file with files to backup */
 };
 
-/*
+/**
  * This defines the arguments that the plugin parser understands.
  */
 enum plugin_argument_type {
    argument_none,
    argument_volume_spec,
    argument_snapdir,
+   argument_basedir,
    argument_gf_file_list
 };
 
@@ -168,6 +173,7 @@ struct plugin_argument {
 static plugin_argument plugin_arguments[] = {
    { "volume", argument_volume_spec },
    { "snapdir", argument_snapdir },
+   { "basedir", argument_basedir },
    { "gffilelist", argument_gf_file_list },
    { NULL, argument_none }
 };
@@ -180,7 +186,7 @@ enum gluster_find_type {
    gf_type_delete
 };
 
-/*
+/**
  * If we recurse into a subdir we push the current directory onto
  * a stack so we can pop it after we have processed the subdir.
  */
@@ -237,7 +243,7 @@ static inline int to_hex(char ch)
    return retval;
 }
 
-/*
+/**
  * Quick and dirty version of RFC 3986 percent-decoding
  * It decodes the entries returned by glusterfind which uses
  * the python urllib.quote_plus method.
@@ -301,7 +307,7 @@ bail_out:
 extern "C" {
 #endif
 
-/*
+/**
  * loadPlugin() and unloadPlugin() are entry points that are exported, so Bareos can
  * directly call these two entry points they are common to all Bareos plugins.
  *
@@ -320,7 +326,7 @@ bRC DLL_IMP_EXP loadPlugin(bInfo *lbinfo,
    return bRC_OK;
 }
 
-/*
+/**
  * External entry point to unload the plugin
  */
 bRC DLL_IMP_EXP unloadPlugin()
@@ -332,7 +338,7 @@ bRC DLL_IMP_EXP unloadPlugin()
 }
 #endif
 
-/*
+/**
  * The following entry points are accessed through the function pointers we supplied to Bareos.
  * Each plugin type (dir, fd, sd) has its own set of entry points that the plugin must define.
  *
@@ -381,7 +387,7 @@ static bRC newPlugin(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Free a plugin instance, i.e. release our private storage
  */
 static bRC freePlugin(bpContext *ctx)
@@ -438,6 +444,10 @@ static bRC freePlugin(bpContext *ctx)
       free(p_ctx->gfapi_volume_spec);
    }
 
+   if (p_ctx->plugin_definition) {
+      free(p_ctx->plugin_definition);
+   }
+
    if (p_ctx->plugin_options) {
       free(p_ctx->plugin_options);
    }
@@ -450,7 +460,7 @@ static bRC freePlugin(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * Return some plugin value (none defined)
  */
 static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
@@ -458,7 +468,7 @@ static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Set a plugin value (none defined)
  */
 static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
@@ -466,7 +476,7 @@ static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
    return bRC_OK;
 }
 
-/*
+/**
  * Handle an event that was generated in Bareos
  */
 static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
@@ -531,7 +541,7 @@ static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
    return retval;
 }
 
-/*
+/**
  * Get the next file to backup.
  */
 static bRC get_next_file_to_backup(bpContext *ctx)
@@ -623,6 +633,7 @@ static bRC get_next_file_to_backup(bpContext *ctx)
           * Strip the newline.
           */
          strip_trailing_junk(p_ctx->next_filename);
+         Dmsg(ctx, dbglvl, "gfapi-fd: Processing glusterfind entry %s\n", p_ctx->next_filename);
 
          /*
           * Lookup mapping to see what type of entry we are processing.
@@ -653,8 +664,10 @@ static bRC get_next_file_to_backup(bpContext *ctx)
                continue;
             }
             *bp++ = '\0';
-            urllib_unquote_plus(p_ctx->next_filename);
-            bfuncs->ClearSeenBitmap(ctx, false, p_ctx->next_filename);
+            if (p_ctx->is_accurate) {
+               urllib_unquote_plus(p_ctx->next_filename);
+               bfuncs->ClearSeenBitmap(ctx, false, p_ctx->next_filename);
+            }
             bstrinlinecpy(p_ctx->next_filename, bp);
             urllib_unquote_plus(p_ctx->next_filename);
             break;
@@ -662,9 +675,11 @@ static bRC get_next_file_to_backup(bpContext *ctx)
             /*
              * DELETE means we clear the seen bitmap for this file and continue.
              */
-            bstrinlinecpy(p_ctx->next_filename, p_ctx->next_filename + gf_mapping->compare_size);
-            urllib_unquote_plus(p_ctx->next_filename);
-            bfuncs->ClearSeenBitmap(ctx, false, p_ctx->next_filename);
+            if (p_ctx->is_accurate) {
+               bstrinlinecpy(p_ctx->next_filename, p_ctx->next_filename + gf_mapping->compare_size);
+               urllib_unquote_plus(p_ctx->next_filename);
+               bfuncs->ClearSeenBitmap(ctx, false, p_ctx->next_filename);
+            }
             continue;
          default:
             Jmsg(ctx, M_ERROR, "Unrecognized glusterfind entry %s\n", p_ctx->next_filename);
@@ -685,8 +700,23 @@ static bRC get_next_file_to_backup(bpContext *ctx)
          if (status != 0) {
             berrno be;
 
-            Jmsg(ctx, M_ERROR, "gfapi-fd: glfs_stat(%s) failed: %s\n", p_ctx->cwd, be.bstrerror());
-            return bRC_Error;
+            switch (errno) {
+            case ENOENT:
+               /*
+                * Note: This was silently ignored before, now at least emit a warning
+                * in the job log that does not trigger the "OK -- with warnings" termination
+                */
+               Jmsg(ctx, M_WARNING, "gfapi-fd: glfs_stat(%s) failed: %s (skipped)\n", p_ctx->next_filename, be.bstrerror());
+               continue;
+            case GF_ERROR_CODE_STALE:
+               Jmsg(ctx, M_ERROR, "gfapi-fd: glfs_stat(%s) failed: %s (skipped)\n", p_ctx->next_filename, be.bstrerror());
+               continue;
+            default:
+               Dmsg(ctx, dbglvl, "gfapi-fd: glfs_stat(%s) failed: %s errno: %d\n",
+                    p_ctx->next_filename, be.bstrerror(), errno);
+               Jmsg(ctx, M_FATAL, "gfapi-fd: glfs_stat(%s) failed: %s\n", p_ctx->next_filename, be.bstrerror());
+               return bRC_Error;
+            }
          }
       } else {
 #ifndef HAVE_GLFS_READDIRPLUS
@@ -704,7 +734,7 @@ static bRC get_next_file_to_backup(bpContext *ctx)
             if (status != 0) {
                berrno be;
 
-               Jmsg(ctx, M_ERROR, "glfs_stat(%s) failed: %s\n", p_ctx->cwd, be.bstrerror());
+               Jmsg(ctx, M_FATAL, "glfs_stat(%s) failed: %s\n", p_ctx->cwd, be.bstrerror());
                return bRC_Error;
             }
 
@@ -805,7 +835,7 @@ static bRC get_next_file_to_backup(bpContext *ctx)
    return bRC_More;
 }
 
-/*
+/**
  * Start the backup of a specific file
  */
 static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
@@ -942,6 +972,23 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
       switch (p_ctx->backup_level) {
       case L_INCREMENTAL:
       case L_DIFFERENTIAL:
+         /*
+          * When sp->type is FT_DIRBEGIN, skip calling checkChanges() because it would be useless.
+          */
+         if (sp->type == FT_DIRBEGIN) {
+            Dmsg(ctx, dbglvl, "gfapi-fd: skip checkChanges() for %s because sp->type is FT_DIRBEGIN\n", p_ctx->next_filename);
+            sp->type = FT_DIRNOCHG;
+            break;
+         }
+         /*
+          * When glfs_chdir() or glfs_opendir() failed, then sp->type is FT_NOOPEN, then
+          * skip calling checkChanges() because it would be useless.
+          */
+         if (sp->type == FT_NOOPEN) {
+            Dmsg(ctx, dbglvl, "gfapi-fd: skip checkChanges() for %s because sp->type is FT_NOOPEN\n", p_ctx->next_filename);
+            break;
+         }
+
          switch (bfuncs->checkChanges(ctx, sp)) {
          case bRC_Seen:
             Dmsg(ctx, dbglvl, "gfapi-fd: skipping %s checkChanges returns bRC_Seen\n", p_ctx->next_filename);
@@ -964,7 +1011,7 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
    return bRC_OK;
 }
 
-/*
+/**
  * Done with backup of this file
  */
 static bRC endBackupFile(bpContext *ctx)
@@ -992,7 +1039,7 @@ static bRC endBackupFile(bpContext *ctx)
    return get_next_file_to_backup(ctx);
 }
 
-/*
+/**
  * Strip any backslashes in the string.
  */
 static inline void strip_back_slashes(char *value)
@@ -1013,7 +1060,7 @@ static inline void strip_back_slashes(char *value)
    }
 }
 
-/*
+/**
  * Only set destination to value when it has no previous setting.
  */
 static inline void set_string_if_null(char **destination, char *value)
@@ -1024,7 +1071,7 @@ static inline void set_string_if_null(char **destination, char *value)
    }
 }
 
-/*
+/**
  * Always set destination to value and clean any previous one.
  */
 static inline void set_string(char **destination, char *value)
@@ -1037,7 +1084,7 @@ static inline void set_string(char **destination, char *value)
    strip_back_slashes(*destination);
 }
 
-/*
+/**
  * Parse the plugin definition passed in.
  *
  * The definition is in this form:
@@ -1055,6 +1102,25 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
       return bRC_Error;
    }
 
+   /*
+    * See if we already got some plugin definition before and its exactly the same.
+    */
+   if (p_ctx->plugin_definition) {
+      if (bstrcmp(p_ctx->plugin_definition, (char *)value)) {
+         return bRC_OK;
+      }
+
+      free(p_ctx->plugin_definition);
+   }
+
+   /*
+    * Keep track of the last processed plugin definition.
+    */
+   p_ctx->plugin_definition = bstrdup((char *)value);
+
+   /*
+    * Keep overrides passed in via pluginoptions.
+    */
    keep_existing = (p_ctx->plugin_options) ? true : false;
 
    /*
@@ -1122,6 +1188,9 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
             case argument_snapdir:
                str_destination = &p_ctx->snapdir;
                break;
+            case argument_basedir:
+               str_destination = &p_ctx->basedir;
+               break;
             case argument_gf_file_list:
                str_destination = &p_ctx->gf_file_list;
                break;
@@ -1166,7 +1235,7 @@ bail_out:
    return bRC_Error;
 }
 
-/*
+/**
  * Create a parent directory using the gfapi.
  */
 static inline bool gfapi_makedirs(plugin_ctx *p_ctx, const char *directory)
@@ -1229,7 +1298,7 @@ static inline bool gfapi_makedirs(plugin_ctx *p_ctx, const char *directory)
    return retval;
 }
 
-/*
+/**
  * Parse a gluster definition into something we can use for setting
  * up the right connection to a gluster management server and get access
  * to a gluster volume.
@@ -1320,7 +1389,7 @@ static inline bool parse_gfapi_devicename(char *devicename,
       /*
        * Validate URI.
        */
-      if (!bp || !*bp == '/') {
+      if (!bp || *(bp + 1) != '/') {
          goto bail_out;
       }
 
@@ -1436,7 +1505,7 @@ bail_out:
    return false;
 }
 
-/*
+/**
  * Open a volume using GFAPI.
  */
 static bRC connect_to_gluster(bpContext *ctx, bool is_backup)
@@ -1457,6 +1526,14 @@ static bRC connect_to_gluster(bpContext *ctx, bool is_backup)
       return bRC_Error;
    }
 
+   /*
+    * If we get called and we already have a handle to gfapi we should tear it down.
+    */
+   if (p_ctx->glfs) {
+      glfs_fini(p_ctx->glfs);
+      p_ctx->glfs = NULL;
+   }
+
    p_ctx->glfs = glfs_new(p_ctx->volumename);
    if (!p_ctx->glfs) {
       goto bail_out;
@@ -1498,17 +1575,28 @@ bail_out:
    return bRC_Error;
 }
 
-/*
+/**
  * Generic setup for performing a backup.
  */
 static bRC setup_backup(bpContext *ctx, void *value)
 {
+   bRC retval = bRC_Error;
    plugin_ctx *p_ctx = (plugin_ctx *)ctx->pContext;
 
    if (!p_ctx || !value) {
       goto bail_out;
    }
 
+   /*
+    * If we are already having a handle to gfapi and we are getting the
+    * same plugin definition there is no need to tear down the whole stuff and
+    * setup exactly the same.
+    */
+   if (p_ctx->glfs &&
+       bstrcmp((char *)value, p_ctx->plugin_definition)) {
+      return bRC_OK;
+   }
+
    if (connect_to_gluster(ctx, true) != bRC_OK) {
       goto bail_out;
    }
@@ -1517,6 +1605,16 @@ static bRC setup_backup(bpContext *ctx, void *value)
     * See if we use an external list with files to backup or should crawl the filesystem ourself.
     */
    if (p_ctx->gf_file_list) {
+      int accurate;
+
+      /*
+       * Get the setting for accurate for this Job.
+       */
+      bfuncs->getBareosValue(ctx, bVarAccurate, (void *)&accurate);
+      if (accurate) {
+         p_ctx->is_accurate = true;
+      }
+
       p_ctx->crawl_fs = false;
       if ((p_ctx->file_list_handle = fopen(p_ctx->gf_file_list, "r")) == (FILE *)NULL) {
          Jmsg(ctx, M_FATAL, "Failed to open %s for reading files to backup\n", p_ctx->gf_file_list);
@@ -1524,21 +1622,23 @@ static bRC setup_backup(bpContext *ctx, void *value)
          goto bail_out;
       }
 
-      /*
-       * Mark all files as seen from the previous backup when this is a incremental or differential backup.
-       * The entries we get from glusterfind are only the changes since that earlier backup.
-       */
-      switch (p_ctx->backup_level) {
-      case L_INCREMENTAL:
-      case L_DIFFERENTIAL:
-         if (bfuncs->SetSeenBitmap(ctx, true, NULL) != bRC_OK) {
-            Jmsg(ctx, M_FATAL, "Failed to enable all entries in Seen bitmap, not an accurate backup ?\n");
-            Dmsg(ctx, dbglvl, "Failed to enable all entries in Seen bitmap, not an accurate backup ?\n");
-            goto bail_out;
+      if (p_ctx->is_accurate) {
+         /*
+          * Mark all files as seen from the previous backup when this is a incremental or differential backup.
+          * The entries we get from glusterfind are only the changes since that earlier backup.
+          */
+         switch (p_ctx->backup_level) {
+         case L_INCREMENTAL:
+         case L_DIFFERENTIAL:
+            if (bfuncs->SetSeenBitmap(ctx, true, NULL) != bRC_OK) {
+               Jmsg(ctx, M_FATAL, "Failed to enable all entries in Seen bitmap, not an accurate backup ?\n");
+               Dmsg(ctx, dbglvl, "Failed to enable all entries in Seen bitmap, not an accurate backup ?\n");
+               goto bail_out;
+            }
+            break;
+         default:
+            break;
          }
-         break;
-      default:
-         break;
       }
 
       /*
@@ -1546,9 +1646,24 @@ static bRC setup_backup(bpContext *ctx, void *value)
        * As we need to get it from the gfflilelist we use get_next_file_to_backup()
        * to do the setup for us it retrieves the entry and does a setup of filetype etc.
        */
-      if (get_next_file_to_backup(ctx) == bRC_Error) {
+      switch (get_next_file_to_backup(ctx)) {
+      case bRC_OK:
+         /*
+          * get_next_file_to_backup() normally returns bRC_More to indicate that there are
+          * more files to backup. But when using glusterfind we use an external filelist which
+          * could be empty in that special case we get bRC_OK back from get_next_file_to_backup()
+          * and then only in setup_backup() we return bRC_Skip which will skip processing of any
+          * more files to backup.
+          */
+         retval = bRC_Skip;
+         break;
+      case bRC_Error:
          Jmsg(ctx, M_FATAL, "Failed to get first file to backup\n");
          Dmsg(ctx, dbglvl, "Failed to get first file to backup\n");
+         goto bail_out;
+      default:
+         retval = bRC_OK;
+         break;
       }
    } else {
       p_ctx->crawl_fs = true;
@@ -1588,15 +1703,15 @@ static bRC setup_backup(bpContext *ctx, void *value)
       } else {
          pm_strcpy(p_ctx->next_filename, "/");
       }
+
+      retval = bRC_OK;
    }
 
-   return bRC_OK;
-
 bail_out:
-   return bRC_Error;
+   return retval;
 }
 
-/*
+/**
  * Generic setup for performing a restore.
  */
 static bRC setup_restore(bpContext *ctx, void *value)
@@ -1607,14 +1722,20 @@ static bRC setup_restore(bpContext *ctx, void *value)
       return bRC_Error;
    }
 
-   if (connect_to_gluster(ctx, false) != bRC_OK) {
-      return bRC_Error;
+   /*
+    * If we are already having a handle to gfapi and we are getting the
+    * same plugin definition there is no need to tear down the whole stuff and
+    * setup exactly the same.
+    */
+   if (p_ctx->glfs &&
+       bstrcmp((char *)value, p_ctx->plugin_definition)) {
+      return bRC_OK;
    }
 
-   return bRC_OK;
+   return connect_to_gluster(ctx, false);
 }
 
-/*
+/**
  * Bareos is calling us to do the actual I/O
  */
 static bRC pluginIO(bpContext *ctx, struct io_pkt *io)
@@ -1631,6 +1752,13 @@ static bRC pluginIO(bpContext *ctx, struct io_pkt *io)
 
    switch(io->func) {
    case IO_OPEN:
+      /*
+       * Close the gfd when it was not closed before.
+       */
+      if (p_ctx->gfd) {
+         glfs_close(p_ctx->gfd);
+      }
+
       if (io->flags & (O_CREAT | O_WRONLY)) {
          p_ctx->gfd = glfs_creat(p_ctx->glfs, io->fname, io->flags, io->mode);
       } else {
@@ -1705,7 +1833,7 @@ bail_out:
    return bRC_Error;
 }
 
-/*
+/**
  * See if we need to do any postprocessing after the restore.
  */
 static bRC end_restore_job(bpContext *ctx, void *value)
@@ -1724,7 +1852,7 @@ static bRC end_restore_job(bpContext *ctx, void *value)
    return retval;
 }
 
-/*
+/**
  * Bareos is notifying us that a plugin name string was found,
  * and passing us the plugin command, so we can prepare for a restore.
  */
@@ -1733,7 +1861,7 @@ static bRC startRestoreFile(bpContext *ctx, const char *cmd)
    return bRC_OK;
 }
 
-/*
+/**
  * Bareos is notifying us that the plugin data has terminated,
  * so the restore for this particular file is done.
  */
@@ -1742,7 +1870,7 @@ static bRC endRestoreFile(bpContext *ctx)
    return bRC_OK;
 }
 
-/*
+/**
  * This is called during restore to create the file (if necessary) We must return in rp->create_status:
  *
  *  CF_ERROR    -- error
@@ -1956,7 +2084,7 @@ static bRC setFileAttributes(bpContext *ctx, struct restore_pkt *rp)
    return bRC_OK;
 }
 
-/*
+/**
  * When using Incremental dump, all previous dumps are necessary
  */
 static bRC checkFile(bpContext *ctx, char *fname)
@@ -1970,7 +2098,7 @@ static bRC checkFile(bpContext *ctx, char *fname)
    return bRC_OK;
 }
 
-/*
+/**
  * Acls are saved using extended attributes.
  */
 static const char *xattr_acl_skiplist[3] = {
