@@ -20,10 +20,12 @@
    02110-1301, USA.
 */
 /*
- * Gluster Filesystem API device abstraction.
- *
  * Marco van Wieringen, May 2014
  */
+/**
+ * @file
+ * Gluster Filesystem API device abstraction.
+ */
 
 #include "bareos.h"
 
@@ -31,7 +33,7 @@
 #include "stored.h"
 #include "backends/cephfs_device.h"
 
-/*
+/**
  * Options that can be specified for this device type.
  */
 enum device_option_type {
@@ -52,7 +54,7 @@ static device_option device_options[] = {
    { NULL, argument_none }
 };
 
-/*
+/**
  * Open a volume using libcephfs.
  */
 int cephfs_device::d_open(const char *pathname, int flags, int mode)
@@ -152,18 +154,26 @@ int cephfs_device::d_open(const char *pathname, int flags, int mode)
     * See if we store in an explicit directory.
     */
    if (m_basedir) {
+#if HAVE_CEPHFS_CEPH_STATX_H
+      struct ceph_statx stx;
+#else
       struct stat st;
+#endif
 
       /*
        * Make sure the dir exists if one is defined.
        */
+#if HAVE_CEPHFS_CEPH_STATX_H
+      status = ceph_statx(m_cmount, m_basedir, &stx, CEPH_STATX_SIZE, AT_SYMLINK_NOFOLLOW);
+#else
       status = ceph_stat(m_cmount, m_basedir, &st);
+#endif
       if (status < 0) {
          switch (status) {
          case -ENOENT:
             status = ceph_mkdirs(m_cmount, m_basedir, 0750);
             if (status < 0) {
-               Mmsg1(errmsg, _("Specified CEPHFS direcory %s cannot be created.\n"), m_basedir);
+               Mmsg1(errmsg, _("Specified CEPHFS directory %s cannot be created.\n"), m_basedir);
                Emsg0(M_FATAL, 0, errmsg);
                goto bail_out;
             }
@@ -172,8 +182,12 @@ int cephfs_device::d_open(const char *pathname, int flags, int mode)
             goto bail_out;
          }
       } else {
+#if HAVE_CEPHFS_CEPH_STATX_H
+         if (!S_ISDIR(stx.stx_mode)) {
+#else
          if (!S_ISDIR(st.st_mode)) {
-            Mmsg1(errmsg, _("Specified CEPHFS direcory %s is not a directory.\n"), m_basedir);
+#endif
+            Mmsg1(errmsg, _("Specified CEPHFS directory %s is not a directory.\n"), m_basedir);
             Emsg0(M_FATAL, 0, errmsg);
             goto bail_out;
          }
@@ -204,7 +218,7 @@ bail_out:
    return -1;
 }
 
-/*
+/**
  * Read data from a volume using libcephfs.
  */
 ssize_t cephfs_device::d_read(int fd, void *buffer, size_t count)
@@ -217,7 +231,7 @@ ssize_t cephfs_device::d_read(int fd, void *buffer, size_t count)
    }
 }
 
-/*
+/**
  * Write data to a volume using libcephfs.
  */
 ssize_t cephfs_device::d_write(int fd, const void *buffer, size_t count)
@@ -271,7 +285,11 @@ boffset_t cephfs_device::d_lseek(DCR *dcr, boffset_t offset, int whence)
 bool cephfs_device::d_truncate(DCR *dcr)
 {
    int status;
+#if HAVE_CEPHFS_CEPH_STATX_H
+   struct ceph_statx stx;
+#else
    struct stat st;
+#endif
 
    if (m_fd >= 0) {
       status = ceph_ftruncate(m_cmount, m_fd, 0);
@@ -291,16 +309,28 @@ bool cephfs_device::d_truncate(DCR *dcr)
        * 3. open new file with same mode
        * 4. change ownership to original
        */
+#if HAVE_CEPHFS_CEPH_STATX_H
+      status = ceph_fstatx(m_cmount, m_fd, &stx, CEPH_STATX_MODE, 0);
+#else
       status = ceph_fstat(m_cmount, m_fd, &st);
+#endif
       if (status < 0) {
          berrno be;
 
+#if HAVE_CEPHFS_CEPH_STATX_H
+         Mmsg2(errmsg, _("Unable to ceph_statx device %s. ERR=%s\n"), prt_name, be.bstrerror(-status));
+#else
          Mmsg2(errmsg, _("Unable to stat device %s. ERR=%s\n"), prt_name, be.bstrerror(-status));
+#endif
          Dmsg1(100, "%s", errmsg);
          return false;
       }
 
+#if HAVE_CEPHFS_CEPH_STATX_H
+      if (stx.stx_size != 0) {             /* ceph_ftruncate() didn't work */
+#else
       if (st.st_size != 0) {             /* ceph_ftruncate() didn't work */
+#endif
          ceph_close(m_cmount, m_fd);
          ceph_unlink(m_cmount, m_virtual_filename);
 
@@ -308,7 +338,11 @@ bool cephfs_device::d_truncate(DCR *dcr)
           * Recreate the file -- of course, empty
           */
          oflags = O_CREAT | O_RDWR | O_BINARY;
+#if HAVE_CEPHFS_CEPH_STATX_H
+         m_fd = ceph_open(m_cmount, m_virtual_filename, oflags, stx.stx_mode);
+#else
          m_fd = ceph_open(m_cmount, m_virtual_filename, oflags, st.st_mode);
+#endif
          if (m_fd < 0) {
             berrno be;
 
@@ -323,7 +357,11 @@ bool cephfs_device::d_truncate(DCR *dcr)
          /*
           * Reset proper owner
           */
+#if HAVE_CEPHFS_CEPH_STATX_H
+         ceph_chown(m_cmount, m_virtual_filename, stx.stx_uid, stx.stx_gid);
+#else
          ceph_chown(m_cmount, m_virtual_filename, st.st_uid, st.st_gid);
+#endif
       }
    }
 
