@@ -20,10 +20,12 @@
    02110-1301, USA.
 */
 /*
- * CEPH (librados) API device abstraction.
- *
  * Marco van Wieringen, February 2014
  */
+/**
+ * @file
+ * CEPH (librados) API device abstraction.
+ */
 
 #include "bareos.h"
 
@@ -31,7 +33,7 @@
 #include "stored.h"
 #include "rados_device.h"
 
-/*
+/**
  * Options that can be specified for this device type.
  */
 enum device_option_type {
@@ -43,6 +45,7 @@ enum device_option_type {
    argument_username,
    argument_striped,
    argument_stripe_unit,
+   argument_object_size,
    argument_stripe_count
 };
 
@@ -64,12 +67,13 @@ static device_option device_options[] = {
 #ifdef HAVE_RADOS_STRIPER
    { "striped", argument_striped, 7 },
    { "stripe_unit=", argument_stripe_unit, 12 },
+   { "object_size=", argument_object_size, 12 },
    { "stripe_count=", argument_stripe_count, 13 },
 #endif
    { NULL, argument_none }
 };
 
-/*
+/**
  * Open a volume using librados.
  */
 int rados_device::d_open(const char *pathname, int flags, int mode)
@@ -137,6 +141,10 @@ int rados_device::d_open(const char *pathname, int flags, int mode)
                   size_to_uint64(bp + device_options[i].compare_size, &m_stripe_unit);
                   done = true;
                   break;
+               case argument_object_size:
+                  size_to_uint64(bp + device_options[i].compare_size, &m_object_size);
+                  done = true;
+                  break;
                case argument_stripe_count:
                   m_stripe_count = str_to_int64(bp + device_options[i].compare_size);
                   done = true;
@@ -254,6 +262,13 @@ int rados_device::d_open(const char *pathname, int flags, int mode)
             Emsg0(M_FATAL, 0, errmsg);
             goto bail_out;
          }
+
+         status = rados_striper_set_object_layout_object_size(m_striper, m_object_size);
+         if (status < 0) {
+            Mmsg3(errmsg, _("Unable to set RADOS striper object size to %d  for pool %s: ERR=%s\n"), m_object_size, m_rados_poolname, be.bstrerror(-status));
+             Emsg0(M_FATAL, 0, errmsg);
+             goto bail_out;
+         }
       }
 #endif
    }
@@ -314,14 +329,14 @@ int rados_device::d_open(const char *pathname, int flags, int mode)
 
 bail_out:
    if (m_cluster_initialized) {
-      rados_shutdown(&m_cluster);
+      rados_shutdown(m_cluster);
       m_cluster_initialized = false;
    }
 
    return -1;
 }
 
-/*
+/**
  * Read some data from an object at a given offset.
  */
 ssize_t rados_device::read_object_data(boffset_t offset, char *buffer, size_t count)
@@ -337,7 +352,7 @@ ssize_t rados_device::read_object_data(boffset_t offset, char *buffer, size_t co
 #endif
 }
 
-/*
+/**
  * Read data from a volume using librados.
  */
 ssize_t rados_device::d_read(int fd, void *buffer, size_t count)
@@ -360,7 +375,7 @@ ssize_t rados_device::d_read(int fd, void *buffer, size_t count)
    }
 }
 
-/*
+/**
  * Write some data to an object at a given offset.
  * Seems the API changed everything earlier then 0.69 returns bytes written.
  */
@@ -390,7 +405,7 @@ ssize_t rados_device::write_object_data(boffset_t offset, char *buffer, size_t c
 #endif
 }
 
-/*
+/**
  * Write data to a volume using librados.
  */
 ssize_t rados_device::d_write(int fd, const void *buffer, size_t count)
@@ -594,7 +609,7 @@ rados_device::~rados_device()
    }
 
    if (m_cluster_initialized) {
-      rados_shutdown(&m_cluster);
+      rados_shutdown(m_cluster);
       m_cluster_initialized = false;
    }
 
@@ -633,8 +648,9 @@ rados_device::rados_device()
    m_ctx = NULL;
 #ifdef HAVE_RADOS_STRIPER
    m_stripe_volume = false;
-   m_stripe_unit = 0;
-   m_stripe_count = 0;
+   m_stripe_unit = 4194304;
+   m_stripe_count = 1;
+   m_object_size = 4194304;
    m_striper = NULL;
 #endif
    m_virtual_filename = get_pool_memory(PM_FNAME);
