@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2013-2013 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,18 +21,19 @@
    02110-1301, USA.
 */
 /*
- * UNIX FIFO API device abstraction.
- *
  * Kern Sibbald, MM
- *
  * Extracted from other source files Marco van Wieringen, December 2013
  */
+/**
+ * @file
+ * UNIX FIFO API device abstraction.
+ */
 
 #include "bareos.h"
 #include "stored.h"
 #include "unix_fifo_device.h"
 
-/*
+/**
  * Open a fifo device
  */
 void unix_fifo_device::open_device(DCR *dcr, int omode)
@@ -140,7 +141,7 @@ bool unix_fifo_device::eod(DCR *dcr)
    return true;
 }
 
-/*
+/**
  * (Un)mount the device (For a FILE device)
  */
 static bool do_mount(DCR *dcr, bool mount, int dotimeout)
@@ -217,7 +218,12 @@ static bool do_mount(DCR *dcr, bool mount, int dotimeout)
       entry = (struct dirent *)malloc(sizeof(struct dirent) + name_max + 1000);
       count = 0;
       while (1) {
+#ifdef USE_READDIR_R
          if ((readdir_r(dp, entry, &result) != 0) || (result == NULL)) {
+#else
+         result = readdir(dp);
+         if (result == NULL) {
+#endif
             dcr->dev->dev_errno = EIO;
             Dmsg2(129, "do_mount: failed to find suitable file in dir %s (dev=%s)\n",
                   device->mount_point, dcr->dev->print_name());
@@ -260,7 +266,7 @@ get_out:
    return true;
 }
 
-/*
+/**
  * Mount the device.
  *
  * If timeout, wait until the mount command returns 0.
@@ -277,7 +283,7 @@ bool unix_fifo_device::mount_backend(DCR *dcr, int timeout)
    return retval;
 }
 
-/*
+/**
  * Unmount the device
  *
  * If timeout, wait until the unmount command returns 0.
