@@ -2,8 +2,8 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2001-2011 Free Software Foundation Europe e.V.
-   Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
+   Copyright (C) 2011-2016 Planets Communications B.V.
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,11 +21,13 @@
    02110-1301, USA.
 */
 /*
+ * Kern E. Sibbald, December 2001
+ */
+/**
+ * @file
  * Program to scan a Bareos Volume and compare it with
  * the catalog and optionally synchronize the catalog
  * with the tape.
- *
- * Kern E. Sibbald, December 2001
  */
 
 #include "bareos.h"
@@ -33,7 +35,6 @@
 #include "lib/crypto_cache.h"
 #include "findlib/find.h"
 #include "cats/cats.h"
-#include "cats/sql_glue.h"
 
 /* Dummy functions */
 extern bool parse_sd_config(CONFIG *config, const char *configfile, int exit_code);
@@ -125,7 +126,8 @@ PROG_COPYRIGHT
 "       -w <directory>    specify working directory (default from configuration file)\n"
 "       -?                print this message\n\n"
 "example:\n"
-"bscan -B postgresql -V Full-0001 FileStorage\n"), 2001, VERSION, BDATE, backend_directory);
+"bscan -B postgresql -V Full-0001 FileStorage\n"),
+            2001, VERSION, BDATE, backend_directory);
    exit(1);
 }
 
@@ -152,7 +154,7 @@ int main (int argc, char *argv[])
 
    OSDependentInit();
 
-   while ((ch = getopt(argc, argv, "a:B:b:c:d:D:h:p:mn:pP:rsSt:u:vV:w:?")) != -1) {
+   while ((ch = getopt(argc, argv, "a:B:b:c:d:D:h:p:mn:pP:q:rsSt:u:vV:w:?")) != -1) {
       switch (ch) {
       case 'a':
          backend_directory = optarg;
@@ -207,10 +209,6 @@ int main (int argc, char *argv[])
          db_name = optarg;
          break;
 
-      case 'u':
-         db_user = optarg;
-         break;
-
       case 'P':
          db_password = optarg;
          break;
@@ -231,6 +229,10 @@ int main (int argc, char *argv[])
          update_db = true;
          break;
 
+      case 'u':
+         db_user = optarg;
+         break;
+
       case 'V':                    /* Volume name */
          VolumeName = optarg;
          break;
@@ -335,8 +337,8 @@ int main (int argc, char *argv[])
    if (db == NULL) {
       Emsg0(M_ERROR_TERM, 0, _("Could not init Bareos database\n"));
    }
-   if (!db_open_database(NULL, db)) {
-      Emsg0(M_ERROR_TERM, 0, db_strerror(db));
+   if (!db->open_database(NULL)) {
+      Emsg0(M_ERROR_TERM, 0, db->strerror());
    }
    Dmsg0(200, "Database opened\n");
    if (verbose) {
@@ -363,7 +365,7 @@ int main (int argc, char *argv[])
    return 0;
 }
 
-/*
+/**
  * We are at the end of reading a tape. Now, we simulate handling
  * the end of writing a tape by wiffling through the attached
  * jcrs creating jobmedia records.
@@ -433,13 +435,13 @@ static void do_scan()
    read_records(bjcr->read_dcr, record_cb, bscan_mount_next_read_volume);
 
    if (update_db) {
-      db_write_batch_file_records(bjcr); /* used by bulk batch file insert */
+      db->write_batch_file_records(bjcr); /* used by bulk batch file insert */
    }
 
    free_attr(attr);
 }
 
-/*
+/**
  * Returns: true  if OK
  *          false if error
  */
@@ -474,7 +476,7 @@ static inline bool unpack_restore_object(JCR *jcr, int32_t stream, char *rec, in
    return true;
 }
 
-/*
+/**
  * Returns: true  if OK
  *          false if error
  */
@@ -530,7 +532,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          bstrncpy(pr.Name, dev->VolHdr.PoolName, sizeof(pr.Name));
          bstrncpy(pr.PoolType, dev->VolHdr.PoolType, sizeof(pr.PoolType));
          num_pools++;
-         if (db_get_pool_record(bjcr, db, &pr)) {
+         if (db->get_pool_record(bjcr, &pr)) {
             if (verbose) {
                Pmsg1(000, _("Pool record for %s found in DB.\n"), pr.Name);
             }
@@ -556,7 +558,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          bstrncpy(mr.VolumeName, dev->VolHdr.VolumeName, sizeof(mr.VolumeName));
          mr.PoolId = pr.PoolId;
          num_media++;
-         if (db_get_media_record(bjcr, db, &mr)) {
+         if (db->get_media_record(bjcr, &mr)) {
             if (verbose) {
                Pmsg1(000, _("Media record for %s found in DB.\n"), mr.VolumeName);
             }
@@ -609,7 +611,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
             unser_session_label(&label, rec);
             memset(&jr, 0, sizeof(jr));
             bstrncpy(jr.Job, label.Job, sizeof(jr.Job));
-            if (db_get_job_record(bjcr, db, &jr)) {
+            if (db->get_job_record(bjcr, &jr)) {
                /*
                 * Job record already exists in DB
                 */
@@ -658,7 +660,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
              * has already been bscan'd
              */
             Mmsg(sql_buffer, "SELECT count(*) from JobMedia where JobId=%d", jr.JobId);
-            db_sql_query(db, sql_buffer.c_str(), db_int64_handler, &jmr_count);
+            db->sql_query(sql_buffer.c_str(), db_int64_handler, &jmr_count);
             if( jmr_count.value > 0 ) {
                mjcr->insert_jobmedia_records = false;
             } else {
@@ -666,15 +668,18 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
             }
 
             if (rec->VolSessionId != jr.VolSessionId) {
-               Pmsg3(000, _("SOS_LABEL: VolSessId mismatch for JobId=%u. DB=%d Vol=%d\n"), jr.JobId, jr.VolSessionId, rec->VolSessionId);
+               Pmsg3(000, _("SOS_LABEL: VolSessId mismatch for JobId=%u. DB=%d Vol=%d\n"),
+                  jr.JobId, jr.VolSessionId, rec->VolSessionId);
                return true;              /* ignore error */
             }
             if (rec->VolSessionTime != jr.VolSessionTime) {
-               Pmsg3(000, _("SOS_LABEL: VolSessTime mismatch for JobId=%u. DB=%d Vol=%d\n"), jr.JobId, jr.VolSessionTime, rec->VolSessionTime);
+               Pmsg3(000, _("SOS_LABEL: VolSessTime mismatch for JobId=%u. DB=%d Vol=%d\n"),
+                  jr.JobId, jr.VolSessionTime, rec->VolSessionTime);
                return true;              /* ignore error */
             }
             if (jr.PoolId != pr.PoolId) {
-               Pmsg3(000, _("SOS_LABEL: PoolId mismatch for JobId=%u. DB=%d Vol=%d\n"), jr.JobId, jr.PoolId, pr.PoolId);
+               Pmsg3(000, _("SOS_LABEL: PoolId mismatch for JobId=%u. DB=%d Vol=%d\n"),
+               jr.JobId, jr.PoolId, pr.PoolId);
                return true;              /* ignore error */
             }
          }
@@ -699,7 +704,8 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
 
             mjcr = get_jcr_by_session(rec->VolSessionId, rec->VolSessionTime);
             if (!mjcr) {
-               Pmsg2(000, _("Could not find SessId=%d SessTime=%d for EOS record.\n"), rec->VolSessionId, rec->VolSessionTime);
+               Pmsg2(000, _("Could not find SessId=%d SessTime=%d for EOS record.\n"),
+                  rec->VolSessionId, rec->VolSessionTime);
                break;
             }
 
@@ -745,8 +751,8 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
                jr.VolSessionTime = mjcr->VolSessionTime;
                jr.JobTDate = (utime_t)mjcr->start_time;
                jr.ClientId = mjcr->ClientId;
-               if (!db_update_job_end_record(bjcr, db, &jr)) {
-                  Pmsg1(0, _("Could not update job record. ERR=%s\n"), db_strerror(db));
+               if (!db->update_job_end_record(bjcr, &jr)) {
+                  Pmsg1(0, _("Could not update job record. ERR=%s\n"), db->strerror());
                }
                mjcr->read_dcr = NULL;
                free_jcr(mjcr);
@@ -820,7 +826,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
 
 
       if (update_db) {
-         db_create_restore_object_record(mjcr, db, &rop);
+         db->create_restore_object_record(mjcr, &rop);
       }
 
       num_restoreobjects++;
@@ -998,7 +1004,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
    return true;
 }
 
-/*
+/**
  * Free the Job Control Record if no one is still using it.
  *  Called from main free_jcr() routine in src/lib/jcr.c so
  *  that we can do our Director specific cleanup of the jcr.
@@ -1038,7 +1044,7 @@ static void bscan_free_jcr(JCR *jcr)
    Dmsg0(200, "End bscan free_jcr\n");
 }
 
-/*
+/**
  * We got a File Attributes record on the tape.  Now, lookup the Job
  * record, and then create the attributes record.
  */
@@ -1068,8 +1074,8 @@ static bool create_file_attributes_record(B_DB *db, JCR *mjcr,
       return true;
    }
 
-   if (!db_create_file_attributes_record(bjcr, db, &ar)) {
-      Pmsg1(0, _("Could not create File Attributes record. ERR=%s\n"), db_strerror(db));
+   if (!db->create_file_attributes_record(bjcr, &ar)) {
+      Pmsg1(0, _("Could not create File Attributes record. ERR=%s\n"), db->strerror());
       return false;
    }
    mjcr->FileId = ar.FileId;
@@ -1081,7 +1087,7 @@ static bool create_file_attributes_record(B_DB *db, JCR *mjcr,
    return true;
 }
 
-/*
+/**
  * For each Volume we see, we create a Medium record
  */
 static bool create_media_record(B_DB *db, MEDIA_DBR *mr, VOLUME_LABEL *vl)
@@ -1126,12 +1132,12 @@ static bool create_media_record(B_DB *db, MEDIA_DBR *mr, VOLUME_LABEL *vl)
       return true;
    }
 
-   if (!db_create_media_record(bjcr, db, mr)) {
-      Pmsg1(000, _("Could not create media record. ERR=%s\n"), db_strerror(db));
+   if (!db->create_media_record(bjcr, mr)) {
+      Pmsg1(000, _("Could not create media record. ERR=%s\n"), db->strerror());
       return false;
    }
-   if (!db_update_media_record(bjcr, db, mr)) {
-      Pmsg1(000, _("Could not update media record. ERR=%s\n"), db_strerror(db));
+   if (!db->update_media_record(bjcr, mr)) {
+      Pmsg1(000, _("Could not update media record. ERR=%s\n"), db->strerror());
       return false;
    }
    if (verbose) {
@@ -1141,7 +1147,7 @@ static bool create_media_record(B_DB *db, MEDIA_DBR *mr, VOLUME_LABEL *vl)
    return true;
 }
 
-/*
+/**
  * Called at end of media to update it
  */
 static bool update_media_record(B_DB *db, MEDIA_DBR *mr)
@@ -1151,8 +1157,8 @@ static bool update_media_record(B_DB *db, MEDIA_DBR *mr)
    }
 
    mr->LastWritten = lasttime;
-   if (!db_update_media_record(bjcr, db, mr)) {
-      Pmsg1(000, _("Could not update media record. ERR=%s\n"), db_strerror(db));
+   if (!db->update_media_record(bjcr, mr)) {
+      Pmsg1(000, _("Could not update media record. ERR=%s\n"), db->strerror());
       return false;
    }
 
@@ -1173,8 +1179,8 @@ static bool create_pool_record(B_DB *db, POOL_DBR *pr)
       return true;
    }
 
-   if (!db_create_pool_record(bjcr, db, pr)) {
-      Pmsg1(000, _("Could not create pool record. ERR=%s\n"), db_strerror(db));
+   if (!db->create_pool_record(bjcr, pr)) {
+      Pmsg1(000, _("Could not create pool record. ERR=%s\n"), db->strerror());
       return false;
    }
 
@@ -1185,7 +1191,7 @@ static bool create_pool_record(B_DB *db, POOL_DBR *pr)
    return true;
 }
 
-/*
+/**
  * Called from SOS to create a client for the current Job
  */
 static bool create_client_record(B_DB *db, CLIENT_DBR *cr)
@@ -1196,16 +1202,16 @@ static bool create_client_record(B_DB *db, CLIENT_DBR *cr)
     */
    if (!update_db) {
       cr->ClientId = 0;
-      if (!db_get_client_record(bjcr, db, cr)) {
-        Pmsg1(0, _("Could not get Client record. ERR=%s\n"), db_strerror(db));
+      if (!db->get_client_record(bjcr, cr)) {
+        Pmsg1(0, _("Could not get Client record. ERR=%s\n"), db->strerror());
         return false;
       }
 
       return true;
    }
 
-   if (!db_create_client_record(bjcr, db, cr)) {
-      Pmsg1(000, _("Could not create Client record. ERR=%s\n"), db_strerror(db));
+   if (!db->create_client_record(bjcr, cr)) {
+      Pmsg1(000, _("Could not create Client record. ERR=%s\n"), db->strerror());
       return false;
    }
 
@@ -1228,13 +1234,13 @@ static bool create_fileset_record(B_DB *db, FILESET_DBR *fsr)
       fsr->MD5[1] = 0;
    }
 
-   if (db_get_fileset_record(bjcr, db, fsr)) {
+   if (db->get_fileset_record(bjcr, fsr)) {
       if (verbose) {
          Pmsg1(000, _("Fileset \"%s\" already exists.\n"), fsr->FileSet);
       }
    } else {
-      if (!db_create_fileset_record(bjcr, db, fsr)) {
-         Pmsg2(000, _("Could not create FileSet record \"%s\". ERR=%s\n"), fsr->FileSet, db_strerror(db));
+      if (!db->create_fileset_record(bjcr, fsr)) {
+         Pmsg2(000, _("Could not create FileSet record \"%s\". ERR=%s\n"), fsr->FileSet, db->strerror());
          return false;
       }
 
@@ -1246,7 +1252,7 @@ static bool create_fileset_record(B_DB *db, FILESET_DBR *fsr)
    return true;
 }
 
-/*
+/**
  * Simulate the two calls on the database to create the Job record and
  * to update it when the Job actually begins running.
  */
@@ -1286,16 +1292,16 @@ static JCR *create_job_record(B_DB *db, JOB_DBR *jr, SESSION_LABEL *label, DEV_R
    /*
     * This creates the bare essentials
     */
-   if (!db_create_job_record(bjcr, db, jr)) {
-      Pmsg1(0, _("Could not create JobId record. ERR=%s\n"), db_strerror(db));
+   if (!db->create_job_record(bjcr, jr)) {
+      Pmsg1(0, _("Could not create JobId record. ERR=%s\n"), db->strerror());
       return mjcr;
    }
 
    /*
     * This adds the client, StartTime, JobTDate, ...
     */
-   if (!db_update_job_start_record(bjcr, db, jr)) {
-      Pmsg1(0, _("Could not update job start record. ERR=%s\n"), db_strerror(db));
+   if (!db->update_job_start_record(bjcr, jr)) {
+      Pmsg1(0, _("Could not update job start record. ERR=%s\n"), db->strerror());
       return mjcr;
    }
 
@@ -1305,7 +1311,7 @@ static JCR *create_job_record(B_DB *db, JOB_DBR *jr, SESSION_LABEL *label, DEV_R
    return mjcr;
 }
 
-/*
+/**
  * Simulate the database call that updates the Job at Job termination time.
  */
 static bool update_job_record(B_DB *db, JOB_DBR *jr, SESSION_LABEL *elabel,
@@ -1352,8 +1358,8 @@ static bool update_job_record(B_DB *db, JOB_DBR *jr, SESSION_LABEL *elabel,
       return true;
    }
 
-   if (!db_update_job_end_record(bjcr, db, jr)) {
-      Pmsg2(0, _("Could not update JobId=%u record. ERR=%s\n"), jr->JobId,  db_strerror(db));
+   if (!db->update_job_end_record(bjcr, jr)) {
+      Pmsg2(0, _("Could not update JobId=%u record. ERR=%s\n"), jr->JobId,  db->strerror());
       free_jcr(mjcr);
       return false;
    }
@@ -1447,8 +1453,8 @@ static bool create_jobmedia_record(B_DB *db, JCR *mjcr)
       return true;
    }
 
-   if (!db_create_jobmedia_record(bjcr, db, &jmr)) {
-      Pmsg1(0, _("Could not create JobMedia record. ERR=%s\n"), db_strerror(db));
+   if (!db->create_jobmedia_record(bjcr, &jmr)) {
+      Pmsg1(0, _("Could not create JobMedia record. ERR=%s\n"), db->strerror());
       return false;
    }
    if (verbose) {
@@ -1458,7 +1464,7 @@ static bool create_jobmedia_record(B_DB *db, JCR *mjcr)
    return true;
 }
 
-/*
+/**
  * Simulate the database call that updates the MD5/SHA1 record
  */
 static bool update_digest_record(B_DB *db, char *digest, DEV_RECORD *rec, int type)
@@ -1481,8 +1487,8 @@ static bool update_digest_record(B_DB *db, char *digest, DEV_RECORD *rec, int ty
       return true;
    }
 
-   if (!db_add_digest_to_file_record(bjcr, db, mjcr->FileId, digest, type)) {
-      Pmsg1(0, _("Could not add MD5/SHA1 to File record. ERR=%s\n"), db_strerror(db));
+   if (!db->add_digest_to_file_record(bjcr, mjcr->FileId, digest, type)) {
+      Pmsg1(0, _("Could not add MD5/SHA1 to File record. ERR=%s\n"), db->strerror());
       free_jcr(mjcr);
       return false;
    }
@@ -1495,7 +1501,7 @@ static bool update_digest_record(B_DB *db, char *digest, DEV_RECORD *rec, int ty
    return true;
 }
 
-/*
+/**
  * Create a JCR as if we are really starting the job
  */
 static JCR *create_jcr(JOB_DBR *jr, DEV_RECORD *rec, uint32_t JobId)
