@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,9 +21,11 @@
    02110-1301, USA.
 */
 /*
- * dev.c  -- low level operations on device (storage device)
- *
  * Kern Sibbald, MM
+ */
+/**
+ * @file
+ * low level operations on device (storage device)
  *
  * NOTE!!!! None of these routines are reentrant. You must
  * use dev->rLock() and dev->Unlock() at a higher level,
@@ -78,7 +80,8 @@
 #include "backends/gfapi_device.h"
 #endif
 #ifdef HAVE_OBJECTSTORE
-#include "backends/object_store_device.h"
+#include "backends/chunked_device.h"
+#include "backends/droplet_device.h"
 #endif
 #ifdef HAVE_RADOS
 #include "backends/rados_device.h"
@@ -161,8 +164,8 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
          device->dev_type = B_FIFO_DEV;
       } else if (!bit_is_set(CAP_REQMOUNT, device->cap_bits)) {
          Jmsg2(jcr, M_ERROR, 0,
-               _("%s is an unknown device type. Must be tape or directory, st_mode=%x\n"),
-               device->device_name, statp.st_mode);
+               _("%s is an unknown device type. Must be tape or directory, st_mode=%04o\n"),
+               device->device_name, (statp.st_mode & ~S_IFMT));
          return NULL;
       }
    }
@@ -182,8 +185,8 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
       break;
 #endif
 #ifdef HAVE_OBJECTSTORE
-   case B_OBJECT_STORE_DEV:
-      dev = New(object_store_device);
+   case B_DROPLET_DEV:
+      dev = New(droplet_device);
       break;
 #endif
 #ifdef HAVE_RADOS
@@ -256,7 +259,7 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
     */
    Mmsg(dev->prt_name, "\"%s\" (%s)", device->name(), device->device_name);
    Dmsg1(400, "Allocate dev=%s\n", dev->print_name());
-   copy_bits(CAP_MAX, device->cap_bits, dev->capabilities);
+   copy_set_bits(CAP_MAX, device->cap_bits, dev->capabilities);
 
    /*
     * current block sizes
@@ -398,7 +401,7 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
    return dev;
 }
 
-/*
+/**
  * Allocate and initialize the DEVICE structure
  * Note, if dev is non-NULL, it is already allocated,
  * thus we neither allocate it nor free it. This allows
@@ -416,7 +419,7 @@ DEVICE *init_dev(JCR *jcr, DEVRES *device)
    return dev;
 }
 
-/*
+/**
  * Set the block size of the device.
  * If the volume block size is zero, we set the max block size to what is
  * configured in the device resource i.e. dev->device->max_block_size.
@@ -490,7 +493,7 @@ void DEVICE::set_blocksizes(DCR *dcr) {
    }
 }
 
-/*
+/**
  * Set the block size of the device to the label_block_size
  * to read labels as we want to always use that blocksize when
  * writing volume labels
@@ -518,7 +521,7 @@ void DEVICE::set_label_blocksize(DCR *dcr)
    }
 }
 
-/*
+/**
  * Open the device with the operating system and
  * initialize buffer pointers.
  *
@@ -584,9 +587,9 @@ bool DEVICE::open(DCR *dcr, int omode)
    /*
     * Reset any important state info
     */
-   clone_bits(ST_MAX, preserve, state);
+   copy_set_bits(ST_MAX, preserve, state);
 
-   Dmsg2(100, "preserve=0x%x fd=%d\n", preserve, m_fd);
+   Dmsg2(100, "preserve=%08o fd=%d\n", preserve, m_fd);
 
    return m_fd >= 0;
 }
@@ -611,7 +614,7 @@ void DEVICE::set_mode(int mode)
    }
 }
 
-/*
+/**
  * Open a device.
  */
 void DEVICE::open_device(DCR *dcr, int omode)
@@ -651,7 +654,7 @@ void DEVICE::open_device(DCR *dcr, int omode)
    /*
     * If creating file, give 0640 permissions
     */
-   Dmsg3(100, "open disk: mode=%s open(%s, 0x%x, 0640)\n", mode_to_str(omode),
+   Dmsg3(100, "open disk: mode=%s open(%s, %08o, 0640)\n", mode_to_str(omode),
          archive_name.c_str(), oflags);
 
    if ((m_fd = d_open(archive_name.c_str(), oflags, 0640)) < 0) {
@@ -671,7 +674,7 @@ void DEVICE::open_device(DCR *dcr, int omode)
    Dmsg1(100, "open dev: disk fd=%d opened\n", m_fd);
 }
 
-/*
+/**
  * Rewind the device.
  *
  * Returns: true  on success
@@ -703,14 +706,14 @@ bool DEVICE::rewind(DCR *dcr)
    if (lseek(dcr, (boffset_t)0, SEEK_SET) < 0) {
       berrno be;
       dev_errno = errno;
-      Mmsg2(errmsg, _("lseek error on %s. ERR=%s.\n"), print_name(), be.bstrerror());
+      Mmsg2(errmsg, _("lseek error on %s. ERR=%s"), print_name(), be.bstrerror());
       return false;
    }
 
    return true;
 }
 
-/*
+/**
  * Called to indicate that we have just read an EOF from the device.
  */
 void DEVICE::set_ateof()
@@ -721,7 +724,7 @@ void DEVICE::set_ateof()
    block_num = 0;
 }
 
-/*
+/**
  * Called to indicate we are now at the end of the volume, and writing is not possible.
  */
 void DEVICE::set_ateot()
@@ -735,7 +738,7 @@ void DEVICE::set_ateot()
    clear_append();
 }
 
-/*
+/**
  * Position device to end of medium (end of data)
  *
  * Returns: true  on succes
@@ -783,7 +786,7 @@ bool DEVICE::eod(DCR *dcr)
    return false;
 }
 
-/*
+/**
  * Set the position of the device.
  *
  * Returns: true  on succes
@@ -882,7 +885,7 @@ void DEVICE::clear_slot()
    }
 }
 
-/*
+/**
  * Reposition the device to file, block
  *
  * Returns: false on failure
@@ -915,7 +918,7 @@ bool DEVICE::reposition(DCR *dcr, uint32_t rfile, uint32_t rblock)
    return true;
 }
 
-/*
+/**
  * Set to unload the current volume in the drive.
  */
 void DEVICE::set_unload()
@@ -926,7 +929,7 @@ void DEVICE::set_unload()
    }
 }
 
-/*
+/**
  * Clear volume header.
  */
 void DEVICE::clear_volhdr()
@@ -936,7 +939,7 @@ void DEVICE::clear_volhdr()
    setVolCatInfo(false);
 }
 
-/*
+/**
  * Close the device.
  */
 bool DEVICE::close(DCR *dcr)
@@ -1015,7 +1018,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Mount the device.
  * If timeout, wait until the mount command returns 0.
  * If !timeout, try to mount the device only once.
@@ -1050,7 +1053,7 @@ bool DEVICE::mount(DCR *dcr, int timeout)
    return retval;
 }
 
-/*
+/**
  * Unmount the device
  * If timeout, wait until the unmount command returns 0.
  * If !timeout, try to unmount the device only once.
@@ -1090,7 +1093,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Edit codes into (Un)MountCommand
  *  %% = %
  *  %a = archive device name
@@ -1140,7 +1143,7 @@ void DEVICE::edit_mount_codes(POOL_MEM &omsg, const char *imsg)
    }
 }
 
-/*
+/**
  * Return the last timer interval (ms) or 0 if something goes wrong
  */
 btime_t DEVICE::get_timer_count()
@@ -1152,7 +1155,7 @@ btime_t DEVICE::get_timer_count()
    return (temp > 0) ? temp : 0; /* take care of skewed clock */
 }
 
-/*
+/**
  * Read from device.
  */
 ssize_t DEVICE::read(void *buf, size_t len)
@@ -1175,7 +1178,7 @@ ssize_t DEVICE::read(void *buf, size_t len)
    return read_len;
 }
 
-/*
+/**
  * Write to device.
  */
 ssize_t DEVICE::write(const void *buf, size_t len)
@@ -1198,7 +1201,7 @@ ssize_t DEVICE::write(const void *buf, size_t len)
    return write_len;
 }
 
-/*
+/**
  * Return the resource name for the device
  */
 const char *DEVICE::name() const
@@ -1206,7 +1209,7 @@ const char *DEVICE::name() const
    return device->name();
 }
 
-/*
+/**
  * Free memory allocated for the device
  */
 void DEVICE::term()
@@ -1254,7 +1257,7 @@ void DEVICE::term()
    delete this;
 }
 
-/*
+/**
  * This routine initializes the device wait timers
  */
 void init_device_wait_timers(DCR *dcr)
@@ -1290,7 +1293,7 @@ void init_jcr_device_wait_timers(JCR *jcr)
    jcr->num_wait = 0;
 }
 
-/*
+/**
  * The dev timers are used for waiting on a particular device
  *
  * Returns: true if time doubled
