@@ -3,7 +3,7 @@
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,6 +21,10 @@
    02110-1301, USA.
 */
 /*
+ * Kern Sibbald, May MMI
+ */
+/**
+ * @file
  * This file handles accepting Director Commands
  *
  * Most Director commands are handled here, with the
@@ -35,8 +39,6 @@
  * we can do things to a blocked device. CAREFUL!!!!
  *
  * File daemon commands are handled in fd_cmds.c
- *
- * Kern Sibbald, May MMI
  */
 
 #include "bareos.h"
@@ -163,7 +165,7 @@ struct s_cmds {
    bool monitoraccess;                      /* set if monitors can access this cmd */
 };
 
-/*
+/**
  * The following are the recognized commands from the Director.
  *
  * Keywords are sorted first longest match when the keywords start with the same string.
@@ -174,30 +176,30 @@ static struct s_cmds cmds[] = {
    { "bootstrap", bootstrap_cmd, false },
    { "cancel", cancel_cmd, false },
    { ".die", die_cmd, false },
-   { "finish", finish_cmd, false },         /* End of backup */
-   { "JobId=", job_cmd, false },            /* Start Job */
-   { "label", label_cmd, false },           /* Label a tape */
-   { "listen", listen_cmd, false },         /* Listen for an incoming Storage Job */
+   { "finish", finish_cmd, false },         /**< End of backup */
+   { "JobId=", job_cmd, false },            /**< Start Job */
+   { "label", label_cmd, false },           /**< Label a tape */
+   { "listen", listen_cmd, false },         /**< Listen for an incoming Storage Job */
    { "mount", mount_cmd, false },
-   { "nextrun", nextrun_cmd, false },       /* Prepare for next backup/restore part of same Job */
+   { "nextrun", nextrun_cmd, false },       /**< Prepare for next backup/restore part of same Job */
    { "passive", passive_cmd, false },
    { "pluginoptions", pluginoptions_cmd, false },
 // { "query", query_cmd, false },
    { "readlabel", readlabel_cmd, false },
-   { "relabel", relabel_cmd, false },       /* Relabel a tape */
+   { "relabel", relabel_cmd, false },       /**< Relabel a tape */
    { "release", release_cmd, false },
    { "resolve", resolve_cmd, false },
-   { "replicate", replicate_cmd, false },   /* Replicate data to an external SD */
-   { "run", run_cmd, false },               /* Start of Job */
+   { "replicate", replicate_cmd, false },   /**< Replicate data to an external SD */
+   { "run", run_cmd, false },               /**< Start of Job */
    { "getSecureEraseCmd", secureerasereq_cmd, false },
    { "setbandwidth=", setbandwidth_cmd, false },
-   { "setdebug=", setdebug_cmd, false },    /* Set debug level */
+   { "setdebug=", setdebug_cmd, false },    /**< Set debug level */
    { "stats", stats_cmd, false },
    { "status", status_cmd, true },
    { ".status", dotstatus_cmd, true },
    { "unmount", unmount_cmd, false },
    { "use storage=", use_cmd, false },
-   { NULL, NULL, false } /* list terminator */
+   { NULL, NULL, false } /**< list terminator */
 };
 
  /*
@@ -216,7 +218,7 @@ static inline bool count_running_jobs()
    return (cnt >= me->MaxConcurrentJobs) ? false : true;
 }
 
-/*
+/**
  * Connection request from an director.
  *
  * Basic tasks done here:
@@ -336,7 +338,7 @@ bail_out:
    return NULL;
 }
 
-/*
+/**
  * Force SD to die, and hopefully dump itself.  Turned on only in development version.
  */
 static bool die_cmd(JCR *jcr)
@@ -361,7 +363,7 @@ static bool die_cmd(JCR *jcr)
 }
 
 
-/*
+/**
  * Handles the secureerase request
  * replies the configured secure erase command
  * or "*None*"
@@ -411,7 +413,7 @@ static bool setbandwidth_cmd(JCR *jcr)
    return dir->fsend(OKBandwidth);
 }
 
-/*
+/**
  * Set debug level as requested by the Director
  */
 static bool setdebug_cmd(JCR *jcr)
@@ -445,7 +447,7 @@ static bool setdebug_cmd(JCR *jcr)
    }
 }
 
-/*
+/**
  * Cancel a Job
  *   Be careful, we switch to using the job's JCR! So, using
  *   BSOCKs on that jcr can have two threads in the same code.
@@ -526,7 +528,7 @@ static bool cancel_cmd(JCR *cjcr)
     * When canceling a NDMP job make sure we call the end_of_ndmp_* functions.
     */
    switch (jcr->getJobProtocol()) {
-   case PT_NDMP:
+   case PT_NDMP_BAREOS:
       switch (jcr->getJobType()) {
       case JT_BACKUP:
          end_of_ndmp_backup(jcr);
@@ -540,6 +542,7 @@ static bool cancel_cmd(JCR *cjcr)
    }
 
    pthread_cond_signal(&jcr->job_end_wait); /* wake waiting job */
+   jcr->my_thread_send_signal(TIMEOUT_SIGNAL);
 
    dir->fsend(_("3000 JobId=%ld Job=\"%s\" marked to be %s.\n"), jcr->JobId, jcr->Job, reason);
    free_jcr(jcr);
@@ -549,7 +552,7 @@ bail_out:
    return true;
 }
 
-/*
+/**
  * Resolve a hostname
  */
 static bool resolve_cmd(JCR *jcr)
@@ -672,7 +675,7 @@ static bool do_label(JCR *jcr, bool relabel)
    return true;
 }
 
-/*
+/**
  * Label a Volume
  */
 static bool label_cmd(JCR *jcr)
@@ -685,7 +688,7 @@ static bool relabel_cmd(JCR *jcr)
    return do_label(jcr, true);
 }
 
-/*
+/**
  * Read the tape label and determine if we can safely
  * label the tape (not a Bareos volume), then label it.
  *
@@ -809,7 +812,7 @@ bail_out:
    return;
 }
 
-/*
+/**
  * Read the tape label
  *
  *  Enter with the mutex set
@@ -842,7 +845,7 @@ static bool read_label(DCR *dcr)
    return ok;
 }
 
-/*
+/**
  * Searches for device by name, and if found, creates a dcr and returns it.
  */
 static DCR *find_device(JCR *jcr, POOL_MEM &devname, drive_number_t drive, BLOCKSIZES *blocksizes)
@@ -921,7 +924,7 @@ static DCR *find_device(JCR *jcr, POOL_MEM &devname, drive_number_t drive, BLOCK
    return dcr;
 }
 
-/*
+/**
  * Mount command from Director
  */
 static bool mount_cmd(JCR *jcr)
@@ -968,7 +971,7 @@ static bool mount_cmd(JCR *jcr)
             }
             /* We freed the device, so reopen it and wake any waiting threads */
             if (!dev->open(dcr, OPEN_READ_ONLY)) {
-               dir->fsend(_("3901 Unable to open device \"%s\": ERR=%s\n"), dev->print_name(), dev->bstrerror());
+               dir->fsend(_("3901 Unable to open device %s: ERR=%s\n"), dev->print_name(), dev->bstrerror());
                if (dev->blocked() == BST_UNMOUNTED) {
                   /* We blocked the device, so unblock it */
                   Dmsg0(100, "Unmounted. Unblocking device\n");
@@ -987,9 +990,9 @@ static bool mount_cmd(JCR *jcr)
                dev->set_blocked(BST_MOUNT);
             }
             if (dev->is_labeled()) {
-               dir->fsend(_("3001 Device \"%s\" is mounted with Volume \"%s\"\n"), dev->print_name(), dev->VolHdr.VolumeName);
+               dir->fsend(_("3001 Device %s is mounted with Volume \"%s\"\n"), dev->print_name(), dev->VolHdr.VolumeName);
             } else {
-               dir->fsend(_("3905 Device \"%s\" open but no Bareos volume is mounted.\n"
+               dir->fsend(_("3905 Device %s open but no Bareos volume is mounted.\n"
                             "If this is not a blank tape, try unmounting and remounting the Volume.\n"), dev->print_name());
             }
             pthread_cond_broadcast(&dev->wait_next_vol);
@@ -998,11 +1001,11 @@ static bool mount_cmd(JCR *jcr)
             break;
 
          case BST_DOING_ACQUIRE:
-            dir->fsend(_("3001 Device \"%s\" is doing acquire.\n"), dev->print_name());
+            dir->fsend(_("3001 Device %s is doing acquire.\n"), dev->print_name());
             break;
 
          case BST_WRITING_LABEL:
-            dir->fsend(_("3903 Device \"%s\" is being labeled.\n"), dev->print_name());
+            dir->fsend(_("3903 Device %s is being labeled.\n"), dev->print_name());
             break;
 
          case BST_NOT_BLOCKED:
@@ -1012,21 +1015,21 @@ static bool mount_cmd(JCR *jcr)
             }
             if (dev->is_open()) {
                if (dev->is_labeled()) {
-                  dir->fsend(_("3001 Device \"%s\" is mounted with Volume \"%s\"\n"), dev->print_name(), dev->VolHdr.VolumeName);
+                  dir->fsend(_("3001 Device %s is mounted with Volume \"%s\"\n"), dev->print_name(), dev->VolHdr.VolumeName);
                } else {
-                  dir->fsend(_("3905 Device \"%s\" open but no Bareos volume is mounted.\n"
+                  dir->fsend(_("3905 Device %s open but no Bareos volume is mounted.\n"
                                "If this is not a blank tape, try unmounting and remounting the Volume.\n"), dev->print_name());
                }
             } else if (dev->is_tape()) {
                if (!dev->open(dcr, OPEN_READ_ONLY)) {
-                  dir->fsend(_("3901 Unable to open device \"%s\": ERR=%s\n"), dev->print_name(), dev->bstrerror());
+                  dir->fsend(_("3901 Unable to open device %s: ERR=%s\n"), dev->print_name(), dev->bstrerror());
                   break;
                }
                read_label(dcr);
                if (dev->is_labeled()) {
-                  dir->fsend(_("3001 Device \"%s\" is already mounted with Volume \"%s\"\n"), dev->print_name(), dev->VolHdr.VolumeName);
+                  dir->fsend(_("3001 Device %s is already mounted with Volume \"%s\"\n"), dev->print_name(), dev->VolHdr.VolumeName);
                } else {
-                  dir->fsend(_("3905 Device \"%s\" open but no Bareos volume is mounted.\n"
+                  dir->fsend(_("3905 Device %s open but no Bareos volume is mounted.\n"
                                "If this is not a blank tape, try unmounting and remounting the Volume.\n"), dev->print_name());
                }
                if (dev->is_open() && !dev->has_cap(CAP_ALWAYSOPEN)) {
@@ -1034,12 +1037,12 @@ static bool mount_cmd(JCR *jcr)
                }
             } else if (dev->is_unmountable()) {
                if (dev->mount(dcr, 1)) {
-                  dir->fsend(_("3002 Device \"%s\" is mounted.\n"), dev->print_name());
+                  dir->fsend(_("3002 Device %s is mounted.\n"), dev->print_name());
                } else {
                   dir->fsend(_("3907 %s"), dev->bstrerror());
                }
             } else { /* must be file */
-               dir->fsend(_("3906 File device \"%s\" is always mounted.\n"), dev->print_name());
+               dir->fsend(_("3906 File device %s is always mounted.\n"), dev->print_name());
                pthread_cond_broadcast(&dev->wait_next_vol);
                Dmsg1(100, "JobId=%u broadcast wait_device_release\n", (uint32_t)dcr->jcr->JobId);
                release_device_cond();
@@ -1047,7 +1050,7 @@ static bool mount_cmd(JCR *jcr)
             break;
 
          case BST_RELEASING:
-            dir->fsend(_("3930 Device \"%s\" is being released.\n"), dev->print_name());
+            dir->fsend(_("3930 Device %s is being released.\n"), dev->print_name());
             break;
 
          default:
@@ -1057,7 +1060,7 @@ static bool mount_cmd(JCR *jcr)
          dev->Unlock();
          free_dcr(dcr);
       } else {
-         dir->fsend(_("3999 Device \"%s\" not found or could not be opened.\n"), devname.c_str());
+         dir->fsend(_("3999 Device %s not found or could not be opened.\n"), devname.c_str());
       }
    } else {
       pm_strcpy(jcr->errmsg, dir->msg);
@@ -1067,7 +1070,7 @@ static bool mount_cmd(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * unmount command from Director
  */
 static bool unmount_cmd(JCR *jcr)
@@ -1156,7 +1159,7 @@ static bool unmount_cmd(JCR *jcr)
 }
 
 #if 0
-/*
+/**
  * The truncate command will recycle a volume. The director can call this
  * after purging a volume so that disk space will not be wasted. Only useful
  * for File Storage, of course.
@@ -1193,7 +1196,7 @@ done:
 }
 #endif
 
-/*
+/**
  * Release command from Director. This rewinds the device and if
  *   configured does a offline and ensures that Bareos will
  *   re-read the label of the tape before continuing. This gives
@@ -1326,7 +1329,7 @@ static bool bootstrap_cmd(JCR *jcr)
    return get_bootstrap_file(jcr, jcr->dir_bsock);
 }
 
-/*
+/**
  * Autochanger command from Director
  */
 static bool changer_cmd(JCR *jcr)
@@ -1399,7 +1402,7 @@ static bool changer_cmd(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * Read and return the Volume label
  */
 static bool readlabel_cmd(JCR *jcr)
@@ -1440,7 +1443,7 @@ static bool readlabel_cmd(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * Read the tape label
  *
  *  Enter with the mutex set
@@ -1480,7 +1483,7 @@ bail_out:
    return;
 }
 
-/*
+/**
  * Try autoloading a device.
  *
  * Returns: 1 on success
@@ -1553,7 +1556,7 @@ static inline void set_storage_auth_key(JCR *jcr, char *key)
    Dmsg0(5, "set sd auth key\n");
 }
 
-/*
+/**
  * Listen for incoming replication session from other SD.
  */
 static bool listen_cmd(JCR *jcr)
@@ -1563,7 +1566,7 @@ static bool listen_cmd(JCR *jcr)
    return do_listen_run(jcr);
 }
 
-/*
+/**
  * Get address of storage daemon from Director
  */
 static bool replicate_cmd(JCR *jcr)
