@@ -2,6 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2016-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -19,10 +20,12 @@
    02110-1301, USA.
 */
 /*
- * Definitions for locking and blocking functions in the SD
- *
  * Kern Sibbald, pulled out of dev.h June 2007
  */
+/**
+ * @file
+ * Definitions for locking and blocking functions in the SD
+ */
 
 #ifndef __LOCK_H
 #define __LOCK_H 1
@@ -39,26 +42,28 @@
 #define steal_device_lock(d, p, s)  _steal_device_lock(__FILE__, __LINE__, (d), (p), s)
 #define give_back_device_lock(d, p) _give_back_device_lock(__FILE__, __LINE__, (d), (p))
 
-/* m_blocked states (mutually exclusive) */
+/**
+ * m_blocked states (mutually exclusive)
+ */
 enum {
-   BST_NOT_BLOCKED = 0,               /* Not blocked */
-   BST_UNMOUNTED,                     /* User unmounted device */
-   BST_WAITING_FOR_SYSOP,             /* Waiting for operator to mount tape */
-   BST_DOING_ACQUIRE,                 /* Opening/validating/moving tape */
-   BST_WRITING_LABEL,                 /* Labeling a tape */
-   BST_UNMOUNTED_WAITING_FOR_SYSOP,   /* User unmounted during wait for op */
-   BST_MOUNT,                         /* Mount request */
-   BST_DESPOOLING,                    /* Despooling -- i.e. multiple writes */
-   BST_RELEASING                      /* Releasing the device */
+   BST_NOT_BLOCKED = 0,               /**< Not blocked */
+   BST_UNMOUNTED,                     /**< User unmounted device */
+   BST_WAITING_FOR_SYSOP,             /**< Waiting for operator to mount tape */
+   BST_DOING_ACQUIRE,                 /**< Opening/validating/moving tape */
+   BST_WRITING_LABEL,                 /**< Labeling a tape */
+   BST_UNMOUNTED_WAITING_FOR_SYSOP,   /**< User unmounted during wait for op */
+   BST_MOUNT,                         /**< Mount request */
+   BST_DESPOOLING,                    /**< Despooling -- i.e. multiple writes */
+   BST_RELEASING                      /**< Releasing the device */
 };
 
 typedef struct s_steal_lock {
-   pthread_t  no_wait_id;             /* id of no wait thread */
-   int        dev_blocked;            /* state */
-   int        dev_prev_blocked;       /* previous blocked state */
+   pthread_t  no_wait_id;             /**< id of no wait thread */
+   int        dev_blocked;            /**< state */
+   int        dev_prev_blocked;       /**< previous blocked state */
 } bsteal_lock_t;
 
-/*
+/**
  * Used in unblock() call
  */
 enum {
