@@ -3,7 +3,7 @@
 
    Copyright (C) 2006-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2017 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,11 +21,13 @@
    02110-1301, USA.
 */
 /*
- * SD -- mac.c --  responsible for doing
- * migration, archive, copy, and virtual backup jobs.
- *
  * Kern Sibbald, January MMVI
  */
+/**
+ * @file
+ * responsible for doing
+ * migration, archive, copy, and virtual backup jobs.
+ */
 
 #include "bareos.h"
 #include "stored.h"
@@ -34,13 +36,13 @@
 
 /* Forward referenced subroutines */
 
-/*
+/**
  * Responses sent to the Director
  */
 static char Job_end[] =
    "3099 Job %s end JobStatus=%d JobFiles=%d JobBytes=%s JobErrors=%u\n";
 
-/*
+/**
  * Responses received from Storage Daemon
  */
 static char OK_start_replicate[] =
@@ -52,7 +54,7 @@ static char OK_replicate[] =
 static char OK_end_replicate[] =
    "3000 OK end replicate\n";
 
-/*
+/**
  * Commands sent to Storage Daemon
  */
 static char start_replicate[] =
@@ -62,7 +64,13 @@ static char replicate_data[] =
 static char end_replicate[] =
    "end replicate\n";
 
-/*
+
+/* get information from first original SOS label for our job */
+static bool found_first_sos_label = false;
+
+
+
+/**
  * Get response from Storage daemon to a command we sent.
  * Check that the response is OK.
  *
@@ -93,7 +101,7 @@ static bool response(JCR *jcr, BSOCK *sd, char *resp, const char *cmd)
    return false;
 }
 
-/*
+/**
  * Called here for each record from read_records()
  * This function is used when we do a internal clone of a Job e.g.
  * this SD is both the reading and writing SD.
@@ -103,6 +111,8 @@ static bool response(JCR *jcr, BSOCK *sd, char *resp, const char *cmd)
  */
 static bool clone_record_internally(DCR *dcr, DEV_RECORD *rec)
 {
+   bool retval = false;
+   bool translated_record = false;
    JCR *jcr = dcr->jcr;
    DEVICE *dev = jcr->dcr->dev;
    char buf1[100], buf2[100];
@@ -114,47 +124,92 @@ static bool clone_record_internally(DCR *dcr, DEV_RECORD *rec)
 #endif
 
    /*
-    * If label and not for us, discard it
+    * If label, discard it as we create our SOS and EOS Labels
+    * However, we still want the first Start Of Session label as that contains
+    * the timestamps from the first original Job.
+    *
+    * We need to put this info in our own SOS and EOS Labels so that
+    * bscan-ing of virtual and always incremental consolidated jobs
+    * works.
     */
-   if (rec->FileIndex < 0 && rec->match_stat <= 0) {
-      return true;
+
+   if (rec->FileIndex < 0) {
+      if (rec->FileIndex == SOS_LABEL) {
+         if (!found_first_sos_label) {
+            Dmsg0(200, "Found first SOS_LABEL and adopting job info\n");
+            found_first_sos_label = true;
+
+            static SESSION_LABEL the_label;
+            static SESSION_LABEL* label= &the_label;
+
+            struct date_time dt;
+            struct tm tm;
+
+            unser_session_label(label, rec);
+
+            /*
+             * set job info from first SOS label
+             */
+            jcr->setJobType(label->JobType);
+            jcr->setJobLevel(label->JobLevel);
+            Dmsg1(200, "joblevel from SOS_LABEL is now %c\n", label->JobLevel);
+            bstrncpy(jcr->Job, label->Job, sizeof(jcr->Job));
+
+            if (label->VerNum >= 11) {
+               jcr->sched_time = btime_to_unix(label->write_btime);
+
+            } else {
+               dt.julian_day_number = label->write_date;
+               dt.julian_day_fraction = label->write_time;
+               tm_decode(&dt, &tm);
+               tm.tm_isdst = 0;
+               jcr->sched_time = mktime(&tm);
+            }
+            jcr->start_time = jcr->sched_time;
+
+            /* write the SOS Label with the existing timestamp infos */
+            if (!write_session_label(jcr->dcr, SOS_LABEL)) {
+               Jmsg1(jcr, M_FATAL, 0, _("Write session label failed. ERR=%s\n"),
+                     dev->bstrerror());
+               jcr->setJobStatus(JS_ErrorTerminated);
+               retval = false;
+               goto bail_out;
+            }
+         } else {
+            Dmsg0(200, "Found additional SOS_LABEL, ignoring! \n");
+            retval = true;
+            goto bail_out;
+         }
+
+      } else  {
+         /* Other label than SOS -> skip */
+         retval = true;
+         goto bail_out;
+      }
    }
 
    /*
-    * We want to write SOS_LABEL and EOS_LABEL discard all others
+    * For normal migration jobs, FileIndex values are sequential because
+    *  we are dealing with one job.  However, for Vbackup (consolidation),
+    *  we will be getting records from multiple jobs and writing them back
+    *  out, so we need to ensure that the output FileIndex is sequential.
+    *  We do so by detecting a FileIndex change and incrementing the
+    *  JobFiles, which we then use as the output FileIndex.
     */
-   switch (rec->FileIndex) {
-   case PRE_LABEL:
-   case VOL_LABEL:
-   case EOT_LABEL:
-   case EOM_LABEL:
-      return true;                    /* don't write vol labels */
-   }
-
-//   if (jcr->is_JobType(JT_BACKUP)) {
+   if (rec->FileIndex >= 0) {
       /*
-       * For normal migration jobs, FileIndex values are sequential because
-       *  we are dealing with one job.  However, for Vbackup (consolidation),
-       *  we will be getting records from multiple jobs and writing them back
-       *  out, so we need to ensure that the output FileIndex is sequential.
-       *  We do so by detecting a FileIndex change and incrementing the
-       *  JobFiles, which we then use as the output FileIndex.
+       * If something changed, increment FileIndex
        */
-      if (rec->FileIndex >= 0) {
-         /*
-          * If something changed, increment FileIndex
-          */
-         if (rec->VolSessionId != rec->last_VolSessionId ||
-             rec->VolSessionTime != rec->last_VolSessionTime ||
-             rec->FileIndex != rec->last_FileIndex) {
-            jcr->JobFiles++;
-            rec->last_VolSessionId = rec->VolSessionId;
-            rec->last_VolSessionTime = rec->VolSessionTime;
-            rec->last_FileIndex = rec->FileIndex;
-         }
-         rec->FileIndex = jcr->JobFiles;     /* set sequential output FileIndex */
+      if (rec->VolSessionId != rec->last_VolSessionId ||
+            rec->VolSessionTime != rec->last_VolSessionTime ||
+            rec->FileIndex != rec->last_FileIndex) {
+         jcr->JobFiles++;
+         rec->last_VolSessionId = rec->VolSessionId;
+         rec->last_VolSessionTime = rec->VolSessionTime;
+         rec->last_FileIndex = rec->FileIndex;
       }
-//   }
+      rec->FileIndex = jcr->JobFiles;     /* set sequential output FileIndex */
+   }
 
    /*
     * Modify record SessionId and SessionTime to correspond to output.
@@ -166,15 +221,35 @@ static bool clone_record_internally(DCR *dcr, DEV_RECORD *rec)
          jcr->JobId, FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
          stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len);
 
-   while (!write_record_to_block(jcr->dcr, rec)) {
+   /*
+    * Perform record translations.
+    */
+   jcr->dcr->before_rec = rec;
+   jcr->dcr->after_rec = NULL;
+   if (generate_plugin_event(jcr, bsdEventWriteRecordTranslation, jcr->dcr) != bRC_OK) {
+      goto bail_out;
+   }
+
+   /*
+    * The record got translated when we got an after_rec pointer after calling the
+    * bsdEventWriteRecordTranslation plugin event. If no translation has taken place
+    * we just point the after_rec pointer to same DEV_RECORD as in the before_rec pointer.
+    */
+   if (!jcr->dcr->after_rec) {
+      jcr->dcr->after_rec = jcr->dcr->before_rec;
+   } else {
+      translated_record = true;
+   }
+
+   while (!write_record_to_block(jcr->dcr, jcr->dcr->after_rec)) {
       Dmsg4(200, "!write_record_to_block blkpos=%u:%u len=%d rem=%d\n",
-            dev->file, dev->block_num, rec->data_len, rec->remainder);
+            dev->file, dev->block_num, jcr->dcr->after_rec->data_len, jcr->dcr->after_rec->remainder);
       if (!jcr->dcr->write_block_to_device()) {
          Dmsg2(90, "Got write_block_to_dev error on device %s. %s\n",
             dev->print_name(), dev->bstrerror());
          Jmsg2(jcr, M_FATAL, 0, _("Fatal append error on device %s: ERR=%s\n"),
                dev->print_name(), dev->bstrerror());
-         return false;
+         goto bail_out;
       }
       Dmsg2(200, "===== Wrote block new pos %u:%u\n", dev->file, dev->block_num);
    }
@@ -182,24 +257,34 @@ static bool clone_record_internally(DCR *dcr, DEV_RECORD *rec)
    /*
     * Restore packet
     */
-   rec->VolSessionId = rec->last_VolSessionId;
-   rec->VolSessionTime = rec->last_VolSessionTime;
-   if (rec->FileIndex < 0) {
-      return true;                    /* don't send LABELs to Dir */
+   jcr->dcr->after_rec->VolSessionId = jcr->dcr->after_rec->last_VolSessionId;
+   jcr->dcr->after_rec->VolSessionTime = jcr->dcr->after_rec->last_VolSessionTime;
+
+   if (jcr->dcr->after_rec->FileIndex < 0) {
+      retval = true;                    /* don't send LABELs to Dir */
+      goto bail_out;
    }
 
-   jcr->JobBytes += rec->data_len;   /* increment bytes of this job */
+   jcr->JobBytes += jcr->dcr->after_rec->data_len;   /* increment bytes of this job */
 
    Dmsg5(500, "wrote_record JobId=%d FI=%s SessId=%d Strm=%s len=%d\n",
-         jcr->JobId, FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
-         stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len);
+         jcr->JobId, FI_to_ascii(buf1, jcr->dcr->after_rec->FileIndex), jcr->dcr->after_rec->VolSessionId,
+         stream_to_ascii(buf2, jcr->dcr->after_rec->Stream, jcr->dcr->after_rec->FileIndex), jcr->dcr->after_rec->data_len);
 
-   send_attrs_to_dir(jcr, rec);
+   send_attrs_to_dir(jcr, jcr->dcr->after_rec);
 
-   return true;
+   retval = true;
+
+bail_out:
+   if (translated_record) {
+      free_record(jcr->dcr->after_rec);
+      jcr->dcr->after_rec = NULL;
+   }
+
+   return retval;
 }
 
-/*
+/**
  * Called here for each record from read_records()
  * This function is used when we do a external clone of a Job e.g.
  * this SD is the reading SD. And a remote SD is the writing SD.
@@ -331,14 +416,14 @@ static bool clone_record_to_remote_sd(DCR *dcr, DEV_RECORD *rec)
    jcr->JobBytes += sd->msglen;
    sd->msg = msgsave;
 
-   Dmsg5(500, "wrote_record JobId=%d FI=%s SessId=%d Strm=%s len=%d\n",
+   Dmsg5(200, "wrote_record JobId=%d FI=%s SessId=%d Strm=%s len=%d\n",
          jcr->JobId, FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
          stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len);
 
    return true;
 }
 
-/*
+/**
  * Check autoinflation/autodeflation settings.
  */
 static inline void check_auto_xflation(JCR *jcr)
@@ -402,17 +487,17 @@ static inline void check_auto_xflation(JCR *jcr)
    }
 }
 
-/*
+/**
  * Read Data and commit to new job.
  */
 bool do_mac_run(JCR *jcr)
 {
-   DEVICE *dev;
    utime_t now;
    char ec1[50];
    const char *Type;
    bool ok = true;
    BSOCK *dir = jcr->dir_bsock;
+   DEVICE *dev = jcr->dcr->dev;
 
    switch(jcr->getJobType()) {
    case JT_MIGRATE:
@@ -564,8 +649,9 @@ bool do_mac_run(JCR *jcr)
       /* Inform Storage daemon that we are done */
       sd->signal(BNET_TERMINATE);
    } else {
-      if (!jcr->read_dcr || !jcr->dcr) {
-         Jmsg(jcr, M_FATAL, 0, _("Read and write devices not properly initialized.\n"));
+
+      if (!jcr->read_dcr) {
+         Jmsg(jcr, M_FATAL, 0, _("Read device not properly initialized.\n"));
          goto bail_out;
       }
 
@@ -625,9 +711,36 @@ bail_out:
        */
       int32_t job_elapsed;
 
-      dev = jcr->dcr->dev;
       Dmsg1(100, "ok=%d\n", ok);
+
       if (ok || dev->can_write()) {
+
+         /*
+            memorize current JobStatus and set to
+            JS_Terminated to write into EOS_LABEL
+          */
+         char currentJobStatus = jcr->JobStatus;
+         jcr->setJobStatus(JS_Terminated);
+
+         /*
+          * Write End Of Session Label
+          */
+         DCR *dcr = jcr->dcr;
+         if (!write_session_label(dcr, EOS_LABEL)) {
+            /*
+             * Print only if ok and not cancelled to avoid spurious messages
+             */
+
+            if (ok && !jcr->is_job_canceled()) {
+               Jmsg1(jcr, M_FATAL, 0, _("Error writing end session label. ERR=%s\n"),
+                     dev->bstrerror());
+            }
+            jcr->setJobStatus(JS_ErrorTerminated);
+            ok = false;
+         } else {
+            /* restore JobStatus */
+            jcr->setJobStatus(currentJobStatus);
+         }
          /*
           * Flush out final partial block of this session
           */
@@ -638,8 +751,10 @@ bail_out:
             ok = false;
          }
          Dmsg2(200, "Flush block to device pos %u:%u\n", dev->file, dev->block_num);
+
       }
 
+
       if (!ok) {
          discard_data_spool(jcr->dcr);
       } else {
