@@ -2,6 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
+   Copyright (C) 2016-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -19,16 +20,18 @@
    02110-1301, USA.
 */
 /*
- * record.c -- Volume (tape/disk) record handling functions
- *
  * Kern Sibbald, April MMI
  * added BB02 format October MMII
  */
+/**
+ * @file
+ * Volume (tape/disk) record handling functions
+ */
 
 #include "bareos.h"
 #include "stored.h"
 
-/*
+/**
  * Convert a FileIndex into a printable
  * ASCII string.  Not reentrant.
  *
@@ -68,7 +71,128 @@ const char *FI_to_ascii(char *buf, int fi)
    }
 }
 
-/*
+
+static const char *compression_to_str(POOL_MEM &resultbuffer,
+                                      const char *compression_algorithm,
+                                      uint32_t data_length,
+                                      uint16_t compression_level,
+                                      uint16_t compression_algorithm_version)
+{
+   POOL_MEM tmp(PM_MESSAGE);
+   tmp.bsprintf("%s, level=%u, version=%u, length=%u",
+                compression_algorithm, compression_level,
+                compression_algorithm_version, data_length);
+   resultbuffer.strcat(tmp);
+   return resultbuffer.c_str();
+}
+
+static const char *record_compression_to_str(POOL_MEM &resultbuffer, const DEV_RECORD *rec)
+{
+   int32_t maskedStream = rec->maskedStream;
+   POOLMEM *buf = rec->data;
+   POOL_MEM tmp(PM_MESSAGE);
+   unser_declare;
+
+   if (maskedStream == STREAM_SPARSE_GZIP_DATA ||
+       maskedStream == STREAM_SPARSE_COMPRESSED_DATA) {
+      uint64_t faddr = 0;
+
+      ser_begin(buf, sizeof(uint64_t));
+      unser_uint64(faddr);
+      ser_end(buf, sizeof(uint64_t));
+
+      buf += sizeof(uint64_t);
+
+      Dmsg1(400, "Sparse data stream found: start address=%llu\n", faddr);
+      tmp.bsprintf("Sparse: StartAddress=%llu. ", faddr);
+      resultbuffer.strcat(tmp);
+   }
+
+   Dmsg1(400, "Stream found in decompress_data(): %d\n", maskedStream);
+   switch (maskedStream) {
+   case STREAM_COMPRESSED_DATA:
+   case STREAM_SPARSE_COMPRESSED_DATA:
+   case STREAM_WIN32_COMPRESSED_DATA:
+   case STREAM_ENCRYPTED_FILE_COMPRESSED_DATA:
+   case STREAM_ENCRYPTED_WIN32_COMPRESSED_DATA: {
+      uint32_t comp_magic, comp_len;
+      uint16_t comp_level, comp_version;
+
+      /*
+       * Read compress header
+       */
+      unser_begin(buf, sizeof(comp_stream_header));
+      unser_uint32(comp_magic);
+      unser_uint32(comp_len);
+      unser_uint16(comp_level);
+      unser_uint16(comp_version);
+      unser_end(buf, sizeof(comp_stream_header));
+
+      Dmsg4(400, "Compressed data stream found: magic=0x%x, len=%d, level=%d, ver=0x%x\n",
+            comp_magic, comp_len, comp_level, comp_version);
+
+      switch (comp_magic) {
+      case COMPRESS_GZIP:
+         compression_to_str(resultbuffer, "GZIP", comp_len, comp_level, comp_version);
+         break;
+      case COMPRESS_LZO1X:
+         compression_to_str(resultbuffer, "LZO1X", comp_len, comp_level, comp_version);
+         break;
+      case COMPRESS_FZFZ:
+         compression_to_str(resultbuffer, "FZFZ", comp_len, comp_level, comp_version);
+         break;
+      case COMPRESS_FZ4L:
+         compression_to_str(resultbuffer, "FZ4L", comp_len, comp_level, comp_version);
+         break;
+      case COMPRESS_FZ4H:
+         compression_to_str(resultbuffer, "FZ4H", comp_len, comp_level, comp_version);
+         break;
+      default:
+         tmp.bsprintf(_("Compression algorithm 0x%x found, but not supported!\n"), comp_magic);
+         resultbuffer.strcat(tmp);
+         break;
+      }
+      break;
+   }
+   case STREAM_GZIP_DATA:
+   case STREAM_SPARSE_GZIP_DATA:
+      /* deprecated */
+      compression_to_str(resultbuffer, "GZIP", 0, 0, 0);
+      break;
+   default:
+      break;
+   }
+
+   return resultbuffer.c_str();
+}
+
+static const char *record_digest_to_str(POOL_MEM &resultbuffer, const DEV_RECORD *rec)
+{
+   char digest[BASE64_SIZE(CRYPTO_DIGEST_MAX_SIZE)];
+
+   switch (rec->maskedStream) {
+      case STREAM_MD5_DIGEST:
+         bin_to_base64(digest, sizeof(digest), (char *)rec->data, CRYPTO_DIGEST_MD5_SIZE, true);
+         break;
+      case STREAM_SHA1_DIGEST:
+         bin_to_base64(digest, sizeof(digest), (char *)rec->data, CRYPTO_DIGEST_SHA1_SIZE, true);
+         break;
+      case STREAM_SHA256_DIGEST:
+         bin_to_base64(digest, sizeof(digest), (char *)rec->data, CRYPTO_DIGEST_SHA256_SIZE, true);
+         break;
+      case STREAM_SHA512_DIGEST:
+         bin_to_base64(digest, sizeof(digest), (char *)rec->data, CRYPTO_DIGEST_SHA512_SIZE, true);
+         break;
+      default:
+         return "";
+   }
+
+   resultbuffer.bsprintf("%s (base64)", digest);
+
+   return resultbuffer.c_str();
+}
+
+/**
  * Convert a Stream ID into a printable
  * ASCII string. Not reentrant.
  *
@@ -256,6 +380,78 @@ static const char *findex_to_str(int32_t index, char *buf, size_t bufsz)
    return buf;
 }
 
+
+static const char *record_unix_attributes_to_str(POOL_MEM &resultbuffer, JCR *jcr, const DEV_RECORD *rec)
+{
+   ATTR *attr = new_attr(NULL);
+
+   if (!unpack_attributes_record(jcr, rec->Stream, rec->data, rec->data_len, attr)) {
+      resultbuffer.bsprintf("ERROR");
+      return NULL;
+   }
+
+   attr->data_stream = decode_stat(attr->attr, &attr->statp, sizeof(attr->statp), &attr->LinkFI);
+   build_attr_output_fnames(jcr, attr);
+   attr_to_str(resultbuffer, jcr, attr);
+
+   free_attr(attr);
+
+   return resultbuffer.c_str();
+}
+
+
+static const char *get_record_short_info(POOL_MEM &resultbuffer, JCR *jcr, const DEV_RECORD *rec)
+{
+   switch (rec->maskedStream) {
+   case STREAM_UNIX_ATTRIBUTES:
+   case STREAM_UNIX_ATTRIBUTES_EX:
+      record_unix_attributes_to_str(resultbuffer, jcr, rec);
+      break;
+   case STREAM_MD5_DIGEST:
+   case STREAM_SHA1_DIGEST:
+   case STREAM_SHA256_DIGEST:
+   case STREAM_SHA512_DIGEST:
+      record_digest_to_str(resultbuffer, rec);
+      break;
+   case STREAM_PLUGIN_NAME: {
+      char data[100];
+      int len = MIN(rec->data_len+1, sizeof(data));
+      bstrncpy(data, rec->data, len);
+      resultbuffer.bsprintf("data: %s\n", data);
+      break;
+   }
+   case STREAM_RESTORE_OBJECT:
+      resultbuffer.bsprintf("Restore Object record");
+      break;
+   case STREAM_COMPRESSED_DATA:
+   case STREAM_SPARSE_COMPRESSED_DATA:
+   case STREAM_WIN32_COMPRESSED_DATA:
+   case STREAM_ENCRYPTED_FILE_COMPRESSED_DATA:
+   case STREAM_ENCRYPTED_WIN32_COMPRESSED_DATA:
+   case STREAM_GZIP_DATA: /* deprecated */
+   case STREAM_SPARSE_GZIP_DATA: /* deprecated */
+      record_compression_to_str(resultbuffer, rec);
+      break;
+   default:
+      break;
+   }
+   return resultbuffer.c_str();
+}
+
+const char *record_to_str(POOL_MEM &resultbuffer, JCR *jcr, const DEV_RECORD *rec)
+{
+   POOL_MEM record_info_buf(PM_MESSAGE);
+   char stream_buf[100];
+
+   resultbuffer.bsprintf("FileIndex=%-5d Stream=%-2d %-25s DataLen=%-5d",
+                         rec->FileIndex, rec->Stream,
+                         stream_to_ascii(stream_buf, rec->Stream, rec->FileIndex),
+                         rec->data_len);
+   indent_multiline_string(resultbuffer, get_record_short_info(record_info_buf, jcr, rec), " | ");
+
+   return resultbuffer.c_str();
+}
+
 void dump_record(const char *tag, const DEV_RECORD *rec)
 {
    char stream[128];
@@ -287,7 +483,7 @@ void dump_record(const char *tag, const DEV_RECORD *rec)
    Dmsg2(100, "%-14s %s\n", "own_mempool", rec->own_mempool ? "true" : "false");
 }
 
-/*
+/**
  * Return a new record entity
  */
 DEV_RECORD *new_record(bool with_data)
@@ -345,7 +541,7 @@ void copy_record_state(DEV_RECORD *dst, DEV_RECORD *src)
    dst->own_mempool = own_mempool;
 }
 
-/*
+/**
  * Free the record entity
  */
 void free_record(DEV_RECORD *rec)
@@ -434,7 +630,7 @@ static inline ssize_t write_data_to_block(DEV_BLOCK *block, const DEV_RECORD *re
    return len;
 }
 
-/*
+/**
  * Write a Record to the block
  *
  * Returns: false means the block could not be written to tape/disk.
@@ -498,7 +694,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Write a Record to the block
  *
  *  Returns: false on failure (none or partially written)
@@ -652,7 +848,7 @@ bail_out:
    return retval;
 }
 
-/*
+/**
  * Test if we can write whole record to the block
  *
  *  Returns: false on failure
@@ -668,7 +864,7 @@ uint64_t get_record_address(const DEV_RECORD *rec)
    return ((uint64_t)rec->File)<<32 | rec->Block;
 }
 
-/*
+/**
  * Read a Record from the block
  *
  * Returns: false if nothing read or if the continuation record does not match.
