@@ -1,7 +1,7 @@
 /*
    BAREOS® - Backup Archiving REcovery Open Sourced
 
-   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -18,7 +18,8 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 */
-/*
+/**
+ * @file
  * Storage Daemon statistics gatherer.
  *
  * Written by Marco van Wieringen and Philipp Storz, November 2013
@@ -292,7 +293,7 @@ void update_job_statistics(JCR *jcr, utime_t now)
    job_stat->timestamp = now;
    job_stat->JobFiles = jcr->JobFiles;
    job_stat->JobBytes = jcr->JobBytes;
-   if (jcr->dcr) {
+   if (jcr->dcr && jcr->dcr->device) {
       job_stat->DevName = bstrdup(jcr->dcr->device->name());
    } else {
       job_stat->DevName = bstrdup("unknown");
@@ -340,7 +341,7 @@ static inline void cleanup_cached_statistics()
    V(mutex);
 }
 
-/*
+/**
  * Entry point for a separate statistics thread.
  */
 extern "C"
