@@ -3,7 +3,7 @@
 
    Copyright (C) 2003-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -21,10 +21,12 @@
    02110-1301, USA.
 */
 /*
- * This file handles the status command
- *
  * Kern Sibbald, May MMIII
  */
+/**
+ * @file
+ * This file handles the status command
+ */
 
 #include "bareos.h"
 #include "stored.h"
@@ -52,10 +54,6 @@ static void sendit(const char *msg, int len, STATUS_PKT *sp);
 static void sendit(POOL_MEM &msg, int len, STATUS_PKT *sp);
 static void sendit(const char *msg, int len, void *arg);
 
-static void trigger_device_status_hook(JCR *jcr,
-                                       DEVRES *device,
-                                       STATUS_PKT *sp,
-                                       bsdEventType eventType);
 static void send_blocked_status(DEVICE *dev, STATUS_PKT *sp);
 static void send_device_status(DEVICE *dev, STATUS_PKT *sp);
 static void list_terminated_jobs(STATUS_PKT *sp);
@@ -63,10 +61,11 @@ static void list_running_jobs(STATUS_PKT *sp);
 static void list_jobs_waiting_on_reservation(STATUS_PKT *sp);
 static void list_status_header(STATUS_PKT *sp);
 static void list_devices(JCR *jcr, STATUS_PKT *sp, const char *devicenames);
+static void list_volumes(STATUS_PKT *sp, const char *devicenames);
 
 static const char *level_to_str(int level);
 
-/*
+/**
  * Status command from Director
  */
 static void output_status(JCR *jcr, STATUS_PKT *sp, const char *devicenames)
@@ -101,7 +100,7 @@ static void output_status(JCR *jcr, STATUS_PKT *sp, const char *devicenames)
       sendit(msg, len, sp);
    }
 
-   list_volumes(sendit, (void *)sp);
+   list_volumes(sp, devicenames);
    if (!sp->api) {
       len = pm_strcpy(msg, "====\n\n");
       sendit(msg, len, sp);
@@ -163,6 +162,9 @@ static bool need_to_list_device(const char *devicenames, const char *devicename)
    char *cur, *bp;
    POOL_MEM namelist;
 
+   Dmsg2(200, "need_to_list_device devicenames %s, devicename %s\n",
+         devicenames, devicename);
+
    /*
     * Make a local copy that we can split on ','
     */
@@ -185,6 +187,8 @@ static bool need_to_list_device(const char *devicenames, const char *devicename)
       cur = bp;
    }
 
+   Dmsg0(200, "need_to_list_device no listing needed\n");
+
    return false;
 }
 
@@ -214,6 +218,49 @@ static bool need_to_list_device(const char *devicenames, DEVRES *device)
    return true;
 }
 
+/*
+ * Trigger the specific eventtype to get status information from any plugin that
+ * registered the event to return specific device information.
+ */
+static void trigger_device_status_hook(JCR *jcr,
+                                       DEVRES *device,
+                                       STATUS_PKT *sp,
+                                       bsdEventType eventType)
+{
+   bsdDevStatTrig dst;
+
+   dst.device = device;
+   dst.status = get_pool_memory(PM_MESSAGE);
+   dst.status_length = 0;
+
+   if (generate_plugin_event(jcr, eventType, &dst) == bRC_OK) {
+      if (dst.status_length > 0) {
+         sendit(dst.status, dst.status_length, sp);
+      }
+   }
+   free_pool_memory(dst.status);
+}
+
+/*
+ * Ask the device if it want to log something specific in the status overview.
+ */
+static void get_device_specific_status(DEVRES *device,
+                                       STATUS_PKT *sp)
+{
+   bsdDevStatTrig dst;
+
+   dst.device = device;
+   dst.status = get_pool_memory(PM_MESSAGE);
+   dst.status_length = 0;
+
+   if (device->dev->device_status(&dst)) {
+      if (dst.status_length > 0) {
+         sendit(dst.status, dst.status_length, sp);
+      }
+   }
+   free_pool_memory(dst.status);
+}
+
 static void list_devices(JCR *jcr, STATUS_PKT *sp, const char *devicenames)
 {
    int len;
@@ -300,7 +347,9 @@ static void list_devices(JCR *jcr, STATUS_PKT *sp, const char *devicenames)
             sendit(msg, len, sp);
          }
 
+         get_device_specific_status(device, sp);
          trigger_device_status_hook(jcr, device, sp, bsdEventDriveStatus);
+
          send_blocked_status(dev, sp);
 
          if (dev->can_append()) {
@@ -346,6 +395,8 @@ static void list_devices(JCR *jcr, STATUS_PKT *sp, const char *devicenames)
             len = Mmsg(msg, _("\nDevice \"%s\" is not open or does not exist.\n"), device->name());
             sendit(msg, len, sp);
          }
+
+         get_device_specific_status(device, sp);
       }
 
       if (!sp->api) {
@@ -360,6 +411,60 @@ static void list_devices(JCR *jcr, STATUS_PKT *sp, const char *devicenames)
    }
 }
 
+/*
+ * List Volumes
+ */
+static void list_volumes(STATUS_PKT *sp, const char *devicenames)
+{
+   int len;
+   VOLRES *vol;
+   POOL_MEM msg(PM_MESSAGE);
+
+   foreach_vol(vol) {
+      DEVICE *dev = vol->dev;
+
+      if (dev) {
+         if (devicenames && !need_to_list_device(devicenames, dev->device)) {
+            continue;
+         }
+
+         len = Mmsg(msg, "%s on device %s\n", vol->vol_name, dev->print_name());
+         sendit(msg.c_str(), len, sp);
+         len = Mmsg(msg, "    Reader=%d writers=%d reserves=%d volinuse=%d\n",
+                    dev->can_read() ? 1 : 0, dev->num_writers, dev->num_reserved(),
+                    vol->is_in_use());
+         sendit(msg.c_str(), len, sp);
+      } else {
+         len = Mmsg(msg, "Volume %s no device. volinuse= %d\n",
+                    vol->vol_name, vol->is_in_use());
+         sendit(msg.c_str(), len, sp);
+      }
+   }
+   endeach_vol(vol);
+
+   foreach_read_vol(vol) {
+      DEVICE *dev = vol->dev;
+
+      if (dev) {
+         if (devicenames && !need_to_list_device(devicenames, dev->device)) {
+            continue;
+         }
+
+         len = Mmsg(msg, "Read volume: %s on device %s\n", vol->vol_name, dev->print_name());
+         sendit(msg.c_str(), len, sp);
+         len = Mmsg(msg, "    Reader=%d writers=%d reserves=%d volinuse=%d JobId=%d\n",
+                    dev->can_read() ? 1 : 0, dev->num_writers, dev->num_reserved(),
+                    vol->is_in_use(), vol->get_jobid());
+         sendit(msg.c_str(), len, sp);
+      } else {
+         len = Mmsg(msg, "Read Volume: %s no device. volinuse= %d\n",
+                    vol->vol_name, vol->is_in_use());
+         sendit(msg.c_str(), len, sp);
+      }
+   }
+   endeach_read_vol(vol);
+}
+
 static void list_status_header(STATUS_PKT *sp)
 {
    int len;
@@ -452,25 +557,6 @@ static void list_status_header(STATUS_PKT *sp)
    }
 }
 
-static void trigger_device_status_hook(JCR *jcr,
-                                       DEVRES *device,
-                                       STATUS_PKT *sp,
-                                       bsdEventType eventType)
-{
-   bsdDevStatTrig dst;
-
-   dst.device = device;
-   dst.status = get_pool_memory(PM_MESSAGE);
-   dst.status_length = 0;
-
-   if (generate_plugin_event(jcr, eventType, &dst) == bRC_OK) {
-      if (dst.status_length > 0) {
-         sendit(dst.status, dst.status_length, sp);
-      }
-   }
-   free_pool_memory(dst.status);
-}
-
 static void send_blocked_status(DEVICE *dev, STATUS_PKT *sp)
 {
    int len;
@@ -744,6 +830,34 @@ static void list_running_jobs(STATUS_PKT *sp)
    }
 }
 
+/*
+ * Send any reservation messages queued for this jcr
+ */
+static inline void send_drive_reserve_messages(JCR *jcr, STATUS_PKT *sp)
+{
+   int i;
+   alist *msgs;
+   char *msg;
+
+   jcr->lock();
+   msgs = jcr->reserve_msgs;
+   if (!msgs || msgs->size() == 0) {
+      goto bail_out;
+   }
+   for (i = msgs->size() - 1; i >= 0; i--) {
+      msg = (char *)msgs->get(i);
+      if (msg) {
+         sendit("   ", 3, sp);
+         sendit(msg, strlen(msg), sp);
+      } else {
+         break;
+      }
+   }
+
+bail_out:
+   jcr->unlock();
+}
+
 static void list_jobs_waiting_on_reservation(STATUS_PKT *sp)
 {
    int len;
@@ -759,7 +873,7 @@ static void list_jobs_waiting_on_reservation(STATUS_PKT *sp)
       if (!jcr->reserve_msgs) {
          continue;
       }
-      send_drive_reserve_messages(jcr, sendit, sp);
+      send_drive_reserve_messages(jcr, sp);
    }
    endeach_jcr(jcr);
 
@@ -876,7 +990,7 @@ static void list_terminated_jobs(STATUS_PKT *sp)
    }
 }
 
-/*
+/**
  * Convert Job Level into a string
  */
 static const char *level_to_str(int level)
@@ -924,7 +1038,7 @@ static const char *level_to_str(int level)
    return str;
 }
 
-/*
+/**
  * Send to Director
  */
 static void sendit(const char *msg, int len, STATUS_PKT *sp)
@@ -958,7 +1072,7 @@ static void sendit(POOL_MEM &msg, int len, STATUS_PKT *sp)
    }
 }
 
-/*
+/**
  * Status command from Director
  */
 bool status_cmd(JCR *jcr)
@@ -985,7 +1099,7 @@ bool status_cmd(JCR *jcr)
    return true;
 }
 
-/*
+/**
  * .status command from Director
  */
 bool dotstatus_cmd(JCR *jcr)
@@ -1037,7 +1151,7 @@ bool dotstatus_cmd(JCR *jcr)
        list_devices(jcr, &sp, NULL);
    } else if (bstrcasecmp(cmd.c_str(), "volumes")) {
        sp.api = true;
-       list_volumes(sendit, &sp);
+       list_volumes(&sp, NULL);
    } else if (bstrcasecmp(cmd.c_str(), "spooling")) {
        sp.api = true;
        list_spool_stats(sendit, &sp);
@@ -1061,7 +1175,7 @@ bool dotstatus_cmd(JCR *jcr)
 #if defined(HAVE_WIN32)
 int bareosstat = 0;
 
-/*
+/**
  * Return a one line status for the tray monitor
  */
 char *bareos_status(char *buf, int buf_len)
