@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2013 Free Software Foundation Europe e.V.
-   Copyright (C) 2015-2015 Bareos GmbH & Co. KG
+   Copyright (C) 2015-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -20,12 +20,13 @@
    02110-1301, USA.
 */
 /*
- * Volume management functions for Storage Daemon
- *
  * Kern Sibbald, MM
- *
  * Split from reserve.c October 2008
  */
+/**
+ * @file
+ * Volume management functions for Storage Daemon
+ */
 
 #include "bareos.h"
 #include "stored.h"
@@ -38,14 +39,21 @@ static dlist *read_vol_list = NULL;
 static bthread_mutex_t read_vol_lock = BTHREAD_MUTEX_PRIORITY(PRIO_SD_READ_VOL_LIST);
 
 /* Global static variables */
+#ifdef SD_DEBUG_LOCK
 int vol_list_lock_count = 0;
+int read_vol_list_lock_count = 0;
+#else
+static int vol_list_lock_count = 0;
+static int read_vol_list_lock_count = 0;
+#endif
 
 /* Forward referenced functions */
 static void free_vol_item(VOLRES *vol);
+static void free_read_vol_item(VOLRES *vol);
 static VOLRES *new_vol_item(DCR *dcr, const char *VolumeName);
 static void debug_list_volumes(const char *imsg);
 
-/*
+/**
  * For append volumes the key is the VolumeName.
  */
 static int compare_by_volumename(void *item1, void *item2)
@@ -59,7 +67,7 @@ static int compare_by_volumename(void *item1, void *item2)
    return strcmp(vol1->vol_name, vol2->vol_name);
 }
 
-/*
+/**
  * For read volumes the key is JobId, VolumeName.
  */
 static int read_compare(void *item1, void *item2)
@@ -86,14 +94,14 @@ bool is_vol_list_empty()
    return vol_list->empty();
 }
 
-/*
+/**
  *  Initialized the main volume list. Note, we are using a recursive lock.
  */
 void init_vol_list_lock()
 {
    int errstat;
 
-   if ((errstat=rwl_init(&vol_list_lock, PRIO_SD_VOL_LIST)) != 0) {
+   if ((errstat = rwl_init(&vol_list_lock, PRIO_SD_VOL_LIST)) != 0) {
       berrno be;
       Emsg1(M_ABORT, 0, _("Unable to initialize volume list lock. ERR=%s\n"),
             be.bstrerror(errstat));
@@ -105,7 +113,7 @@ void term_vol_list_lock()
    rwl_destroy(&vol_list_lock);
 }
 
-/*
+/**
  * This allows a given thread to recursively call to lock_volumes()
  */
 void _lock_volumes(const char *file, int line)
@@ -113,10 +121,10 @@ void _lock_volumes(const char *file, int line)
    int errstat;
 
    vol_list_lock_count++;
-   if ((errstat=rwl_writelock_p(&vol_list_lock, file, line)) != 0) {
+   if ((errstat = rwl_writelock_p(&vol_list_lock, file, line)) != 0) {
       berrno be;
       Emsg2(M_ABORT, 0, "rwl_writelock failure. stat=%d: ERR=%s\n",
-           errstat, be.bstrerror(errstat));
+            errstat, be.bstrerror(errstat));
    }
 }
 
@@ -125,25 +133,26 @@ void _unlock_volumes()
    int errstat;
 
    vol_list_lock_count--;
-   if ((errstat=rwl_writeunlock(&vol_list_lock)) != 0) {
+   if ((errstat = rwl_writeunlock(&vol_list_lock)) != 0) {
       berrno be;
       Emsg2(M_ABORT, 0, "rwl_writeunlock failure. stat=%d: ERR=%s\n",
-           errstat, be.bstrerror(errstat));
+            errstat, be.bstrerror(errstat));
    }
 }
 
-#define lock_read_volumes() lock_read_volumes_p(__FILE__, __LINE__)
-static void lock_read_volumes_p(const char *file, int line)
+void _lock_read_volumes(const char *file, int line)
 {
+   read_vol_list_lock_count++;
    bthread_mutex_lock_p(&read_vol_lock, file, line);
 }
 
-static void unlock_read_volumes()
+void _unlock_read_volumes()
 {
+   read_vol_list_lock_count--;
    bthread_mutex_unlock(&read_vol_lock);
 }
 
-/*
+/**
  * Add a volume to the read list.
  *
  * Note, we use VOLRES because it simplifies the code
@@ -165,15 +174,15 @@ void add_read_volume(JCR *jcr, const char *VolumeName)
    lock_read_volumes();
    vol = (VOLRES *)read_vol_list->binary_insert(nvol, read_compare);
    if (vol != nvol) {
-      free_vol_item(nvol);
+      free_read_vol_item(nvol);
       Dmsg2(dbglvl, "read_vol=%s JobId=%d already in list.\n", VolumeName, jcr->JobId);
    } else {
-      Dmsg2(dbglvl, "add read_vol=%s JobId=%d\n", VolumeName, jcr->JobId);
+      Dmsg2(dbglvl, "add_read_vol=%s JobId=%d\n", VolumeName, jcr->JobId);
    }
    unlock_read_volumes();
 }
 
-/*
+/**
  * Remove a given volume name from the read list.
  */
 void remove_read_volume(JCR *jcr, const char *VolumeName)
@@ -194,13 +203,13 @@ void remove_read_volume(JCR *jcr, const char *VolumeName)
    }
    if (fvol) {
       read_vol_list->remove(fvol);
-      free_vol_item(fvol);
+      free_read_vol_item(fvol);
    }
    unlock_read_volumes();
 // pthread_cond_broadcast(&wait_next_vol);
 }
 
-/*
+/**
  * Search for a Volume name in the read Volume list.
  *
  * Returns: VOLRES entry on success
@@ -235,7 +244,7 @@ static VOLRES *find_read_volume(const char *VolumeName)
    return fvol;
 }
 
-/*
+/**
  * List Volumes -- this should be moved to status.c
  */
 enum {
@@ -261,52 +270,7 @@ static void debug_list_volumes(const char *imsg)
    endeach_vol(vol);
 }
 
-/*
- * List Volumes -- this should be moved to status.c
- */
-void list_volumes(void sendit(const char *msg, int len, void *sarg), void *arg)
-{
-   VOLRES *vol;
-   POOL_MEM msg(PM_MESSAGE);
-   int len;
-
-   foreach_vol(vol) {
-      DEVICE *dev = vol->dev;
-      if (dev) {
-         len = Mmsg(msg, "%s on device %s\n", vol->vol_name, dev->print_name());
-         sendit(msg.c_str(), len, arg);
-         len = Mmsg(msg, "    Reader=%d writers=%d reserves=%d volinuse=%d\n",
-                    dev->can_read() ? 1 : 0, dev->num_writers, dev->num_reserved(),
-                    vol->is_in_use());
-         sendit(msg.c_str(), len, arg);
-      } else {
-         len = Mmsg(msg, "Volume %s no device. volinuse= %d\n",
-                    vol->vol_name, vol->is_in_use());
-         sendit(msg.c_str(), len, arg);
-      }
-   }
-   endeach_vol(vol);
-
-   lock_read_volumes();
-   foreach_dlist(vol, read_vol_list) {
-      DEVICE *dev = vol->dev;
-      if (dev) {
-         len = Mmsg(msg, "Read volume: %s on device %s\n", vol->vol_name, dev->print_name());
-         sendit(msg.c_str(), len, arg);
-         len = Mmsg(msg, "    Reader=%d writers=%d reserves=%d volinuse=%d JobId=%d\n",
-                    dev->can_read() ? 1 : 0, dev->num_writers, dev->num_reserved(),
-                    vol->is_in_use(), vol->get_jobid());
-         sendit(msg.c_str(), len, arg);
-      } else {
-         len = Mmsg(msg, "Volume: %s no device. volinuse= %d\n",
-                    vol->vol_name, vol->is_in_use());
-         sendit(msg.c_str(), len, arg);
-      }
-   }
-   unlock_read_volumes();
-}
-
-/*
+/**
  * Create a Volume item to put in the Volume list
  * Ensure that the device points to it.
  */
@@ -350,7 +314,29 @@ static void free_vol_item(VOLRES *vol)
    }
 }
 
-/*
+static void free_read_vol_item(VOLRES *vol)
+{
+   DEVICE *dev = NULL;
+
+   vol->dec_use_count();
+   vol->Lock();
+   if (vol->use_count() > 0) {
+      vol->Unlock();
+      return;
+   }
+   vol->Unlock();
+   free(vol->vol_name);
+   if (vol->dev) {
+      dev = vol->dev;
+   }
+   vol->destroy_mutex();
+   free(vol);
+   if (dev) {
+      dev->vol = NULL;
+   }
+}
+
+/**
  * Put a new Volume entry in the Volume list. This
  * effectively reserves the volume so that it will
  * not be mounted again.
@@ -594,7 +580,7 @@ get_out:
    return vol;
 }
 
-/*
+/**
  * Start walk of vol chain
  * The proper way to walk the vol chain is:
  *
@@ -624,7 +610,7 @@ VOLRES *vol_walk_start()
    return vol;
 }
 
-/*
+/**
  * Get next vol from chain, and release current one
  */
 VOLRES *vol_walk_next(VOLRES *prev_vol)
@@ -646,7 +632,7 @@ VOLRES *vol_walk_next(VOLRES *prev_vol)
    return vol;
 }
 
-/*
+/**
  * Release last vol referenced
  */
 void vol_walk_end(VOLRES *vol)
@@ -661,6 +647,72 @@ void vol_walk_end(VOLRES *vol)
 }
 
 /*
+ * Start walk of vol chain
+ * The proper way to walk the vol chain is:
+ *
+ * VOLRES *vol;
+ * foreach_read_vol(vol) {
+ *    ...
+ * }
+ * endeach_read_vol(vol);
+ *
+ * It is possible to leave out the endeach_read_vol(vol), but in that case,
+ * the last vol referenced must be explicitly released with:
+ *
+ * free_read_vol_item(vol);
+ */
+VOLRES *read_vol_walk_start()
+{
+   VOLRES *vol;
+   lock_read_volumes();
+   vol = (VOLRES *)read_vol_list->first();
+   if (vol) {
+      vol->inc_use_count();
+      Dmsg2(dbglvl, "Inc walk_start use_count=%d volname=%s\n",
+            vol->use_count(), vol->vol_name);
+   }
+   unlock_read_volumes();
+
+   return vol;
+}
+
+/*
+ * Get next vol from chain, and release current one
+ */
+VOLRES *read_vol_walk_next(VOLRES *prev_vol)
+{
+   VOLRES *vol;
+
+   lock_read_volumes();
+   vol = (VOLRES *)read_vol_list->next(prev_vol);
+   if (vol) {
+      vol->inc_use_count();
+      Dmsg2(dbglvl, "Inc walk_next use_count=%d volname=%s\n",
+            vol->use_count(), vol->vol_name);
+   }
+   if (prev_vol) {
+      free_read_vol_item(prev_vol);
+   }
+   unlock_read_volumes();
+
+   return vol;
+}
+
+/*
+ * Release last vol referenced
+ */
+void read_vol_walk_end(VOLRES *vol)
+{
+   if (vol) {
+      lock_read_volumes();
+      Dmsg2(dbglvl, "Free walk_end use_count=%d volname=%s\n",
+            vol->use_count(), vol->vol_name);
+      free_read_vol_item(vol);
+      unlock_read_volumes();
+   }
+}
+
+/**
  * Search for a Volume name in the Volume list.
  *
  * Returns: VOLRES entry on success
@@ -688,7 +740,7 @@ static VOLRES *find_volume(const char *VolumeName)
    return fvol;
 }
 
-/*
+/**
  * Free a Volume from the Volume list if it is no longer used
  * Note, for tape drives we want to remember where the Volume
  * was when last used, so rather than free the volume entry,
@@ -742,7 +794,7 @@ bool volume_unused(DCR *dcr)
    }
 }
 
-/*
+/**
  * Unconditionally release the volume entry
  */
 bool free_volume(DEVICE *dev)
@@ -790,7 +842,7 @@ bool free_volume(DEVICE *dev)
    return true;
 }
 
-/*
+/**
  * Create the Volume list
  */
 void create_volume_lists()
@@ -804,7 +856,7 @@ void create_volume_lists()
    }
 }
 
-/*
+/**
  * Free normal append volumes list
  */
 static inline void free_volume_list(const char *what, dlist *vollist)
@@ -823,7 +875,7 @@ static inline void free_volume_list(const char *what, dlist *vollist)
    }
 }
 
-/*
+/**
  * Release all Volumes from the list
  */
 void free_volume_lists()
@@ -845,7 +897,7 @@ void free_volume_lists()
    }
 }
 
-/*
+/**
  * Determine if caller can write on volume
  */
 bool DCR::can_i_write_volume()
@@ -861,7 +913,7 @@ bool DCR::can_i_write_volume()
    return can_i_use_volume();
 }
 
-/*
+/**
  * Determine if caller can read or write volume
  */
 bool DCR::can_i_use_volume()
@@ -902,7 +954,7 @@ get_out:
    return rtn;
 }
 
-/*
+/**
  * Create a temporary copy of the volume list.  We do this,
  * to avoid having the volume list locked during the
  * call to reserve_device(), which would cause a deadlock.
@@ -940,7 +992,7 @@ dlist *dup_vol_list(JCR *jcr)
    return temp_vol_list;
 }
 
-/*
+/**
  * Free the specified temp list.
  */
 void free_temp_vol_list(dlist *temp_vol_list)
