@@ -2,7 +2,7 @@
    BAREOS® - Backup Archiving REcovery Open Sourced
 
    Copyright (C) 2000-2013 Free Software Foundation Europe e.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -25,6 +25,11 @@
  * Kern Sibbald, MMXIII
  *
  */
+/**
+ * @file
+ * volume management definitions
+ *
+ */
 
 /*
  * Some details of how volume reservations work
@@ -45,22 +50,25 @@ class VOLRES;
 VOLRES *vol_walk_start();
 VOLRES *vol_walk_next(VOLRES *prev_vol);
 void vol_walk_end(VOLRES *vol);
+VOLRES *read_vol_walk_start();
+VOLRES *read_vol_walk_next(VOLRES *prev_vol);
+void read_vol_walk_end(VOLRES *vol);
 
-/*
+/**
  * Volume reservation class -- see vol_mgr.c and reserve.c
  */
 class VOLRES {
-   bool m_swapping;                   /* set when swapping to another drive */
-   bool m_in_use;                     /* set when volume reserved or in use */
-   bool m_reading;                    /* set when reading */
-   slot_number_t m_slot;              /* slot of swapping volume */
-   uint32_t m_JobId;                  /* JobId for read volumes */
-   volatile int32_t m_use_count;      /* Use count */
-   pthread_mutex_t m_mutex;           /* Vol muntex */
+   bool m_swapping;                   /**< set when swapping to another drive */
+   bool m_in_use;                     /**< set when volume reserved or in use */
+   bool m_reading;                    /**< set when reading */
+   slot_number_t m_slot;              /**< slot of swapping volume */
+   uint32_t m_JobId;                  /**< JobId for read volumes */
+   volatile int32_t m_use_count;      /**< Use count */
+   pthread_mutex_t m_mutex;           /**< Vol muntex */
 public:
    dlink link;
-   char *vol_name;                    /* Volume name */
-   DEVICE *dev;                       /* Pointer to device to which we are attached */
+   char *vol_name;                    /**< Volume name */
+   DEVICE *dev;                       /**< Pointer to device to which we are attached */
 
    void init_mutex() { pthread_mutex_init(&m_mutex, NULL); };
    void destroy_mutex() { pthread_mutex_destroy(&m_mutex); };
@@ -87,8 +95,13 @@ public:
 };
 
 #define foreach_vol(vol) \
-   for (vol=vol_walk_start(); vol; (vol=vol_walk_next(vol)) )
+   for (vol=vol_walk_start(); vol; (vol = vol_walk_next(vol)) )
 
 #define endeach_vol(vol) vol_walk_end(vol)
 
+#define foreach_read_vol(vol) \
+   for (vol=read_vol_walk_start(); vol; (vol = read_vol_walk_next(vol)) )
+
+#define endeach_read_vol(vol) read_vol_walk_end(vol)
+
 #endif
