@@ -21,18 +21,18 @@
    02110-1301, USA.
 */
 /*
- * Windows File API device abstraction.
- *
  * Kern Sibbald, MM
- *
- * Extracted from other source files Marco van Wieringen, December 2013
+ */
+/**
+ * @file
+ * Windows File API device abstraction.
  */
 
 #include "bareos.h"
 #include "stored.h"
 #include "win32_file_device.h"
 
-/*
+/**
  * (Un)mount the device (For a FILE device)
  */
 static bool do_mount(DCR *dcr, bool mount, int dotimeout)
@@ -109,7 +109,14 @@ static bool do_mount(DCR *dcr, bool mount, int dotimeout)
       entry = (struct dirent *)malloc(sizeof(struct dirent) + name_max + 1000);
       count = 0;
       while (1) {
+
+#ifdef USE_READDIR_R
          if ((readdir_r(dp, entry, &result) != 0) || (result == NULL)) {
+#else
+         result = readdir(dp);
+         if (result == NULL) {
+#endif
+
             dcr->dev->dev_errno = EIO;
             Dmsg2(129, "do_mount: failed to find suitable file in dir %s (dev=%s)\n",
                   device->mount_point, dcr->dev->print_name());
@@ -152,7 +159,7 @@ get_out:
    return true;
 }
 
-/*
+/**
  * Mount the device.
  *
  * If timeout, wait until the mount command returns 0.
@@ -169,7 +176,7 @@ bool win32_file_device::mount_backend(DCR *dcr, int timeout)
    return retval;
 }
 
-/*
+/**
  * Unmount the device
  *
  * If timeout, wait until the unmount command returns 0.
